/*
ssc (static site checker)
Copyright (c) 2020-2022 Dylan Harris
https://dylanharris.org/

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public Licence as published by
the Free Software Foundation, either version 3 of the Licence, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public Licence for more details.

You should have received a copy of the GNU General Public
Licence along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "main/standard.h"
#include "microdata/microdata_itemid.h"
#include "element/element.h"
#include "utility/quote.h"

struct itemid
{   ::std::string path_;
    int line_ = 0;
    itemid (const ::std::string& path, const int line)
        :   path_ (path), line_ (line) { } };

typedef ::std::map < ::std::string, itemid > miid_t;
miid_t miid;

bool note_itemid (nitpick& nits, const html_version& , const ::std::string& id, const ::std::string& path, const int line)
{   auto i = miid.emplace (id, itemid (path, line));
    if (i.second) return true;
    nits.pick (nit_bad_itemid, ed_microdata, "5.1.4 Global identifiers for items", es_warning, ec_microdata, quote (id), " is not unique; see line ", i.first -> second.line_, " of ", i.first -> second.path_);
    return false; }

::std::string report_itemids ()
{   ::std::string res;
    for (auto i : miid)
    {   mmac_t mac;
        mac.emplace (nm_id_name, i.first);
        mac.emplace (nm_id_page, i.second.path_);
        mac.emplace (nm_id_line, ::boost::lexical_cast < ::std::string > (i.second.line_));
        res += macro.apply (ns_itemid, mac); }
    if (! res.empty ())
        res = macro.apply (ns_id_head) + res + macro.apply (ns_id_foot);
    return res; }

bool empty_itemid ()
{   return miid.empty (); }

bool invalid_itemid (nitpick& nits, const html_version& v, const ::std::string& id, const element* const e)
{   VERIFY_NOT_NULL (e, __FILE__, __LINE__);
    return ! note_itemid (nits, v, id, e -> name (), e -> line ()); }
