/*
ssc (static site checker)
Copyright (c) 2020-2022 Dylan Harris
https://dylanharris.org/

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public Licence as published by
the Free Software Foundation, either version 3 of the Licence, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public Licence for more details.

You should have received a copy of the GNU General Public
Licence along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "main/standard.h"
#include "schema/schema_name.h"
#include "utility/common.h"

struct property_name
{   const char* name_ = nullptr;
    e_schema root_;
    e_schema_property prop_; };

property_name namtab [] =
{   // open graph article
    { "author", s_article, og_article_author },
    { "expiration_time", s_article, og_article_expiration_time },
    { "modified_time", s_article, og_article_modified_time },
    { "published_time", s_article, og_article_published_time },
    { "section", s_article, og_article_section },
    { "tag", s_article, og_article_tag },

    // activity streams etc.
    { "accuracy", s_as, asp_accuracy },
    { "actor", s_as, asp_actor },
    { "altitude", s_as, asp_altitude },
    { "anyOf", s_as, asp_anyof },
    { "attachment", s_as, asp_attachment },
    { "attachments", s_as, asp_attachments },
    { "attributedTo", s_as, asp_attributedto },
    { "audience", s_as, asp_audience },
    { "author", s_as, asp_author },
    { "bcc", s_as, asp_bcc },
    { "bto", s_as, asp_bto },
    { "cc", s_as, asp_cc },
    { "closed", s_as, asp_closed },
    { "content", s_as, asp_content },
    { "contentMap", s_as, asp_contentmap },
    { "context", s_as, asp_context },
    { "current", s_as, asp_current },
    { "deleted", s_as, asp_deleted },
    { "describes", s_as, asp_describes },
    { "displayName", s_as, asp_displayname },
    { "downstreamDuplicates", s_as, asp_downstreamduplicates },
    { "duration", s_as, asp_duration },
    { "endTime", s_as, asp_endtime },
    { "first", s_as, asp_first },
    { "formerType", s_as, asp_formertype },
    { "generator", s_as, asp_generator },
    { "height", s_as, asp_height },
    { "href", s_as, asp_href },
    { "hreflang", s_as, asp_hreflang },
    { "icon", s_as, asp_icon },
    { "id", s_as, asp_id },
    { "image", s_as, asp_image },
    { "inReplyTo", s_as, asp_inreplyto },
    { "instrument", s_as, asp_instrument },
    { "items", s_as, asp_items },
    { "last", s_as, asp_last },
    { "latitude", s_as, asp_latitude },
    { "location", s_as, asp_location },
    { "longitude", s_as, asp_longitude },
    { "mediaType", s_as, asp_mediatype },
    { "name", s_as, asp_name },
    { "nameMap", s_as, asp_namemap },
    { "next", s_as, asp_next },
    { "object", s_as, asp_object },
    { "objectType", s_as, asp_objecttype },
    { "oneOf", s_as, asp_oneof },
    { "orderedItems", s_as, asp_ordereditems },
    { "origin", s_as, asp_origin },
    { "partOf", s_as, asp_partof },
    { "prev", s_as, asp_prev },
    { "preview", s_as, asp_preview },
    { "provider", s_as, asp_provider },
    { "published", s_as, asp_published },
    { "radius", s_as, asp_radius },
    { "rating", s_as, asp_rating },
    { "rel", s_as, asp_rel },
    { "relationship", s_as, asp_relationship },
    { "replies", s_as, asp_replies },
    { "result", s_as, asp_result },
    { "startIndex", s_as, asp_startindex },
    { "startTime", s_as, asp_starttime },
    { "subject", s_as, asp_subject },
    { "summary", s_as, asp_summary },
    { "summaryMap", s_as, asp_summarymap },
    { "tag", s_as, asp_tag },
    { "tags", s_as, asp_tags },
    { "target", s_as, asp_target },
    { "title", s_as, asp_title },
    { "to", s_as, asp_to },
    { "totalItems", s_as, asp_totalitems },
    { "type", s_as, asp_type },
    { "units", s_as, asp_units },
    { "updated", s_as, asp_updated },
    { "upstreamDuplicates", s_as, asp_upstreamduplicates },
    { "url", s_as, asp_url },
    { "verb", s_as, asp_verb },
    { "width", s_as, asp_width },

    // bibo (dreadful spec, guessed)
    { "abstract", s_bibo, bibo_abstract },
    { "affirmedBy", s_bibo, bibo_affirmedby },
    { "annotates", s_bibo, bibo_annotates },
    { "argued", s_bibo, bibo_argued },
    { "asin", s_bibo, bibo_asin },
    { "authorList", s_bibo, bibo_authorlist },
    { "chapter", s_bibo, bibo_chapternumber },
    { "citedBy", s_bibo, bibo_citedby },
    { "cites", s_bibo, bibo_cites },
    { "coden", s_bibo, bibo_coden },
    { "content", s_bibo, bibo_content },
    { "contributorList", s_bibo, bibo_contributorlist },
    { "court", s_bibo, bibo_court },
    { "degree", s_bibo, bibo_degree },
    { "direcor", s_bibo, bibo_direcor },
    { "distributor", s_bibo, bibo_distributor },
    { "doi", s_bibo, bibo_doi },
    { "eanucc13", s_bibo, bibo_eanucc13 },
    { "edition", s_bibo, bibo_edition },
    { "editor", s_bibo, bibo_editor },
    { "editorList", s_bibo, bibo_editorlist },
    { "eissn", s_bibo, bibo_eissn },
    { "gtin14", s_bibo, bibo_gtin14 },
    { "handle", s_bibo, bibo_handle },
    { "identifier", s_bibo, bibo_identifier },
    { "interviewee", s_bibo, bibo_interviewee },
    { "interviewer", s_bibo, bibo_interviewer },
    { "isbn", s_bibo, bibo_isbn },
    { "isbn10", s_bibo, bibo_isbn10 },
    { "isbn13", s_bibo, bibo_isbn13 },
    { "issn", s_bibo, bibo_issn },
    { "issue", s_bibo, bibo_issuenumber },
    { "issuer", s_bibo, bibo_issuer },
    { "lccn", s_bibo, bibo_lccn },
    { "locator", s_bibo, bibo_locator },
    { "number", s_bibo, bibo_number },
    { "numPages", s_bibo, bibo_numpages },
    { "numVolumes", s_bibo, bibo_numvolumes },
    { "oclcnum", s_bibo, bibo_oclcnum },
    { "organizer", s_bibo, bibo_organiser },
    { "owner", s_bibo, bibo_owner },
    { "pageEnd", s_bibo, bibo_pageend },
    { "pages", s_bibo, bibo_pages },
    { "pageStart", s_bibo, bibo_pagestart },
    { "performer", s_bibo, bibo_performer },
    { "pmid", s_bibo, bibo_pmid },
    { "prefixName", s_bibo, bibo_prefixname },
    { "presentedAt", s_bibo, bibo_presentedat },
    { "presents", s_bibo, bibo_presents },
    { "producer", s_bibo, bibo_producer },
    { "recipient", s_bibo, bibo_recipient },
    { "reproducedIn", s_bibo, bibo_reproducedin },
    { "reversedBy", s_bibo, bibo_reversedby },
    { "reviewOf", s_bibo, bibo_reviewof },
    { "section", s_bibo, bibo_section },
    { "shortDescription", s_bibo, bibo_shortdescription },
    { "shortTitle", s_bibo, bibo_shorttitle },
    { "sici", s_bibo, bibo_sici },
    { "status", s_bibo, bibo_status },
    { "subsequentLegalDecision", s_bibo, bibo_subsequentlegaldecision },
    { "suffixName", s_bibo, bibo_suffixname },
    { "transcriptOf", s_bibo, bibo_transcriptof },
    { "translationof", s_bibo, bibo_translationof },
    { "translator", s_bibo, bibo_translator },
    { "upc", s_bibo, bibo_upc },
    { "uri", s_bibo, bibo_uri },
    { "volume", s_bibo, bibo_volume },

    // open graph book
    { "author", s_book, og_book_author },
    { "isbn", s_book, og_book_isbn },
    { "release_date", s_book, og_book_release_date },
    { "tag", s_book, og_book_tag },

    // creative commons
    { "attributionName", s_cc, cc_attributionname },
    { "attributionURL", s_cc, cc_attributionurl },
    { "deprecatedOn", s_cc, cc_deprecatedon },
    { "jurisdiction", s_cc, cc_jurisdiction },
    { "legalCode", s_cc, cc_legalcode },
    { "license", s_cc, cc_licence },
    { "morePermissions", s_cc, cc_morepermissions },
    { "permits", s_cc, cc_permits },
    { "prohibits", s_cc, cc_prohibits },
    { "requires", s_cc, cc_requires },

    // common tag
    { "label", s_ctag, ctag_label },
    { "means", s_ctag, ctag_means },
    { "tagged", s_ctag, ctag_tagged },
    { "taggingDate", s_ctag, ctag_taggingdate },

    // content
    { "encoded", s_content, content_encoded },
    { "encoding", s_content, content_encoding },
    { "format", s_content, content_format },
    { "item", s_content, content_item },
    { "items", s_content, content_items },
    { "value", s_content, content_value },

    // contrary sausages
    { "aboutUrl", s_csvw, csvw_abouturl, },
    { "base", s_csvw, csvw_base, },
    { "column", s_csvw, csvw_column, },
    { "columnReference", s_csvw, csvw_columnreference, },
    { "commentPrefix", s_csvw, csvw_commentprefix, },
    { "datatype", s_csvw, csvw_datatype, },
    { "decimalChar", s_csvw, csvw_decimalchar, },
    { "default", s_csvw, csvw_default, },
    { "delimiter", s_csvw, csvw_delimiter, },
    { "describes", s_csvw, csvw_describes, },
    { "dialect", s_csvw, csvw_dialect, },
    { "doubleQuote", s_csvw, csvw_doublequote, },
    { "encoding", s_csvw, csvw_encoding, },
    { "foreignKey", s_csvw, csvw_foreignkey, },
    { "format", s_csvw, csvw_format, },
    { "groupChar", s_csvw, csvw_groupchar, },
    { "header", s_csvw, csvw_header, },
    { "headerRowCount", s_csvw, csvw_headerrowcount, },
    { "lang", s_csvw, csvw_lang, },
    { "length", s_csvw, csvw_length, },
    { "lineTerminators", s_csvw, csvw_lineterminators, },
    { "maxExclusive", s_csvw, csvw_maxexclusive, },
    { "maxInclusive", s_csvw, csvw_maxinclusive, },
    { "maxLength", s_csvw, csvw_maxlength, },
    { "minExclusive", s_csvw, csvw_minexclusive, },
    { "minInclusive", s_csvw, csvw_mininclusive, },
    { "minLength", s_csvw, csvw_minlength, },
    { "name", s_csvw, csvw_name, },
    { "note", s_csvw, csvw_note, },
    { "null", s_csvw, csvw_null, },
    { "ordered", s_csvw, csvw_ordered, },
    { "pattern", s_csvw, csvw_pattern, },
    { "primaryKey", s_csvw, csvw_primarykey, },
    { "propertyUrl", s_csvw, csvw_propertyurl, },
    { "quoteChar", s_csvw, csvw_quotechar, },
    { "reference", s_csvw, csvw_reference, },
    { "referencedRow", s_csvw, csvw_referencedrow, },
    { "required", s_csvw, csvw_required, },
    { "resource", s_csvw, csvw_resource, },
    { "row", s_csvw, csvw_row, },
    { "rownum", s_csvw, csvw_rownum, },
    { "rowTitle", s_csvw, csvw_rowtitle, },
    { "schemaReference", s_csvw, csvw_schemareference, },
    { "scriptFormat", s_csvw, csvw_scriptformat, },
    { "separator", s_csvw, csvw_separator, },
    { "skipColumns", s_csvw, csvw_skipcolumns, },
    { "skipBlankRows", s_csvw, csvw_skipblankrows, },
    { "skipInitialSpace", s_csvw, csvw_skipinitialspace, },
    { "skipRows", s_csvw, csvw_skiprows, },
    { "source", s_csvw, csvw_source, },
    { "suppressOutput", s_csvw, csvw_suppressoutput, },
    { "table", s_csvw, csvw_table, },
    { "tableDirection", s_csvw, csvw_tabledirection, },
    { "tableSchema", s_csvw, csvw_tableschema, },
    { "targetFormat", s_csvw, csvw_targetformat, },
    { "textDirection", s_csvw, csvw_textdirection, },
    { "title", s_csvw, csvw_title, },
    { "transformations", s_csvw, csvw_transformations, },
    { "trim", s_csvw, csvw_trim, },
    { "url", s_csvw, csvw_url, },
    { "valueUrl", s_csvw, csvw_valueurl, },
    { "virtual", s_csvw, csvw_virtual, },

    // data quality
    { "computedon", s_daq, daq_computedon, },
    { "expecteddatatype", s_daq, daq_expecteddatatype, },
    { "hasdimension", s_daq, daq_hasdimension, },
    { "hasmetric", s_daq, daq_hasmetric, },
    { "hasobservation", s_daq, daq_hasobservation, },
    { "isestimate", s_daq, daq_isestimate, },
    { "metric", s_daq, daqp_metric, },
    { "requires", s_daq, daq_requires, },
    { "value", s_daq, daq_value, },

    // dublin core elements
    { "contributor", s_dc, dc_contributor },
    { "coverage", s_dc, dc_coverage },
    { "creator", s_dc, dc_creator },
    { "date", s_dc, dc_date },
    { "description", s_dc, dc_description },
    { "format", s_dc, dc_format },
    { "identifier", s_dc, dc_identifier },
    { "language", s_dc, dc_language },
    { "publisher", s_dc, dc_publisher },
    { "relation", s_dc, dc_relation },
    { "rights", s_dc, dc_rights },
    { "source", s_dc, dc_source },
    { "subject", s_dc, dc_subject },
    { "title", s_dc, dc_title },
    { "type", s_dc, dc_type },

    // dublin core vocabulary description
    { "domainIncludes", s_dcam, dcam_domainincludes },
    { "memberOf", s_dcam, dcam_memberof },
    { "rangeIncludes", s_dcam, dcam_rangeincludes },

    // data catalogue
    { "accessURL", s_dcat, dcat_accessurl },
    { "accessService", s_dcat, dcat_accessservice },
    { "beginning", s_dcat, dcat_beginning },
    { "bbox", s_dcat, dcat_bbox },
    { "byteSize", s_dcat, dcat_bytesize },
    { "catalog", s_dcat, dcat_catalogue },
    { "centroid", s_dcat, dcat_centroid },
    { "compressFormat", s_dcat, dcat_compressformat },
    { "contactpoint", s_dcat, dcat_contactpoint },
    { "dataset", s_dcat, dcat_dataset },
    { "distribution", s_dcat, dcat_distribution },
    { "downloadURL", s_dcat, dcat_downloadurl },
    { "end", s_dcat, dcat_end },
    { "endDate", s_dcat, dcat_enddate },
    { "endpointURL", s_dcat, dcat_endpointurl },
    { "endpointDescription", s_dcat, dcat_endpointdescription },
    { "format", s_dcat, dcat_format },
    { "frequency", s_dcat, dcat_frequency },
    { "geometry", s_dcat, dcat_geometry },
    { "hadRole", s_dcat, dcat_hadrole },
    { "hasPart", s_dcat, dcat_haspart },
    { "homepage", s_dcat, dcat_homepage },
    { "keyword", s_dcat, dcat_keyword },
    { "landingPage", s_dcat, dcat_landingpage },
    { "listingdate", s_dcat, dcat_listingdate },
    { "mediaType", s_dcat, dcat_mediatype },
    { "packageFormat", s_dcat, dcat_packageformat },
    { "primarytopic", s_dcat, dcat_primarytopic },
    { "qualifiedRelation", s_dcat, dcat_qualifiedrelation },
    { "record", s_dcat, dcat_record },
    { "relation", s_dcat, dcat_relation },
    { "servesDataset", s_dcat, dcat_servesdataset },
    { "service", s_dcat, dcat_service },
    { "spatialResolutionInMeters", s_dcat, dcat_spatialresolutioninmeters },
    { "startDate", s_dcat, dcat_startdate },
    { "temporalResolution", s_dcat, dcat_temporalresolution },
    { "theme", s_dcat, dcat_theme },
    { "themeTaxonomy", s_dcat, dcat_themetaxonomy },

    // dublin core terms
    { "abstract", s_dct, dct_abstract },
    { "accessRights", s_dct, dct_accessrights },
    { "accrualMethod", s_dct, dct_accrualmethod },
    { "accrualPeriodicity", s_dct, dct_accrualperiodicity },
    { "accrualPolicy", s_dct, dct_accrualpolicy },
    { "alternative", s_dct, dct_alternative },
    { "audience", s_dct, dct_audience },
    { "available", s_dct, dct_available },
    { "bibliographicCitation", s_dct, dct_bibliographiccitation },
    { "conformsTo", s_dct, dct_conformsto },
    { "contributor", s_dct, dct_contributor },
    { "coverage", s_dct, dct_coverage },
    { "created", s_dct, dct_created },
    { "creator", s_dct, dct_creator },
    { "date", s_dct, dct_date },
    { "dateAccepted", s_dct, dct_dateaccepted },
    { "dateCopyrighted", s_dct, dct_datecopyrighted },
    { "dateSubmitted", s_dct, dct_datesubmitted },
    { "description", s_dct, dct_description },
    { "educationLevel", s_dct, dct_educationlevel },
    { "extent", s_dct, dct_extent },
    { "format", s_dct, dct_format },
    { "hasFormat", s_dct, dct_hasformat },
    { "hasPart", s_dct, dct_haspart },
    { "hasVersion", s_dct, dct_hasversion },
    { "identifier", s_dct, dct_identifier },
    { "instructionalMethod", s_dct, dct_instructionalmethod },
    { "isFormatOf", s_dct, dct_isformatof },
    { "isPartOf", s_dct, dct_ispartof },
    { "isReferencedBy", s_dct, dct_isreferencedby },
    { "isReplacedBy", s_dct, dct_isreplacedby },
    { "isRequiredBy", s_dct, dct_isrequiredby },
    { "issued", s_dct, dct_issued },
    { "isVersionOf", s_dct, dct_isversionof },
    { "language", s_dct, dct_language },
    { "license", s_dct, dct_licence },
    { "mediator", s_dct, dct_mediator },
    { "medium", s_dct, dct_medium },
    { "modified", s_dct, dct_modified },
    { "provenance", s_dct, dct_provenance },
    { "publisher", s_dct, dct_publisher },
    { "references", s_dct, dct_references },
    { "relation", s_dct, dct_relation },
    { "replaces", s_dct, dct_replaces },
    { "requires", s_dct, dct_requires },
    { "rights", s_dct, dct_rights },
    { "rightsHolder", s_dct, dct_rightsholder },
    { "source", s_dct, dct_source },
    { "spatial", s_dct, dct_spatial },
    { "subject", s_dct, dct_subject },
    { "tableOfContents", s_dct, dct_tableofcontents },
    { "temporal", s_dct, dct_temporal },
    { "title", s_dct, dct_title },
    { "type", s_dct, dct_type },
    { "valid", s_dct, dct_valid },

    // doap
    { "anonroot", s_doap, doap_anonroot },
    { "audience", s_doap, doap_audience },
    { "blog", s_doap, doap_blog },
    { "browse", s_doap, doap_browse },
    { "bug-database", s_doap, doap_bugdatabase },
    { "category", s_doap, doap_category },
    { "created", s_doap, doap_created },
    { "description", s_doap, doap_description },
    { "developer", s_doap, doap_developer },
    { "developer-forum", s_doap, doap_developerforum },
    { "documenter", s_doap, doap_documenter },
    { "download-mirror", s_doap, doap_downloadmirror },
    { "download-page", s_doap, doap_downloadpage },
    { "file-release", s_doap, doap_filerelease },
    { "helper", s_doap, doap_helper },
    { "homepage", s_doap, doap_homepage },
    { "implements", s_doap, doap_implements },
    { "language", s_doap, doap_language },
    { "license", s_doap, doap_licence },
    { "location", s_doap, doap_location },
    { "mailing-list", s_doap, doap_mailinglist },
    { "maintainer", s_doap, doap_maintainer },
    { "module", s_doap, doap_module },
    { "name", s_doap, doap_name },
    { "old-homepage", s_doap, doap_oldhomepage },
    { "os", s_doap, doap_os },
    { "platform", s_doap, doap_platform },
    { "programming-language", s_doap, doap_programminglanguage },
    { "release", s_doap, doap_release },
    { "repository", s_doap, doap_hasrepository },
    { "repositoryOf", s_doap, doap_repositoryof },
    { "revision", s_doap, doap_revision },
    { "screenshots", s_doap, doap_screenshots },
    { "service-endpoint", s_doap, doap_serviceendpoint },
    { "shortdesc", s_doap, doap_shortdesc },
    { "support-forum", s_doap, doap_supportforum },
    { "tester", s_doap, doap_tester },
    { "translator", s_doap, doap_translator },
    { "vendor", s_doap, doap_vendor },
    { "wiki", s_doap, doap_wiki },

    // data quality
    { "computedOn", s_dqv, dqv_computedon },
    { "expectedDataType", s_dqv, dqv_expecteddatatype },
    { "hasQualityAnnotation", s_dqv, dqv_hasqualityannotation },
    { "hasQualityMeasurement", s_dqv, dqv_hasqualitymeasurement },
    { "hasQualityMetadata", s_dqv, dqv_hasqualitymetadata },
    { "inCategory", s_dqv, dqv_incategory },
    { "inDimension", s_dqv, dqv_indimension },
    { "isMeasurementOf", s_dqv, dqv_ismeasurementof },
    { "value", s_dqv, dqv_value },

    // earl
    { "assertedBy", s_earl, earl_assertedby },
    { "info", s_earl, earl_info },
    { "mainAssertor", s_earl, earl_mainassertor },
    { "mode", s_earl, earl_mode },
    { "outcome", s_earl, earl_outcome },
    { "pointer", s_earl, earl_pointer },
    { "result", s_earl, earl_result },
    { "subject", s_earl, earl_subject },
    { "test", s_earl, earl_test },

    // event
    { "agent", s_event, ep_agent },
    { "agentIn", s_event, ep_agentin },
    { "factor", s_event, ep_factor },
    { "factorIf", s_event, ep_factorof },
    { "hasAgent", s_event, ep_hasagent },
    { "hasFactor", s_event, ep_hasfactor },
    { "hasLiteralFactor", s_event, ep_hasliteralfactor },
    { "hasProduct", s_event, ep_hasproduct },
    { "hasSubEvent", s_event, ep_hassubevent },
    { "isAgentIn", s_event, ep_isagentin },
    { "isFactorOf", s_event, ep_isfactorof },
    { "literalFactor", s_event, ep_literalfactor },
    { "place", s_event, ep_place },
    { "producedIn", s_event, ep_producedin },
    { "product", s_event, ep_product },
    { "subEvent", s_event, ep_subevent },
    { "time", s_event, ep_time },

    // foaf
    { "account", s_foaf, foaf_account },
    { "accountName", s_foaf, foaf_accountname },
    { "accountServiceHomepage", s_foaf, foaf_accountservicehomepage },
    { "age", s_foaf, foaf_age },
    { "aimChatID", s_foaf, foaf_aimchatid },
    { "based_near", s_foaf, foaf_basednear },
    { "birthday", s_foaf, foaf_birthday },
    { "currentProject", s_foaf, foaf_currentproject },
    { "depiction", s_foaf, foaf_depiction },
    { "depicts", s_foaf, foaf_depicts },
    { "dnaChecksum", s_foaf, foaf_dnachecksum },
    { "familyName", s_foaf, foaf_familyname },
    { "family_name", s_foaf, foaf_familyname },
    { "firstName", s_foaf, foaf_firstname },
    { "focus", s_foaf, foaf_focus },
    { "fundedBy", s_foaf, foaf_fundedby },
    { "geekcode", s_foaf, foaf_geekcode },
    { "gender", s_foaf, foaf_gender },
    { "givenName", s_foaf, foaf_givenname },
    { "holdsAccount", s_foaf, foaf_holdsaccount },
    { "homepage", s_foaf, foaf_homepage },
    { "icqChatID", s_foaf, foaf_icqchatid },
    { "img", s_foaf, foaf_img },
    { "interest", s_foaf, foaf_interest },
    { "isPrimaryTopicOf", s_foaf, foaf_isprimarytopicof },
    { "jabberID", s_foaf, foaf_jabberid },
    { "knows", s_foaf, foaf_knows },
    { "lastName", s_foaf, foaf_lastname },
    { "logo", s_foaf, foaf_logo },
    { "made", s_foaf, foaf_made },
    { "maker", s_foaf, foaf_maker },
    { "mbox", s_foaf, foaf_mbox },
    { "mbox_sha1sum", s_foaf, foaf_mboxsha1sum },
    { "member", s_foaf, foaf_member },
    { "membershipClass", s_foaf, foaf_membershipclass },
    { "msnChatID", s_foaf, foaf_msnchatid },
    { "myersBriggs", s_foaf, foaf_myersbriggs },
    { "name", s_foaf, foaf_name },
    { "nick", s_foaf, foaf_nick },
    { "openid", s_foaf, foaf_openid },
    { "page", s_foaf, foaf_page },
    { "pastProject", s_foaf, foaf_pastproject },
    { "phone", s_foaf, foaf_phone },
    { "plan", s_foaf, foaf_plan },
    { "primaryTopic", s_foaf, foaf_primarytopic },
    { "publications", s_foaf, foaf_publications },
    { "schoolHomepage", s_foaf, foaf_schoolhomepage },
    { "sha1", s_foaf, foaf_sha1 },
    { "skypeID", s_foaf, foaf_skypeid },
    { "status", s_foaf, foaf_status },
    { "surname", s_foaf, foaf_surname },
    { "theme", s_foaf, foaf_theme },
    { "thumbnail", s_foaf, foaf_thumbnail },
    { "tipjar", s_foaf, foaf_tipjar },
    { "title", s_foaf, foaf_title },
    { "topic", s_foaf, foaf_topic },
    { "topic_interest", s_foaf, foaf_topic_interest },
    { "weblog", s_foaf, foaf_weblog },
    { "workinfoHomepage", s_foaf, foaf_workinfohomepage },
    { "workplaceHomepage", s_foaf, foaf_workplacehomepage },
    { "yahooChatID", s_foaf, foaf_yahoochatid },

    // frbr core
    { "abridgement", s_frbr, fp_abridgement },
    { "abridgementOf", s_frbr, fp_abridgementof },
    { "adaption", s_frbr, fp_adaption },
    { "adaptionOf", s_frbr, fp_adaptionof },
    { "alternate", s_frbr, fp_alternative },
    { "alternateOf", s_frbr, fp_alternativeof },
    { "arrangement", s_frbr, fp_arrangement },
    { "arrangementOf", s_frbr, fp_arrangementof },
    { "complement", s_frbr, fp_complement },
    { "complementOf", s_frbr, fp_complementof },
    { "creator", s_frbr, fp_creator },
    { "creatorOf", s_frbr, fp_creatorof },
    { "embodiment", s_frbr, fp_embodiment },
    { "embodimentOf", s_frbr, fp_embodimentof },
    { "exemplar", s_frbr, fp_exemplar },
    { "exemplarOf", s_frbr, fp_exemplarof },
    { "imitation", s_frbr, fp_imitation },
    { "imitationOf", s_frbr, fp_imitationof },
    { "owner", s_frbr, fp_owner },
    { "ownerOf", s_frbr, fp_ownerof },
    { "part", s_frbr, fp_part },
    { "partOf", s_frbr, fp_partof },
    { "producer", s_frbr, fp_producer },
    { "producerOf", s_frbr, fp_producerof },
    { "realization", s_frbr, fp_realisation },
    { "realizationOf", s_frbr, fp_realisationof },
    { "realizer", s_frbr, fp_realiser },
    { "realizerOf", s_frbr, fp_realiserof },
    { "reconfiguration", s_frbr, fp_reconfiguration },
    { "reconfigurationOf", s_frbr, fp_reconfigurationof },
    { "relatedEndeavour", s_frbr, fp_relatedendeavour },
    { "reproduction", s_frbr, fp_reproduction },
    { "reproductionOf", s_frbr, fp_reproductionof },
    { "responsibleEntity", s_frbr, fp_responsibleentity },
    { "responsibleEntityOf", s_frbr, fp_responsibleentityof },
    { "revision", s_frbr, fp_revision },
    { "revisionOf", s_frbr, fp_revisionof },
    { "subject", s_frbr, fp_subject },
    { "successor", s_frbr, fp_successor },
    { "successorOf", s_frbr, fp_successorof },
    { "summarization", s_frbr, fp_summary },
    { "summarizationOf", s_frbr, fp_summaryof },
    { "supplement", s_frbr, fp_supplement },
    { "supplementOf", s_frbr, fp_supplementof },
    { "transformation", s_frbr, fp_transformation },
    { "transformationOf", s_frbr, fp_transformationof },
    { "translation", s_frbr, fp_translation },
    { "translationOf", s_frbr, fp_translationof },

    // good relations
    { "acceptedPaymentMethods", s_gr, gr_acceptedpaymentmethods },
    { "addOn", s_gr, gr_addon },
    { "advanceBookingRequirement", s_gr, gr_advancebookingrequirement },
    { "amountOfThisGood", s_gr, gr_amountofgoods },
    { "appliesToDeliveryMethod", s_gr, gr_appliestodeliverymethod },
    { "appliesToPaymentMethod", s_gr, gr_appliestopaymentmethod },
    { "availabilityEnds", s_gr, gr_availabilityends },
    { "availabilityStarts", s_gr, gr_availabilitystarts },
    { "availableAtOrFrom", s_gr, gr_availableatorfrom },
    { "availableDeliveryMethods", s_gr, gr_availabledeliverymethods },
    { "billingIncrement", s_gr, gr_billingincrement },
    { "category", s_gr, gr_category },
    { "closes", s_gr, gr_closes },
    { "color", s_gr, gr_colour },
    { "condition", s_gr, gr_condition },
    { "datatypeProductOrServiceProperty", s_gr, gr_datatypeproductorserviceproperty },
    { "deliveryLeadTime", s_gr, gr_deliveryleadtime },
    { "depth", s_gr, gr_depth },
    { "description", s_gr, gr_description },
    { "displayPosition", s_gr, gr_displayposition },
    { "durationOfWarrantyInMonths", s_gr, gr_durationofwarrantyinmonths },
    { "eligibleCustomerTypes", s_gr, gr_eligiblecustomertypes },
    { "eligibleDuration", s_gr, gr_eligibleduration },
    { "eligibleRegions", s_gr, gr_eligibleregions },
    { "eligibleTransactionVolume", s_gr, gr_eligibletransactionvolume },
    { "equal", s_gr, gr_equal },
    { "greater", s_gr, gr_greater },
    { "greaterOrEqual", s_gr, gr_greaterorequal },
    { "hasBrand", s_gr, gr_hasbrand },
    { "hasBusinessFunction", s_gr, gr_hasbusinessfunction },
    { "hasCurrency", s_gr, gr_hascurrency },
    { "hasCurrencyValue", s_gr, gr_hascurrencyvalue },
    { "hasDUNS", s_gr, gr_hasduns },
    { "hasEAN_UCC-13", s_gr, gr_hasean_ucc_13 },
    { "hasEligibleQuantity", s_gr, gr_haseligiblequantity },
    { "hasGTIN-14", s_gr, gr_hasgtin_14 },
    { "hasGTIN-8", s_gr, gr_hasgtin_8 },
    { "hasGlobalLocationNumber", s_gr, gr_hasgloballocationnumber },
    { "hasISICv4", s_gr, gr_hasisicv4 },
    { "hasInventoryLevel", s_gr, gr_hasinventorylevel },
    { "hasMakeAndModel", s_gr, gr_hasmakeandmodel },
    { "hasManufacturer", s_gr, gr_hasmanufacturer },
    { "hasMaxCurrencyValue", s_gr, gr_hasmaxcurrencyvalue },
    { "hasMaxValue", s_gr, gr_hasmaxvalue },
    { "hasMaxValueFloat", s_gr, gr_hasmaxvaluefloat },
    { "hasMaxValueInteger", s_gr, gr_hasmaxvalueinteger },
    { "hasMinCurrencyValue", s_gr, gr_hasmincurrencyvalue },
    { "hasMinValue", s_gr, gr_hasminvalue },
    { "hasMinValueFloat", s_gr, gr_hasminvaluefloat },
    { "hasMinValueInteger", s_gr, gr_hasminvalueinteger },
    { "hasMPN", s_gr, gr_hasmpn },
    { "hasNAICS", s_gr, gr_hasnaics },
    { "hasNext", s_gr, gr_hasnext },
    { "hasOpeningHoursDayOfWeek", s_gr, gr_hasopeninghoursdayofweek },
    { "hasOpeningHoursSpecification", s_gr, gr_hasopeninghoursspecification },
    { "hasPOS", s_gr, gr_haspos },
    { "hasPrevious", s_gr, gr_hasprevious },
    { "hasPriceSpecification", s_gr, gr_haspricespecification },
    { "hasStockKeepingUnit", s_gr, gr_hasstockkeepingunit },
    { "hasUnitOfMeasurement", s_gr, gr_hasunitofmeasurement },
    { "hasValue", s_gr, gr_hasvalue },
    { "hasValueFloat", s_gr, gr_hasvaluefloat },
    { "hasValueInteger", s_gr, gr_hasvalueinteger },
    { "hasWarrantyPromise", s_gr, gr_haswarrantypromise },
    { "hasWarrantyScope", s_gr, gr_haswarrantyscope },
    { "height", s_gr, gr_height },
    { "includes", s_gr, gr_includes },
    { "includesObject", s_gr, gr_includesobject },
    { "isAccessoryOrSparePartFor", s_gr, gr_isaccessoryorsparepartfor },
    { "isConsumableFor", s_gr, gr_isconsumablefor },
    { "isListPrice", s_gr, gr_islistprice },
    { "isSimilarTo", s_gr, gr_issimilarto },
    { "isVariantOf", s_gr, gr_isvariantof },
    { "legalName", s_gr, gr_legalname },
    { "lesser", s_gr, gr_lesser },
    { "lesserOrEqual", s_gr, gr_lesserorequal },
    { "name", s_gr, gr_name },
    { "nonEqual", s_gr, gr_nonequal },
    { "offers", s_gr, gr_offers },
    { "opens", s_gr, gr_opens },
    { "owns", s_gr, gr_owns },
    { "predecessorOf", s_gr, gr_predecessorof },
    { "priceType", s_gr, gr_pricetype },
    { "qualitativeProductOrServiceProperty", s_gr, gr_qualitativeproductorserviceproperty },
    { "quantitativeProductOrServiceProperty", s_gr, gr_quantitativeproductorserviceproperty },
    { "relatedWebService", s_gr, gr_relatedwebservice },
    { "seeks", s_gr, gr_seeks },
    { "serialNumber", s_gr, gr_serialnumber },
    { "successorOf", s_gr, gr_successorof },
    { "taxID", s_gr, gr_taxid },
    { "typeOfGood", s_gr, gr_typeofgoods },
    { "validFrom", s_gr, gr_validfrom },
    { "validThrough", s_gr, gr_validto },
    { "valueAddedTaxIncluded", s_gr, gr_valueaddedtaxincluded },
    { "valueReference", s_gr, gr_valuereference },
    { "vatID", s_gr, gr_vatid },
    { "weight", s_gr, gr_weight },
    { "width", s_gr, gr_width },

    // global standards
    { "acceptedPaymentMethod", s_gs1, gs1p_acceptedpaymentmethod },
    { "activityIdeas", s_gs1, gs1p_activityideas },
    { "additionalOrganizationIdentificationTypeValue", s_gs1, gs1p_additionalorganizationidentificationtypevalue },
    { "additionalProductClassification", s_gs1, gs1p_additionalproductclassification },
    { "additionalProductClassificationCode", s_gs1, gs1p_additionalproductclassificationcode },
    { "additionalProductClassificationCodeDescription", s_gs1, gs1p_additionalproductclassificationcodedescription },
    { "additionalProductClassificationValue", s_gs1, gs1p_additionalproductclassificationvalue },
    { "additionalProductDescription", s_gs1, gs1p_additionalproductdescription },
    { "additive", s_gs1, gs1p_additive },
    { "additiveLevelOfContainment", s_gs1, gs1p_additivelevelofcontainment },
    { "additiveName", s_gs1, gs1p_additivename },
    { "address", s_gs1, gs1p_address },
    { "addressCountry", s_gs1, gs1p_addresscountry },
    { "addressLocality", s_gs1, gs1p_addresslocality },
    { "addressRegion", s_gs1, gs1p_addressregion },
    { "afterHoursContact", s_gs1, gs1p_afterhourscontact },
    { "alcoholicBeverageSubregion", s_gs1, gs1p_alcoholicbeveragesubregion },
    { "allergenInfo", s_gs1, gs1p_allergeninfo },
    { "allergenLevelOfContainmentCode", s_gs1, gs1p_allergenlevelofcontainmentcode },
    { "allergenSpecificationAgency", s_gs1, gs1p_allergenspecificationagency },
    { "allergenSpecificationName", s_gs1, gs1p_allergenspecificationname },
    { "allergenStatement", s_gs1, gs1p_allergenstatement },
    { "allergenType", s_gs1, gs1p_allergentype },
    { "anatomicalForm", s_gs1, gs1p_anatomicalform },
    { "applicableTo", s_gs1, gs1p_applicableto },
    { "audioFile", s_gs1, gs1p_audiofile },
    { "authenticity", s_gs1, gs1p_authenticity },
    { "authenticitySecurityFeatureInstructions", s_gs1, gs1p_authenticitysecurityfeatureinstructions },
    { "authenticitySecurityFeatureInstructionsURL", s_gs1, gs1p_authenticitysecurityfeatureinstructionsurl },
    { "authenticitySecurityFeatureRegularExpression", s_gs1, gs1p_authenticitysecurityfeatureregularexpression },
    { "authenticitySecurityFeatureType", s_gs1, gs1p_authenticitysecurityfeaturetype },
    { "authenticitySecurityFeatureValue", s_gs1, gs1p_authenticitysecurityfeaturevalue },
    { "availabilityEnds", s_gs1, gs1p_availabilityends },
    { "availabilityStarts", s_gs1, gs1p_availabilitystarts },
    { "availableAtOrFrom", s_gs1, gs1p_availableatorfrom },
    { "availableLanguage", s_gs1, gs1p_availablelanguage },
    { "awardPrize", s_gs1, gs1p_awardprize },
    { "awardPrizeCode", s_gs1, gs1p_awardprizecode },
    { "awardPrizeCountryCode", s_gs1, gs1p_awardprizecountrycode },
    { "awardPrizeDescription", s_gs1, gs1p_awardprizedescription },
    { "awardPrizeJury", s_gs1, gs1p_awardprizejury },
    { "awardPrizeName", s_gs1, gs1p_awardprizename },
    { "awardPrizeYear", s_gs1, gs1p_awardprizeyear },
    { "bestBeforeDate", s_gs1, gs1p_bestbeforedate },
    { "beverageVintage", s_gs1, gs1p_beveragevintage },
    { "biotinPerNutrientBasis", s_gs1, gs1p_biotinpernutrientbasis },
    { "bonelessClaim", s_gs1, gs1p_bonelessclaim },
    { "brand", s_gs1, gs1p_brand },
    { "brandHomepageClinical", s_gs1, gs1p_brandhomepageclinical },
    { "brandHomepagePatient", s_gs1, gs1p_brandhomepagepatient },
    { "brandName", s_gs1, gs1p_brandname },
    { "brandOwner", s_gs1, gs1p_brandowner },
    { "calciumPerNutrientBasis", s_gs1, gs1p_calciumpernutrientbasis },
    { "carbohydratesPerNutrientBasis", s_gs1, gs1p_carbohydratespernutrientbasis },
    { "careersInfo", s_gs1, gs1p_careersinfo },
    { "catchZone", s_gs1, gs1p_catchzone },
    { "certification", s_gs1, gs1p_certification },
    { "certificationAgency", s_gs1, gs1p_certificationagency },
    { "certificationInfo", s_gs1, gs1p_certificationinfo },
    { "certificationStandard", s_gs1, gs1p_certificationstandard },
    { "certificationValue", s_gs1, gs1p_certificationvalue },
    { "cheeseFirmness", s_gs1, gs1p_cheesefirmness },
    { "cheeseMaturationPeriodDescription", s_gs1, gs1p_cheesematurationperioddescription },
    { "chloridePerNutrientBasis", s_gs1, gs1p_chloridepernutrientbasis },
    { "cholesterolPerNutrientBasis", s_gs1, gs1p_cholesterolpernutrientbasis },
    { "chromiumPerNutrientBasis", s_gs1, gs1p_chromiumpernutrientbasis },
    { "city", s_gs1, gs1p_city },
    { "clothingCut", s_gs1, gs1p_clothingcut },
    { "collarType", s_gs1, gs1p_collartype },
    { "colourCode", s_gs1, gs1p_colourcode },
    { "colourCodeList", s_gs1, gs1p_colourcodelist },
    { "colourCodeValue", s_gs1, gs1p_colourcodevalue },
    { "colourDescription", s_gs1, gs1p_colourdescription },
    { "consumerFirstAvailabilityDateTime", s_gs1, gs1p_consumerfirstavailabilitydatetime },
    { "consumerHandlingStorage", s_gs1, gs1p_consumerhandlingstorage },
    { "consumerHandlingStorageInfo", s_gs1, gs1p_consumerhandlingstorageinfo },
    { "consumerLifestage", s_gs1, gs1p_consumerlifestage },
    { "consumerPackageDisclaimer", s_gs1, gs1p_consumerpackagedisclaimer },
    { "consumerProductVariant", s_gs1, gs1p_consumerproductvariant },
    { "consumerSafetyInformation", s_gs1, gs1p_consumersafetyinformation },
    { "consumerSalesCondition", s_gs1, gs1p_consumersalescondition },
    { "consumerStorageInstructions", s_gs1, gs1p_consumerstorageinstructions },
    { "consumerUsageInstructions", s_gs1, gs1p_consumerusageinstructions },
    { "contactPoint", s_gs1, gs1p_contactpoint },
    { "contactTitle", s_gs1, gs1p_contacttitle },
    { "contactType", s_gs1, gs1p_contacttype },
    { "convenienceLevelPercent", s_gs1, gs1p_conveniencelevelpercent },
    { "copperPerNutrientBasis", s_gs1, gs1p_copperpernutrientbasis },
    { "countryCode", s_gs1, gs1p_countrycode },
    { "countryOfAssembly", s_gs1, gs1p_countryofassembly },
    { "countryOfLastProcessing", s_gs1, gs1p_countryoflastprocessing },
    { "countryOfOrigin", s_gs1, gs1p_countryoforigin },
    { "countryOfOriginStatement", s_gs1, gs1p_countryoforiginstatement },
    { "countrySubdivisionCode", s_gs1, gs1p_countrysubdivisioncode },
    { "countyCode", s_gs1, gs1p_countycode },
    { "crossStreet", s_gs1, gs1p_crossstreet },
    { "customerSupportCentre", s_gs1, gs1p_customersupportcentre },
    { "dailyValueIntakePercent", s_gs1, gs1p_dailyvalueintakepercent },
    { "defaultLink", s_gs1, gs1p_defaultlink },
    { "defaultLinkMulti", s_gs1, gs1p_defaultlinkmulti },
    { "department", s_gs1, gs1p_department },
    { "dependentProprietaryProduct", s_gs1, gs1p_dependentproprietaryproduct },
    { "descriptiveSize", s_gs1, gs1p_descriptivesize },
    { "dietCode", s_gs1, gs1p_dietcode },
    { "dietType", s_gs1, gs1p_diettype },
    { "dietTypeDescription", s_gs1, gs1p_diettypedescription },
    { "dietTypeSubcode", s_gs1, gs1p_diettypesubcode },
    { "discountRepeatsPerMultipleMinimum", s_gs1, gs1p_discountrepeatspermultipleminimum },
    { "discountType", s_gs1, gs1p_discounttype },
    { "drainedWeight", s_gs1, gs1p_drainedweight },
    { "dueDate", s_gs1, gs1p_duedate },
    { "durationOfWarranty", s_gs1, gs1p_durationofwarranty },
    { "dutyFeeTaxAmount", s_gs1, gs1p_dutyfeetaxamount },
    { "dutyFeeTaxDescription", s_gs1, gs1p_dutyfeetaxdescription },
    { "dutyFeeTaxRate", s_gs1, gs1p_dutyfeetaxrate },
    { "eligibleQuantity", s_gs1, gs1p_eligiblequantity },
    { "eligibleQuantityMaximum", s_gs1, gs1p_eligiblequantitymaximum },
    { "eligibleQuantityMinimum", s_gs1, gs1p_eligiblequantityminimum },
    { "eligibleTradeChannel", s_gs1, gs1p_eligibletradechannel },
    { "email", s_gs1, gs1p_email },
    { "energyFromFatPerNutrientBasis", s_gs1, gs1p_energyfromfatpernutrientbasis },
    { "energyPerNutrientBasis", s_gs1, gs1p_energypernutrientbasis },
    { "epcis", s_gs1, gs1p_epcis },
    { "epil", s_gs1, gs1p_epil },
    { "equivalentProduct", s_gs1, gs1p_equivalentproduct },
    { "eventsInfo", s_gs1, gs1p_eventsinfo },
    { "exactDiscountAmount", s_gs1, gs1p_exactdiscountamount },
    { "exactDiscountPercentage", s_gs1, gs1p_exactdiscountpercentage },
    { "exclusionDescription", s_gs1, gs1p_exclusiondescription },
    { "expirationDate", s_gs1, gs1p_expirationdate },
    { "expirationDateTime", s_gs1, gs1p_expirationdatetime },
    { "faqs", s_gs1, gs1p_faqs },
    { "fatInMilkContent", s_gs1, gs1p_fatinmilkcontent },
    { "fatPerNutrientBasis", s_gs1, gs1p_fatpernutrientbasis },
    { "fatpercentageInDryMatter", s_gs1, gs1p_fatpercentageindrymatter },
    { "faxNumber", s_gs1, gs1p_faxnumber },
    { "fibrePerNutrientBasis", s_gs1, gs1p_fibrepernutrientbasis },
    { "fileLanguageCode", s_gs1, gs1p_filelanguagecode },
    { "filePixelHeight", s_gs1, gs1p_filepixelheight },
    { "filePixelWidth", s_gs1, gs1p_filepixelwidth },
    { "firstFreezeDate", s_gs1, gs1p_firstfreezedate },
    { "fishType", s_gs1, gs1p_fishtype },
    { "fluoridePerNutrientBasis", s_gs1, gs1p_fluoridepernutrientbasis },
    { "folicAcidPerNutrientBasis", s_gs1, gs1p_folicacidpernutrientbasis },
    { "foodBeverageRefrigerationClaim", s_gs1, gs1p_foodbeveragerefrigerationclaim },
    { "foodBeverageTargetUse", s_gs1, gs1p_foodbeveragetargetuse },
    { "footwearFasteningType", s_gs1, gs1p_footwearfasteningtype },
    { "footwearUpperType", s_gs1, gs1p_footwearuppertype },
    { "freshOrSeawaterFarmed", s_gs1, gs1p_freshorseawaterfarmed },
    { "functionalName", s_gs1, gs1p_functionalname },
    { "geneticallyModifiedDeclaration", s_gs1, gs1p_geneticallymodifieddeclaration },
    { "geo", s_gs1, gs1p_geo },
    { "globalLocationNumber", s_gs1, gs1p_globallocationnumber },
    { "gpcCategoryCode", s_gs1, gs1p_gpccategorycode },
    { "gpcCategoryDescription", s_gs1, gs1p_gpccategorydescription },
    { "grossArea", s_gs1, gs1p_grossarea },
    { "grossVolume", s_gs1, gs1p_grossvolume },
    { "grossWeight", s_gs1, gs1p_grossweight },
    { "growingMethod", s_gs1, gs1p_growingmethod },
    { "gtin", s_gs1, gs1p_gtin },
    { "handledBy", s_gs1, gs1p_handledby },
    { "harvestDate", s_gs1, gs1p_harvestdate },
    { "harvestDateEnd", s_gs1, gs1p_harvestdateend },
    { "harvestDateStart", s_gs1, gs1p_harvestdatestart },
    { "hasAllergen", s_gs1, gs1p_hasallergen },
    { "hasBatchLotNumber", s_gs1, gs1p_hasbatchlotnumber },
    { "hasRetailers", s_gs1, gs1p_hasretailers },
    { "hasReturnablePackageDeposit", s_gs1, gs1p_hasreturnablepackagedeposit },
    { "hasSerialNumber", s_gs1, gs1p_hasserialnumber },
    { "hasThirdPartyControlledSerialNumber", s_gs1, gs1p_hasthirdpartycontrolledserialnumber },
    { "healthClaimDescription", s_gs1, gs1p_healthclaimdescription },
    { "homepage", s_gs1, gs1p_homepage },
    { "image", s_gs1, gs1p_image },
    { "inPackageDepth", s_gs1, gs1p_inpackagedepth },
    { "inPackageDiameter", s_gs1, gs1p_inpackagediameter },
    { "inPackageHeight", s_gs1, gs1p_inpackageheight },
    { "inPackageWidth", s_gs1, gs1p_inpackagewidth },
    { "includedAccessories", s_gs1, gs1p_includedaccessories },
    { "ingredient", s_gs1, gs1p_ingredient },
    { "ingredientContentPercentage", s_gs1, gs1p_ingredientcontentpercentage },
    { "ingredientName", s_gs1, gs1p_ingredientname },
    { "ingredientOfConcern", s_gs1, gs1p_ingredientofconcern },
    { "ingredientSequence", s_gs1, gs1p_ingredientsequence },
    { "ingredientStatement", s_gs1, gs1p_ingredientstatement },
    { "ingredientsInfo", s_gs1, gs1p_ingredientsinfo },
    { "instructions", s_gs1, gs1p_instructions },
    { "instructionsForUse", s_gs1, gs1p_instructionsforuse },
    { "iodinePerNutrientBasis", s_gs1, gs1p_iodinepernutrientbasis },
    { "ironPerNutrientBasis", s_gs1, gs1p_ironpernutrientbasis },
    { "irradiatedCode", s_gs1, gs1p_irradiatedcode },
    { "isCarbonated", s_gs1, gs1p_iscarbonated },
    { "isDecaffeinated", s_gs1, gs1p_isdecaffeinated },
    { "isFromConcentrate", s_gs1, gs1p_isfromconcentrate },
    { "isHomogenised", s_gs1, gs1p_ishomogenised },
    { "isInstant", s_gs1, gs1p_isinstant },
    { "isMaternity", s_gs1, gs1p_ismaternity },
    { "isOnlyAvailableThroughRetailer", s_gs1, gs1p_isonlyavailablethroughretailer },
    { "isOnlyWithMailingListSignup", s_gs1, gs1p_isonlywithmailinglistsignup },
    { "isOnlyWithPaymentCard", s_gs1, gs1p_isonlywithpaymentcard },
    { "isOnlyWithRetailerLoyaltyCard", s_gs1, gs1p_isonlywithretailerloyaltycard },
    { "isOnlyWithRetailerPaymentCard", s_gs1, gs1p_isonlywithretailerpaymentcard },
    { "isPatterned", s_gs1, gs1p_ispatterned },
    { "isPittedStoned", s_gs1, gs1p_ispittedstoned },
    { "isProductRecalled", s_gs1, gs1p_isproductrecalled },
    { "isPromoterExclusiveOffer", s_gs1, gs1p_ispromoterexclusiveoffer },
    { "isRindEdible", s_gs1, gs1p_isrindedible },
    { "isSeedless", s_gs1, gs1p_isseedless },
    { "isShelledPeeled", s_gs1, gs1p_isshelledpeeled },
    { "isSliced", s_gs1, gs1p_issliced },
    { "isThermal", s_gs1, gs1p_isthermal },
    { "isVintage", s_gs1, gs1p_isvintage },
    { "isWashedReadyToEat", s_gs1, gs1p_iswashedreadytoeat },
    { "isWaterproof", s_gs1, gs1p_iswaterproof },
    { "isWearableItemDisposable", s_gs1, gs1p_iswearableitemdisposable },
    { "itemOffered", s_gs1, gs1p_itemoffered },
    { "juiceContentPercent", s_gs1, gs1p_juicecontentpercent },
    { "jws", s_gs1, gs1p_jws },
    { "latitude", s_gs1, gs1p_latitude },
    { "leaveReview", s_gs1, gs1p_leavereview },
    { "linkType", s_gs1, gs1p_linktype },
    { "location", s_gs1, gs1p_location },
    { "locationInfo", s_gs1, gs1p_locationinfo },
    { "logisticsInfo", s_gs1, gs1p_logisticsinfo },
    { "longitude", s_gs1, gs1p_longitude },
    { "magnesiumPerNutrientBasis", s_gs1, gs1p_magnesiumpernutrientbasis },
    { "makesOffer", s_gs1, gs1p_makesoffer },
    { "manganesePerNutrientBasis", s_gs1, gs1p_manganesepernutrientbasis },
    { "manufacturer", s_gs1, gs1p_manufacturer },
    { "manufacturerPreparationCode", s_gs1, gs1p_manufacturerpreparationcode },
    { "manufacturersWarranty", s_gs1, gs1p_manufacturerswarranty },
    { "manufacturingPlant", s_gs1, gs1p_manufacturingplant },
    { "massPerUnitArea", s_gs1, gs1p_massperunitarea },
    { "masterData", s_gs1, gs1p_masterdata },
    { "maturationMethod", s_gs1, gs1p_maturationmethod },
    { "maxPrice", s_gs1, gs1p_maxprice },
    { "maximumDiscountAmount", s_gs1, gs1p_maximumdiscountamount },
    { "maximumDiscountPercentage", s_gs1, gs1p_maximumdiscountpercentage },
    { "maximumOptimumConsumptionTemperature", s_gs1, gs1p_maximumoptimumconsumptiontemperature },
    { "maximumQualifyingItems", s_gs1, gs1p_maximumqualifyingitems },
    { "maximumQualifyingSpend", s_gs1, gs1p_maximumqualifyingspend },
    { "meatPoultryType", s_gs1, gs1p_meatpoultrytype },
    { "menuInfo", s_gs1, gs1p_menuinfo },
    { "minPrice", s_gs1, gs1p_minprice },
    { "minimumDiscountAmount", s_gs1, gs1p_minimumdiscountamount },
    { "minimumDiscountPercentage", s_gs1, gs1p_minimumdiscountpercentage },
    { "minimumFishContent", s_gs1, gs1p_minimumfishcontent },
    { "minimumMeatPoultryContent", s_gs1, gs1p_minimummeatpoultrycontent },
    { "minimumOptimumConsumptionTemperature", s_gs1, gs1p_minimumoptimumconsumptiontemperature },
    { "minimumQualifyingItems", s_gs1, gs1p_minimumqualifyingitems },
    { "minimumQualifyingSpend", s_gs1, gs1p_minimumqualifyingspend },
    { "molybdenumPerNutrientBasis", s_gs1, gs1p_molybdenumpernutrientbasis },
    { "monounsaturatedFatPerNutrientBasis", s_gs1, gs1p_monounsaturatedfatpernutrientbasis },
    { "netArea", s_gs1, gs1p_netarea },
    { "netContent", s_gs1, gs1p_netcontent },
    { "netWeight", s_gs1, gs1p_netweight },
    { "niacinPerNutrientBasis", s_gs1, gs1p_niacinpernutrientbasis },
    { "numberOfServingsPerPackage", s_gs1, gs1p_numberofservingsperpackage },
    { "numberOfServingsPerPackageMeasurementPrecision", s_gs1, gs1p_numberofservingsperpackagemeasurementprecision },
    { "numberOfServingsRangeDescription", s_gs1, gs1p_numberofservingsrangedescription },
    { "nutrientBasisQuantity", s_gs1, gs1p_nutrientbasisquantity },
    { "nutrientBasisQuantityType", s_gs1, gs1p_nutrientbasisquantitytype },
    { "nutrientMeasurementPrecision", s_gs1, gs1p_nutrientmeasurementprecision },
    { "nutritionalClaim", s_gs1, gs1p_nutritionalclaim },
    { "nutritionalClaimStatement", s_gs1, gs1p_nutritionalclaimstatement },
    { "nutritionalInfo", s_gs1, gs1p_nutritionalinfo },
    { "offerDescription", s_gs1, gs1p_offerdescription },
    { "offerDiscount", s_gs1, gs1p_offerdiscount },
    { "offerRedemptionType", s_gs1, gs1p_offerredemptiontype },
    { "offerRedemptionURL", s_gs1, gs1p_offerredemptionurl },
    { "offerRestrictionDescription", s_gs1, gs1p_offerrestrictiondescription },
    { "openingHoursInfo", s_gs1, gs1p_openinghoursinfo },
    { "organicClaim", s_gs1, gs1p_organicclaim },
    { "organicClaimAgency", s_gs1, gs1p_organicclaimagency },
    { "organicPercentClaim", s_gs1, gs1p_organicpercentclaim },
    { "organizationName", s_gs1, gs1p_organisationname },
    { "organizationRole", s_gs1, gs1p_organisationrole },
    { "originalCodeValue", s_gs1, gs1p_originalcodevalue },
    { "outOfPackageDepth", s_gs1, gs1p_outofpackagedepth },
    { "outOfPackageDiameter", s_gs1, gs1p_outofpackagediameter },
    { "outOfPackageHeight", s_gs1, gs1p_outofpackageheight },
    { "outOfPackageWidth", s_gs1, gs1p_outofpackagewidth },
    { "packaging", s_gs1, gs1p_packaging },
    { "packagingDate", s_gs1, gs1p_packagingdate },
    { "packagingFeature", s_gs1, gs1p_packagingfeature },
    { "packagingFunction", s_gs1, gs1p_packagingfunction },
    { "packagingMarkedDietAllergenType", s_gs1, gs1p_packagingmarkeddietallergentype },
    { "packagingMarkedFreeFrom", s_gs1, gs1p_packagingmarkedfreefrom },
    { "packagingMarkedLabelAccreditation", s_gs1, gs1p_packagingmarkedlabelaccreditation },
    { "packagingMaterial", s_gs1, gs1p_packagingmaterial },
    { "packagingMaterialCompositionQuantity", s_gs1, gs1p_packagingmaterialcompositionquantity },
    { "packagingMaterialThickness", s_gs1, gs1p_packagingmaterialthickness },
    { "packagingMaterialType", s_gs1, gs1p_packagingmaterialtype },
    { "packagingRecyclingProcessType", s_gs1, gs1p_packagingrecyclingprocesstype },
    { "packagingRecyclingScheme", s_gs1, gs1p_packagingrecyclingscheme },
    { "packagingShape", s_gs1, gs1p_packagingshape },
    { "packagingType", s_gs1, gs1p_packagingtype },
    { "pantothenicAcidPerNutrientBasis", s_gs1, gs1p_pantothenicacidpernutrientbasis },
    { "partyName", s_gs1, gs1p_partyname },
    { "paymentLink", s_gs1, gs1p_paymentlink },
    { "paymentTerms", s_gs1, gs1p_paymentterms },
    { "percentageOfAlcoholByVolume", s_gs1, gs1p_percentageofalcoholbyvolume },
    { "phosphorusPerNutrientBasis", s_gs1, gs1p_phosphoruspernutrientbasis },
    { "pip", s_gs1, gs1p_pip },
    { "POBoxNumber", s_gs1, gs1p_poboxnumber },
    { "polyolsPerNutrientBasis", s_gs1, gs1p_polyolspernutrientbasis },
    { "polyunsaturatedFatPerNutrientBasis", s_gs1, gs1p_polyunsaturatedfatpernutrientbasis },
    { "postOfficeBoxNumber", s_gs1, gs1p_postofficeboxnumber },
    { "postalCode", s_gs1, gs1p_postalcode },
    { "potassiumPerNutrientBasis", s_gs1, gs1p_potassiumpernutrientbasis },
    { "preparationCode", s_gs1, gs1p_preparationcode },
    { "preparationConsumptionPrecautions", s_gs1, gs1p_preparationconsumptionprecautions },
    { "preparationInformation", s_gs1, gs1p_preparationinformation },
    { "preparationInstructions", s_gs1, gs1p_preparationinstructions },
    { "preservationTechnique", s_gs1, gs1p_preservationtechnique },
    { "price", s_gs1, gs1p_price },
    { "priceCurrency", s_gs1, gs1p_pricecurrency },
    { "priceSpecification", s_gs1, gs1p_pricespecification },
    { "primaryAlternateProduct", s_gs1, gs1p_primaryalternateproduct },
    { "productDescription", s_gs1, gs1p_productdescription },
    { "productFeatureBenefit", s_gs1, gs1p_productfeaturebenefit },
    { "productFormDescription", s_gs1, gs1p_productformdescription },
    { "productID", s_gs1, gs1p_productid },
    { "productMarketingMessage", s_gs1, gs1p_productmarketingmessage },
    { "productName", s_gs1, gs1p_productname },
    { "productRange", s_gs1, gs1p_productrange },
    { "productSustainabilityInfo", s_gs1, gs1p_productsustainabilityinfo },
    { "productYield", s_gs1, gs1p_productyield },
    { "productYieldType", s_gs1, gs1p_productyieldtype },
    { "productYieldVariationPercentage", s_gs1, gs1p_productyieldvariationpercentage },
    { "productionDate", s_gs1, gs1p_productiondate },
    { "productionDateTime", s_gs1, gs1p_productiondatetime },
    { "productionVariantDescription", s_gs1, gs1p_productionvariantdescription },
    { "productionVariantEffectiveDateTime", s_gs1, gs1p_productionvarianteffectivedatetime },
    { "promotion", s_gs1, gs1p_promotion },
    { "proteinPerNutrientBasis", s_gs1, gs1p_proteinpernutrientbasis },
    { "provenanceStatement", s_gs1, gs1p_provenancestatement },
    { "provinceStateCode", s_gs1, gs1p_provincestatecode },
    { "purchaseSuppliesOrAccessories", s_gs1, gs1p_purchasesuppliesoraccessories },
    { "qualifyingBrandName", s_gs1, gs1p_qualifyingbrandname },
    { "qualifyingGPCs", s_gs1, gs1p_qualifyinggpcs },
    { "qualifyingProductCategoryDescription", s_gs1, gs1p_qualifyingproductcategorydescription },
    { "qualifyingProductClassificationCode", s_gs1, gs1p_qualifyingproductclassificationcode },
    { "qualifyingProductGTINs", s_gs1, gs1p_qualifyingproductgtins },
    { "qualifyingSubBrandName", s_gs1, gs1p_qualifyingsubbrandname },
    { "quickStartGuide", s_gs1, gs1p_quickstartguide },
    { "recallStatus", s_gs1, gs1p_recallstatus },
    { "recipeInfo", s_gs1, gs1p_recipeinfo },
    { "recipeWebsite", s_gs1, gs1p_recipewebsite },
    { "referencedFile", s_gs1, gs1p_referencedfile },
    { "referencedFileEffectiveEndDateTime", s_gs1, gs1p_referencedfileeffectiveenddatetime },
    { "referencedFileEffectiveStartDateTime", s_gs1, gs1p_referencedfileeffectivestartdatetime },
    { "referencedFileSize", s_gs1, gs1p_referencedfilesize },
    { "referencedFileType", s_gs1, gs1p_referencedfiletype },
    { "referencedFileURL", s_gs1, gs1p_referencedfileurl },
    { "registerProduct", s_gs1, gs1p_registerproduct },
    { "registryEntry", s_gs1, gs1p_registryentry },
    { "regulatedProductName", s_gs1, gs1p_regulatedproductname },
    { "reheatingClaim", s_gs1, gs1p_reheatingclaim },
    { "relatedVideo", s_gs1, gs1p_relatedvideo },
    { "replacedByProduct", s_gs1, gs1p_replacedbyproduct },
    { "replacedProduct", s_gs1, gs1p_replacedproduct },
    { "responsibility", s_gs1, gs1p_responsibility },
    { "returnablePackageDepositAmount", s_gs1, gs1p_returnablepackagedepositamount },
    { "returnablePackageDepositRegion", s_gs1, gs1p_returnablepackagedepositregion },
    { "review", s_gs1, gs1p_review },
    { "riboflavinPerNutrientBasis", s_gs1, gs1p_riboflavinpernutrientbasis },
    { "safetyInfo", s_gs1, gs1p_safetyinfo },
    { "saltPerNutrientBasis", s_gs1, gs1p_saltpernutrientbasis },
    { "saturatedFatPerNutrientBasis", s_gs1, gs1p_saturatedfatpernutrientbasis },
    { "scheduleTime", s_gs1, gs1p_scheduletime },
    { "seasonCalendarYear", s_gs1, gs1p_seasoncalendaryear },
    { "seasonName", s_gs1, gs1p_seasonname },
    { "seasonParameter", s_gs1, gs1p_seasonparameter },
    { "seeker", s_gs1, gs1p_seeker },
    { "selectedProductsOnly", s_gs1, gs1p_selectedproductsonly },
    { "seleniumPerNutrientBasis", s_gs1, gs1p_seleniumpernutrientbasis },
    { "sellByDate", s_gs1, gs1p_sellbydate },
    { "seller", s_gs1, gs1p_seller },
    { "serviceInfo", s_gs1, gs1p_serviceinfo },
    { "servingSize", s_gs1, gs1p_servingsize },
    { "servingSizeDescription", s_gs1, gs1p_servingsizedescription },
    { "servingSuggestion", s_gs1, gs1p_servingsuggestion },
    { "sharpnessOfCheese", s_gs1, gs1p_sharpnessofcheese },
    { "size", s_gs1, gs1p_size },
    { "sizeCode", s_gs1, gs1p_sizecode },
    { "sizeCodeListCode", s_gs1, gs1p_sizecodelistcode },
    { "sizeCodeValue", s_gs1, gs1p_sizecodevalue },
    { "sizeDimension", s_gs1, gs1p_sizedimension },
    { "sizeGroup", s_gs1, gs1p_sizegroup },
    { "sizeSystem", s_gs1, gs1p_sizesystem },
    { "sizeType", s_gs1, gs1p_sizetype },
    { "smartLabel", s_gs1, gs1p_smartlabel },
    { "smpc", s_gs1, gs1p_smpc },
    { "socialMedia", s_gs1, gs1p_socialmedia },
    { "sodiumPerNutrientBasis", s_gs1, gs1p_sodiumpernutrientbasis },
    { "sourceAnimal", s_gs1, gs1p_sourceanimal },
    { "sportingActivityType", s_gs1, gs1p_sportingactivitytype },
    { "starchPerNutrientBasis", s_gs1, gs1p_starchpernutrientbasis },
    { "statisticInfo", s_gs1, gs1p_statisticinfo },
    { "streetAddress", s_gs1, gs1p_streetaddress },
    { "streetAddressOne", s_gs1, gs1p_streetaddress1 },
    { "streetAddressThree", s_gs1, gs1p_streetaddress3 },
    { "streetAddressTwo", s_gs1, gs1p_streetaddress2 },
    { "styleDescription", s_gs1, gs1p_styledescription },
    { "subBrandName", s_gs1, gs1p_subbrandname },
    { "sugarsPerNutrientBasis", s_gs1, gs1p_sugarspernutrientbasis },
    { "supplierSpecifiedMinimumConsumerStorageDays", s_gs1, gs1p_supplierspecifiedminimumconsumerstoragedays },
    { "support", s_gs1, gs1p_support },
    { "sustainabilityInfo", s_gs1, gs1p_sustainabilityinfo },
    { "targetConsumerAge", s_gs1, gs1p_targetconsumerage },
    { "targetConsumerGender", s_gs1, gs1p_targetconsumergender },
    { "targetMarket", s_gs1, gs1p_targetmarket },
    { "targetMarketCountries", s_gs1, gs1p_targetmarketcountries },
    { "telephone", s_gs1, gs1p_telephone },
    { "textileMaterial", s_gs1, gs1p_textilematerial },
    { "textileMaterialContent", s_gs1, gs1p_textilematerialcontent },
    { "textileMaterialDescription", s_gs1, gs1p_textilematerialdescription },
    { "textileMaterialPercentage", s_gs1, gs1p_textilematerialpercentage },
    { "textileMaterialThreadCount", s_gs1, gs1p_textilematerialthreadcount },
    { "textileMaterialWeight", s_gs1, gs1p_textilematerialweight },
    { "thiaminPerNutrientBasis", s_gs1, gs1p_thiaminpernutrientbasis },
    { "traceability", s_gs1, gs1p_traceability },
    { "transFatPerNutrientBasis", s_gs1, gs1p_transfatpernutrientbasis },
    { "tutorial", s_gs1, gs1p_tutorial },
    { "unitCode", s_gs1, gs1p_unitcode },
    { "upperMaterialType", s_gs1, gs1p_uppermaterialtype },
    { "userAgreement", s_gs1, gs1p_useragreement },
    { "validFrom", s_gs1, gs1p_validfrom },
    { "validThrough", s_gs1, gs1p_validto },
    { "value", s_gs1, gs1p_value },
    { "variantDescription", s_gs1, gs1p_variantdescription },
    { "verificationService", s_gs1, gs1p_verificationservice },
    { "vintner", s_gs1, gs1p_vintner },
    { "vitaminAPerNutrientBasis", s_gs1, gs1p_vitaminapernutrientbasis },
    { "vitaminB12PerNutrientBasis", s_gs1, gs1p_vitaminb12pernutrientbasis },
    { "vitaminB6PerNutrientBasis", s_gs1, gs1p_vitaminb6pernutrientbasis },
    { "vitaminCPerNutrientBasis", s_gs1, gs1p_vitamincpernutrientbasis },
    { "vitaminDPerNutrientBasis", s_gs1, gs1p_vitamindpernutrientbasis },
    { "vitaminEPerNutrientBasis", s_gs1, gs1p_vitaminepernutrientbasis },
    { "vitaminKPerNutrientBasis", s_gs1, gs1p_vitaminkpernutrientbasis },
    { "warningCopyDescription", s_gs1, gs1p_warningcopydescription },
    { "warranty", s_gs1, gs1p_warranty },
    { "warrantyScopeDescription", s_gs1, gs1p_warrantyscopedescription },
    { "whatsInTheBox", s_gs1, gs1p_whatsinthebox },
    { "yield", s_gs1, gs1p_yield },
    { "zincPerNutrientBasis", s_gs1, gs1p_zincpernutrientbasis },

    //  ical
    { "action", s_ical, ical_action },
    { "attendee", s_ical, ical_attendee },
    { "byday", s_ical, ical_byday },
    { "bymonth", s_ical, ical_bymonth },
    { "calAddress", s_ical, ical_caladdress },
    { "calscale", s_ical, ical_calscale },
    { "categories", s_ical, ical_categories },
    { "class", s_ical, ical_class },
    { "cn", s_ical, ical_cn },
    { "comment", s_ical, ical_comment },
    { "component", s_ical, ical_component },
    { "cutype", s_ical, ical_cutype },
    { "date", s_ical, ical_date },
    { "dateTime", s_ical, ical_datetime },
    { "daylight", s_ical, ical_daylight },
    { "description", s_ical, ical_description },
    { "dtend", s_ical, ical_dtend },
    { "dstamp", s_ical, ical_dstamp },
    { "dtstart", s_ical, ical_dtstart },
    { "duration", s_ical, ical_duration },
    { "freq", s_ical, ical_freq },
    { "interval", s_ical, ical_interval },
    { "language", s_ical, ical_language },
    { "location", s_ical, ical_location },
    { "organizer", s_ical, ical_organiser },
    { "partstat", s_ical, ical_partstat },
    { "prodid", s_ical, ical_prodid },
    { "related", s_ical, ical_related },
    { "role", s_ical, ical_role },
    { "rrule", s_ical, ical_rrule },
    { "rsvp", s_ical, ical_rsvp },
    { "sequence", s_ical, ical_sequence },
    { "standard", s_ical, ical_standard },
    { "summary", s_ical, ical_summary },
    { "transp", s_ical, ical_transp },
    { "trigger", s_ical, ical_trigger },
    { "tzid", s_ical, ical_tzid },
    { "tzname", s_ical, ical_tzname },
    { "tzoffsetfrom", s_ical, ical_tzoffsetfrom },
    { "tzoffsetto", s_ical, ical_tzoffsetto },
    { "uid", s_ical, ical_uid },
    { "valarm", s_ical, ical_valarm },
    { "version", s_ical, ical_version },

    // jsonld
    { "base", s_jsonld, jsonld_base },
    { "container", s_jsonld, jsonld_container },
    { "context", s_jsonld, jsonld_context },
    { "definition", s_jsonld, jsonld_definition },
    { "direction", s_jsonld, jsonld_direction },
    { "id", s_jsonld, jsonld_id },
    { "import", s_jsonld, jsonld_import },
    { "language", s_jsonld, jsonld_language },
    { "nest", s_jsonld, jsonld_nest },
    { "prefix", s_jsonld, jsonld_prefix },
    { "propagate", s_jsonld, jsonld_propagate },
    { "protected", s_jsonld, jsonld_protected },
    { "reverse", s_jsonld, jsonld_reverse },
    { "term", s_jsonld, jsonld_term },
    { "type", s_jsonld, jsonld_type },
    { "version", s_jsonld, jsonld_version },
    { "vocab", s_jsonld, jsonld_vocab },

    // ldp
    { "constrainedBy", s_ldp, ldp_constrainedby },
    { "contains", s_ldp, ldp_contains },
    { "hasMemberRelation", s_ldp, ldp_hasmemberrelation },
    { "inbox", s_ldp, ldp_inbox },
    { "insertedContentRelation", s_ldp, ldp_insertedcontentrelation },
    { "isMemberRelationof", s_ldp, ldp_ismemberrelationof },
    { "member", s_ldp, ldp_member },
    { "membershipResource", s_ldp, ldp_membershipresource },
    { "pageSequence", s_ldp, ldp_pagesequence },
    { "pageSortCollation", s_ldp, ldp_pagesortcollation },
    { "pageSortCriteria", s_ldp, ldp_pagesortcriteria },
    { "pageSortOrder", s_ldp, ldp_pagesortorder },
    { "pageSortPredicate", s_ldp, ldp_pagesortpredicate },

    // locn
    { "address", s_locn, locn_address },
    { "addressArea", s_locn, locn_addressarea },
    { "addressId", s_locn, locn_addressid },
    { "adminUnitL1", s_locn, locn_adminunitlevel1 },
    { "adminUnitL2", s_locn, locn_adminunitlevel2 },
    { "fullAddress", s_locn, locn_fulladdress },
    { "geographicName", s_locn, locn_geographicname },
    { "geometry", s_locn, locn_geometry },
    { "location", s_locn, locn_location },
    { "locatorDesignator", s_locn, locn_locatordesignator },
    { "locatorName", s_locn, locn_locatorname },
    { "poBox", s_locn, locn_pobox },
    { "postCode", s_locn, locn_postcode },
    { "postName", s_locn, locn_postname },
    { "thoroughfare", s_locn, locn_thoroughfare },

    // media resources
    { "alternativeTitle", s_ma, ma_alternativetitle },
    { "averageBitRate", s_ma, ma_averagebitrate },
    { "collectionName", s_ma, ma_collectionname },
    { "copyright", s_ma, ma_copyright },
    { "createdIn", s_ma, ma_createdin },
    { "creationDate", s_ma, ma_creationdate },
    { "date", s_ma, ma_date },
    { "depictsFictionalLocation", s_ma, ma_depictsfictionallocation },
    { "description", s_ma, ma_description },
    { "duration", s_ma, ma_duration },
    { "editDate", s_ma, ma_editdate },
    { "features", s_ma, ma_features },
    { "fragmentName", s_ma, ma_fragmentname },
    { "frameHeight", s_ma, ma_frameheight },
    { "frameRate", s_ma, ma_framerate },
    { "frameSizeUnit", s_ma, ma_framesizeunit },
    { "frameWidth", s_ma, ma_framewidth },
    { "hasAccessConditions", s_ma, ma_hasaccessconditions },
    { "hasAudioDescription", s_ma, ma_hasaudiodescription },
    { "hasCaptioning", s_ma, ma_hascaptioning },
    { "hasChapter", s_ma, ma_haschapter },
    { "hasClassification", s_ma, ma_hasclassification },
    { "hasClassificationSystem", s_ma, ma_hasclassificationsystem },
    { "hasCompression", s_ma, ma_hascompression },
    { "hasContributedTo", s_ma, ma_hascontributedto },
    { "hasContributor", s_ma, ma_hascontributor },
    { "hasCopyrightOver", s_ma, ma_hascopyrightover },
    { "hasCreated", s_ma, ma_hascreated },
    { "hasCreator", s_ma, ma_hascreator },
    { "hasFormat", s_ma, ma_hasformat },
    { "hasFragment", s_ma, ma_hasfragment },
    { "hasGenre", s_ma, ma_hasgenre },
    { "hasKeyword", s_ma, ma_haskeyword },
    { "hasLanguage", s_ma, ma_haslanguage },
    { "hasLocationCoordinateSystem", s_ma, ma_haslocationcoordinatesystem },
    { "hasMember", s_ma, ma_hasmember },
    { "hasNamedFragment", s_ma, ma_hasnamedfragment },
    { "hasPermissions", s_ma, ma_haspermissions },
    { "hasPolicy", s_ma, ma_haspolicy },
    { "hasPublished", s_ma, ma_haspublished },
    { "hasPublisher", s_ma, ma_haspublisher },
    { "hasRating", s_ma, ma_hasrating },
    { "hasRatingSystem", s_ma, ma_hasratingsystem },
    { "hasRelatedImage", s_ma, ma_hasrelatedimage },
    { "hasRelatedLocation", s_ma, ma_hasrelatedlocation },
    { "hasRelatedResource", s_ma, ma_hasrelatedresource },
    { "hasSigning", s_ma, ma_hassigning },
    { "hasSource", s_ma, ma_hassource },
    { "hasSubtitling", s_ma, ma_hassubtitling },
    { "hasTargetAudience", s_ma, ma_hastargetaudience },
    { "hasTrack", s_ma, ma_hastrack },
    { "isCaptioningOf", s_ma, ma_iscaptioningof },
    { "isChapterOf", s_ma, ma_ischapterof },
    { "isCopyrightedBy", s_ma, ma_iscopyrightedby },
    { "isCreationLocationOf", s_ma, ma_iscreationlocationof },
    { "isFictionalLocationDepictedIn", s_ma, ma_isfictionallocationdepictedin },
    { "isFragmentOf", s_ma, ma_isfragmentof },
    { "isImageRelatedTo", s_ma, ma_isimagerelatedto },
    { "isLocationRelatedTo", s_ma, ma_islocationrelatedto },
    { "isMemberOf", s_ma, ma_ismemberof },
    { "isNamedFragmentOf", s_ma, ma_isnamedfragmentof },
    { "isProvidedBy", s_ma, ma_isprovidedby },
    { "isRatingOf", s_ma, ma_isratingof },
    { "isRelatedTo", s_ma, ma_isrelatedto },
    { "isSigningOf", s_ma, ma_issigningof },
    { "isSourceOf", s_ma, ma_issourceof },
    { "isTargetAudienceOf", s_ma, ma_istargetaudienceof },
    { "isTrackOf", s_ma, ma_istrackof },
    { "locationAltitude", s_ma, ma_locationaltitude },
    { "locationLatitude", s_ma, ma_locationlatitude },
    { "locationLongitude", s_ma, ma_locationlongitude },
    { "locationName", s_ma, ma_locationname },
    { "locator", s_ma, ma_locator },
    { "mainOriginalTitle", s_ma, ma_mainoriginaltitle },
    { "numberOfTracks", s_ma, ma_numberoftracks },
    { "playsIn", s_ma, ma_playsin },
    { "provides", s_ma, ma_provides },
    { "ratingScaleMax", s_ma, ma_ratingscalemax },
    { "ratingScaleMin", s_ma, ma_ratingscalemin },
    { "ratingValue", s_ma, ma_ratingvalue },
    { "recordDate", s_ma, ma_recorddate },
    { "releaseDate", s_ma, ma_releasedate },
    { "samplingRate", s_ma, ma_samplingrate },
    { "title", s_ma, ma_title },
    { "trackName", s_ma, ma_trackname },

    // living standard / microformats v1
    { "additional-name", s_microformats, mp_additional_name },
    { "adr", s_microformats, mp_adr },
    { "affliation", s_microformats, mp_affliation },
    { "agent", s_microformats, mp_agent },
    { "album", s_microformats, mp_album },
    { "anniversary", s_microformats, mp_anniversary },
    { "attach", s_microformats, mp_attach },
    { "attendee", s_microformats, mp_attendee },
    { "author", s_microformats, mp_author },
    { "availability", s_microformats, mp_availability },
    { "bday", s_microformats, mp_bday },
    { "best", s_microformats, mp_best },
    { "bookmark", s_microformats, mp_bookmark },
    { "categories", s_microformats, mp_categories },
    { "category", s_microformats, mp_category },
    { "class", s_microformats, mp_class },
    { "comment", s_microformats, mp_comment },
    { "contact", s_microformats, mp_contact },
    { "contributor", s_microformats, mp_contributor },
    { "count", s_microformats, mp_count },
    { "country-name", s_microformats, mp_country_name },
    { "created", s_microformats, mp_created },
    { "dateline", s_microformats, mp_dateline },
    { "description", s_microformats, mp_description },
    { "dtend", s_microformats, mp_dtend },
    { "dtexpired", s_microformats, mp_dtexpired },
    { "dtlisted", s_microformats, mp_dtlisted },
    { "dtreviewed", s_microformats, mp_dtreviewed },
    { "dtstart", s_microformats, mp_dtstart },
    { "duration", s_microformats, mp_duration },
    { "education", s_microformats, mp_education },
    { "email", s_microformats, mp_email },
    { "enclosure", s_microformats, mp_enclosure },
    { "entry", s_microformats, mp_entry },
    { "entry-content", s_microformats, mp_entry_content },
    { "entry-summary", s_microformats, mp_entry_summary },
    { "entry-title", s_microformats, mp_entry_title },
    { "exdate", s_microformats, mp_exdate },
    { "experience", s_microformats, mp_experience },
    { "extended-address", s_microformats, mp_extended_address },
    { "family-name", s_microformats, mp_family_name },
    { "feed-category", s_microformats, mp_feed_category },
    { "fn", s_microformats, mp_fn },
    { "gender-identity", s_microformats, mp_gender_identity },
    { "geo", s_microformats, mp_geo },
    { "given-name", s_microformats, mp_given_name },
    { "h", s_microformats, mp_h },
    { "hentry", s_microformats, mp_hentry },
    { "honourific-prefix", s_microformats, mp_honourific_prefix },
    { "honourific-suffixr", s_microformats, mp_honourific_suffix },
    { "impp", s_microformats, mp_impp },
    { "include", s_microformats, mp_include },
    { "info", s_microformats, mp_info },
    { "ingredient", s_microformats, mp_ingredient },
    { "instructions", s_microformats, mp_instructions },
    { "item", s_microformats, mp_item },
    { "item-license", s_microformats, mp_licence },
    { "key", s_microformats, mp_key },
    { "kind", s_microformats, mp_kind },
    { "label", s_microformats, mp_label },
    { "lang", s_microformats, mp_lang },
    { "last-modified", s_microformats, mp_last_modified },
    { "latitude", s_microformats, mp_latitude },
    { "license", s_microformats, mp_licence },
    { "lister", s_microformats, mp_lister },
    { "listing-action", s_microformats, mp_listing_action },
    { "locality", s_microformats, mp_locality },
    { "location", s_microformats, mp_location },
    { "logo", s_microformats, mp_logo },
    { "longitude", s_microformats, mp_longitude },
    { "mailer", s_microformats, mp_mailer },
    { "member", s_microformats, mp_member },
    { "method", s_microformats, mp_method },
    { "min", s_microformats, mp_min },
    { "n", s_microformats, mp_n },
    { "nickname", s_microformats, mp_nickname },
    { "note", s_microformats, mp_note },
    { "nutrition", s_microformats, mp_nutrition },
    { "org", s_microformats, mp_org },
    { "organization-name", s_microformats, mp_organisation_name },
    { "organization-unit", s_microformats, mp_organisation_unit },
    { "organizer", s_microformats, mp_organiser },
    { "partstat", s_microformats, mp_partstat },
    { "payment", s_microformats, mp_payment },
    { "permalink", s_microformats, mp_permalink },
    { "photo", s_microformats, mp_photo },
    { "player", s_microformats, mp_player },
    { "position", s_microformats, mp_position },
    { "post-office-box", s_microformats, mp_post_office_box },
    { "postal-code", s_microformats, mp_postal_code },
    { "price", s_microformats, mp_price },
    { "principles", s_microformats, mp_principles },
    { "publications", s_microformats, mp_publications },
    { "published", s_microformats, mp_published },
    { "rating", s_microformats, mp_rating },
    { "rdate", s_microformats, mp_rdate },
    { "region", s_microformats, mp_region },
    { "rel", s_microformats, mp_rel },
    { "related", s_microformats, mp_related },
    { "rev", s_microformats, mp_rev },
    { "review", s_microformats, mp_review },
    { "reviewer", s_microformats, mp_reviewer },
    { "role", s_microformats, mp_role },
    { "rrule", s_microformats, mp_rrule },
    { "resources", s_microformats, mp_resources },
    { "s", s_microformats, mp_s },
    { "sample", s_microformats, mp_sample },
    { "sequence", s_microformats, mp_sequence },
    { "self", s_microformats, mp_self },
    { "sex", s_microformats, mp_sex },
    { "skill", s_microformats, mp_skill },
    { "skills", s_microformats, mp_skill },
    { "sort-string", s_microformats, mp_sort_string },
    { "sound", s_microformats, mp_sound },
    { "source-org", s_microformats, mp_source_org },
    { "status", s_microformats, mp_status },
    { "street-address", s_microformats, mp_street_address },
    { "summary", s_microformats, mp_summary },
    { "tag", s_microformats, mp_tag },
    { "tags", s_microformats, mp_tags },
    { "tel", s_microformats, mp_tel },
    { "term", s_microformats, mp_term },
    { "title", s_microformats, mp_title },
    { "transp", s_microformats, mp_transp },
    { "type", s_microformats, mp_type },
    { "tz", s_microformats, mp_tz },
    { "uid", s_microformats, mp_uid },
    { "updated", s_microformats, mp_updated },
    { "url", s_microformats, mp_url },
    { "value", s_microformats, mp_value },
    { "value-title", s_microformats, mp_value_title },
    { "version", s_microformats, mp_version },
    { "votes", s_microformats, mp_votes },
    { "worst", s_microformats, mp_worst },
    { "yield", s_microformats, mp_yield },

    // microformats v2
    { "dt-accessed", s_microformats, mp2_dt_accessed },
    { "dt-anniversary", s_microformats, mp2_dt_anniversary },
    { "dt-bday", s_microformats, mp2_dt_bday },
    { "dt-duration", s_microformats, mp2_dt_duration },
    { "dt-end", s_microformats, mp2_dt_end },
    { "dt-expired", s_microformats, mp2_dt_expired },
    { "dt-listed", s_microformats, mp2_dt_listed },
    { "dt-published", s_microformats, mp2_dt_published },
    { "dt-start", s_microformats, mp2_dt_start },
    { "dt-updated", s_microformats, mp2_dt_updated },
    { "e-content", s_microformats, mp2_e_content },
    { "e-description", s_microformats, mp2_e_description },
    { "e-instructions", s_microformats, mp2_e_instructions },
    { "h-breadcrumb", s_microformats, mp2_h_breadcrumb },
    { "h-entry", s_microformats, mp2_h_entry },
    { "p-action", s_microformats, mp2_p_action },
    { "p-additional-name", s_microformats, mp2_p_additional_name },
    { "p-adr", s_microformats, mp2_p_adr },
    { "p-affiliation", s_microformats, mp2_p_affiliation },
    { "p-altitude", s_microformats, mp2_p_altitude },
    { "p-attendee", s_microformats, mp2_p_attendee },
    { "p-author", s_microformats, mp2_p_author },
    { "p-best", s_microformats, mp2_p_best },
    { "p-category", s_microformats, mp2_p_category },
    { "p-comment", s_microformats, mp2_p_comment },
    { "p-contact", s_microformats, mp2_p_contact },
    { "p-content", s_microformats, mp2_p_content },
    { "p-country-name", s_microformats, mp2_p_country_name },
    { "p-description", s_microformats, mp2_p_description },
    { "p-duration", s_microformats, mp2_p_duration },
    { "p-education", s_microformats, mp2_p_education },
    { "p-entry", s_microformats, mp2_p_entry },
    { "p-experience", s_microformats, mp2_p_experience },
    { "p-extended-address", s_microformats, mp2_p_extended_address },
    { "p-family-name", s_microformats, mp2_p_family_name },
    { "p-region", s_microformats, mp2_p_region },
    { "p-gender-identity", s_microformats, mp2_p_gender_identity },
    { "p-geo", s_microformats, mp2_p_geo },
    { "p-given_name", s_microformats, mp2_p_given_name },
    { "p-honourific-prefix", s_microformats, mp2_p_honourific_prefix },
    { "p-honourific-suffix", s_microformats, mp2_p_honourific_suffix },
    { "p-ingredient", s_microformats, mp2_p_ingredient },
    { "p-item", s_microformats, mp2_p_item },
    { "p-job-title", s_microformats, mp2_p_job_title },
    { "p-label", s_microformats, mp2_p_label },
    { "p-latitude", s_microformats, mp2_p_latitude },
    { "p-lister", s_microformats, mp2_p_lister },
    { "p-locality", s_microformats, mp2_p_locality },
    { "p-location", s_microformats, mp2_p_location },
    { "p-longitude", s_microformats, mp2_p_longitude },
    { "p-name", s_microformats, mp2_p_name },
    { "p-nickname", s_microformats, mp2_p_nickname },
    { "p-note", s_microformats, mp2_p_note },
    { "p-nutrition", s_microformats, mp2_p_nutrition },
    { "p-post-office-box", s_microformats, mp2_p_post_office_box },
    { "p-postal-code", s_microformats, mp2_p_postal_code },
    { "p-publication", s_microformats, mp2_p_publication },
    { "p-org", s_microformats, mp2_p_org },
    { "p-price", s_microformats, mp2_p_price },
    { "p-rating", s_microformats, mp2_p_rating },
    { "p-read-of", s_microformats, mp2_p_read_of },
    { "p-role", s_microformats, mp2_p_role },
    { "p-rsvp", s_microformats, mp2_p_rsvp },
    { "p-sex", s_microformats, mp2_p_sex },
    { "p-size", s_microformats, mp2_p_size },
    { "p-skill", s_microformats, mp2_p_skill },
    { "p-sort-string", s_microformats, mp2_p_sort_string },
    { "p-street-address", s_microformats, mp2_p_street_address },
    { "p-summary", s_microformats, mp2_p_summary },
    { "p-tz", s_microformats, mp2_p_tz },
    { "p-worst", s_microformats, mp2_p_worst },
    { "p-yield", s_microformats, mp2_p_yield },
    { "u-audio", s_microformats, mp2_u_audio },
    { "u-bookmark-of", s_microformats, mp2_u_bookmark_of },
    { "u-email", s_microformats, mp2_u_email },
    { "u-featured", s_microformats, mp2_u_featured },
    { "u-geo", s_microformats, mp2_u_geo },
    { "u-impp", s_microformats, mp2_u_impp },
    { "u-in-reply-to", s_microformats, mp2_u_in_reply_to },
    { "u-like", s_microformats, mp2_u_like },
    { "u-like-of", s_microformats, mp2_u_like_of },
    { "u-listen-of", s_microformats, mp2_u_listen_of },
    { "u-logo", s_microformats, mp2_u_logo },
    { "u-key", s_microformats, mp2_u_key },
    { "u-photo", s_microformats, mp2_u_photo },
    { "u-read-of", s_microformats, mp2_u_read_of },
    { "u-repost", s_microformats, mp2_u_repost },
    { "u-repost-of", s_microformats, mp2_u_repost_of },
    { "u-sound", s_microformats, mp2_u_sound },
    { "u-syndication", s_microformats, mp2_u_syndication },
    { "u-translation-of", s_microformats, mp2_u_translation_of },
    { "u-uid", s_microformats, mp2_u_uid },
    { "u-url", s_microformats, mp2_u_url },
    { "u-video", s_microformats, mp2_u_video },
    { "u-watch-of", s_microformats, mp2_u_watch_of },

    // open graph music
    { "album", s_music, og_music_album },
    { "album:disc", s_music, og_music_album_disc },
    { "album:track", s_music, og_music_album_track },
    { "creator", s_music, og_music_creator },
    { "musician", s_music, og_music_musician },
    { "release_date", s_music, og_music_release_date },
    { "song", s_music, og_music_song },
    { "song:disc", s_music, og_music_song_disc },
    { "song:track", s_music, og_music_song_track },

    // web annotation
    { "annotationService", s_oa, oa_annotationservice },
    { "bodyValue", s_oa, oa_bodyvalue },
    { "cachedSource", s_oa, oa_cachedsource },
    { "canonical", s_oa, oa_canonical },
    { "end", s_oa, oa_end },
    { "exact", s_oa, oa_exact },
    { "hasBody", s_oa, oa_hasbody },
    { "hasEndSelector", s_oa, oa_hasendselector },
    { "hasPurpose", s_oa, oa_haspurpose },
    { "hasScope", s_oa, oa_hasscope },
    { "hasSelector", s_oa, oa_hasselector },
    { "hasSource", s_oa, oa_hassource },
    { "hasStartSelector", s_oa, oa_hasstartselector },
    { "hasState", s_oa, oa_hasstate },
    { "hasTarget", s_oa, oa_hastarget },
    { "motivatedBy", s_oa, oa_motivatedby },
    { "prefix", s_oa, oa_prefix },
    { "processingLanguage", s_oa, oa_processinglanguage },
    { "refinedBy", s_oa, oa_refinedby },
    { "renderedVia", s_oa, oa_renderedvia },
    { "sourceDate", s_oa, oa_sourcedate },
    { "sourceDateEnd", s_oa, oa_sourcedateend },
    { "sourceDateStart", s_oa, oa_sourcedatestart },
    { "start", s_oa, oa_start },
    { "styleClass", s_oa, oa_styleclass },
    { "styledBy", s_oa, oa_styledby },
    { "suffix", s_oa, oa_suffix },
    { "textDirection", s_oa, oa_textdirection },
    { "via", s_oa, oa_via },

    // open graph
    { "audio", s_og, og_audio },
    { "audio:secure_url", s_og, og_audio_secure_url },
    { "audio:type", s_og, og_audio_type },
    { "audio:url", s_og, og_audio_url },
    { "description", s_og, og_description },
    { "determiner", s_og, og_determiner },
    { "image", s_og, og_image },
    { "image:alt", s_og, og_image_alt },
    { "image:height", s_og, og_image_height },
    { "image:secure_url", s_og, og_image_secure_url },
    { "image:type", s_og, og_image_type },
    { "image:url", s_og, og_image_url },
    { "image:width", s_og, og_image_width },
    { "locale", s_og, og_locale },
    { "locale:alternate", s_og, og_locale_alternative },
    { "site_name", s_og, og_site_name },
    { "title", s_og, og_title },
    { "type", s_og, og_type },
    { "url", s_og, og_url },
    { "video", s_og, og_video },
    { "video:alt", s_og, og_video_alt },
    { "video:height", s_og, og_video_height },
    { "video:secure_url", s_og, og_video_secure_url },
    { "video:type", s_og, og_video_type },
    { "video:url", s_og, og_video_url },
    { "video:width", s_og, og_video_width },

    // odrl
    { "action", s_odrl, odrlp_action },
    { "and", s_odrl, odrlp_and },
    { "andSequence", s_odrl, odrlp_andsequence },
    { "assignee", s_odrl, odrlp_assignee },
    { "assigneeOf", s_odrl, odrlp_assigneeof },
    { "assigner", s_odrl, odrlp_assigner },
    { "assignerOf", s_odrl, odrlp_assignerof },
    { "attributedParty", s_odrl, odrlp_attributedparty },
    { "attributingParty", s_odrl, odrlp_attributingparty },
    { "compensatedParty", s_odrl, odrlp_compensatedparty },
    { "compensatingParty", s_odrl, odrlp_compensatingparty },
    { "conflict", s_odrl, odrlp_conflict },
    { "consentedParty", s_odrl, odrlp_consentedparty },
    { "consentingParty", s_odrl, odrlp_consentingparty },
    { "consequence", s_odrl, odrlp_consequence },
    { "constraint", s_odrl, odrlp_constraint },
    { "contractedParty", s_odrl, odrlp_contractedparty },
    { "contractingParty", s_odrl, odrlp_contractingparty },
    { "dataType", s_odrl, odrlp_datatype },
    { "duty", s_odrl, odrlp_duty },
    { "failure", s_odrl, odrlp_failure },
    { "function", s_odrl, odrlp_function },
    { "hasPolicy", s_odrl, odrlp_haspolicy },
    { "informedParty", s_odrl, odrlp_informedparty },
    { "informingParty", s_odrl, odrlp_informingparty },
    { "inheritFrom", s_odrl, odrlp_inheritfrom },
    { "leftOperand", s_odrl, odrlp_leftoperand },
    { "obligation", s_odrl, odrlp_obligation },
    { "output", s_odrl, odrlp_output },
    { "partOf", s_odrl, odrlp_partof },
    { "permission", s_odrl, odrlp_permission },
    { "profile", s_odrl, odrlp_profile },
    { "prohibition", s_odrl, odrlp_prohibition },
    { "refinement", s_odrl, odrlp_refinement },
    { "relation", s_odrl, odrlp_relation },
    { "remedy", s_odrl, odrlp_remedy },
    { "operand", s_odrl, odrlp_operand },
    { "operator", s_odrl, odrlp_operator },
    { "or", s_odrl, odrlp_or },
    { "rightOperand", s_odrl, odrlp_rightoperand },
    { "rightOperandReference", s_odrl, odrlp_rightoperandreference },
    { "source", s_odrl, odrlp_source },
    { "status", s_odrl, odrlp_status },
    { "target", s_odrl, odrlp_target },
    { "trackedParty", s_odrl, odrlp_trackedparty },
    { "trackingParty", s_odrl, odrlp_trackingparty },
    { "uid", s_odrl, odrlp_uid },
    { "unit", s_odrl, odrlp_unit },
    { "xone", s_odrl, odrlp_xone },

    // org
    { "basedAt", s_org, orgp_basedat },
    { "changedBy", s_org, orgp_changedby },
    { "classification", s_org, orgp_classification },
    { "hasMember", s_org, orgp_hasmember },
    { "hasMembership", s_org, orgp_hasmembership },
    { "hasPost", s_org, orgp_haspost },
    { "hasPrimarySite", s_org, orgp_hasprimarysite },
    { "hasRegisteredSite", s_org, orgp_hasregisteredsite },
    { "hasSite", s_org, orgp_hassite },
    { "hasSubOrganization", s_org, orgp_hassuborganisation },
    { "hasUnit", s_org, orgp_hasunit },
    { "headOf", s_org, orgp_headof },
    { "heldBy", s_org, orgp_heldby },
    { "holds", s_org, orgp_holds },
    { "identifier", s_org, orgp_identifier },
    { "linkedTo", s_org, orgp_linkedto },
    { "location", s_org, orgp_location },
    { "member", s_org, orgp_member },
    { "memberDuring", s_org, orgp_memberduring },
    { "memberOf", s_org, orgp_memberof },
    { "organization", s_org, orgp_organisation },
    { "originalOrganization", s_org, orgp_originalorganisation },
    { "postIn", s_org, orgp_postin },
    { "purpose", s_org, orgp_purpose },
    { "remuneration", s_org, orgp_remuneration },
    { "reportsTo", s_org, orgp_reportsto },
    { "resultedFrom", s_org, orgp_resultedfrom },
    { "resultingOrganization", s_org, orgp_resultingorganisation },
    { "role", s_org, orgp_role },
    { "roleProperty", s_org, orgp_roleproperty },
    { "siteAddress", s_org, orgp_siteaddress },
    { "siteOf", s_org, orgp_siteof },
    { "subOrganizationOf", s_org, orgp_suborganisationof },
    { "transitiveSubOrganizationOf", s_org, orgp_transitivesuborganisationof },
    { "unitOf", s_org, orgp_unitof },

    // owl
    { "allValuesFrom", s_owl, owl_allvaluesfrom },
    { "annotatedProperty", s_owl, owl_annotatedproperty },
    { "annotatedSource", s_owl, owl_annotatedsource },
    { "annotatedTarget", s_owl, owl_annotatedtarget },
    { "assertionProperty", s_owl, owl_assertionproperty },
    { "backwardCompatibleWith", s_owl, owl_backwardcompatiblewith },
    { "bottomDataProperty", s_owl, owl_bottomdataproperty },
    { "bottomObjectProperty", s_owl, owl_bottomobjectproperty },
    { "cardinality", s_owl, owl_cardinality },
    { "complementOf", s_owl, owl_complementof },
    { "datatypeComplementOf", s_owl, owl_datatypecomplementof },
    { "deprecated", s_owl, owl_deprecated },
    { "differentFrom", s_owl, owl_differentfrom },
    { "disjointUnionOf", s_owl, owl_disjointunionof },
    { "disjointWith", s_owl, owl_disjointwith },
    { "distinctMembers", s_owl, owl_distinctmembers },
    { "equivalentClass", s_owl, owl_equivalentclass },
    { "equivalentProperty", s_owl, owl_equivalentproperty },
    { "hasKey", s_owl, owl_haskey },
    { "hasSelf", s_owl, owl_hasself },
    { "hasValue", s_owl, owl_hasvalue },
    { "imports", s_owl, owl_imports },
    { "incompatibleWith", s_owl, owl_incompatiblewith },
    { "intersectionOf", s_owl, owl_intersectionof },
    { "inverseOf", s_owl, owl_inverseof },
    { "maxCardinality", s_owl, owl_maxcardinality },
    { "maxQualifiedCardinality", s_owl, owl_maxqualifiedcardinality },
    { "members", s_owl, owl_members },
    { "minCardinality", s_owl, owl_mincardinality },
    { "minQualifiedCardinality", s_owl, owl_minqualifiedcardinality },
    { "onClass", s_owl, owl_onclass },
    { "onDataRange", s_owl, owl_ondatarange },
    { "onDatatype", s_owl, owl_ondatatype },
    { "oneOf", s_owl, owl_oneof },
    { "onProperties", s_owl, owl_onproperties },
    { "onProperty", s_owl, owl_onproperty },
    { "priorVersion", s_owl, owl_priorversion },
    { "propertyChainAxiom", s_owl, owl_propertychainaxiom },
    { "propertyDisjointWith", s_owl, owl_propertydisjointwith },
    { "qualifiedCardinality", s_owl, owl_qualifiedcardinality },
    { "sameAs", s_owl, owl_sameas },
    { "someValuesFrom", s_owl, owl_somevaluesfrom },
    { "sourceIndividual", s_owl, owl_sourceindividual },
    { "targetIndividual", s_owl, owl_targetindividual },
    { "targetValue", s_owl, owl_targetvalue },
    { "topDataProperty", s_owl, owl_topdataproperty },
    { "topObjectProperty", s_owl, owl_topobjectproperty },
    { "unionOf", s_owl, owl_unionof },
    { "versionInfo", s_owl, owl_versioninfo },
    { "versionIRI", s_owl, owl_versioniri },
    { "withRestrictions", s_owl, owl_withrestrictions },

    // poetry
    { "age", s_poetry, poetry_age },
    { "containedinPoem", s_poetry, poetry_containedinpoem },
    { "containsPoem", s_poetry, poetry_containspoem },
    { "hasForm", s_poetry, poetry_hasform },
    { "hasStanza", s_poetry, poetry_hasstanza },
    { "inDialect", s_poetry, poetry_indialect },
    { "isChorus", s_poetry, poetry_ischorus },
    { "isLaidOut", s_poetry, poetry_islaidout },
    { "keepsStrictForm", s_poetry, poetry_keepsstrictform },
    { "lines", s_poetry, poetry_lines },
    { "medium", s_poetry, poetry_medium },
    { "metre", s_poetry, poetry_metre },
    { "poet", s_poetry, poetry_poet },
    { "region", s_poetry, poetry_region },
    { "rhymingScheme", s_poetry, poetry_rhymingscheme },
    { "subForm", s_poetry, poetry_subform },

    // open graph profile
    { "first_name", s_profile, og_profile_first_name },
    { "last_name", s_profile, og_profile_last_name },
    { "username", s_profile, og_profile_username },
    { "gender", s_profile, og_profile_gender },

    // provincial
    { "actedOnBehalfOf", s_prov, pp_actedonbehalfof },
    { "activity", s_prov, pp_activity },
    { "agent", s_prov, pp_agent },
    { "alternateOf", s_prov, pp_alternativeto },
    { "atLocation", s_prov, pp_atlocation },
    { "atTime", s_prov, pp_attime },
    { "endedAtTime", s_prov, pp_endedattime },
    { "entity", s_prov, pp_entity },
    { "generated", s_prov, pp_generated },
    { "generatedAtTime", s_prov, pp_generatedattime },
    { "hadActivity", s_prov, pp_hadactivity },
    { "hadGeneration", s_prov, pp_hadgeneration },
    { "hadMember", s_prov, pp_hadmember },
    { "hadPlan", s_prov, pp_hadplan },
    { "hadPrimarySource", s_prov, pp_hadprimarysource },
    { "hadRole", s_prov, pp_hadrole },
    { "hadUsage", s_prov, pp_hadusage },
    { "influenced", s_prov, pp_influenced },
    { "influencer", s_prov, pp_influencer },
    { "invalidated", s_prov, pp_invalidated },
    { "invalidatedAtTime", s_prov, pp_invalidatedattime },
    { "qualifiedAssociation", s_prov, pp_qualifiedassociation },
    { "qualifiedAttribution", s_prov, pp_qualifiedattribution },
    { "qualifiedCommunication", s_prov, pp_qualifiedcommunication },
    { "qualifiedDelegation", s_prov, pp_qualifieddelegation },
    { "qualifiedDerivation", s_prov, pp_qualifiedderivation },
    { "qualifiedEnd", s_prov, pp_qualifiedend },
    { "qualifiedGeneration", s_prov, pp_qualifiedgeneration },
    { "qualifiedInfluence", s_prov, pp_qualifiedinfluence },
    { "qualifiedInvalidation", s_prov, pp_qualifiedinvalidation },
    { "qualifiedPrimarySource", s_prov, pp_qualifiedprimarysource },
    { "qualifiedQuotation", s_prov, pp_qualifiedquotation },
    { "qualifiedRevision", s_prov, pp_qualifiedrevision },
    { "qualifiedStart", s_prov, pp_qualifiedstart },
    { "qualifiedUsage", s_prov, pp_qualifiedusage },
    { "specializationOf", s_prov, pp_specialisationof },
    { "startedAtTime", s_prov, pp_startedattime },
    { "used", s_prov, pp_used },
    { "value", s_prov, pp_value },
    { "wasAssociatedWith", s_prov, pp_wasassociatedwith },
    { "wasAttributedTo", s_prov, pp_wasattributedto },
    { "wasDerivedFrom", s_prov, pp_wasderivedfrom },
    { "wasEndedBy", s_prov, pp_wasendedby },
    { "wasGeneratedBy", s_prov, pp_wasgeneratedby },
    { "wasInfluencedBy", s_prov, pp_wasinfluencedby },
    { "wasInformedBy", s_prov, pp_wasinformedby },
    { "wasInvalidatedBy", s_prov, pp_wasinvalidatedby },
    { "wasQuotedFrom", s_prov, pp_wasquotedfrom },
    { "wasRevisionOf", s_prov, pp_wasrevisionof },
    { "wasStartedBy", s_prov, pp_wasstartedby },

    // ptr
    { "bytes", s_ptr, ptr_bytes },
    { "characterEncoding", s_ptr, ptr_characterencoding },
    { "chars", s_ptr, ptr_chars },
    { "declaredEncoding", s_ptr, ptr_declaredencoding },
    { "doctypeName", s_ptr, ptr_doctypename },
    { "dtDecl", s_ptr, ptr_dtdecl },
    { "internalSubset", s_ptr, ptr_internalsubset },
    { "leadingMisc", s_ptr, ptr_leadingmisc },
    { "publicId", s_ptr, ptr_publicid },
    { "rest", s_ptr, ptr_rest },
    { "standalone", s_ptr, ptr_standalone },
    { "systemId", s_ptr, ptr_systemid },
    { "version", s_ptr, ptr_version },

    // data cube
     { "attribute", s_qb, qbp_attribute },
     { "codeList", s_qb, qbp_codelist },
     { "component", s_qb, qbp_component },
     { "componentAttachment", s_qb, qbp_componentattachment },
     { "componentProperty", s_qb, qbp_componentproperty },
     { "componentRequired", s_qb, qbp_componentrequired },
     { "concept", s_qb, qbp_concept },
     { "dataSet", s_qb, qbp_dataset },
     { "dimension", s_qb, qbp_dimension },
     { "hierarchyRoot", s_qb, qbp_hierarchyroot },
     { "measure", s_qb, qbp_measure },
     { "measureDimension", s_qb, qbp_measuredimension },
     { "measureType", s_qb, qbp_measuretype },
     { "observation", s_qb, qbp_observation },
     { "observationGroup", s_qb, qbp_observationgroup },
     { "order", s_qb, qbp_order },
     { "parentChildProperty", s_qb, qbp_parentchildproperty },
     { "slice", s_qb, qbp_slice },
     { "sliceKey", s_qb, qbp_slicekey },
     { "sliceStructure", s_qb, qbp_slicestructure },
     { "structure", s_qb, qbp_structure },

    // RDF
    { "description", s_rdf, rdf_description },
    { "direction", s_rdf, rdf_direction },
    { "first", s_rdf, rdf_first },
    { "language", s_rdf, rdf_language },
    { "object", s_rdf, rdf_object },
    { "predicate", s_rdf, rdf_predicate },
    { "resource", s_rdf, rdf_resource },
    { "rest", s_rdf, rdf_rest },
    { "subject", s_rdf, rdf_subject },
    { "type", s_rdf, rdf_type },
    { "value", s_rdf, rdf_value },

    // RDFg
    { "equivalentGraph", s_rdfg, rdfg_equivalentgraph },
    { "subGraphOf", s_rdfg, rdfg_subgraphof },

    // RDFs
    { "comment", s_rdfs, rdfs_comment },
    { "domain", s_rdfs, rdfs_domain },
    { "isDefinedBy", s_rdfs, rdfs_isdefinedby },
    { "label", s_rdfs, rdfs_label },
    { "member", s_rdfs, rdfs_member },
    { "range", s_rdfs, rdfs_range },
    { "seeAlso", s_rdfs, rdfs_seealso },
    { "subClassOf", s_rdfs, rdfs_subclassof },
    { "subPropertyOf", s_rdfs, rdfs_subpropertyof },

    // RDF Reviews
    { "commenter", s_rev, rev_commenter },
    { "hasComment", s_rev, rev_hascomment },
    { "hasFeedback", s_rev, rev_hasfeedback },
    { "hasReview", s_rev, rev_hasreview },
    { "maxRating", s_rev, rev_maxrating },
    { "minRating", s_rev, rev_minrating },
    { "positiveVotes", s_rev, rev_positivevotes },
    { "rating", s_rev, rev_rating },
    { "reviewer", s_rev, rev_reviewer },
    { "text", s_rev, rev_text },
    { "totalVotes", s_rev, rev_totalvotes },
    { "type", s_rev, rev_type },

    // rdb to rdf
    { "child", s_rr, rrp_child },
    { "class", s_rr, rrp_class },
    { "column", s_rr, rrp_column },
    { "constant", s_rr, rrp_constant },
    { "datatype", s_rr, rrp_datatype },
    { "graph", s_rr, rrp_graph },
    { "graphMap", s_rr, rrp_graphmap },
    { "inverseExpression", s_rr, rrp_inverseexpression },
    { "joinCondition", s_rr, rrp_joincondition },
    { "language", s_rr, rrp_language },
    { "logicalTable", s_rr, rrp_logicaltable },
    { "object", s_rr, rrp_object },
    { "objectMap", s_rr, rrp_objectmap },
    { "parent", s_rr, rrp_parent },
    { "parentTriplesMap", s_rr, rrp_parenttriplesmap },
    { "predicate", s_rr, rrp_predicate },
    { "predicateMap", s_rr, rrp_predicatemap },
    { "predicateObjectMap", s_rr, rrp_predicateobjectmap },
    { "sqlQuery", s_rr, rrp_sqlquery },
    { "sqlVersion", s_rr, rrp_sqlversion },
    { "subject", s_rr, rrp_subject },
    { "subjectMap", s_rr, rrp_subjectmap },
    { "tableName", s_rr, rrp_tablename },
    { "template", s_rr, rrp_template },
    { "termType", s_rr, rrp_termtype },

    // schema.org
    { "about", s_schema, sp_about },
    { "abridged", s_schema, sp_abridged },
    { "abstract", s_schema, sp_abstract },
    { "accelerationTime", s_schema, sp_accelerationtime },
    { "acceptedAnswer", s_schema, sp_acceptedanswer },
    { "acceptedOffer", s_schema, sp_acceptedoffer },
    { "acceptedPaymentMethod", s_schema, sp_acceptedpaymentmethod },
    { "acceptsReservations", s_schema, sp_acceptsreservations },
    { "accessCode", s_schema, sp_accesscode },
    { "accessMode", s_schema, sp_accessmode },
    { "accessModeSufficient", s_schema, sp_accessmodesufficient },
    { "accessibilityAPI", s_schema, sp_accessibilityapi },
    { "accessibilityControl", s_schema, sp_accessibilitycontrol },
    { "accessibilityFeature", s_schema, sp_accessibilityfeature },
    { "accessibilityHazard", s_schema, sp_accessibilityhazard },
    { "accessibilitySummary", s_schema, sp_accessibilitysummary },
    { "accommodationCategory", s_schema, sp_accommodationcategory },
    { "accommodationFloorPlan", s_schema, sp_accommodationfloorplan },
    { "accountId", s_schema, sp_accountid },
    { "accountMinimumInflow", s_schema, sp_accountminimuminflow },
    { "accountOverdraftLimit", s_schema, sp_accountoverdraftlimit },
    { "accountablePerson", s_schema, sp_accountableperson },
    { "acquireLicensePage", s_schema, sp_acquirelicensepage },
    { "acquiredFrom", s_schema, sp_acquiredfrom },
    { "acrissCode", s_schema, sp_acrisscode },
    { "action", s_schema, sp_action },
    { "actionAccessibilityRequirement", s_schema, sp_actionaccessibilityrequirement },
    { "actionApplication", s_schema, sp_actionapplication },
    { "actionOption", s_schema, sp_actionoption },
    { "actionPlatform", s_schema, sp_actionplatform },
    { "actionStatus", s_schema, sp_actionstatus },
    { "actionableFeedbackPolicy", s_schema, sp_actionablefeedbackpolicy },
    { "activeIngredient", s_schema, sp_activeingredient },
    { "activityDuration", s_schema, sp_activityduration },
    { "activityFrequency", s_schema, sp_activityfrequency },
    { "actor", s_schema, sp_actor },
    { "actors", s_schema, sp_actors },
    { "addOn", s_schema, sp_addon },
    { "additionalName", s_schema, sp_additionalname },
    { "additionalNumberOfGuests", s_schema, sp_additionalnumberofguests },
    { "additionalProperty", s_schema, sp_additionalproperty },
    { "additionalType", s_schema, sp_additionaltype },
    { "additionalVariable", s_schema, sp_additionalvariable },
    { "address", s_schema, sp_address },
    { "addressCountry", s_schema, sp_addresscountry },
    { "addressLocality", s_schema, sp_addresslocality },
    { "addressRegion", s_schema, sp_addressregion },
    { "administrationRoute", s_schema, sp_administrationroute },
    { "advanceBookingRequirement", s_schema, sp_advancebookingrequirement },
    { "adverseOutcome", s_schema, sp_adverseoutcome },
    { "affectedBy", s_schema, sp_affectedby },
    { "affiliation", s_schema, sp_affiliation },
    { "afterMedia", s_schema, sp_aftermedia },
    { "agent", s_schema, sp_agent },
    { "aggregateRating", s_schema, sp_aggregaterating },
    { "aircraft", s_schema, sp_aircraft },
    { "album", s_schema, sp_album },
    { "albums", s_schema, sp_albums },
    { "albumProductionType", s_schema, sp_albumproductiontype },
    { "albumRelease", s_schema, sp_albumrelease },
    { "albumReleaseType", s_schema, sp_albumreleasetype },
    { "alcoholWarning", s_schema, sp_alcoholwarning },
    { "algorithm", s_schema, sp_algorithm },
    { "alignmentType", s_schema, sp_alignmenttype },
    { "alternateName", s_schema, sp_alternativename },
    { "alternativeHeadline", s_schema, sp_alternativeheadline },
    { "alternativeOf", s_schema, sp_alternativeof },
    { "alumni", s_schema, sp_alumni },
    { "alumniOf", s_schema, sp_alumniof },
    { "amenityFeature", s_schema, sp_amenityfeature },
    { "amount", s_schema, sp_amount },
    { "amountOfThisGood", s_schema, sp_amountofthisgood },
    { "announcementLocation", s_schema, sp_announcementlocation },
    { "annualPercentageRate", s_schema, sp_annualpercentagerate },
    { "answerCount", s_schema, sp_answercount },
    { "antagonist", s_schema, sp_antagonist },
    { "appearance", s_schema, sp_appearance },
    { "applicableCountry", s_schema, sp_applicablecountry },
    { "applicableLocation", s_schema, sp_applicablelocation },
    { "applicantContact", s_schema, sp_applicantcontact },
    { "applicantLocationRequirements", s_schema, sp_applicantlocationrequirements },
    { "application", s_schema, sp_application },
    { "applicationCategory", s_schema, sp_applicationcategory },
    { "applicationDeadline", s_schema, sp_applicationdeadline },
    { "applicationStartDate", s_schema, sp_applicationstartdate },
    { "applicationSubCategory", s_schema, sp_applicationsubcategory },
    { "applicationSuite", s_schema, sp_applicationsuite },
    { "appliesToDeliveryMethod", s_schema, sp_appliestodeliverymethod },
    { "appliesToPaymentMethod", s_schema, sp_appliestopaymentmethod },
    { "area", s_schema, sp_area },
    { "archivedAt", s_schema, sp_archivedat },
    { "archiveHeld", s_schema, sp_archiveheld },
    { "areaServed", s_schema, sp_areaserved },
    { "arrivalAirport", s_schema, sp_arrivalairport },
    { "arrivalBoatTerminal", s_schema, sp_arrivalboatterminal },
    { "arrivalBusStop", s_schema, sp_arrivalbusstop },
    { "arrivalGate", s_schema, sp_arrivalgate },
    { "arrivalPlatform", s_schema, sp_arrivalplatform },
    { "arrivalStation", s_schema, sp_arrivalstation },
    { "arrivalTerminal", s_schema, sp_arrivalterminal },
    { "arrivalTime", s_schema, sp_arrivaltime },
    { "artEdition", s_schema, sp_artedition },
    { "artMedium", s_schema, sp_artmedium },
    { "arterialBranch", s_schema, sp_arterialbranch },
    { "artform", s_schema, sp_artform },
    { "articleBody", s_schema, sp_articlebody },
    { "articleSection", s_schema, sp_articlesection },
    { "artist", s_schema, sp_artist },
    { "artworkSurface", s_schema, sp_artworksurface },
    { "aspect", s_schema, sp_aspect },
    { "assemblyVersion", s_schema, sp_assemblyversion },
    { "assess", s_schema, sp_assess },
    { "assesses", s_schema, sp_assesses },
    { "associatedAnatomy", s_schema, sp_associatedanatomy },
    { "associatedArticle", s_schema, sp_associatedarticle },
    { "associatedClaimReview", s_schema, sp_associatedclaimreview },
    { "associatedDisease", s_schema, sp_associateddisease },
    { "associatedMedia", s_schema, sp_associatedmedia },
    { "associatedMediaReview", s_schema, sp_associatedmediareview },
    { "associatedPathophysiology", s_schema, sp_associatedpathophysiology },
    { "associatedReview", s_schema, sp_associatedreview },
    { "athlete", s_schema, sp_athlete },
    { "attendee", s_schema, sp_attendee },
    { "attendees", s_schema, sp_attendees },
    { "audience", s_schema, sp_audience },
    { "audienceType", s_schema, sp_audiencetype },
    { "audio", s_schema, sp_audio },
    { "authenticator", s_schema, sp_authenticator },
    { "author", s_schema, sp_author },
    { "availability", s_schema, sp_availability },
    { "availabilityEnds", s_schema, sp_availabilityends },
    { "availabilityStarts", s_schema, sp_availabilitystarts },
    { "availableAtOrFrom", s_schema, sp_availableatorfrom },
    { "availableChannel", s_schema, sp_availablechannel },
    { "availableDeliveryMethod", s_schema, sp_availabledeliverymethod },
    { "availableFrom", s_schema, sp_availablefrom },
    { "availableIn", s_schema, sp_availablein },
    { "availableLanguage", s_schema, sp_availablelanguage },
    { "availableLocation", s_schema, sp_availablelocation },
    { "availableOnDevice", s_schema, sp_availableondevice },
    { "availableService", s_schema, sp_availableservice },
    { "availableStrength", s_schema, sp_availablestrength },
    { "availableTest", s_schema, sp_availabletest },
    { "availableThrough", s_schema, sp_availableuntil }, // in standard English, available through means supplier or distributor
    { "award", s_schema, sp_award },
    { "awards", s_schema, sp_awards },
    { "awayTeam", s_schema, sp_awayteam },
    { "background", s_schema, sp_background },
    { "backstory", s_schema, sp_backstory },
    { "bankAccountType", s_schema, sp_bankaccounttype },
    { "baseSalary", s_schema, sp_basesalary },
    { "bccRecipient", s_schema, sp_bccrecipient },
    { "bed", s_schema, sp_bed },
    { "beforeMedia", s_schema, sp_beforemedia },
    { "beneficiaryBank", s_schema, sp_beneficiarybank },
    { "benefits", s_schema, sp_benefits },
    { "benefitsSummaryUrl", s_schema, sp_benefitssummaryurl },
    { "bestRating", s_schema, sp_bestrating },
    { "billingAddress", s_schema, sp_billingaddress },
    { "billingDuration", s_schema, sp_billingduration },
    { "billingIncrement", s_schema, sp_billingincrement },
    { "billingPeriod", s_schema, sp_billingperiod },
    { "billingStart", s_schema, sp_billingstart },
    { "bioChemInteraction", s_schema, sp_biocheminteraction },
    { "bioChemSimilarity", s_schema, sp_biochemsimilarity },
    { "biologicalRole", s_schema, sp_biologicalrole },
    { "biomechanicalClass", s_schema, sp_biomechanicalclass },
    { "birthDate", s_schema, sp_birthdate },
    { "birthPlace", s_schema, sp_birthplace },
    { "bitrate", s_schema, sp_bitrate },
    { "blogPost", s_schema, sp_blogpost },
    { "blogPosts", s_schema, sp_blogposts },
    { "bloodSupply", s_schema, sp_bloodsupply },
    { "boardingGroup", s_schema, sp_boardinggroup },
    { "boardingPolicy", s_schema, sp_boardingpolicy },
    { "bodyLocation", s_schema, sp_bodylocation },
    { "bodyType", s_schema, sp_bodytype },
    { "bookEdition", s_schema, sp_bookedition },
    { "bookFormat", s_schema, sp_bookformat },
    { "bookingAgent", s_schema, sp_bookingagent },
    { "bookingTime", s_schema, sp_bookingtime },
    { "box", s_schema, sp_box },
    { "borrower", s_schema, sp_borrower },
    { "branch", s_schema, sp_branch },
    { "branchCode", s_schema, sp_branchcode },
    { "branchOf", s_schema, sp_branchof },
    { "brand", s_schema, sp_brand },
    { "breadcrumb", s_schema, sp_breadcrumb },
    { "breastfeedingWarning", s_schema, sp_breastfeedingwarning },
    { "broadcastAffiliateOf", s_schema, sp_broadcastaffiliateof },
    { "broadcastChannelId", s_schema, sp_broadcastchannelid },
    { "broadcastDisplayName", s_schema, sp_broadcastdisplayname },
    { "broadcastFrequency", s_schema, sp_broadcastfrequency },
    { "broadcastFrequencyValue", s_schema, sp_broadcastfrequencyvalue },
    { "broadcastOfEvent", s_schema, sp_broadcastofevent },
    { "broadcastServiceTier", s_schema, sp_broadcastservicetier },
    { "broadcastSignalModulation", s_schema, sp_broadcastsignalmodulation },
    { "broadcastSubChannel", s_schema, sp_broadcastsubchannel },
    { "broadcastTimezone", s_schema, sp_broadcasttimezone },
    { "broadcaster", s_schema, sp_broadcaster },
    { "broker", s_schema, sp_broker },
    { "browserRequirements", s_schema, sp_browserrequirements },
    { "busName", s_schema, sp_busname },
    { "busNumber", s_schema, sp_busnumber },
    { "businessDays", s_schema, sp_businessdays },
    { "businessFunction", s_schema, sp_businessfunction },
    { "buyer", s_schema, sp_buyer },
    { "byArtist", s_schema, sp_byartist },
    { "byDay", s_schema, sp_byday },
    { "byMonth", s_schema, sp_bymonth },
    { "byMonthDay", s_schema, sp_bymonthday },
    { "byMonthWeek", s_schema, sp_bymonthweek },
    { "callSign", s_schema, sp_callsign },
    { "calories", s_schema, sp_calories },
    { "candidate", s_schema, sp_candidate },
    { "caption", s_schema, sp_caption },
    { "carbohydrateContent", s_schema, sp_carbohydratecontent },
    { "cargoVolume", s_schema, sp_cargovolume },
    { "carrierRequirements", s_schema, sp_carrierrequirements },
    { "cashBack", s_schema, sp_cashback },
    { "catalog", s_schema, sp_catalogue },
    { "catalogNumber", s_schema, sp_cataloguenumber },
    { "category", s_schema, sp_category },
    { "cause", s_schema, sp_cause },
    { "causeOf", s_schema, sp_causeof },
    { "ccRecipient", s_schema, sp_ccrecipient },
    { "character", s_schema, sp_character },
    { "characterAttribute", s_schema, sp_characterattribute },
    { "characterName", s_schema, sp_charactername },
    { "cheatCode", s_schema, sp_cheatcode },
    { "checkinTime", s_schema, sp_checkintime },
    { "checkoutTime", s_schema, sp_checkouttime },
    { "chemicalComposition", s_schema, sp_chemicalcomposition },
    { "chemicalRole", s_schema, sp_chemicalrole },
    { "childMaxAge", s_schema, sp_childmaxage },
    { "childMinAge", s_schema, sp_childminage },
    { "children", s_schema, sp_children },
    { "childTaxon", s_schema, sp_childtaxon },
    { "cholesterolContent", s_schema, sp_cholesterolcontent },
    { "circle", s_schema, sp_circle },
    { "citation", s_schema, sp_citation },
    { "claimInterpreter", s_schema, sp_claiminterpreter },
    { "claimReviewed", s_schema, sp_claimreviewed },
    { "clinicalPharamcology", s_schema, sp_clinicalpharamcology },
    { "clipNumber", s_schema, sp_clipnumber },
    { "closes", s_schema, sp_closes },
    { "coach", s_schema, sp_coach },
    { "code", s_schema, sp_code },
    { "codeRepository", s_schema, sp_coderepository },
    { "codeSampleType", s_schema, sp_codesampletype },
    { "codeValue", s_schema, sp_codevalue },
    { "codingSystem", s_schema, sp_codingsystem },
    { "colleague", s_schema, sp_colleague },
    { "colleagues", s_schema, sp_colleagues },
    { "collection", s_schema, sp_collection },
    { "collectionSize", s_schema, sp_collectionsize },
    { "color", s_schema, sp_colour },
    { "colorist", s_schema, sp_colourist },
    { "comment", s_schema, sp_comment },
    { "commentCount", s_schema, sp_commentcount },
    { "commentText", s_schema, sp_commenttext },
    { "commentTime", s_schema, sp_commenttime },
    { "competencyRequired", s_schema, sp_competencyrequired },
    { "competitor", s_schema, sp_competitor },
    { "composer", s_schema, sp_composer },
    { "comprisedOf", s_schema, sp_comprisedof },
    { "conditionsOfAccess", s_schema, sp_conditionsofaccess },
    { "confirmationNumber", s_schema, sp_confirmationnumber },
    { "connectedTo", s_schema, sp_connectedto },
    { "constrainingProperty", s_schema, sp_constrainingproperty },
    { "contactOption", s_schema, sp_contactoption },
    { "contactPoint", s_schema, sp_contactpoint },
    { "contactPoints", s_schema, sp_contactpoints },
    { "contactType", s_schema, sp_contacttype },
    { "contactlessPayment", s_schema, sp_contactlesspayment },
    { "containedIn", s_schema, sp_containedin },
    { "containedInPlace", s_schema, sp_containedinplace },
    { "containsPlace", s_schema, sp_containsplace },
    { "containsSeason", s_schema, sp_containsseason },
    { "contentLocation", s_schema, sp_contentlocation },
    { "contentRating", s_schema, sp_contentrating },
    { "contentReferenceTime", s_schema, sp_contentreferencetime },
    { "contentSize", s_schema, sp_contentsize },
    { "contentType", s_schema, sp_contenttype },
    { "contentUrl", s_schema, sp_contenturl },
    { "contraindication", s_schema, sp_contraindication },
    { "contributor", s_schema, sp_contributor },
    { "cookTime", s_schema, sp_cooktime },
    { "cookingMethod", s_schema, sp_cookingmethod },
    { "copyrightHolder", s_schema, sp_copyrightholder },
    { "copyrightNotice", s_schema, sp_copyrightnotice },
    { "copyrightYear", s_schema, sp_copyrightyear },
    { "correction", s_schema, sp_correction },
    { "correctionsPolicy", s_schema, sp_correctionspolicy },
    { "cost", s_schema, sp_cost },
    { "costCategory", s_schema, sp_costcategory },
    { "costCurrency", s_schema, sp_costcurrency },
    { "costOrigin", s_schema, sp_costorigin },
    { "costPerUnit", s_schema, sp_costperunit },
    { "countriesNotSupported", s_schema, sp_countriesnotsupported },
    { "countriesSupported", s_schema, sp_countriessupported },
    { "countryOfAssembly", s_schema, sp_countryofassembly },
    { "countryOfLastProcessing", s_schema, sp_countryoflastprocessing },
    { "countryOfOrigin", s_schema, sp_countryoforigin },
    { "course", s_schema, sp_course },
    { "courseCode", s_schema, sp_coursecode },
    { "courseMode", s_schema, sp_coursemode },
    { "coursePresequisites", s_schema, sp_coursepresequisites },
    { "courseWorkload", s_schema, sp_courseworkload },
    { "coverageEndTime", s_schema, sp_coverageendtime },
    { "coverageStartTime", s_schema, sp_coveragestarttime },
    { "creativeWorkStatus", s_schema, sp_creativeworkstatus },
    { "creator", s_schema, sp_creator },
    { "credentialCategory", s_schema, sp_credentialcategory },
    { "creditText", s_schema, sp_credittext },
    { "creditedTo", s_schema, sp_creditedto },
    { "cssSelector", s_schema, sp_cssselector },
    { "currenciesAccepted", s_schema, sp_currenciesaccepted },
    { "currency", s_schema, sp_currency },
    { "currentExchangeRate", s_schema, sp_currentexchangerate },
    { "customer", s_schema, sp_customer },
    { "cutoffTime", s_schema, sp_cutofftime },
    { "cvdCollectionDate", s_schema, sp_cvdcollectiondate },
    { "cvdFacilityCounty", s_schema, sp_cvdfacilitycounty },
    { "cvdFacilityId", s_schema, sp_cvdfacilityid },
    { "cvdNumBeds", s_schema, sp_cvdnumbeds },
    { "cvdNumBedsOcc", s_schema, sp_cvdnumbedsocc },
    { "cvdNumC19Died", s_schema, sp_cvdnumc19died },
    { "cvdNumC19HOPats", s_schema, sp_cvdnumc19hopats },
    { "cvdNumC19HospPats", s_schema, sp_cvdnumc19hosppats },
    { "cvdNumC19MechVentPats", s_schema, sp_cvdnumc19mechventpats },
    { "cvdNumC19OFMechVentPats", s_schema, sp_cvdnumc19ofmechventpats },
    { "cvdNumC19OverflowPats", s_schema, sp_cvdnumc19overflowpats },
    { "cvdNumICUBeds", s_schema, sp_cvdnumicubeds },
    { "cvdNumICUBedsOcc", s_schema, sp_cvdnumicubedsocc },
    { "cvdNumTotBeds", s_schema, sp_cvdnumtotbeds },
    { "cvdNumVent", s_schema, sp_cvdnumvent },
    { "cvdNumVentUse", s_schema, sp_cvdnumventuse },
    { "customerRemorseReturnFees", s_schema, sp_customerremorsereturnfees },
    { "customerRemorseReturnLabelSource", s_schema, sp_customerremorsereturnlabelsource },
    { "customerRemorseReturnShippingFeesAmount", s_schema, sp_customerremorsereturnshippingfeesamount },
    { "dataFeedElement", s_schema, sp_datafeedelement },
    { "dataset", s_schema, sp_dataset },
    { "datasetTimeInterval", s_schema, sp_datasettimeinterval },
    { "dateCreated", s_schema, sp_datecreated },
    { "dateDeleted", s_schema, sp_datedeleted },
    { "dateIssued", s_schema, sp_dateissued },
    { "dateModified", s_schema, sp_datemodified },
    { "datePosted", s_schema, sp_dateposted },
    { "datePublished", s_schema, sp_datepublished },
    { "dateRead", s_schema, sp_dateread },
    { "dateReceived", s_schema, sp_datereceived },
    { "dateSent", s_schema, sp_datesent },
    { "dateVehicleFirstRegistered", s_schema, sp_datevehiclefirstregistered },
    { "dateline", s_schema, sp_dateline },
    { "dayOfWeek", s_schema, sp_dayofweek },
    { "deathDate", s_schema, sp_deathdate },
    { "deathPlace", s_schema, sp_deathplace },
    { "defaultValue", s_schema, sp_defaultvalue },
    { "deliveryAddress", s_schema, sp_deliveryaddress },
    { "deliveryLeadTime", s_schema, sp_deliveryleadtime },
    { "deliveryMethod", s_schema, sp_deliverymethod },
    { "deliveryStatus", s_schema, sp_deliverystatus },
    { "deliveryTime", s_schema, sp_deliverytime },
    { "department", s_schema, sp_department },
    { "departureAirport", s_schema, sp_departureairport },
    { "departureBoatTerminal", s_schema, sp_departureboatterminal },
    { "departureBusStop", s_schema, sp_departurebusstop },
    { "departureGate", s_schema, sp_departuregate },
    { "departurePlatform", s_schema, sp_departureplatform },
    { "departureStation", s_schema, sp_departurestation },
    { "departureTerminal", s_schema, sp_departureterminal },
    { "departureTime", s_schema, sp_departuretime },
    { "dependencies", s_schema, sp_dependencies },
    { "depth", s_schema, sp_depth },
    { "description", s_schema, sp_description },
    { "device", s_schema, sp_device },
    { "diagnosis", s_schema, sp_diagnosis },
    { "diagram", s_schema, sp_diagram },
    { "diet", s_schema, sp_diet },
    { "dietFeatures", s_schema, sp_dietfeatures },
    { "differentialDiagnosis", s_schema, sp_differentialdiagnosis },
    { "director", s_schema, sp_director },
    { "directors", s_schema, sp_directors },
    { "directApply", s_schema, sp_directapply },
    { "disambiguatingDescription", s_schema, sp_disambiguatingdescription },
    { "discount", s_schema, sp_discount },
    { "discountCode", s_schema, sp_discountcode },
    { "discountCurrency", s_schema, sp_discountcurrency },
    { "discusses", s_schema, sp_discusses },
    { "discussionUrl", s_schema, sp_discussionurl },
    { "diseasePreventionInfo", s_schema, sp_diseasepreventioninfo },
    { "diseaseSpreadStatistics", s_schema, sp_diseasespreadstatistics },
    { "dissolutionDate", s_schema, sp_dissolutiondate },
    { "distance", s_schema, sp_distance },
    { "distinguishingSign", s_schema, sp_distinguishingsign },
    { "distribution", s_schema, sp_distribution },
    { "diversityPolicy", s_schema, sp_diversitypolicy },
    { "diversityStaffingReport", s_schema, sp_diversitystaffingreport },
    { "documentation", s_schema, sp_documentation },
    { "doesNotShip", s_schema, sp_doesnotship },
    { "domainIncludes", s_schema, sp_domainincludes },
    { "domiciledMortgage", s_schema, sp_domiciledmortgage },
    { "doorTime", s_schema, sp_doortime },
    { "dosageForm", s_schema, sp_dosageform },
    { "doseSchedule", s_schema, sp_doseschedule },
    { "doseUnit", s_schema, sp_doseunit },
    { "doseValue", s_schema, sp_dosevalue },
    { "downPayment", s_schema, sp_downpayment },
    { "downloadUrl", s_schema, sp_downloadurl },
    { "downvoteCount", s_schema, sp_downvotecount },
    { "drainsTo", s_schema, sp_drainsto },
    { "driveWheelConfiguration", s_schema, sp_drivewheelconfiguration },
    { "dropoffLocation", s_schema, sp_dropofflocation },
    { "dropoffTime", s_schema, sp_dropofftime },
    { "drug", s_schema, sp_drug },
    { "drugClass", s_schema, sp_drugclass },
    { "drugUnit", s_schema, sp_drugunit },
    { "duns", s_schema, sp_duns },
    { "duplicateTherapy", s_schema, sp_duplicatetherapy },
    { "duration", s_schema, sp_duration },
    { "durationOfWarranty", s_schema, sp_durationofwarranty },
    { "duringMedia", s_schema, sp_duringmedia },
    { "earlyPrepaymentPenalty", s_schema, sp_earlyprepaymentpenalty },
    { "editEIDR", s_schema, sp_editeidr },
    { "editor", s_schema, sp_editor },
    { "eduQuestionType", s_schema, sp_eduquestiontype },
    { "educationCredentialAwarded", s_schema, sp_educationcredentialawarded },
    { "educationRequirements", s_schema, sp_educationrequirements },
    { "educationalAlignment", s_schema, sp_educationalalignment },
    { "educationalCredentialAwarded", s_schema, sp_educationalcredentialawarded },
    { "educationalFramework", s_schema, sp_educationalframework },
    { "educationalLevel", s_schema, sp_educationallevel },
    { "educationalProgramMode", s_schema, sp_educationalprogrammode },
    { "educationalRole", s_schema, sp_educationalrole },
    { "educationalUse", s_schema, sp_educationaluse },
    { "elevation", s_schema, sp_elevation },
    { "eligibileCustomerType", s_schema, sp_eligibilecustomertype },
    { "eligibilityToWorkRequirement", s_schema, sp_eligibilitytoworkrequirement },
    { "eligibleCustomerType", s_schema, sp_eligiblecustomertype },
    { "eligibleDuration", s_schema, sp_eligibleduration },
    { "eligibleQuantity", s_schema, sp_eligiblequantity },
    { "eligibleRegion", s_schema, sp_eligibleregion },
    { "eligibleTransactionVolume", s_schema, sp_eligibletransactionvolume },
    { "email", s_schema, sp_email },
    { "embeddedTextCaption", s_schema, sp_embeddedtextcaption },
    { "embedUrl", s_schema, sp_embedurl },
    { "emmissionsCO2", s_schema, sp_emmissionsco2 },
    { "employee", s_schema, sp_employee },
    { "employees", s_schema, sp_employees },
    { "employerOverview", s_schema, sp_employeroverview },
    { "employmentType", s_schema, sp_employmenttype },
    { "employmentUnit", s_schema, sp_employmentunit },
    { "encodesBioChementity", s_schema, sp_encodesbiochementity },
    { "encodesCreativeWork", s_schema, sp_encodescreativework },
    { "encoding", s_schema, sp_encoding },
    { "encodings", s_schema, sp_encodings },
    { "encodingFormat", s_schema, sp_encodingformat },
    { "encodingType", s_schema, sp_encodingtype },
    { "endDate", s_schema, sp_enddate },
    { "endOffset", s_schema, sp_endoffset },
    { "endTime", s_schema, sp_endtime },
    { "endorsee", s_schema, sp_endorsee },
    { "endorsers", s_schema, sp_endorsers },
    { "energyEfficiencyScaleMax", s_schema, sp_energyefficiencyscalemax },
    { "energyEfficiencyScaleMin", s_schema, sp_energyefficiencyscalemin },
    { "engineDisplacement", s_schema, sp_enginedisplacement },
    { "enginePower", s_schema, sp_enginepower },
    { "engineType", s_schema, sp_enginetype },
    { "entertainmentBusiness", s_schema, sp_entertainmentbusiness },
    { "epidemiology", s_schema, sp_epidemiology },
    { "episode", s_schema, sp_episode },
    { "episodes", s_schema, sp_episodes },
    { "episodeNumber", s_schema, sp_episodenumber },
    { "equal", s_schema, sp_equal },
    { "error", s_schema, sp_error },
    { "estimatedCost", s_schema, sp_estimatedcost },
    { "estimatedFlightDuration", s_schema, sp_estimatedflightduration },
    { "estimatedSalary", s_schema, sp_estimatedsalary },
    { "estimatesRiskOf", s_schema, sp_estimatesriskof },
    { "ethicsPolicy", s_schema, sp_ethicspolicy },
    { "event", s_schema, sp_event },
    { "events", s_schema, sp_events },
    { "eventAttendanceMode", s_schema, sp_eventattendancemode },
    { "eventSchedule", s_schema, sp_eventschedule },
    { "eventStatus", s_schema, sp_eventstatus },
    { "evidenceLevel", s_schema, sp_evidencelevel },
    { "evidenceOrigin", s_schema, sp_evidenceorigin },
    { "exampleOfWork", s_schema, sp_exampleofwork },
    { "exceptDate", s_schema, sp_exceptdate },
    { "exchangeRateSpread", s_schema, sp_exchangeratespread },
    { "executableLibraryName", s_schema, sp_executablelibraryname },
    { "exerciseCourse", s_schema, sp_exercisecourse },
    { "exercisePlan", s_schema, sp_exerciseplan },
    { "exerciseRelatedDiet", s_schema, sp_exerciserelateddiet },
    { "exerciseType", s_schema, sp_exercisetype },
    { "exifData", s_schema, sp_exifdata },
    { "expectedArrivalFrom", s_schema, sp_expectedarrivalfrom },
    { "expectedArrivalUntil", s_schema, sp_expectedarrivaluntil },
    { "expectedPrognosis", s_schema, sp_expectedprognosis },
    { "expectsAcceptanceOf", s_schema, sp_expectsacceptanceof },
    { "experienceRequirements", s_schema, sp_experiencerequirements },
    { "expeienceRequirements", s_schema, sp_experiencerequirements }, // typo in early original schema.org docs
    { "experienceInPlaceOfEducation", s_schema, sp_experienceinplaceofeducation },
    { "expires", s_schema, sp_expires },
    { "expressedIn", s_schema, sp_expressedin },
    { "exportConsiderations", s_schema, sp_exportconsiderations },
    { "familyName", s_schema, sp_familyname },
    { "fatContent", s_schema, sp_fatcontent },
    { "faxNumber", s_schema, sp_faxnumber },
    { "featureList", s_schema, sp_featurelist },
    { "feesAndCommissionsSpecification", s_schema, sp_feesandcommissionsspecification },
    { "fiberContent", s_schema, sp_fibrecontent },
    { "fileFormat", s_schema, sp_fileformat },
    { "fileSize", s_schema, sp_filesize },
    { "financialAidEligible", s_schema, sp_financialaideligible },
    { "firstAppearance", s_schema, sp_firstappearance },
    { "firstPerformance", s_schema, sp_firstperformance },
    { "flightDistance", s_schema, sp_flightdistance },
    { "flightNumber", s_schema, sp_flightnumber },
    { "floorLevel", s_schema, sp_floorlevel },
    { "floorLimit", s_schema, sp_floorlimit },
    { "floorSize", s_schema, sp_floorsize },
    { "followee", s_schema, sp_followee },
    { "follows", s_schema, sp_follows },
    { "followup", s_schema, sp_followup },
    { "foodEstablishment", s_schema, sp_foodestablishment },
    { "foodEvent", s_schema, sp_foodevent },
    { "foodWarning", s_schema, sp_foodwarning },
    { "founder", s_schema, sp_founder },
    { "founders", s_schema, sp_founders },
    { "foundingDate", s_schema, sp_foundingdate },
    { "foundingLocation", s_schema, sp_foundinglocation },
    { "free", s_schema, sp_free },
    { "freeShippingThreshold", s_schema, sp_freeshippingthreshold },
    { "frequency", s_schema, sp_frequency },
    { "fromLocation", s_schema, sp_fromlocation },
    { "fuelCapacity", s_schema, sp_fuelcapacity },
    { "fuelConsumption", s_schema, sp_fuelconsumption },
    { "fuelEfficiency", s_schema, sp_fuelefficiency },
    { "fuelType", s_schema, sp_fueltype },
    { "funcionalClass", s_schema, sp_funcionalclass },
    { "function", s_schema, sp_function },
    { "fundedItem", s_schema, sp_fundeditem },
    { "funder", s_schema, sp_funder },
    { "funding", s_schema, sp_funding },
    { "game", s_schema, sp_game },
    { "gameAvailabilityType", s_schema, sp_gameavailabilitytype },
    { "gameEdition", s_schema, sp_gameedition },
    { "gameItem", s_schema, sp_gameitem },
    { "gameLocation", s_schema, sp_gamelocation },
    { "gamePlatform", s_schema, sp_gameplatform },
    { "gameServer", s_schema, sp_gameserver },
    { "gameTip", s_schema, sp_gametip },
    { "gender", s_schema, sp_gender },
    { "genre", s_schema, sp_genre },
    { "geo", s_schema, sp_geo },
    { "geoContains", s_schema, sp_geocontains },
    { "geoCoveredBy", s_schema, sp_geocoveredby },
    { "geoCovers", s_schema, sp_geocovers },
    { "geoCrosses", s_schema, sp_geocrosses },
    { "geoDisjoint", s_schema, sp_geodisjoint },
    { "geoEquals", s_schema, sp_geoequals },
    { "geoIntersects", s_schema, sp_geointersects },
    { "geoMidpoint", s_schema, sp_geomidpoint },
    { "geoOverlaps", s_schema, sp_geooverlaps },
    { "geoRadius", s_schema, sp_georadius },
    { "geoTouches", s_schema, sp_geotouches },
    { "geoWithin", s_schema, sp_geowithin },
    { "geographicArea", s_schema, sp_geographicarea },
    { "gettingTestedInfo", s_schema, sp_gettingtestedinfo },
    { "givenName", s_schema, sp_givenname },
    { "globalLocationNumber", s_schema, sp_globallocationnumber },
    { "governmentBenefitsInfo", s_schema, sp_governmentbenefitsinfo },
    { "gracePeriod", s_schema, sp_graceperiod },
    { "grantee", s_schema, sp_grantee },
    { "greater", s_schema, sp_greater },
    { "greaterOrEqual", s_schema, sp_greaterorequal },
    { "gtin", s_schema, sp_gtin },
    { "gtin12", s_schema, sp_gtin12 },
    { "gtin13", s_schema, sp_gtin13 },
    { "gtin14", s_schema, sp_gtin14 },
    { "gtin8", s_schema, sp_gtin8 },
    { "guideline", s_schema, sp_guideline },
    { "guidelineDate", s_schema, sp_guidelinedate },
    { "guidelineSubject", s_schema, sp_guidelinesubject },
    { "handlingTime", s_schema, sp_handlingtime },
    { "hasAdultConsideration", s_schema, sp_hasadultconsideration },
    { "hasBioPolymerSequence", s_schema, sp_hasbiopolymersequence },
    { "hasBroadcastChannel", s_schema, sp_hasbroadcastchannel },
    { "hasCategoryCode", s_schema, sp_hascategorycode },
    { "hasCourse", s_schema, sp_hascourse },
    { "hasCourseInstance", s_schema, sp_hascourseinstance },
    { "hasCredential", s_schema, sp_hascredential },
    { "hasDefinedTerm", s_schema, sp_hasdefinedterm },
    { "hasDeliveryMethod", s_schema, sp_hasdeliverymethod },
    { "hasDigitalDocumentPermission", s_schema, sp_hasdigitaldocumentpermission },
    { "hasDriveThroughService", s_schema, sp_hasdrivethroughservice },
    { "hasEnergyConsumptionDetails", s_schema, sp_hasenergyconsumptiondetails },
    { "hasEnergyEfficiencyCategory", s_schema, sp_hasenergyefficiencycategory },
    { "hasHealthAspect", s_schema, sp_hashealthaspect },
    { "hasMap", s_schema, sp_hasmap },
    { "hasMeasurement", s_schema, sp_hasmeasurement },
    { "hasMenu", s_schema, sp_hasmenu },
    { "hasMenuItem", s_schema, sp_hasmenuitem },
    { "hasMenuSection", s_schema, sp_hasmenusection },
    { "hasMerchantReturnPolicy", s_schema, sp_hasmerchantreturnpolicy },
    { "hasOccupation", s_schema, sp_hasoccupation },
    { "hasOfferCatalog", s_schema, sp_hasoffercatalogue },
    { "hasPOS", s_schema, sp_haspos },
    { "hasPart", s_schema, sp_haspart },
    { "hasProductReturnPolicy", s_schema, sp_hasproductreturnpolicy },
    { "hasVariant", s_schema, sp_hasvariant },
    { "headline", s_schema, sp_headline },
    { "healthCondition", s_schema, sp_healthcondition },
    { "healthPlanCoinsuranceOption", s_schema, sp_healthplancoinsuranceoption },
    { "healthPlanCoinsuranceRate", s_schema, sp_healthplancoinsurancerate },
    { "healthPlanCopay", s_schema, sp_healthplancopay },
    { "healthPlanCopayOption", s_schema, sp_healthplancopayoption },
    { "healthPlanCostSharing", s_schema, sp_healthplancostsharing },
    { "healthPlanDrugOption", s_schema, sp_healthplandrugoption },
    { "healthPlanDrugTier", s_schema, sp_healthplandrugtier },
    { "healthPlanId", s_schema, sp_healthplanid },
    { "healthPlanMarketingUrl", s_schema, sp_healthplanmarketingurl },
    { "healthPlanNetworkId", s_schema, sp_healthplannetworkid },
    { "healthPlanNetworkTier", s_schema, sp_healthplannetworktier },
    { "healthPlanPharmacyCategory", s_schema, sp_healthplanpharmacycategory },
    { "healthcareReportingData", s_schema, sp_healthcarereportingdata },
    { "height", s_schema, sp_height },
    { "highPrice", s_schema, sp_highprice },
    { "hiringOrganization", s_schema, sp_hiringorganisation },
    { "holdingArchive", s_schema, sp_holdingarchive },
    { "homeLocation", s_schema, sp_homelocation },
    { "homeTeam", s_schema, sp_hometeam },
    { "honorificPrefix", s_schema, sp_honourificprefix },
    { "honorificSuffix", s_schema, sp_honourificsuffix },
    { "hospitalAffiliation", s_schema, sp_hospitalaffiliation },
    { "hostingOrganization", s_schema, sp_hostingorganisation },
    { "hoursAvailable", s_schema, sp_hoursavailable },
    { "howPerformed", s_schema, sp_howperformed },
    { "httpMethod", s_schema, sp_httpmethod },
    { "iataCode", s_schema, sp_iatacode },
    { "identifier", s_schema, sp_identifier },
    { "identifyingExam", s_schema, sp_identifyingexam },
    { "identifyingTest", s_schema, sp_identifyingtest },
    { "illustrator", s_schema, sp_illustrator },
    { "image", s_schema, sp_image },
    { "imagingTechnique", s_schema, sp_imagingtechnique },
    { "inAlbum", s_schema, sp_inalbum },
    { "inBroadcastLineup", s_schema, sp_inbroadcastlineup },
    { "inChI", s_schema, sp_inchi },
    { "inChIKey", s_schema, sp_inchikey },
    { "inCodeSet", s_schema, sp_incodeset },
    { "inDefinedTermSet", s_schema, sp_indefinedtermset },
    { "inLanguage", s_schema, sp_inlanguage },
    { "inPlaylist", s_schema, sp_inplaylist },
    { "inProductGroupWithID", s_schema, sp_inproductgroupwithid },
    { "inStoreReturnsOffered", s_schema, sp_instorereturnsoffered },
    { "inSupportOf", s_schema, sp_insupportof },
    { "incentives", s_schema, sp_incentives },
    { "incentiveCompensation", s_schema, sp_incentivecompensation },
    { "includedComposition", s_schema, sp_includedcomposition },
    { "includedDataCatalog", s_schema, sp_includeddatacatalog },
    { "includedInHealthInsurancePlan", s_schema, sp_includedinhealthinsuranceplan },
    { "includedRiskFactor", s_schema, sp_includedriskfactor },
    { "includesAttraction", s_schema, sp_includesattraction },
    { "includesHealthPlanFormulary", s_schema, sp_includeshealthplanformulary },
    { "includesHealthPlanNetwork", s_schema, sp_includeshealthplannetwork },
    { "includesObject", s_schema, sp_includesobject },
    { "increasesRiskOf", s_schema, sp_increasesriskof },
    { "indication", s_schema, sp_indication },
    { "industry", s_schema, sp_industry },
    { "ineligibleRegion", s_schema, sp_ineligibleregion },
    { "infectiousAgent", s_schema, sp_infectiousagent },
    { "infectiousAgentClass", s_schema, sp_infectiousagentclass },
    { "ingredients", s_schema, sp_ingredients },
    { "inker", s_schema, sp_inker },
    { "insertion", s_schema, sp_insertion },
    { "installUrl", s_schema, sp_installurl },
    { "instructor", s_schema, sp_instructor },
    { "instrument", s_schema, sp_instrument },
    { "intensity", s_schema, sp_intensity },
    { "interactingDrug", s_schema, sp_interactingdrug },
    { "interactionCount", s_schema, sp_interactioncount },
    { "interactionService", s_schema, sp_interactionservice },
    { "interactionStatistic", s_schema, sp_interactionstatistic },
    { "interactionType", s_schema, sp_interactiontype },
    { "interactivityType", s_schema, sp_interactivitytype },
    { "interestRate", s_schema, sp_interestrate },
    { "interpretedAsClaim", s_schema, sp_interpretedasclaim },
    { "inventoryLevel", s_schema, sp_inventorylevel },
    { "inverseOf", s_schema, sp_inverseof },
    { "isAcceptingNewPatients", s_schema, sp_isacceptingnewpatients },
    { "isAccessibleForFree", s_schema, sp_isaccessibleforfree },
    { "isAccessoryOrSparePartFor", s_schema, sp_isaccessoryorsparepartfor },
    { "isAvailableGenerally", s_schema, sp_isavailablegenerally },
    { "isBasedOn", s_schema, sp_isbasedon },
    { "isBasedOnUrl", s_schema, sp_isbasedonurl },
    { "isConsumableFor", s_schema, sp_isconsumablefor },
    { "isFamilyFriendly", s_schema, sp_isfamilyfriendly },
    { "isGift", s_schema, sp_isgift },
    { "isLiveBroadcast", s_schema, sp_islivebroadcast },
    { "isPartOf", s_schema, sp_ispartof },
    { "isPlanForApartment", s_schema, sp_isplanforapartment },
    { "isProprietary", s_schema, sp_isproprietary },
    { "isRelatedTo", s_schema, sp_isrelatedto },
    { "isResizable", s_schema, sp_isresizable },
    { "isSimilarTo", s_schema, sp_issimilarto },
    { "isUnlabelledFallback", s_schema, sp_isunlabelledfallback },
    { "isVariantOf", s_schema, sp_isvariantof },
    { "isbn", s_schema, sp_isbn },
    { "isicV4", s_schema, sp_isicv4 },
    { "iso6523Code", s_schema, sp_iso6523code },
    { "isrcCode", s_schema, sp_isrccode },
    { "issn", s_schema, sp_issn },
    { "issueNumber", s_schema, sp_issuenumber },
    { "issuedBy", s_schema, sp_issuedby },
    { "issuedThrough", s_schema, sp_issuedthrough },
    { "iswcCode", s_schema, sp_iswccode },
    { "item", s_schema, sp_item },
    { "itemCondition", s_schema, sp_itemcondition },
    { "itemDefectReturnFees", s_schema, sp_itemdefectreturnfees },
    { "itemDefectReturnLabelSource", s_schema, sp_itemdefectreturnlabelsource },
    { "itemDefectReturnShippingFeesAmount", s_schema, sp_itemdefectreturnshippingfeesamount },
    { "itemListElement", s_schema, sp_itemlistelement },
    { "itemListOrder", s_schema, sp_itemlistorder },
    { "itemLocation", s_schema, sp_itemlocation },
    { "itemOffered", s_schema, sp_itemoffered },
    { "itemReviewed", s_schema, sp_itemreviewed },
    { "itemShipped", s_schema, sp_itemshipped },
    { "itinerary", s_schema, sp_itinerary },
    { "iupacName", s_schema, sp_iupacname },
    { "jobBenefits", s_schema, sp_jobbenefits },
    { "jobImmediateStart", s_schema, sp_jobimmediatestart },
    { "jobLocation", s_schema, sp_joblocation },
    { "jobLocationType", s_schema, sp_joblocationtype },
    { "jobStartDate", s_schema, sp_jobstartdate },
    { "jobTitle", s_schema, sp_jobtitle },
    { "jurisdiction", s_schema, sp_jurisdiction },
    { "keywords", s_schema, sp_keywords },
    { "knownVehicleDamages", s_schema, sp_knownvehicledamages },
    { "knows", s_schema, sp_knows },
    { "knowsAbout", s_schema, sp_knowsabout },
    { "knowsLanguage", s_schema, sp_knowslanguage },
    { "labelDetails", s_schema, sp_labeldetails },
    { "landlord", s_schema, sp_landlord },
    { "language", s_schema, sp_language },
    { "lastReviewed", s_schema, sp_lastreviewed },
    { "latitude", s_schema, sp_latitude },
    { "layoutImage", s_schema, sp_layoutimage },
    { "learningResourceType", s_schema, sp_learningresourcetype },
    { "leaseLength", s_schema, sp_leaselength },
    { "legalName", s_schema, sp_legalname },
    { "legalStatus", s_schema, sp_legalstatus },
    { "legislationApplies", s_schema, sp_legislationapplies },
    { "legislationChanges", s_schema, sp_legislationchanges },
    { "legislationConsolidates", s_schema, sp_legislationconsolidates },
    { "legislationDate", s_schema, sp_legislationdate },
    { "legislationDateVersion", s_schema, sp_legislationdateversion },
    { "legislationIdentifier", s_schema, sp_legislationidentifier },
    { "legislationJurisdiction", s_schema, sp_legislationjurisdiction },
    { "legislationLegalForce", s_schema, sp_legislationlegalforce },
    { "legislationLegalValue", s_schema, sp_legislationlegalvalue },
    { "legislationPassedBy", s_schema, sp_legislationpassedby },
    { "legislationResponsible", s_schema, sp_legislationresponsible },
    { "legislationTransposes", s_schema, sp_legislationtransposes },
    { "legislationType", s_schema, sp_legislationtype },
    { "leiCode", s_schema, sp_leicode },
    { "lender", s_schema, sp_lender },
    { "lesser", s_schema, sp_lesser },
    { "lesserOrEqual", s_schema, sp_lesserorequal },
    { "letterer", s_schema, sp_letterer },
    { "license", s_schema, sp_license },
    { "line", s_schema, sp_line },
    { "linkRelationship", s_schema, sp_linkrelationship },
    { "liveBlogUpdate", s_schema, sp_liveblogupdate },
    { "loanMortgagaMandateAmount", s_schema, sp_loanmortgagamandateamount },
    { "loanPaymentAmount", s_schema, sp_loanpaymentamount },
    { "loanPaymentFrequency", s_schema, sp_loanpaymentfrequency },
    { "loanRepaymentForm", s_schema, sp_loanrepaymentform },
    { "loanTerm", s_schema, sp_loanterm },
    { "loanType", s_schema, sp_loantype },
    { "location", s_schema, sp_location },
    { "locationCreated", s_schema, sp_locationcreated },
    { "lodgingUnitDescription", s_schema, sp_lodgingunitdescription },
    { "lodgingUnitType", s_schema, sp_lodgingunittype },
    { "logo", s_schema, sp_logo },
    { "longitude", s_schema, sp_longitude },
    { "loser", s_schema, sp_loser },
    { "lowPrice", s_schema, sp_lowprice },
    { "lyricist", s_schema, sp_lyricist },
    { "lyrics", s_schema, sp_lyrics },
    { "mainContentOfPage", s_schema, sp_maincontentofpage },
    { "mainEntity", s_schema, sp_mainentity },
    { "mainEntityOfPage", s_schema, sp_mainentityofpage },
    { "maintainer", s_schema, sp_maintainer },
    { "makesOffer", s_schema, sp_makesoffer },
    { "manufacturer", s_schema, sp_manufacturer },
    { "map", s_schema, sp_map },
    { "maps", s_schema, sp_maps },
    { "mapType", s_schema, sp_maptype },
    { "marginOfError", s_schema, sp_marginoferror },
    { "masthead", s_schema, sp_masthead },
    { "material", s_schema, sp_material },
    { "materialExtent", s_schema, sp_materialextent },
    { "mathExpression", s_schema, sp_mathexpression },
    { "maxPrice", s_schema, sp_maxprice },
    { "maxValue", s_schema, sp_maxvalue },
    { "maximumAttendeeCapacity", s_schema, sp_maximumattendeecapacity },
    { "maximumEnrollment", s_schema, sp_maximumenrollment },
    { "maximumIntake", s_schema, sp_maximumintake },
    { "maximumPhysicalAttendeeCapacity", s_schema, sp_maximumphysicalattendeecapacity },
    { "maximumVirtualAttendeeCapacity", s_schema, sp_maximumvirtualattendeecapacity },
    { "mealService", s_schema, sp_mealservice },
    { "measuredProperty", s_schema, sp_measuredproperty },
    { "measuredValue", s_schema, sp_measuredvalue },
    { "measurementTechnique", s_schema, sp_measurementtechnique },
    { "mechanismOfAction", s_schema, sp_mechanismofaction },
    { "mediaAuthenticityCategory", s_schema, sp_mediaauthenticitycategory },
    { "mediaItemAppearance", s_schema, sp_mediaitemappearance },
    { "median", s_schema, sp_median },
    { "medicalAudience", s_schema, sp_medicalaudience },
    { "medicalSpeciality", s_schema, sp_medicalspeciality },
    { "medicineSystem", s_schema, sp_medicinesystem },
    { "meetsEmissionStandard", s_schema, sp_meetsemissionstandard },
    { "member", s_schema, sp_member },
    { "memberOf", s_schema, sp_memberof },
    { "members", s_schema, sp_members },
    { "membershipNumber", s_schema, sp_membershipnumber },
    { "membershipPointsEarned", s_schema, sp_membershippointsearned },
    { "memoryRequirements", s_schema, sp_memoryrequirements },
    { "mentions", s_schema, sp_mentions },
    { "menu", s_schema, sp_menu },
    { "menuAddOn", s_schema, sp_menuaddon },
    { "merchant", s_schema, sp_merchant },
    { "merchantReturnDays", s_schema, sp_merchantreturndays },
    { "merchantReturnLink", s_schema, sp_merchantreturnlink },
    { "messageAttachment", s_schema, sp_messageattachment },
    { "mileageFromOdometer", s_schema, sp_mileagefromodometer },
    { "minPrice", s_schema, sp_minprice },
    { "minValue", s_schema, sp_minvalue },
    { "minimumMonthlyRepaymentAmount", s_schema, sp_minimummonthlyrepaymentamount },
    { "minimumPaymentDue", s_schema, sp_minimumpaymentdue },
    { "missionCoveragePrioritiesPolicy", s_schema, sp_missioncoverageprioritiespolicy },
    { "model", s_schema, sp_model },
    { "modelDate", s_schema, sp_modeldate },
    { "modifiedTime", s_schema, sp_modifiedtime },
    { "molecularFormula", s_schema, sp_molecularformula },
    { "molecularWeight", s_schema, sp_molecularweight },
    { "monoistopicMolecularWeight", s_schema, sp_monoistopicmolecularweight },
    { "monthsOfExperience", s_schema, sp_monthsofexperience },
    { "mpn", s_schema, sp_mpn },
    { "multipleValues", s_schema, sp_multiplevalues },
    { "muscleAction", s_schema, sp_muscleaction },
    { "musicArrangement", s_schema, sp_musicarrangement },
    { "musicBy", s_schema, sp_musicby },
    { "musicCompositionForm", s_schema, sp_musiccompositionform },
    { "musicGroupMember", s_schema, sp_musicgroupmember },
    { "musicReleaseFormat", s_schema, sp_musicreleaseformat },
    { "musicalKey", s_schema, sp_musicalkey },
    { "naics", s_schema, sp_naics },
    { "name", s_schema, sp_name },
    { "nationality", s_schema, sp_nationality },
    { "naturalProgression", s_schema, sp_naturalprogression },
    { "negativeVotes", s_schema, sp_negativevotes },
    { "nerve", s_schema, sp_nerve },
    { "nerveMotor", s_schema, sp_nervemotor },
    { "netWorth", s_schema, sp_networth },
    { "newUpdatesAndGuideLines", s_schema, sp_newupdatesandguidelines },
    { "nextItem", s_schema, sp_nextitem },
    { "noBylinesPolicy", s_schema, sp_nobylinespolicy },
    { "nonEqual", s_schema, sp_nonequal },
    { "nonProprietaryName", s_schema, sp_nonproprietaryname },
    { "nonprofitStatus", s_schema, sp_nonprofitstatus },
    { "normalRange", s_schema, sp_normalrange },
    { "nsn", s_schema, sp_nsn },
    { "numAdults", s_schema, sp_numadults },
    { "numberOfAccommodationUnits", s_schema, sp_numberofaccommodationunits },
    { "numberOfAirbags", s_schema, sp_numberofairbags },
    { "numberOfAvailableAccommodationUnits", s_schema, sp_numberofavailableaccommodationunits },
    { "numberOfAxles", s_schema, sp_numberofaxles },
    { "numberOfBathroomsTotal", s_schema, sp_numberofbathroomstotal },
    { "numberOfBedrooms", s_schema, sp_numberofbedrooms },
    { "numberOfBeds", s_schema, sp_numberofbeds },
    { "numberOfCredits", s_schema, sp_numberofcredits },
    { "numberOfDoors", s_schema, sp_numberofdoors },
    { "numberOfEmployees", s_schema, sp_numberofemployees },
    { "numberOfEpisodes", s_schema, sp_numberofepisodes },
    { "numberOfForwardgears", s_schema, sp_numberofforwardgears },
    { "numberOfFullBathrooms", s_schema, sp_numberoffullbathrooms },
    { "numberOfItems", s_schema, sp_numberofitems },
    { "numberOfLoanPayments", s_schema, sp_numberofloanpayments },
    { "numberOfPages", s_schema, sp_numberofpages },
    { "numberOfPartialBathrooms", s_schema, sp_numberofpartialbathrooms },
    { "numberOfPlayers", s_schema, sp_numberofplayers },
    { "numberOfPreviousowners", s_schema, sp_numberofpreviousowners },
    { "numberOfRooms", s_schema, sp_numberofrooms },
    { "numberOfSeasons", s_schema, sp_numberofseasons },
    { "numberedPosition", s_schema, sp_numberedposition },
    { "numChildren", s_schema, sp_numchildren },
    { "numConstraints", s_schema, sp_numconstraints },
    { "numTracks", s_schema, sp_numtracks },
    { "nutrition", s_schema, sp_nutrition },
    { "object", s_schema, sp_object },
    { "observationDate", s_schema, sp_observationdate },
    { "observedNode", s_schema, sp_observednode },
    { "ocaoCode", s_schema, sp_ocaocode },
    { "occupancy", s_schema, sp_occupancy },
    { "occupationLocation", s_schema, sp_occupationlocation },
    { "occupationalCategory", s_schema, sp_occupationalcategory },
    { "occupationalCredentialAwarded", s_schema, sp_occupationalcredentialawarded },
    { "offerCount", s_schema, sp_offercount },
    { "offeredBy", s_schema, sp_offeredby },
    { "offers", s_schema, sp_offers },
    { "offersPrescriptionByMail", s_schema, sp_offersprescriptionbymail },
    { "openingHours", s_schema, sp_openinghours },
    { "openingHoursSpecification", s_schema, sp_openinghoursspecification },
    { "opens", s_schema, sp_opens },
    { "operatingSystem", s_schema, sp_operatingsystem },
    { "oponent", s_schema, sp_oponent }, // typo in schema.org specs
    { "opponent", s_schema, sp_opponent },
    { "orderDate", s_schema, sp_orderdate },
    { "orderDelivery", s_schema, sp_orderdelivery },
    { "orderItemNumber", s_schema, sp_orderitemnumber },
    { "orderItemStatus", s_schema, sp_orderitemstatus },
    { "orderNumber", s_schema, sp_ordernumber },
    { "orderQuantity", s_schema, sp_orderquantity },
    { "orderStatus", s_schema, sp_orderstatus },
    { "orderedItem", s_schema, sp_ordereditem },
    { "organizer", s_schema, sp_organiser },
    { "origin", s_schema, sp_origin },
    { "originAddress", s_schema, sp_originaddress },
    { "originalMediaContentDescription", s_schema, sp_originalmediacontentdescription },
    { "originalMedialink", s_schema, sp_originalmedialink },
    { "originatesFrom", s_schema, sp_originatesfrom },
    { "outcome", s_schema, sp_outcome },
    { "overdosage", s_schema, sp_overdosage },
    { "overview", s_schema, sp_overview },
    { "ownedFrom", s_schema, sp_ownedfrom },
    { "ownedThrough", s_schema, sp_ownedto },
    { "ownershipFundingInfo", s_schema, sp_ownershipfundinginfo },
    { "owns", s_schema, sp_owns },
    { "pageEnd", s_schema, sp_pageend },
    { "pageStart", s_schema, sp_pagestart },
    { "pagination", s_schema, sp_pagination },
    { "parent", s_schema, sp_parent },
    { "parentItem", s_schema, sp_parentitem },
    { "parentOrganization", s_schema, sp_parentorganisation },
    { "parents", s_schema, sp_parents },
    { "parentService", s_schema, sp_parentservice },
    { "parentTaxon", s_schema, sp_parenttaxon },
    { "partOfEpisode", s_schema, sp_partofepisode },
    { "partOfInvoice", s_schema, sp_partofinvoice },
    { "partOfOrder", s_schema, sp_partoforder },
    { "partOfSeason", s_schema, sp_partofseason },
    { "partOfSeries", s_schema, sp_partofseries },
    { "partOfSystem", s_schema, sp_partofsystem },
    { "partOfTrip", s_schema, sp_partoftrip },
    { "partOfTVSeries", s_schema, sp_partoftvseries },
    { "participant", s_schema, sp_participant },
    { "partySize", s_schema, sp_partysize },
    { "passengerPriorityStatus", s_schema, sp_passengerprioritystatus },
    { "passengerSequenceNumber", s_schema, sp_passengersequencenumber },
    { "pathophysiology", s_schema, sp_pathophysiology },
    { "pattern", s_schema, sp_pattern },
    { "payload", s_schema, sp_payload },
    { "paymentAccepted", s_schema, sp_paymentaccepted },
    { "paymentDue", s_schema, sp_paymentdue },
    { "paymentDueDate", s_schema, sp_paymentduedate },
    { "paymentMethod", s_schema, sp_paymentmethod },
    { "paymentMethodId", s_schema, sp_paymentmethodid },
    { "paymentStatus", s_schema, sp_paymentstatus },
    { "paymentUrl", s_schema, sp_paymenturl },
    { "penciler", s_schema, sp_penciler },
    { "percentile10", s_schema, sp_percentile10 },
    { "percentile25", s_schema, sp_percentile25 },
    { "percentile75", s_schema, sp_percentile75 },
    { "percentile90", s_schema, sp_percentile90 },
    { "performTime", s_schema, sp_performtime },
    { "performer", s_schema, sp_performer },
    { "performers", s_schema, sp_performers },
    { "performerIn", s_schema, sp_performerin },
    { "permissionType", s_schema, sp_permissiontype },
    { "permissions", s_schema, sp_permissions },
    { "permitAudience", s_schema, sp_permitaudience },
    { "permittedUsage", s_schema, sp_permittedusage },
    { "petsAllowed", s_schema, sp_petsallowed },
    { "phase", s_schema, sp_phase },
    { "phoneticText", s_schema, sp_phonetictext },
    { "photo", s_schema, sp_photo },
    { "photos", s_schema, sp_photos },
    { "physicalRequirement", s_schema, sp_physicalrequirement },
    { "physiologicalBenefits", s_schema, sp_physiologicalbenefits },
    { "pickupLocation", s_schema, sp_pickuplocation },
    { "pickupTime", s_schema, sp_pickuptime },
    { "playMode", s_schema, sp_playmode },
    { "playerType", s_schema, sp_playertype },
    { "playersOnline", s_schema, sp_playersonline },
    { "polygon", s_schema, sp_polygon },
    { "population", s_schema, sp_population },
    { "populationType", s_schema, sp_populationtype },
    { "position", s_schema, sp_position },
    { "positiveVotes", s_schema, sp_positivevotes },
    { "possibleComplication", s_schema, sp_possiblecomplication },
    { "possibleTreatment", s_schema, sp_possibletreatment },
    { "postOfficeBoxNumber", s_schema, sp_postofficeboxnumber },
    { "postOp", s_schema, sp_postop },
    { "postalCode", s_schema, sp_postalcode },
    { "postalCodeBegin", s_schema, sp_postalcodebegin },
    { "postalCodeEnd", s_schema, sp_postalcodeend },
    { "postalCodePrefix", s_schema, sp_postalcodeprefix },
    { "postalCodeRange", s_schema, sp_postalcoderange },
    { "potentialAction", s_schema, sp_potentialaction },
    { "potentialUse", s_schema, sp_potentialuse },
    { "preOp", s_schema, sp_preop },
    { "predecessorOf", s_schema, sp_predecessorof },
    { "pregnancyCategory", s_schema, sp_pregnancycategory },
    { "pregnancyWarning", s_schema, sp_pregnancywarning },
    { "prepTime", s_schema, sp_preptime },
    { "preparation", s_schema, sp_preparation },
    { "prescribingInfo", s_schema, sp_prescribinginfo },
    { "prescriptionStatus", s_schema, sp_prescriptionstatus },
    { "previousItem", s_schema, sp_previousitem },
    { "previousStartDate", s_schema, sp_previousstartdate },
    { "price", s_schema, sp_price },
    { "priceComponent", s_schema, sp_pricecomponent },
    { "priceComponentType", s_schema, sp_pricecomponenttype },
    { "priceCurrency", s_schema, sp_pricecurrency },
    { "priceRange", s_schema, sp_pricerange },
    { "priceSpecification", s_schema, sp_pricespecification },
    { "priceType", s_schema, sp_pricetype },
    { "priceValidUntil", s_schema, sp_pricevaliduntil },
    { "primaryImageOfPage", s_schema, sp_primaryimageofpage },
    { "primaryPrevention", s_schema, sp_primaryprevention },
    { "printColumn", s_schema, sp_printcolumn },
    { "printEdition", s_schema, sp_printedition },
    { "printPage", s_schema, sp_printpage },
    { "printSection", s_schema, sp_printsection },
    { "procedure", s_schema, sp_procedure },
    { "procedureType", s_schema, sp_proceduretype },
    { "processingTime", s_schema, sp_processingtime },
    { "processorRequirements", s_schema, sp_processorrequirements },
    { "producer", s_schema, sp_producer },
    { "produces", s_schema, sp_produces },
    { "productGroupID", s_schema, sp_productgroupid },
    { "productID", s_schema, sp_productid },
    { "productReturnDays", s_schema, sp_productreturndays },
    { "productReturnLink", s_schema, sp_productreturnlink },
    { "productSupported", s_schema, sp_productsupported },
    { "productionCompany", s_schema, sp_productioncompany },
    { "productionDate", s_schema, sp_productiondate },
    { "proficiencyLevel", s_schema, sp_proficiencylevel },
    { "programMembershipUse", s_schema, sp_programmemembershipused },
    { "programName", s_schema, sp_programmename },
    { "programPrequisites", s_schema, sp_programprequisites },
    { "programType", s_schema, sp_programtype },
    { "programmingLanguage", s_schema, sp_programminglanguage },
    { "programmingModel", s_schema, sp_programmingmodel },
    { "propertyID", s_schema, sp_propertyid },
    { "propriatryName", s_schema, sp_propriatryname },
    { "proprietaryName", s_schema, sp_proprietaryname },
    { "proteinContent", s_schema, sp_proteincontent },
    { "provider", s_schema, sp_provider },
    { "providerMobility", s_schema, sp_providermobility },
    { "providesBroadcastService", s_schema, sp_providesbroadcastservice },
    { "providesService", s_schema, sp_providesservice },
    { "publicAccess", s_schema, sp_publicaccess },
    { "publicTransportClosuresInfo", s_schema, sp_publictransportclosuresinfo },
    { "publication", s_schema, sp_publication },
    { "publicationType", s_schema, sp_publicationtype },
    { "publishedBy", s_schema, sp_publishedby },
    { "publishedOn", s_schema, sp_publishedon },
    { "publisher", s_schema, sp_publisher },
    { "publisherImprint", s_schema, sp_publisherimprint },
    { "publishingPrinciples", s_schema, sp_publishingprinciples },
    { "purchaseDate", s_schema, sp_purchasedate },
    { "purpose", s_schema, sp_purpose },
    { "qualifications", s_schema, sp_qualifications },
    { "quarantineGuidelines", s_schema, sp_quarantineguidelines },
    { "query", s_schema, sp_query },
    { "quest", s_schema, sp_quest },
    { "question", s_schema, sp_question },
    { "rangeIncludes", s_schema, sp_rangeincludes },
    { "ratingCount", s_schema, sp_ratingcount },
    { "ratingExplanation", s_schema, sp_ratingexplanation },
    { "ratingValue", s_schema, sp_ratingvalue },
    { "readBy", s_schema, sp_readby },
    { "readonlyValue", s_schema, sp_readonlyvalue },
    { "realEstateAgent", s_schema, sp_realestateagent },
    { "recipe", s_schema, sp_recipe },
    { "recipeCategory", s_schema, sp_recipecategory },
    { "recipeCuisine", s_schema, sp_recipecuisine },
    { "recipeIngredient", s_schema, sp_recipeingredient },
    { "recipeInstructions", s_schema, sp_recipeinstructions },
    { "recipeYield", s_schema, sp_recipeyield },
    { "recipient", s_schema, sp_recipient },
    { "recognisedBy", s_schema, sp_recognisedby },
    { "recognisingAuthority", s_schema, sp_recognisingauthority },
    { "recommendationStrength", s_schema, sp_recommendationstrength },
    { "recommendedIntake", s_schema, sp_recommendedintake },
    { "recordLabel", s_schema, sp_recordlabel },
    { "recordedAs", s_schema, sp_recordedas },
    { "recordedAt", s_schema, sp_recordedat },
    { "recordedIn", s_schema, sp_recordedin },
    { "recordingOf", s_schema, sp_recordingof },
    { "recourseLoan", s_schema, sp_recourseloan },
    { "referenceQuantity", s_schema, sp_referencequantity },
    { "referencesOrder", s_schema, sp_referencesorder },
    { "refundType", s_schema, sp_refundtype },
    { "regionDrained", s_schema, sp_regiondrained },
    { "regionsAllowed", s_schema, sp_regionsallowed },
    { "relatedAnatomy", s_schema, sp_relatedanatomy },
    { "relatedCondition", s_schema, sp_relatedcondition },
    { "relatedDrug", s_schema, sp_relateddrug },
    { "relatedLink", s_schema, sp_relatedlink },
    { "relatedStructure", s_schema, sp_relatedstructure },
    { "relatedTherapy", s_schema, sp_relatedtherapy },
    { "relatedTo", s_schema, sp_relatedto },
    { "releaseDate", s_schema, sp_releasedate },
    { "releaseNotes", s_schema, sp_releasenotes },
    { "releaseOf", s_schema, sp_releaseof },
    { "releasedEvent", s_schema, sp_releasedevent },
    { "relevantOccupation", s_schema, sp_relevantoccupation },
    { "relevantSpeciality", s_schema, sp_relevantspeciality },
    { "remainingAttendeeCapacity", s_schema, sp_remainingattendeecapacity },
    { "renegotiableLoan", s_schema, sp_renegotiableloan },
    { "repeatCount", s_schema, sp_repeatcount },
    { "repeatFrequency", s_schema, sp_repeatfrequency },
    { "repetitions", s_schema, sp_repetitions },
    { "replacee", s_schema, sp_replacee },
    { "replacer", s_schema, sp_replacer },
    { "replyToUrl", s_schema, sp_replytourl },
    { "reportNumber", s_schema, sp_reportnumber },
    { "representativeOfPage", s_schema, sp_representativeofpage },
    { "requiredCollateral", s_schema, sp_requiredcollateral },
    { "requiredGender", s_schema, sp_requiredgender },
    { "requiredMaxAge", s_schema, sp_requiredmaxage },
    { "requiredMinAge", s_schema, sp_requiredminage },
    { "requiredQuantity", s_schema, sp_requiredquantity },
    { "requiresSubscription", s_schema, sp_requiressubscription },
    { "reservationFor", s_schema, sp_reservationfor },
    { "reservationId", s_schema, sp_reservationid },
    { "reservationStatus", s_schema, sp_reservationstatus },
    { "reservedTicket", s_schema, sp_reservedticket },
    { "restockingFee", s_schema, sp_restockingfee },
    { "responsibilities", s_schema, sp_responsibilities },
    { "restPeriods", s_schema, sp_restperiods },
    { "result", s_schema, sp_result },
    { "resultComment", s_schema, sp_resultcomment },
    { "resultReview", s_schema, sp_resultreview },
    { "returnFees", s_schema, sp_returnfees },
    { "returnLabelSource", s_schema, sp_returnlabelsource },
    { "returnMethod", s_schema, sp_returnmethod },
    { "returnPolicyCategory", s_schema, sp_returnpolicycategory },
    { "returnPolicyCountry", s_schema, sp_returnpolicycountry },
    { "returnPolicySeasonalOverride", s_schema, sp_returnpolicyseasonaloverride },
    { "returnShippingFeesAmount", s_schema, sp_returnshippingfeesamount },
    { "review", s_schema, sp_review },
    { "reviews", s_schema, sp_reviews },
    { "reviewAspect", s_schema, sp_reviewaspect },
    { "reviewBody", s_schema, sp_reviewbody },
    { "reviewCount", s_schema, sp_reviewcount },
    { "reviewRating", s_schema, sp_reviewrating },
    { "reviewedBy", s_schema, sp_reviewedby },
    { "riskFactor", s_schema, sp_riskfactor },
    { "risks", s_schema, sp_risks },
    { "roleName", s_schema, sp_rolename },
    { "roofLoad", s_schema, sp_roofload },
    { "rsvpResponse", s_schema, sp_rsvpresponse },
    { "runsTo", s_schema, sp_runsto },
    { "runtime", s_schema, sp_runtime },
    { "runtimePlatform", s_schema, sp_runtimeplatform },
    { "rxcui", s_schema, sp_rxcui },
    { "safetyConsideration", s_schema, sp_safetyconsideration },
    { "salaryCurrency", s_schema, sp_salarycurrency },
    { "salaryUponCompletion", s_schema, sp_salaryuponcompletion },
    { "sameAs", s_schema, sp_sameas },
    { "sampleType", s_schema, sp_sampletype },
    { "saturatedFatContent", s_schema, sp_saturatedfatcontent },
    { "scheduledPaymentDate", s_schema, sp_scheduledpaymentdate },
    { "scheduledTime", s_schema, sp_scheduledtime },
    { "schemaVersion", s_schema, sp_schemaversion },
    { "schoolClosuresInfo", s_schema, sp_schoolclosuresinfo },
    { "screenCount", s_schema, sp_screencount },
    { "screenshot", s_schema, sp_screenshot },
    { "sdDatePublished", s_schema, sp_sddatepublished },
    { "sdLicense", s_schema, sp_sdlicence },
    { "sdPublisher", s_schema, sp_sdpublisher },
    { "season", s_schema, sp_season },
    { "seasons", s_schema, sp_seasons },
    { "seasonNumber", s_schema, sp_seasonnumber },
    { "seatNumber", s_schema, sp_seatnumber },
    { "seatRow", s_schema, sp_seatrow },
    { "seatSection", s_schema, sp_seatsection },
    { "seatingCapacity", s_schema, sp_seatingcapacity },
    { "seatingType", s_schema, sp_seatingtype },
    { "secondaryPrevention", s_schema, sp_secondaryprevention },
    { "securityClearanceRequirement", s_schema, sp_securityclearancerequirement },
    { "securityScreening", s_schema, sp_securityscreening },
    { "seeks", s_schema, sp_seeks },
    { "seller", s_schema, sp_seller },
    { "sender", s_schema, sp_sender },
    { "sensoryClearanceRequirement", s_schema, sp_sensoryclearancerequirement },
    { "sensoryUnit", s_schema, sp_sensoryunit },
    { "serialNumber", s_schema, sp_serialnumber },
    { "seriousAdverseOutcome", s_schema, sp_seriousadverseoutcome },
    { "serverStatus", s_schema, sp_serverstatus },
    { "servesCuisine", s_schema, sp_servescuisine },
    { "serviceArea", s_schema, sp_servicearea },
    { "serviceAudience", s_schema, sp_serviceaudience },
    { "serviceLocation", s_schema, sp_servicelocation },
    { "serviceOperator", s_schema, sp_serviceoperator },
    { "serviceOutput", s_schema, sp_serviceoutput },
    { "servicePhone", s_schema, sp_servicephone },
    { "servicePostalAddress", s_schema, sp_servicepostaladdress },
    { "serviceSmsNumber", s_schema, sp_servicesmsnumber },
    { "serviceType", s_schema, sp_servicetype },
    { "serviceUrl", s_schema, sp_serviceurl },
    { "servingSize", s_schema, sp_servingsize },
    { "sha256", s_schema, sp_sha256 },
    { "sharedContent", s_schema, sp_sharedcontent },
    { "shippingDestination", s_schema, sp_shippingdestination },
    { "shippingLabel", s_schema, sp_shippinglabel },
    { "shippingRate", s_schema, sp_shippingrate },
    { "shippingSettingsLink", s_schema, sp_shippingsettingslink },
    { "sibling", s_schema, sp_sibling },
    { "siblings", s_schema, sp_siblings },
    { "signDetected", s_schema, sp_signdetected },
    { "signOrSymptom", s_schema, sp_signorsymptom },
    { "significance", s_schema, sp_significance },
    { "significantLink", s_schema, sp_significantlink },
    { "significantLinks", s_schema, sp_significantlinks },
    { "size", s_schema, sp_size },
    { "sizeGroup", s_schema, sp_sizegroup },
    { "sizeSystem", s_schema, sp_sizesystem },
    { "skills", s_schema, sp_skills },
    { "sku", s_schema, sp_sku },
    { "slogan", s_schema, sp_slogan },
    { "smiles", s_schema, sp_smiles },
    { "smokingAllowed", s_schema, sp_smokingallowed },
    { "sodiumContent", s_schema, sp_sodiumcontent },
    { "softwareAddOn", s_schema, sp_softwareaddon },
    { "softwareHelp", s_schema, sp_softwarehelp },
    { "softwareRequirements", s_schema, sp_softwarerequirements },
    { "softwareVersion", s_schema, sp_softwareversion },
    { "source", s_schema, sp_source },
    { "sourceOrganization", s_schema, sp_sourceorganisation },
    { "sourcedFrom", s_schema, sp_sourcedfrom },
    { "spatial", s_schema, sp_spatial },
    { "spatialCoverage", s_schema, sp_spatialcoverage },
    { "speakable", s_schema, sp_speakable },
    { "specialCommitments", s_schema, sp_specialcommitments },
    { "specialOpeningHoursSpecification", s_schema, sp_specialopeninghoursspecification },
    { "specialty", s_schema, sp_speciality },
    { "speechToTextMarkup", s_schema, sp_speechtotextmarkup },
    { "speed", s_schema, sp_speed },
    { "spokenByCharacter", s_schema, sp_spokenbycharacter },
    { "sponsor", s_schema, sp_sponsor },
    { "sport", s_schema, sp_sport },
    { "sportsActivityLocation", s_schema, sp_sportsactivitylocation },
    { "sportsEvent", s_schema, sp_sportsevent },
    { "sportsTeam", s_schema, sp_sportsteam },
    { "spouse", s_schema, sp_spouse },
    { "sscheduleTimezone", s_schema, sp_scheduletimezone },
    { "stage", s_schema, sp_stage },
    { "stageAsNumber", s_schema, sp_stageasnumber },
    { "starRating", s_schema, sp_starrating },
    { "startDate", s_schema, sp_startdate },
    { "startOffset", s_schema, sp_startoffset },
    { "startTime", s_schema, sp_starttime },
    { "status", s_schema, sp_status },
    { "steeringPosition", s_schema, sp_steeringposition },
    { "step", s_schema, sp_step },
    { "stepValue", s_schema, sp_stepvalue },
    { "steps", s_schema, sp_steps },
    { "storageRequirements", s_schema, sp_storagerequirements },
    { "streetAddress", s_schema, sp_streetaddress },
    { "strengthUnit", s_schema, sp_strengthunit },
    { "strengthValue", s_schema, sp_strengthvalue },
    { "structuralClass", s_schema, sp_structuralclass },
    { "study", s_schema, sp_study },
    { "studyDesign", s_schema, sp_studydesign },
    { "studyLocation", s_schema, sp_studylocation },
    { "studySubject", s_schema, sp_studysubject },
    { "stupidProperty", s_schema, sp_stupidproperty },
    { "subEvent", s_schema, sp_subevent },
    { "subEvents", s_schema, sp_subevents },
    { "subOrganization", s_schema, sp_suborganisation },
    { "subReservation", s_schema, sp_subreservation },
    { "subStageSuffix", s_schema, sp_substagesuffix },
    { "subStructure", s_schema, sp_substructure },
    { "subTrip", s_schema, sp_subtrip },
    { "subjectOf", s_schema, sp_subjectof },
    { "subtitleLanguage", s_schema, sp_subtitlelanguage },
    { "subtype", s_schema, sp_subtype },
    { "successorOf", s_schema, sp_successorof },
    { "sugarContent", s_schema, sp_sugarcontent },
    { "suggestedAge", s_schema, sp_suggestedage },
    { "suggestedAnswer", s_schema, sp_suggestedanswer },
    { "suggestedGender", s_schema, sp_suggestedgender },
    { "suggestedMaxAge", s_schema, sp_suggestedmaxage },
    { "suggestedMeasurement", s_schema, sp_suggestedmeasurement },
    { "suggestedMinAge", s_schema, sp_suggestedminage },
    { "suitableForDiet", s_schema, sp_suitablefordiet },
    { "superEvent", s_schema, sp_superevent },
    { "supersededBy", s_schema, sp_supersededby },
    { "supply", s_schema, sp_supply },
    { "supplyTo", s_schema, sp_supplyto },
    { "supportingData", s_schema, sp_supportingdata },
    { "target", s_schema, sp_target },
    { "targetCollection", s_schema, sp_targetcollection },
    { "targetDescription", s_schema, sp_targetdescription },
    { "targetName", s_schema, sp_targetname },
    { "targetPlatform", s_schema, sp_targetplatform },
    { "targetPopulation", s_schema, sp_targetpopulation },
    { "targetProduct", s_schema, sp_targetproduct },
    { "targetUrl", s_schema, sp_targeturl },
    { "taxID", s_schema, sp_taxid },
    { "taxonRank", s_schema, sp_taxonrank },
    { "teaches", s_schema, sp_teaches },
    { "telephone", s_schema, sp_telephone },
    { "temporal", s_schema, sp_temporal },
    { "temporalCoverage", s_schema, sp_temporalcoverage },
    { "termCode", s_schema, sp_termcode },
    { "termDuration", s_schema, sp_termduration },
    { "termsOfService", s_schema, sp_termsofservice },
    { "termsPerYear", s_schema, sp_termsperyear },
    { "text", s_schema, sp_text },
    { "textValue", s_schema, sp_textvalue },
    { "thumbnail", s_schema, sp_thumbnail },
    { "thumbnailUrl", s_schema, sp_thumbnailurl },
    { "tickerSymbol", s_schema, sp_tickersymbol },
    { "ticketNumber", s_schema, sp_ticketnumber },
    { "ticketToken", s_schema, sp_tickettoken },
    { "ticketedSeat", s_schema, sp_ticketedseat },
    { "timeOfDay", s_schema, sp_timeofday },
    { "timeRequired", s_schema, sp_timerequired },
    { "timeToComplete", s_schema, sp_timetocomplete },
    { "tissueSample", s_schema, sp_tissuesample },
    { "title", s_schema, sp_title },
    { "titleEIDR", s_schema, sp_titleeidr },
    { "toLocation", s_schema, sp_tolocation },
    { "toRecipient", s_schema, sp_torecipient },
    { "tocContinuation", s_schema, sp_toccontinuation },
    { "tocEntry", s_schema, sp_tocentry },
    { "tongueWeight", s_schema, sp_tongueweight },
    { "tool", s_schema, sp_tool },
    { "torque", s_schema, sp_torque },
    { "totalJobOpenings", s_schema, sp_totaljobopenings },
    { "totalPaymentDue", s_schema, sp_totalpaymentdue },
    { "totalPrice", s_schema, sp_totalprice },
    { "totalTime", s_schema, sp_totaltime },
    { "tourBookingPage", s_schema, sp_tourbookingpage },
    { "touristType", s_schema, sp_touristtype },
    { "track", s_schema, sp_track },
    { "tracks", s_schema, sp_tracks },
    { "trackingNumber", s_schema, sp_trackingnumber },
    { "trackingUrl", s_schema, sp_trackingurl },
    { "trailer", s_schema, sp_trailer },
    { "trailerWeight", s_schema, sp_trailerweight },
    { "trainName", s_schema, sp_trainname },
    { "trainNumber", s_schema, sp_trainnumber },
    { "trainingSalary", s_schema, sp_trainingsalary },
    { "transFatContent", s_schema, sp_transfatcontent },
    { "transcript", s_schema, sp_transcript },
    { "transitTime", s_schema, sp_transittime },
    { "transitTimeLabel", s_schema, sp_transittimelabel },
    { "translationOfWork", s_schema, sp_translationofwork },
    { "translator", s_schema, sp_translator },
    { "transmissionMethod", s_schema, sp_transmissionmethod },
    { "travelBans", s_schema, sp_travelbans },
    { "trialDesign", s_schema, sp_trialdesign },
    { "tributary", s_schema, sp_tributary },
    { "typeOfBed", s_schema, sp_typeofbed },
    { "typeOfGood", s_schema, sp_typeofgood },
    { "typicalAgeRange", s_schema, sp_typicalagerange },
    { "typicalCreditsPerTerm", s_schema, sp_typicalcreditsperterm },
    { "typicalTest", s_schema, sp_typicaltest },
    { "underName", s_schema, sp_undername },
    { "unitCode", s_schema, sp_unitcode },
    { "unitText", s_schema, sp_unittext },
    { "unnamedSourcesPolicy", s_schema, sp_unnamedsourcespolicy },
    { "unsaturatedFatContent", s_schema, sp_unsaturatedfatcontent },
    { "uploadDate", s_schema, sp_uploaddate },
    { "upvoteCount", s_schema, sp_upvotecount },
    { "url", s_schema, sp_url },
    { "urlTemplate", s_schema, sp_urltemplate },
    { "usageInfo", s_schema, sp_usageinfo },
    { "usedToDiagnose", s_schema, sp_usedtodiagnose },
    { "userInteractionCount", s_schema, sp_userinteractioncount },
    { "usesDevice", s_schema, sp_usesdevice },
    { "usesHealthPlanIdStandard", s_schema, sp_useshealthplanidstandard },
    { "utterances", s_schema, sp_utterances },
    { "validFor", s_schema, sp_validfor },
    { "validFrom", s_schema, sp_validfrom },
    { "validIn", s_schema, sp_validin },
    { "validThrough", s_schema, sp_validto },
    { "validUntil", s_schema, sp_validuntil },
    { "value", s_schema, sp_value },
    { "valueAddedTaxIncluded", s_schema, sp_valueaddedtaxincluded },
    { "valueMaxLength", s_schema, sp_valuemaxlength },
    { "valueMinLength", s_schema, sp_valueminlength },
    { "valueName", s_schema, sp_valuename },
    { "valuePattern", s_schema, sp_valuepattern },
    { "valueReference", s_schema, sp_valuereference },
    { "valueRequired", s_schema, sp_valuerequired },
    { "variableMeasured", s_schema, sp_variablemeasured },
    { "variablesMeasured", s_schema, sp_variablesmeasured },
    { "variantCover", s_schema, sp_variantcover },
    { "variesBy", s_schema, sp_variesby },
    { "vatID", s_schema, sp_vatid },
    { "vehicleConfiguration", s_schema, sp_vehicleconfiguration },
    { "vehicleEngine", s_schema, sp_vehicleengine },
    { "vehicleIdentificationNumber", s_schema, sp_vehicleidentificationnumber },
    { "vehicleInteriorColor", s_schema, sp_vehicleinteriorcolour },
    { "vehicleInteriorType", s_schema, sp_vehicleinteriortype },
    { "vehicleModelDate", s_schema, sp_vehiclemodeldate },
    { "vehicleSeatingCapacity", s_schema, sp_vehicleseatingcapacity },
    { "vehicleSpecialUsage", s_schema, sp_vehiclespecialusage },
    { "vehicleTransmission", s_schema, sp_vehicletransmission },
    { "vendor", s_schema, sp_vendor },
    { "verificationFactCheckingPolicy", s_schema, sp_verificationfactcheckingpolicy },
    { "version", s_schema, sp_version },
    { "video", s_schema, sp_video },
    { "videoFormat", s_schema, sp_videoformat },
    { "videoFrameSize", s_schema, sp_videoframesize },
    { "videoQuality", s_schema, sp_videoquality },
    { "volumeNumber", s_schema, sp_volumenumber },
    { "warning", s_schema, sp_warning },
    { "warranty", s_schema, sp_warranty },
    { "warrantyPromise", s_schema, sp_warrantypromise },
    { "warrantyScope", s_schema, sp_warrantyscope },
    { "webCheckInTime", s_schema, sp_webcheckintime },
    { "webFeed", s_schema, sp_webfeed },
    { "weight", s_schema, sp_weight },
    { "weightTotal", s_schema, sp_weighttotal },
    { "wheelbase", s_schema, sp_wheelbase },
    { "width", s_schema, sp_width },
    { "winner", s_schema, sp_winner },
    { "wordCount", s_schema, sp_wordcount },
    { "workExample", s_schema, sp_workexample },
    { "workFeatured", s_schema, sp_workfeatured },
    { "workHours", s_schema, sp_workhours },
    { "workLocation", s_schema, sp_worklocation },
    { "workPerformed", s_schema, sp_workperformed },
    { "workPresented", s_schema, sp_workpresented },
    { "workTranslation", s_schema, sp_worktranslation },
    { "workload", s_schema, sp_workload },
    { "worksFor", s_schema, sp_worksfor },
    { "worstRating", s_schema, sp_worstrating },
    { "xpath", s_schema, sp_xpath },
    { "yearBuilt", s_schema, sp_yearbuilt },
    { "yearlyRevenue", s_schema, sp_yearlyrevenue },
    { "yearsInOperation", s_schema, sp_yearsinoperation },
    { "yield", s_schema, sp_yield },

    // sd
    { "availableGraphs", s_sd, sdp_availablegraphs },
    { "defaultDataset", s_sd, sdp_defaultdataset },
    { "defaultEntailmentRegime", s_sd, sdp_defaultentailmentregime },
    { "defaultGraph", s_sd, sdp_defaultgraph },
    { "defaultSupportedEntailmentProfile", s_sd, sdp_defaultsupportedentailmentprofile },
    { "endpoint", s_sd, sdp_endpoint },
    { "entailmentRegime", s_sd, sdp_entailmentregime },
    { "extensionAggregate", s_sd, sdp_extensionaggregate },
    { "extensionFunction", s_sd, sdp_extensionfunction },
    { "feature", s_sd, sdp_feature },
    { "graph", s_sd, sdp_graph },
    { "inputFormat", s_sd, sdp_inputformat },
    { "languageExtension", s_sd, sdp_languageextension },
    { "name", s_sd, sdp_name },
    { "namedGraph", s_sd, sdp_namedgraph },
    { "propertyFeature", s_sd, sdp_propertyfeature },
    { "resultFormat", s_sd, sdp_resultformat },
    { "supportedEntailmentProfile", s_sd, sdp_supportedentailmentprofile },
    { "supportedLanguage", s_sd, sdp_supportedlanguage },

    // sioc (frost)
    { "about", s_sioc, sioc_about },
    { "account_of", s_sioc, sioc_account_of },
    { "administrator_of", s_sioc, sioc_administrator_of },
    { "attachment", s_sioc, sioc_attachment },
    { "avatar", s_sioc, sioc_avatar },
    { "container_of", s_sioc, sioc_container_of },
    { "content", s_sioc, sioc_content },
    { "creator_of", s_sioc, sioc_creator_of },
    { "email", s_sioc, sioc_email },
    { "email_sha1", s_sioc, sioc_email_sha1 },
    { "feed", s_sioc, sioc_feed },
    { "function_of", s_sioc, sioc_function_of },
    { "has_administrator", s_sioc, sioc_has_administrator },
    { "has_container", s_sioc, sioc_has_container },
    { "has_creator", s_sioc, sioc_has_creator },
    { "has_function", s_sioc, sioc_has_function },
    { "has_host", s_sioc, sioc_has_host },
    { "has_member", s_sioc, sioc_has_member },
    { "has_moderator", s_sioc, sioc_has_moderator },
    { "has_modifier", s_sioc, sioc_has_modifier },
    { "has_owner", s_sioc, sioc_has_owner },
    { "has_parent", s_sioc, sioc_has_parent },
    { "has_reply", s_sioc, sioc_has_reply },
    { "has_scope", s_sioc, sioc_has_scope },
    { "has_space", s_sioc, sioc_has_space },
    { "has_subscriber", s_sioc, sioc_has_subscriber },
    { "has_usergroup", s_sioc, sioc_has_usergroup },
    { "host_of", s_sioc, sioc_host_of },
    { "id", s_sioc, sioc_id },
    { "ip_address", s_sioc, sioc_ip_address },
    { "link", s_sioc, sioc_link },
    { "links_to", s_sioc, sioc_links_to },
    { "member_of", s_sioc, sioc_member_of },
    { "moderator_of", s_sioc, sioc_moderator_of },
    { "modifier_of", s_sioc, sioc_modifier_of },
    { "name", s_sioc, sioc_name },
    { "next_by_date", s_sioc, sioc_next_by_date },
    { "next_version", s_sioc, sioc_next_version },
    { "note", s_sioc, sioc_note },
    { "num_replies", s_sioc, sioc_num_replies },
    { "num_views", s_sioc, sioc_num_views },
    { "owner_of", s_sioc, sioc_owner_of },
    { "parent_of", s_sioc, sioc_parent_of },
    { "previous_by_date", s_sioc, sioc_previous_by_date },
    { "previous_version", s_sioc, sioc_previous_version },
    { "related_to", s_sioc, sioc_related_to },
    { "reply_of", s_sioc, sioc_reply_of },
    { "scope_of", s_sioc, sioc_scope_of },
    { "sibling", s_sioc, sioc_sibling },
    { "space_of", s_sioc, sioc_space_of },
    { "subscriber_of", s_sioc, sioc_subscriber_of },
    { "topic", s_sioc, sioc_topic },
    { "usergroup_of", s_sioc, sioc_usergroup_of },

    // skos
    { "altLabel", s_skos, skos_altlabel },
    { "broadMatch", s_skos, skos_broadmatch },
    { "broader", s_skos, skos_broader },
    { "broaderTransitive", s_skos, skos_broadertransitive },
    { "changeNote", s_skos, skos_changenote },
    { "closeMatch", s_skos, skos_closematch },
    { "definition", s_skos, skos_definition },
    { "editorialNote", s_skos, skos_editorialnote },
    { "exactMatch", s_skos, skos_exactmatch },
    { "example", s_skos, skos_example },
    { "hasTopConcept", s_skos, skos_hastopconcept },
    { "hiddenLabel", s_skos, skos_hiddenlabel },
    { "historyNote", s_skos, skos_historynote },
    { "inScheme", s_skos, skos_inscheme },
    { "mappingRelation", s_skos, skos_mappingrelation },
    { "member", s_skos, skos_member },
    { "memberList", s_skos, skos_memberlist },
    { "narrowMatch", s_skos, skos_narrowmatch },
    { "narrower", s_skos, skos_narrower },
    { "narrowerTransitive", s_skos, skos_narrowertransitive },
    { "notation", s_skos, skos_notation },
    { "note", s_skos, skos_note },
    { "prefLabel", s_skos, skos_preflabel },
    { "related", s_skos, skos_related },
    { "relatedMatch", s_skos, skos_relatedmatch },
    { "scopeNote", s_skos, skos_scopenote },
    { "semanticRelation", s_skos, skos_semanticrelation },
    { "topConceptOf", s_skos, skos_topconceptof },

    // skosxl
    { "altLabel", s_skosxl, skosxl_altlabel },
    { "hiddenLabel", s_skosxl, skosxl_hiddenlabel },
    { "labelRelation", s_skosxl, skosxl_labelrelation },
    { "literalForm", s_skosxl, skosxl_literalform },
    { "prefLabel", s_skosxl, skosxl_preflabel },

    // sosa
    { "actsOnProperty", s_sosa, sosa_actsonproperty },
    { "hasFeatureOfInterest", s_sosa, sosa_hasfeatureofinterest },
    { "hasResult", s_sosa, sosa_hasresult },
    { "hasSample", s_sosa, sosa_hassample },
    { "hasSimpleResult", s_sosa, sosa_hassimpleresult },
    { "hosts", s_sosa, sosa_hosts },
    { "isActedOnBy", s_sosa, sosa_isacteduponby },
    { "isFeatureOfInterest", s_sosa, sosa_isfeatureofinterest },
    { "isHostedBy", s_sosa, sosa_ishostedby },
    { "isObservedBy", s_sosa, sosa_isobservedby },
    { "isResultOf", s_sosa, sosa_isresultof },
    { "isSampleOf", s_sosa, sosa_issampleof },
    { "madeActuation", s_sosa, sosa_madeactuation },
    { "madeByActuator", s_sosa, sosa_madebyactuator },
    { "madeBySsampler", s_sosa, sosa_madebysampler },
    { "madeBySensor", s_sosa, sosa_madebysensor },
    { "madeObservation", s_sosa, sosa_madeobservation },
    { "madeSampling", s_sosa, sosa_madesampling },
    { "observedProperty", s_sosa, sosa_observedproperty },
    { "observes", s_sosa, sosa_observes },
    { "phenomenonTime", s_sosa, sosa_phenomenontime },
    { "usedProcedure", s_sosa, sosa_usedprocedure },
    { "resultTime", s_sosa, sosa_resulttime },

    // ssn
    { "deployedPlatform", s_ssn, ssn_deployedplatform },
    { "deployedSystem", s_ssn, ssn_deployedsystem },
    { "detects", s_ssn, ssn_detects },
    { "forProperty", s_ssn, ssn_forproperty },
    { "hasDeployment", s_ssn, ssn_hasdeployment },
    { "hasInput", s_ssn, ssn_hasinput },
    { "hasOutput", s_ssn, ssn_hasoutput },
    { "hasProperty", s_ssn, ssn_hasproperty },
    { "implements", s_ssn, ssn_implements },
    { "inDeployment", s_ssn, ssn_indeployment },
    { "isPropertyOf", s_ssn, ssn_ispropertyof },
    { "implementedBy", s_ssn, ssn_isimplementedby },
    { "isProxyFor", s_ssn, ssn_isproxyfor },
    { "wasOriginatedBy", s_ssn, ssn_wasoriginatedby },
    { "hasSubSystem", s_ssn, ssn_hassubsystem },

    // time
    { "after", s_time, tp_after },
    { "before", s_time, tp_before },
    { "day", s_time, tp_day },
    { "days", s_time, tp_days },
    { "dayOfWeek", s_time, tp_dayofweek },
    { "dayOfYear", s_time, tp_dayofyear },
    { "generalDay", s_time, tp_generalday },
    { "generalMonth", s_time, tp_generalmonth },
    { "generalYear", s_time, tp_generalyear },
    { "hasBeginning", s_time, tp_hasbeginning },
    { "hasDateTimeDescription", s_time, tp_hasdatetimedescription },
    { "hasDuration", s_time, tp_hasduration },
    { "hasDurationDescription", s_time, tp_hasdurationdescription },
    { "hasEnd", s_time, tp_hasend },
    { "hasTemporalDuration", s_time, tp_hastemporalduration },
    { "hasTime", s_time, tp_hastime },
    { "hasTRS", s_time, tp_hastrs },
    { "hasXSDuration", s_time, tp_hasxsduration },
    { "hour", s_time, tp_hour },
    { "hours", s_time, tp_hours },
    { "inDateTime", s_time, tp_indatetime },
    { "inTemporalPosition", s_time, tp_intemporalposition },
    { "inTimePosition", s_time, tp_intimeposition },
    { "inXSDDate", s_time, tp_inxsddate },
    { "inXSDDateTime", s_time, tp_inxsddatetime },
    { "inXSDDateTimeStamp", s_time, tp_inxsddatetimestamp },
    { "inXSDgYear", s_time, tp_inxsdgyear },
    { "inXSDgYearMonth", s_time, tp_inxsdgyearmonth },
    { "inside", s_time, tp_inside },
    { "intervalAfter", s_time, tp_intervalafter },
    { "intervalBefore", s_time, tp_intervalbefore },
    { "intervalContains", s_time, tp_intervalcontains },
    { "intervalDisjoint", s_time, tp_intervaldisjoint },
    { "intervalDuring", s_time, tp_intervalduring },
    { "intervalEquals", s_time, tp_intervalequals },
    { "intervalFinishedBy", s_time, tp_intervalfinishedby },
    { "intervalFinishes", s_time, tp_intervalfinishes },
    { "intervalIn", s_time, tp_intervalin },
    { "intervalMeets", s_time, tp_intervalmeets },
    { "intervalMetBy", s_time, tp_intervalmetby },
    { "intervalOverlappedBy", s_time, tp_intervaloverlappedby },
    { "intervalOverlaps", s_time, tp_intervaloverlaps },
    { "intervalStartedBy", s_time, tp_intervalstartedby },
    { "intervalStarts", s_time, tp_intervalstarts },
    { "minute", s_time, tp_minute },
    { "minutes", s_time, tp_minutes },
    { "month", s_time, tp_month },
    { "monthOfYear", s_time, tp_monthofyear },
    { "nominalPosition", s_time, tp_nominalposition },
    { "numericDuration", s_time, tp_numericduration },
    { "numericPosition", s_time, tp_numericposition },
    { "second", s_time, tp_second },
    { "seconds", s_time, tp_seconds },
    { "timeZone", s_time, tp_timezone },
    { "week", s_time, tp_week },
    { "weeks", s_time, tp_weeks },
    { "year", s_time, tp_year },
    { "years", s_time, tp_years },

    // data vocabulary
    { "acquaintance", s_v, v_acquaintance },
    { "address", s_v, v_address },
    { "adr", s_v, v_adr },
    { "affiliation", s_v, v_affiliation },
    { "availability", s_v, v_availability },
    { "best", s_v, v_best },
    { "brand", s_v, v_brand },
    { "category", s_v, v_category },
    { "child", s_v, v_child },
    { "condition", s_v, v_condition },
    { "contact", s_v, v_contact },
    { "country-name", s_v, v_countryname },
    { "count", s_v, v_count },
    { "currency", s_v, v_currency },
    { "description", s_v, v_description },
    { "dtreviewed", s_v, v_dtreviewed },
    { "duration", s_v, v_duration },
    { "endDate", s_v, v_enddate },
    { "eventType", s_v, v_eventtype },
    { "fn", s_v, v_fn },
    { "friend", s_v, v_friend },
    { "geo", s_v, v_geo },
    { "highPrice", s_v, v_highprice },
    { "identifier", s_v, v_identifier },
    { "image", s_v, v_image },
    { "item", s_v, v_item },
    { "itemOffered", s_v, v_itemoffered },
    { "itemreviewed", s_v, v_itemreviewed },
    { "latitude", s_v, v_latitude },
    { "locality", s_v, v_locality },
    { "location", s_v, v_location },
    { "longitude", s_v, v_longitude },
    { "lowPrice", s_v, v_lowprice },
    { "name", s_v, v_name },
    { "nickname", s_v, v_nickname },
    { "offerCount", s_v, v_offercount },
    { "offerdetails", s_v, v_offerdetails },
    { "offerURL", s_v, v_offerurl },
    { "org", s_v, v_org },
    { "photo", s_v, v_photo },
    { "postal-code", s_v, v_postcode },
    { "price", s_v, v_price },
    { "priceValidUntil", s_v, v_pricevaliduntil },
    { "rating", s_v, v_rating },
    { "region", s_v, v_region },
    { "review", s_v, v_reviewed },
    { "reviewer", s_v, v_reviewer },
    { "role", s_v, v_role },
    { "seller", s_v, v_seller },
    { "startDate", s_v, v_startdate },
    { "streetaddress", s_v, v_streetaddress },
    { "summary", s_v, v_summary },
    { "title", s_v, v_title },
    { "url", s_v, v_url },
    { "votes", s_v, v_votes },
    { "worst", s_v, v_worst },

    // vann
    { "changes", s_vann, vann_changes },
    { "example", s_vann, vann_example },
    { "preferredNamespacePrefix", s_vann, vann_pnp },
    { "preferredNamespaceUri", s_vann, vann_pnu },
    { "termGroup", s_vann, vann_termgroup },
    { "usageNote", s_vann, vann_usagenote },

    // vcard
    { "additional-name", s_vcard, vcard_additional_name },
    { "adr", s_vcard, vcard_adr },
    { "agent", s_vcard, vcard_agent },
    { "anniversary", s_vcard, vcard_anniversary },
    { "bday", s_vcard, vcard_bday },
    { "caladruri", s_vcard, vcard_caladruri },
    { "caluri", s_vcard, vcard_caluri },
    { "category", s_vcard, vcard_category },
    { "categories", s_vcard, vcard_categories },
    { "class", s_vcard, vcard_class },
    { "country", s_vcard, vcard_country },
    { "country-name", s_vcard, vcard_country_name },
    { "email", s_vcard, vcard_pemail },
    { "extadd", s_vcard, vcard_extadd },
    { "extended-address", s_vcard, vcard_extended_address },
    { "family", s_vcard, vcard_family },
    { "family-name", s_vcard, vcard_family_name },
    { "fax", s_vcard, vcard_hasfax },
    { "fburl", s_vcard, vcard_fburl },
    { "fn", s_vcard, vcard_fn },
    { "gender", s_vcard, vcard_gender },
    { "geo", s_vcard, vcard_geo },
    { "given", s_vcard, vcard_given },
    { "given-name", s_vcard, vcard_given_name },
    { "group", s_vcard, vcard_group },
    { "hasAdditionalName", s_vcard, vcard_hasadditionalname },
    { "hasAddress", s_vcard, vcard_hasaddress },
    { "hasCalendarBusy", s_vcard, vcard_hascalendarbusy },
    { "hasCalendarRequest", s_vcard, vcard_hascalendarrequest },
    { "hasCalendarLink", s_vcard, vcard_hascalendarlink },
    { "hasCategory", s_vcard, vcard_hascategory },
    { "hasCountryName", s_vcard, vcard_hascountryname },
    { "hasEmail", s_vcard, vcard_hasemail },
    { "hasFN", s_vcard, vcard_hasfn },
    { "hasFamilyName", s_vcard, vcard_hasfamilyname },
    { "hasGeo", s_vcard, vcard_hasgeo },
    { "hasGender", s_vcard, vcard_hasgender },
    { "hasGivenName", s_vcard, vcard_hasgivenname },
    { "hasHonorificPrefix", s_vcard, vcard_hashonourificprefix },
    { "hasHonorificSuffix", s_vcard, vcard_hashonourificsuffix },
    { "hasKey", s_vcard, vcard_haskey },
    { "hasLocality", s_vcard, vcard_haslocality },
    { "hasLogo", s_vcard, vcard_haslogo },
    { "hasMember", s_vcard, vcard_hasmember },
    { "hasName", s_vcard, vcard_hasname },
    { "hasNickname", s_vcard, vcard_hasnickname },
    { "hasNote", s_vcard, vcard_hasnote },
    { "hasOrganizationName", s_vcard, vcard_hasorganisationname },
    { "hasOrganizationalUnit", s_vcard, vcard_hasorganisationunit },
    { "hasPhoto", s_vcard, vcard_hasphoto },
    { "hasPostalCode", s_vcard, vcard_haspostcode },
    { "hasRegion", s_vcard, vcard_hasregion },
    { "hasRelated", s_vcard, vcard_hasrelated },
    { "hasRole", s_vcard, vcard_hasrole },
    { "hasStreetAddress", s_vcard, vcard_hasstreetaddress },
    { "hasTelephone", s_vcard, vcard_hastelephone },
    { "hasTitle", s_vcard, vcard_hastitle },
    { "homeAdr", s_vcard, vcard_homeadr },
    { "homeTel", s_vcard, vcard_hometel },
    { "honorific-prefix", s_vcard, vcard_honourific_prefix },
    { "honorific-suffix", s_vcard, vcard_honourific_suffix },
    { "impp", s_vcard, vcard_impp },
    { "hasInstantMessage", s_vcard, vcard_impp },
    { "key", s_vcard, vcard_key },
    { "kind", s_vcard, vcard_kind },
    { "label", s_vcard, vcard_labelp },
    { "lang", s_vcard, vcard_lang },
    { "language", s_vcard, vcard_lang },
    { "latitude", s_vcard, vcard_latitude },
    { "locality", s_vcard, vcard_locality },
    { "logo", s_vcard, vcard_logo },
    { "longitude", s_vcard, vcard_longitude },
    { "mailer", s_vcard, vcard_mailer },
    { "member", s_vcard, vcard_member },
    { "mobileEmail", s_vcard, vcard_mobileemail },
    { "mobileTel", s_vcard, vcard_mobiletel },
    { "n", s_vcard, vcard_n },
    { "name", s_vcard, vcard_pname },
    { "nickname", s_vcard, vcard_nickname },
    { "note", s_vcard, vcard_note },
    { "other", s_vcard, vcard_other },
    { "org", s_vcard, vcard_org },
    { "organization-name", s_vcard, vcard_organisation_name },
    { "organization-unit", s_vcard, vcard_organisation_unit },
    { "organizational-unit", s_vcard, vcard_organisational_unit },
    { "orgname", s_vcard, vcard_orgname },
    { "orgunit", s_vcard, vcard_orgunit },
    { "tel", s_vcard, vcard_phone },
    { "pcode", s_vcard, vcard_pcode },
    { "personalEmail", s_vcard, vcard_personalemail },
    { "photo", s_vcard, vcard_photo },
    { "pobox", s_vcard, vcard_pobox },
    { "post-office-box", s_vcard, vcard_post_office_box },
    { "postal-code", s_vcard, vcard_postcode },
    { "pref", s_vcard, vcard_ppref },
    { "prefix", s_vcard, vcard_prefix },
    { "prodid", s_vcard, vcard_prodid },
    { "profile", s_vcard, vcard_profile },
    { "region", s_vcard, vcard_region },
    { "related", s_vcard, vcard_related },
    { "rev", s_vcard, vcard_rev },
    { "role", s_vcard, vcard_role },
    { "sort-string", s_vcard, vcard_sort_string },
    { "sound", s_vcard, vcard_sound },
    { "hasSound", s_vcard, vcard_sound },
    { "source", s_vcard, vcard_source },
    { "street", s_vcard, vcard_street },
    { "street-address", s_vcard, vcard_street_address },
    { "suffix", s_vcard, vcard_suffix },
    { "title", s_vcard, vcard_title },
    { "type", s_vcard, vcard_type },
    { "tz", s_vcard, vcard_tz },
    { "uid", s_vcard, vcard_uid },
    { "hasUID", s_vcard, vcard_uid },
    { "unlabeledAdr", s_vcard, vcard_unlabeledadr },
    { "unlabeledEmail", s_vcard, vcard_unlabeledemail },
    { "unlabeledTel", s_vcard, vcard_unlabeledtel },
    { "url", s_vcard, vcard_url },
    { "hasURL", s_vcard, vcard_url },
    { "workAdr", s_vcard, vcard_workadr },
    { "workEmail", s_vcard, vcard_workemail },
    { "workTel", s_vcard, vcard_worktel },
    { "xml", s_vcard, vcard_xml },

    // open graph video
    { "actor", s_video, og_video_actor },
    { "actor:role", s_video, og_video_actor_role },
    { "director", s_video, og_video_director },
    { "duration", s_video, og_video_duration },
    { "release_date", s_video, og_video_release_date },
    { "series", s_video, og_video_series },
    { "tag", s_video, og_video_tag },
    { "writer", s_video, og_video_writer },

    // void
    { "class", s_void, void_class },
    { "classes", s_void, void_classes },
    { "classPartition", s_void, void_classpartition },
    { "dataDump", s_void, void_datadump },
    { "distinctObjects", s_void, void_distinctobjects },
    { "distinctSubjects", s_void, void_distinctsubjects },
    { "documents", s_void, void_documents },
    { "entities", s_void, void_entities },
    { "exampleResource", s_void, void_exampleresource },
    { "feature", s_void, void_feature },
    { "inDataset", s_void, void_indataset },
    { "linkPredicate", s_void, void_linkpredicate },
    { "objectsTarget", s_void, void_objectstarget },
    { "openSearchDescription", s_void, void_opensearchdescription },
    { "properties", s_void, void_properties },
    { "property", s_void, void_property },
    { "propertyPartition", s_void, void_propertypartition },
    { "rootResource", s_void, void_rootresource },
    { "sparqlEndpoint", s_void, void_sparqlendpoint },
    { "subjectsTarget", s_void, void_subjectstarget },
    { "subset", s_void, void_subset },
    { "target", s_void, void_target },
    { "triples", s_void, void_triples },
    { "uriLookupEndpoint", s_void, void_urilookupendpoint },
    { "uriRegexPattern", s_void, void_uriregexpattern },
    { "uriSpace", s_void, void_urispace },
    { "vocabulary", s_void, void_vocabulary },

    // WDR
    { "abouthosts", s_wdr, wdr_abouthosts },
    { "delimiter", s_wdr, wdr_delimiter },
    { "excludeexactpaths", s_wdr, wdr_excludeexactpaths },
    { "excludehosts", s_wdr, wdr_excludehosts },
    { "excludeiripattern", s_wdr, wdr_excludeiripattern },
    { "excludepathcontains", s_wdr, wdr_excludepathcontains },
    { "excludepathendswith", s_wdr, wdr_excludepathendswith },
    { "excludepathstartswith", s_wdr, wdr_excludepathstartswith },
    { "excludequerycontains", s_wdr, wdr_excludequerycontains },
    { "excludeports", s_wdr, wdr_excludeports },
    { "excluderesources", s_wdr, wdr_excluderesources },
    { "excludeschemes", s_wdr, wdr_excludeschemes },
    { "includeexactpaths", s_wdr, wdr_includeexactpaths },
    { "includehosts", s_wdr, wdr_includehosts },
    { "includeiripattern", s_wdr, wdr_includeiripattern },
    { "includepathcontains", s_wdr, wdr_includepathcontains },
    { "includepathendswith", s_wdr, wdr_includepathendswith },
    { "includepathstartswith", s_wdr, wdr_includepathstartswith },
    { "includequerycontains", s_wdr, wdr_includequerycontains },
    { "includeports", s_wdr, wdr_includeports },
    { "includeresources", s_wdr, wdr_includeresources },
    { "includeschemes", s_wdr, wdr_includeschemes },

    // WDRS
    { "authenticate", s_wdrs, wdrs_authenticate },
    { "certified", s_wdrs, wdrs_certified },
    { "certifiedby", s_wdrs, wdrs_certifiedby },
    { "data_error", s_wdrs, wdrs_data_error },
    { "describedby", s_wdrs, wdrs_describedby },
    { "error_code", s_wdrs, wdrs_error_code },
    { "hasIRI", s_wdrs, wdrs_hasiri },
    { "issuedby", s_wdrs, wdrs_issuedby },
    { "logo", s_wdrs, wdrs_logo },
    { "matchesregex", s_wdrs, wdrs_matchesregex },
    { "notknownto", s_wdrs, wdrs_notknownto },
    { "notmatchesregex", s_wdrs, wdrs_notmatchesregex },
    { "proc_error", s_wdrs, wdrs_proc_error },
    { "sha1sum", s_wdrs, wdrs_sha1sum },
    { "supportedby", s_wdrs, wdrs_supportedby },
    { "tag", s_wdrs, wdrs_tag },
    { "text", s_wdrs, wdrs_text },
    { "validfrom", s_wdrs, wdrs_validfrom },
    { "validuntil", s_wdrs, wdrs_validuntil },

    // living standard
    { "author", s_whatwg, wp_author },
    { "license", s_whatwg, wp_license },
    { "title", s_whatwg, wp_title },
    { "work", s_whatwg, wp_work },

    // xhv
    { "alert", s_xhv, xhv_alert },
    { "alertdialog", s_xhv, xhv_alertdialog },
    { "alternate", s_xhv, xhv_alternate },
    { "appendix", s_xhv, xhv_appendix },
    { "application", s_xhv, xhv_application },
    { "article", s_xhv, xhv_article },
    { "banner", s_xhv, xhv_banner },
    { "bookmark", s_xhv, xhv_bookmark },
    { "button", s_xhv, xhv_button },
    { "chapter", s_xhv, xhv_chapter },
    { "checkbox", s_xhv, xhv_checkbox },
    { "cite", s_xhv, xhv_cite },
    { "columnheader", s_xhv, xhv_columnheader },
    { "combobox", s_xhv, xhv_combobox },
    { "complementary", s_xhv, xhv_complementary },
    { "contentinfo", s_xhv, xhv_contentinfo },
    { "contents", s_xhv, xhv_contents },
    { "copyright", s_xhv, xhv_copyright },
    { "definition", s_xhv, xhv_definition },
    { "dialog", s_xhv, xhv_dialog },
    { "directory", s_xhv, xhv_directory },
    { "document", s_xhv, xhv_document },
    { "first", s_xhv, xhv_first },
    { "form", s_xhv, xhv_form },
    { "glossary", s_xhv, xhv_glossary },
    { "grid", s_xhv, xhv_grid },
    { "gridcell", s_xhv, xhv_gridcell },
    { "group", s_xhv, xhv_group },
    { "heading", s_xhv, xhv_heading },
    { "help", s_xhv, xhv_help },
    { "icon", s_xhv, xhv_icon },
    { "img", s_xhv, xhv_img },
    { "index", s_xhv, xhv_index },
    { "itsRules", s_xhv, xhv_itsrules },
    { "last", s_xhv, xhv_last },
    { "license", s_xhv, xhv_license },
    { "link", s_xhv, xhv_link },
    { "list", s_xhv, xhv_list },
    { "listbox", s_xhv, xhv_listbox },
    { "listitem", s_xhv, xhv_listitem },
    { "log", s_xhv, xhv_log },
    { "main", s_xhv, xhv_main },
    { "marquee", s_xhv, xhv_marquee },
    { "math", s_xhv, xhv_math },
    { "menu", s_xhv, xhv_menu },
    { "menubar", s_xhv, xhv_menubar },
    { "menuitem", s_xhv, xhv_menuitem },
    { "menuitemcheckbox", s_xhv, xhv_menuitemcheckbox },
    { "menuitemradio", s_xhv, xhv_menuitemradio },
    { "meta", s_xhv, xhv_meta },
    { "navigation", s_xhv, xhv_navigation },
    { "next", s_xhv, xhv_next },
    { "note", s_xhv, xhv_note },
    { "option", s_xhv, xhv_option },
    { "p3pv1", s_xhv, xhv_p3pv1 },
    { "presentation", s_xhv, xhv_presentation },
    { "prev", s_xhv, xhv_prev },
    { "progressbar", s_xhv, xhv_progressbar },
    { "radio", s_xhv, xhv_radio },
    { "radiogroup", s_xhv, xhv_radiogroup },
    { "region", s_xhv, xhv_region },
    { "role", s_xhv, xhv_role },
    { "row", s_xhv, xhv_row },
    { "rowgroup", s_xhv, xhv_rowgroup },
    { "rowheader", s_xhv, xhv_rowheader },
    { "scrollbar", s_xhv, xhv_scrollbar },
    { "search", s_xhv, xhv_search },
    { "section", s_xhv, xhv_section },
    { "separator", s_xhv, xhv_separator },
    { "slider", s_xhv, xhv_slider },
    { "spinbutton", s_xhv, xhv_spinbutton },
    { "start", s_xhv, xhv_start },
    { "status", s_xhv, xhv_status },
    { "stylesheet", s_xhv, xhv_stylesheet },
    { "subsection", s_xhv, xhv_subsection },
    { "tab", s_xhv, xhv_tab },
    { "tablist", s_xhv, xhv_tablist },
    { "tabpanel", s_xhv, xhv_tabpanel },
    { "textbox", s_xhv, xhv_textbox },
    { "timer", s_xhv, xhv_timer },
    { "toolbar", s_xhv, xhv_toolbar },
    { "tooltip", s_xhv, xhv_tooltip },
    { "top", s_xhv, xhv_top },
    { "tree", s_xhv, xhv_tree },
    { "treegrid", s_xhv, xhv_treegrid },
    { "treeitem", s_xhv, xhv_treeitem },
    { "up", s_xhv, xhv_up },

    { nullptr, s_none, sp_illegal } };

typedef ssc_mm < ::std::string, property_name* > mpn_t;
typedef ssc_map < e_schema_property, property_name* > mnp_t;
mpn_t mpn;
mnp_t mnp;

void schema_name_init (nitpick& nits)
{   PRESUME (mpn.empty (), __FILE__, __LINE__);
    PRESUME (mnp.empty (), __FILE__, __LINE__);
#ifdef _MSC_VER
#pragma warning (push, 3)
#pragma warning (disable : 26481)
#endif // _MSC_VER
    for (property_name* p = &namtab [0]; p -> prop_ != sp_illegal; ++p)
#ifdef _MSC_VER
#pragma warning (pop)
#endif // _MSC_VER
        if (p -> name_ != nullptr)
        {   ::std::string n (::boost::to_lower_copy (::std::string (p -> name_)));
            for (mpn_t::const_iterator ci = mpn.find (n); (ci != mpn.cend ()) && compare_no_case (ci -> first, n); ++ci)
                if (ci -> second -> root_ == p -> root_)
                {   nits.pick (nit_symbol_aleady_defined, es_catastrophic, ec_init, "property name ", n, " (", p -> root_, ") defined more than once");
                    break; }
            auto i = mnp.find (p -> prop_);
            if (i != mnp.cend ())
            {   nits.pick (nit_symbol_aleady_defined, es_splurge, ec_init, "property ", p -> prop_, " ('", i -> second -> name_, "', '", p -> name_, "') has many names");
                mpn.insert (mpn_t::value_type (n, p)); }
            else
            {   mpn.insert (mpn_t::value_type (n, p));
                mnp.insert (mnp_t::value_type (p -> prop_, p)); } } }

::std::string schema_property_name (const e_schema_property p)
{   mnp_t::const_iterator i = mnp.find (p);
    if (i == mnp.cend ()) return ::std::string ();
    return i -> second -> name_; }

e_schema_property get_schema_property (const ::std::string& s)
{   mpn_t::const_iterator i = mpn.find (s);
    if (i == mpn.cend ()) return sp_illegal;
    VERIFY_NOT_NULL (i -> second, __FILE__, __LINE__);
    return i -> second -> prop_; }

vsp_t get_schema_properties (const ::std::string& s, const e_schema root)
{   vsp_t res;
    for (mpn_t::const_iterator i = mpn.find (s); (i != mpn.cend ()) && compare_no_case (i -> first, s); ++i)
        if ((root == s_none) || (root == i -> second -> root_))
        {   VERIFY_NOT_NULL (i -> second, __FILE__, __LINE__)
            res.emplace_back (i -> second -> prop_); }
    return res; }

e_schema get_property_root (const ::std::string& s)
{   mpn_t::const_iterator ci = mpn.find (s);
    if (ci == mpn.cend ()) return s_none;
    VERIFY_NOT_NULL (ci -> second, __FILE__, __LINE__);
    return ci -> second -> root_; }

e_schema get_property_root (const e_schema_property sp)
{   mnp_t::const_iterator ci = mnp.find (sp);
    if (ci == mnp.cend ()) return s_none;
    VERIFY_NOT_NULL (ci -> second, __FILE__, __LINE__);
    return ci -> second -> root_; }
