/*
ssc (static site checker)
Copyright (c) 2020-2022 Dylan Harris
https://dylanharris.org/

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public Licence as published by
the Free Software Foundation, either version 3 of the Licence, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public Licence for more details.

You should have received a copy of the GNU General Public
Licence along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "main/standard.h"
#include "schema/schema_structure.h"
#include "schema/schema_property.h"
#include "main/context.h"

// only used for microformats
#define SS_REQUIRED         0x00000001
#define SS_REPEATABLE       0x00000002

struct microdata_structure
{   schema_version from_;
    schema_version to_;
    e_schema_type record_ = sty_context;
    e_schema_property property_ = sp_context;
    uint32_t flags_ = 0; };

microdata_structure schema_structure [] =
{   // implied by many specifications but rarely actually stated
    { { s_grddl, 1, 0 }, { 0, 0 }, anything, grddlp_namespacetransformation },
    { { s_grddl, 1, 0 }, { 0, 0 }, anything, grddlp_profiletransformation },
    { { s_grddl, 1, 0 }, { 0, 0 }, anything, grddlp_result }, // presuming grddl's reference to InformationResource is anything
    { { s_grddl, 1, 0 }, { 0, 0 }, anything, grddlp_transformation },

    { { s_jsonld, 1, 0 }, { 0, 0 }, anything, jsonld_context },

    { { s_ldp, 1, 0 }, { 0, 0 }, anything, ldp_pagesequence },
    { { s_ldp, 1, 0 }, { 0, 0 }, anything, ldp_inbox },

    { { s_time, 1, 0 }, { 0, 0 }, anything, tp_hastime },

    { { s_vann, 1, 1 }, { 0, 0 }, anything, vann_changes },
    { { s_vann, 1, 1 }, { 0, 0 }, anything, vann_example },
    { { s_vann, 1, 1 }, { 0, 0 }, anything, vann_pnp },
    { { s_vann, 1, 1 }, { 0, 0 }, anything, vann_pnu },
    { { s_vann, 1, 1 }, { 0, 0 }, anything, vann_termgroup },
    { { s_vann, 1, 1 }, { 0, 0 }, anything, vann_usagenote },

    // activity pub etc.
    { { s_as, 1, 0, }, { 0, 0 }, asc_activity, asp_actor },
    { { s_as, 1, 0, }, { 1, 0 }, asc_activity, asp_content },
    { { s_as, 1, 0, }, { 1, 0 }, asc_activity, asp_generator },
    { { s_as, 1, 0, }, { 1, 0 }, asc_activity, asp_icon },
    { { s_as, 1, 0, }, { 1, 0 }, asc_activity, asp_id },
    { { s_as, 2, 0, }, { 0, 0 }, asc_activity, asp_instrument },
    { { s_as, 1, 0, }, { 1, 0 }, asc_activity, asp_published },
    { { s_as, 1, 0, }, { 1, 0 }, asc_activity, asp_provider },
    { { s_as, 1, 0, }, { 0, 0 }, asc_activity, asp_object },
    { { s_as, 2, 0, }, { 0, 0 }, asc_activity, asp_origin },
    { { s_as, 2, 0, }, { 0, 0 }, asc_activity, asp_result },
    { { s_as, 1, 0, }, { 0, 0 }, asc_activity, asp_target },
    { { s_as, 1, 0, }, { 1, 0 }, asc_activity, asp_title },
    { { s_as, 1, 0, }, { 1, 0 }, asc_activity, asp_updated },
    { { s_as, 1, 0, }, { 1, 0 }, asc_activity, asp_url },
    { { s_as, 1, 0, }, { 1, 0 }, asc_activity, asp_verb },

    { { s_as, 2, 0, }, { 0, 0 }, asc_collection, asp_current },
    { { s_as, 2, 0, }, { 0, 0 }, asc_collection, asp_first },
    { { s_as, 1, 0, }, { 0, 0 }, asc_collection, asp_items },
    { { s_as, 2, 0, }, { 0, 0 }, asc_collection, asp_last },
    { { s_as, 1, 0, }, { 0, 0 }, asc_collection, asp_totalitems },
    { { s_as, 1, 0, }, { 1, 0 }, asc_collection, asp_url },

    { { s_as, 2, 0, }, { 0, 0 }, asc_collectionpage, asp_next },
    { { s_as, 2, 0, }, { 0, 0 }, asc_collectionpage, asp_partof },
    { { s_as, 2, 0, }, { 0, 0 }, asc_collectionpage, asp_prev },

    { { s_as, 2, 0, }, { 0, 0 }, asc_link, asp_height },
    { { s_as, 2, 0, }, { 0, 0 }, asc_link, asp_href },
    { { s_as, 2, 0, }, { 0, 0 }, asc_link, asp_hreflang },
    { { s_as, 2, 0, }, { 0, 0 }, asc_link, asp_id },
    { { s_as, 2, 0, }, { 0, 0 }, asc_link, asp_mediatype },
    { { s_as, 2, 0, }, { 0, 0 }, asc_link, asp_name },
    { { s_as, 2, 0, }, { 0, 0 }, asc_link, asp_preview },
    { { s_as, 2, 0, }, { 0, 0 }, asc_link, asp_rel },
    { { s_as, 2, 0, }, { 0, 0 }, asc_link, asp_width },

    { { s_as, 1, 0, }, { 1, 0 }, asc_medialink, asp_duration },
    { { s_as, 1, 0, }, { 1, 0 }, asc_medialink, asp_height },
    { { s_as, 1, 0, }, { 1, 0 }, asc_medialink, asp_url },
    { { s_as, 1, 0, }, { 1, 0 }, asc_medialink, asp_width },

    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_attachment },
    { { s_as, 1, 0, }, { 1, 0 }, asc_object, asp_attachments },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_attributedto },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_audience },
    { { s_as, 1, 0, }, { 1, 0 }, asc_object, asp_author },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_bcc },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_bto },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_cc },
    { { s_as, 1, 0, }, { 0, 0 }, asc_object, asp_content },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_contentmap },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_context },
    { { s_as, 1, 0, }, { 1, 0 }, asc_object, asp_displayname },
    { { s_as, 1, 0, }, { 1, 0 }, asc_object, asp_downstreamduplicates },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_duration },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_endtime },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_generator },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_icon },
    { { s_as, 1, 0, }, { 0, 0 }, asc_object, asp_id },
    { { s_as, 1, 0, }, { 0, 0 }, asc_object, asp_image },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_inreplyto },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_location },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_mediatype },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_name },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_namemap },
    { { s_as, 1, 0, }, { 1, 0 }, asc_object, asp_objecttype },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_preview },
    { { s_as, 1, 0, }, { 0, 0 }, asc_object, asp_published },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_replies },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_starttime },
    { { s_as, 1, 0, }, { 0, 0 }, asc_object, asp_summary },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_summarymap },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_tag },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_to },
    { { s_as, 1, 0, }, { 0, 0 }, asc_object, asp_updated },
    { { s_as, 1, 0, }, { 1, 0 }, asc_object, asp_upstreamduplicates },
    { { s_as, 2, 0, }, { 0, 0 }, asc_object, asp_url },

    { { s_as, 2, 0, }, { 0, 0 }, asc_orderedcollectionpage, asp_startindex },

    { { s_as, 2, 0, }, { 0, 0 }, asc_place, asp_accuracy },
    { { s_as, 2, 0, }, { 0, 0 }, asc_place, asp_altitude },
    { { s_as, 2, 0, }, { 0, 0 }, asc_place, asp_latitude },
    { { s_as, 2, 0, }, { 0, 0 }, asc_place, asp_longitude },
    { { s_as, 2, 0, }, { 0, 0 }, asc_place, asp_radius },
    { { s_as, 2, 0, }, { 0, 0 }, asc_place, asp_units },

    { { s_as, 2, 0, }, { 0, 0 }, asc_profile, asp_describes },

    { { s_as, 2, 0, }, { 0, 0 }, asc_question, asp_closed },
    { { s_as, 2, 0, }, { 0, 0 }, asc_question, asp_oneof },
    { { s_as, 2, 0, }, { 0, 0 }, asc_question, asp_anyof },

    { { s_as, 2, 0, }, { 0, 0 }, asc_relationship, asp_object },
    { { s_as, 2, 0, }, { 0, 0 }, asc_relationship, asp_relationship },
    { { s_as, 2, 0, }, { 0, 0 }, asc_relationship, asp_subject },

    { { s_as, 2, 0, }, { 0, 0 }, asc_tombstone, asp_formertype },
    { { s_as, 2, 0, }, { 0, 0 }, asc_tombstone, asp_deleted },

    // bibliographical ontology
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_booksection, bibo_chapternumber },

    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_asin },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_coden },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_doi },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_eanucc13 },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_eissn },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_gtin14 },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_handle },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_identifier },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_isbn10 },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_isbn13 },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_issn },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_lccn },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_numvolumes },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_oclcnum },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_pmid },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_sici },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_upc },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_collection, bibo_uri },

    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_asin },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_coden },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_content },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_doi },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_eanucc13 },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_edition },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_eissn },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_gtin14 },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_handle },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_identifier },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_isbn10 },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_isbn13 },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_issn },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_lccn },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_locator },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_number },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_numpages },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_oclcnum },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_pageend },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_pages },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_pagestart },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_pmid },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_section },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_shortdescription },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_shorttitle },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_sici },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_upc },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_uri },
    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_document, bibo_volume },

    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_issue, bibo_issuenumber },

    { { s_bibo, 1, 3, }, { 0, 0 }, bibo_legaldocument, bibo_argued },

    // common tag
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_tag, ctag_label },
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_tag, ctag_means },
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_tag, ctag_tagged },
    { { s_ctag, 1, 0 }, { 0, 0 }, ctag_tag, ctag_taggingdate },

    // contrary sausages
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_abouturl },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_datatype },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_default },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_lang },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_name },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_null },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_ordered },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_propertyurl },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_required },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_separator },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_suppressoutput },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_transformations },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_textdirection },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_title },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_valueurl },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_column, csvw_virtual },

    { { s_csvw, 1, 0 }, { 0, 0 }, csv_datatype, csvw_base },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_datatype, csvw_format },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_datatype, csvw_length },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_datatype, csvw_maxexclusive },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_datatype, csvw_maxinclusive },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_datatype, csvw_maxlength },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_datatype, csvw_minexclusive },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_datatype, csvw_mininclusive },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_datatype, csvw_minlength },

    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_commentprefix },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_delimiter },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_doublequote },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_encoding },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_header },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_headerrowcount },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_lineterminators },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_quotechar },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_skipblankrows },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_skipcolumns },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_skipinitialspace },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_skiprows },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_dialect, csvw_trim },

    { { s_csvw, 1, 0 }, { 0, 0 }, csv_foreignkey, csvw_columnreference },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_foreignkey, csvw_reference },

    { { s_csvw, 1, 0 }, { 0, 0 }, csv_numericformat, csvw_decimalchar },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_numericformat, csvw_groupchar },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_numericformat, csvw_pattern },

    { { s_csvw, 1, 0 }, { 0, 0 }, csv_row, csvw_describes },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_row, csvw_primarykey },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_row, csvw_referencedrow },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_row, csvw_title },

    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_abouturl },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_column },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_datatype },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_default },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_foreignkey },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_lang },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_null },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_ordered },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_primarykey },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_propertyurl },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_required },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_rowtitle },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_separator },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_transformations },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_textdirection },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_schema, csvw_valueurl },

    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_abouturl },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_datatype },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_default },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_dialect },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_foreignkey },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_lang },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_null },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_ordered },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_propertyurl },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_required },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_row },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_separator },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_suppressoutput },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_tabledirection },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_tableschema },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_transformations },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_textdirection },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_url },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_table, csvw_valueurl },

    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_abouturl },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_datatype },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_default },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_dialect },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_lang },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_null },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_ordered },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_propertyurl },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_required },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_separator },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_table },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_tabledirection },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_tableschema },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_transformations },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_textdirection },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablegroup, csvw_valueurl },

    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablereference, csvw_columnreference },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablereference, csvw_resource },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_tablereference, csvw_schemareference },

    { { s_csvw, 1, 0 }, { 0, 0 }, csv_transformation, csvw_scriptformat },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_transformation, csvw_source },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_transformation, csvw_targetformat },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_transformation, csvw_title },
    { { s_csvw, 1, 0 }, { 0, 0 }, csv_transformation, csvw_url },

    // data quality
    { { s_daq, 1, 0 }, { 0, 0 }, daq_category, daq_hasdimension },

    { { s_daq, 1, 0 }, { 0, 0 }, daq_dimension, daq_hasmetric },

    { { s_daq, 1, 0 }, { 0, 0 }, daq_metric, daq_expecteddatatype },
    { { s_daq, 1, 0 }, { 0, 0 }, daq_metric, daq_hasobservation },
    { { s_daq, 1, 0 }, { 0, 0 }, daq_metric, daq_requires },

    { { s_daq, 1, 0 }, { 0, 0 }, daq_observation, daq_computedon },
    { { s_daq, 1, 0 }, { 0, 0 }, daq_observation, daq_isestimate },
    { { s_daq, 1, 0 }, { 0, 0 }, daq_observation, daqp_metric },
    { { s_daq, 1, 0 }, { 0, 0 }, daq_observation, daq_value },

    // data catalogue
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_catalogue, dcat_dataset },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_catalogue, dcat_catalogue },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_catalogue, dcat_record },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_catalogue, foaf_homepage },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_catalogue, dcat_service },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_catalogue, dcat_themetaxonomy },

    { { s_dcat, 1, 0 }, { 0, 0 }, dca_cataloguerecord, dct_conformsto },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_cataloguerecord, dct_description },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_cataloguerecord, dct_issued },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_cataloguerecord, dct_modified },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_cataloguerecord, foaf_primarytopic },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_cataloguerecord, dct_title },

    { { s_dcat, 2, 0 }, { 0, 0 }, dca_dataservice, dcat_endpointurl },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_dataservice, dcat_endpointdescription },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_dataservice, dcat_servesdataset },

    { { s_dcat, 1, 0 }, { 0, 0 }, dca_dataset, dct_accrualperiodicity },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_dataset, dcat_distribution },
    { { s_duv, 1, 0 }, { 0, 0 }, dca_dataset, duv_hasdistributor },
    { { s_duv, 1, 0 }, { 0, 0 }, dca_dataset, duv_hasfeedback },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_dataset, dct_spatial },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_dataset, dcat_spatialresolutioninmeters },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_dataset, dct_temporal },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_dataset, dcat_temporalresolution },
    { { s_dcat, 1, 0 }, { s_dcat, 1, 0 }, dca_dataset, dcat_theme },
//    { { s_dcat, 1, 0 }, { 0, 0 }, dca_dataset, prov_wasgeneratedby },

    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dct_accessrights },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dcat_accessservice },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dcat_accessurl },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dcat_bytesize },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_distribution, dcat_compressformat },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_distribution, dct_conformsto },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dct_description },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dcat_downloadurl },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dct_format },
    { { s_duv, 1, 0 }, { 0, 0 }, dca_distribution, duv_hasdistributor },
    { { s_duv, 1, 0 }, { 0, 0 }, dca_distribution, duv_hasfeedback },
//    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, odrl_haspolicy },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dct_issued },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dct_licence },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dcat_mediatype },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dct_modified },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_distribution, dcat_packageformat },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dct_rights },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_distribution, dcat_spatialresolutioninmeters },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_distribution, dcat_temporalresolution },
    { { s_dcat, 1, 0 }, { 0, 0 }, dca_distribution, dct_title },

    { { s_dcat, 2, 0 }, { 0, 0 }, dca_relationship, dct_relation },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_relationship, dcat_hadrole },

    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_accessrights },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_conformsto },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dcat_contactpoint },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_creator },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_description },
//    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, odrl_haspolicy },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_identifier },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_isreferencedby },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dcat_keyword },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dcat_landingpage },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_language },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_licence },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_modified },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_publisher },
//    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, prov_qualifiedattribution },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dcat_qualifiedrelation },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_issued },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_relation },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_rights },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dcat_theme },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_title },
    { { s_dcat, 2, 0 }, { 0, 0 }, dca_resource, dct_type },

// oa_processinglanguage
// commented out because dublin core doesn't define classes
//    { { s_oa, 1, 0 }, { 0, 0 }, dc_language, oa_processinglanguage },

    // dublin core terms
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agent, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_agentclass, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_bibliographicresource, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_fileformat, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_frequency, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_jurisdiction, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_licencedocument, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_linguisticsystem, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_available },
    { { s_dcat, 2, 0 }, { 0, 0 }, dct_location, dcat_bbox },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_bibliographiccitation },
    { { s_dcat, 2, 0 }, { 0, 0 }, dct_location, dcat_centroid },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_format },
    { { s_dcat, 2, 0 }, { 0, 0 }, dct_location, locn_geometry },
    { { s_locn, 1, 0 }, { 0, 0 }, dct_location, locn_geographicname },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_rightsholder },
    { { s_locn, 1, 0 }, { 0, 0 }, dct_location, rdfs_seealso },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_location, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_locationperiodorjurisdiction, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatype, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_mediatypeorextent, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofaccrual, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_methodofinstruction, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_educationlevel },
    { { s_dcat, 2, 0 }, { 0, 0 }, dct_periodoftime, dcat_enddate },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_hasformat },
//    { { s_dcat, 2, 0 }, { 0, 0 }, dct_periodoftime, time_hasbeginning },
//    { { s_dcat, 2, 0 }, { 0, 0 }, dct_periodoftime, time_hasend },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_spatial },
    { { s_dcat, 2, 0 }, { 0, 0 }, dct_periodoftime, dcat_startdate },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_periodoftime, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalmedium, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_physicalresource, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_policy, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_provenancestatement, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_rightsstatement, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_sizeorduration, dct_valid },

    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_abstract },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_accessrights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_accrualmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_accrualperiodicity },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_accrualpolicy },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_alternative },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_audience },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_available },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_bibliographiccitation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_conformsto },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_contributor },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_coverage },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_created },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_creator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_date },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_dateaccepted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_datecopyrighted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_datesubmitted },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_description },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_educationlevel },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_extent },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_format },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_hasformat },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_haspart },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_hasversion },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_identifier },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_instructionalmethod },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_isformatof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_ispartof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_isreferencedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_isreplacedby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_isrequiredby },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_issued },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_isversionof },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_language },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_licence },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_mediator },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_medium },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_modified },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_provenance },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_publisher },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_references },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_relation },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_replaces },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_requires },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_rights },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_rightsholder },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_source },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_spatial },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_subject },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_tableofcontents },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_temporal },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_title },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_type },
    { { s_dct, 1, 0 }, { 0, 0 }, dct_standard, dct_valid },

    // doap
    { { s_doap, 1, 0 }, { 0, 0 }, doap_archrepository, doap_module },

    { { s_doap, 1, 0 }, { 0, 0 }, doap_bkrepository, doap_module },

    { { s_doap, 1, 0 }, { 0, 0 }, doap_cvsrepository, doap_module },

    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_audience },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_blog },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_bugdatabase },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_developer },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_developerforum },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_documenter },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_downloadpage },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_downloadmirror },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_hasrepository },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_helper },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_homepage },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_implements },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_language },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_mailinglist },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_maintainer },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_oldhomepage },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_os },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_platform },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_programminglanguage },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_release },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_screenshots },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_serviceendpoint },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_supportforum },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_tester },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_translator },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_vendor },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_project, doap_wiki },

    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_anonroot },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_browse },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_location },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_repository, doap_repositoryof },

    { { s_doap, 1, 0 }, { 0, 0 }, doap_version, doap_filerelease },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_version, doap_os },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_version, doap_platform },
    { { s_doap, 1, 0 }, { 0, 0 }, doap_version, doap_revision },

    // data quality
    { { s_dqv, 1, 0 }, { 0, 0 }, dqv_dimension, dqv_incategory },

    { { s_dqv, 1, 0 }, { 0, 0 }, dqv_metric, dqv_expecteddatatype },

    { { s_dqv, 1, 0 }, { 0, 0 }, dqv_qualitymeasurement, dqv_computedon },
    { { s_dqv, 1, 0 }, { 0, 0 }, dqv_qualitymeasurement, dqv_value },

    // earl
    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertion, earl_assertedby },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertion, earl_mode },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertion, earl_result },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertion, earl_subject },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertion, earl_test },

    { { s_earl, 1, 0 }, { 0, 0 }, earl_assertor, earl_mainassertor },

    { { s_earl, 1, 0 }, { 0, 0 }, earl_testresult, earl_info },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_testresult, earl_outcome },
    { { s_earl, 1, 0 }, { 0, 0 }, earl_testresult, earl_pointer },

    // event
    { { s_event, 1, 0 }, { 0, 0 }, event_event, ep_agent },
    { { s_event, 1, 0 }, { 0, 0 }, event_event, ep_factor },
    { { s_event, 1, 0 }, { 0, 0 }, event_event, ep_literalfactor },
    { { s_event, 1, 0 }, { 0, 0 }, event_event, ep_place },
    { { s_event, 1, 0 }, { 0, 0 }, event_event, ep_product },
    { { s_event, 1, 0 }, { 0, 0 }, event_event, ep_subevent },
    { { s_event, 1, 0 }, { 0, 0 }, event_event, ep_time },

    // foaf
    { { s_foaf, 0, 96 }, { 0, 0 }, foaf_agent, foaf_account },
    { { s_foaf, 0, 96 }, { 0, 0 }, foaf_agent, foaf_age },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_aimchatid },
    { { s_foaf, 0, 90 }, { 0, 0 }, foaf_agent, foaf_birthday },
    { { s_org, 1, 0 }, { 0, 0 }, foaf_agent, orgp_hasmembership },
    { { s_org, 1, 0 }, { 0, 0 }, foaf_agent, orgp_headof },
    { { s_org, 1, 0 }, { 0, 0 }, foaf_agent, orgp_holds },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_holdsaccount },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_icqchatid },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_interest },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_jabberid },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_made },
    { { s_org, 1, 0 }, { 0, 0 }, foaf_agent, orgp_memberof },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_mbox },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_mboxsha1sum },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_msnchatid },
    { { s_foaf, 0, 91 }, { 0, 0 }, foaf_agent, foaf_openid },
    { { s_bibo, 1, 3, }, { 0, 0 }, foaf_agent, bibo_prefixname },
    { { s_org, 1, 0 }, { 0, 0 }, foaf_agent, orgp_reportsto },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_skypeid },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_status },
    { { s_bibo, 1, 3, }, { 0, 0 }, foaf_agent, bibo_suffixname },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_tipjar },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_topic_interest },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_weblog },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_agent, foaf_yahoochatid },

    { { s_void, 1, 0 }, { 0, 0 }, foaf_document, void_indataset },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_document, foaf_primarytopic },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_document, foaf_sha1 },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_document, foaf_topic },

    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_group, foaf_member },

    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_image, foaf_depicts },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_image, foaf_thumbnail },

    { { s_org, 1, 0 }, { 0, 0 }, foaf_person, orgp_basedat },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_currentproject },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_familyname },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_firstname },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_geekcode },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_holdsaccount }, // sioc
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_img },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_knows },
    { { s_foaf, 0, 97 }, { 0, 0 }, foaf_person, foaf_lastname },
    { { s_org, 1, 0 }, { 0, 0 }, foaf_person, orgp_location },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_myersbriggs },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_pastproject },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_publications },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_schoolhomepage },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_surname },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_thumbnail },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_workinfohomepage },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_person, foaf_workplacehomepage },

    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_onlineaccount, foaf_accountname },
    { { s_foaf, 0, 1 }, { 0, 0 }, foaf_onlineaccount, foaf_accountservicehomepage },

    //frbr core
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_endeavour, fp_part },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_endeavour, fp_partof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_endeavour, fp_relatedendeavour },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_endeavour, fp_responsibleentity },

    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_abridgement },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_abridgementof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_adaption },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_adaptionof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_arrangement },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_arrangementof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_complement },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_complementof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_embodiment },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_imitation },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_imitationof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_realisationof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_realiser },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_revision },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_revisionof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_successor },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_successorof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_summary },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_summaryof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_supplement },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_supplementof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_transformation },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_transformationof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_translation },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_expression, fp_translationof },

    { { s_frbr, 1, 0 }, { 0, 0 }, fs_item, fp_exemplarof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_item, fp_owner },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_item, fp_reproduction },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_item, fp_reproductionof },

    { { s_frbr, 1, 0 }, { 0, 0 }, fs_manifestation, fp_alternative },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_manifestation, fp_alternativeof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_manifestation, fp_embodimentof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_manifestation, fp_exemplar },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_manifestation, fp_producer },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_manifestation, fp_reproduction },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_manifestation, fp_reproductionof },

    { { s_frbr, 1, 0 }, { 0, 0 }, fs_responsibleentity, fp_creatorof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_responsibleentity, fp_ownerof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_responsibleentity, fp_producerof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_responsibleentity, fp_reconfiguration },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_responsibleentity, fp_reconfigurationof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_responsibleentity, fp_responsibleentityof },

    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_adaption },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_adaptionof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_complement },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_complementof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_creator },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_imitation },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_imitationof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_realisation },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_subject },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_successor },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_successorof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_summary },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_summaryof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_supplement },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_supplementof },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_transformation },
    { { s_frbr, 1, 0 }, { 0, 0 }, fs_work, fp_transformationof },

    // good relations
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_category },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_colour },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_condition },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_datatypeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_depth },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_hasbrand },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_hasean_ucc_13 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_hasgtin_14 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_hasgtin_8 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_hasmpn },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_hasmanufacturer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_hasstockkeepingunit },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_height },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_isaccessoryorsparepartfor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_isconsumablefor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_issimilarto },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_qualitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_quantitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_weight },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_actualproductorserviceinstance, gr_width },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_brand, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_brand, gr_name },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_category },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_hasbrand },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_hasduns },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_hasgloballocationnumber },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_hasisicv4 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_hasnaics },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_haspos },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_legalname },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_offers },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_owns },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_seeks },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_taxid },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentity, gr_vatid },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentitytype, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessentitytype, gr_name },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_businessfunction, gr_name },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_hasnext },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_hasprevious },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_dayofweek, gr_name },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_appliestodeliverymethod },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_eligibleregions },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_eligibletransactionvolume },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_hascurrency },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_hascurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_haseligiblequantity },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_hasmaxcurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_hasmincurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_validfrom },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_validto },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverychargespecification, gr_valueaddedtaxincluded },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_category },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_colour },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_condition },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_datatypeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_depth },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_hasbrand },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_hasean_ucc_13 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_hasgtin_14 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_hasgtin_8 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_hasmpn },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_hasmakeandmodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_hasmanufacturer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_hasstockkeepingunit },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_height },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_isaccessoryorsparepartfor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_isconsumablefor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_issimilarto },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_qualitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_quantitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_serialnumber },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_weight },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_deliverymethod, gr_width },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_licence, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_licence, gr_eligibleduration },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_licence, gr_eligibleregions },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_licence, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_licence, gr_validfrom },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_licence, gr_validto },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_location, gr_category },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_location, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_location, gr_hasgloballocationnumber },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_location, gr_hasisicv4 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_location, gr_hasopeninghoursspecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_location, gr_name },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_acceptedpaymentmethods },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_addon },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_advancebookingrequirement },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_availabilityends },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_availabilitystarts },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_availableatorfrom },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_availabledeliverymethods },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_category },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_condition },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_deliveryleadtime },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_eligiblecustomertypes },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_eligibleduration },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_eligibleregions },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_eligibletransactionvolume },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_hasbusinessfunction },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_hasean_ucc_13 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_haseligiblequantity },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_hasgtin_14 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_hasgtin_8 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_hasinventorylevel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_hasmpn },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_haspricespecification },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_hasstockkeepingunit },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_haswarrantypromise },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_includes },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_includesobject },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_serialnumber },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_validfrom },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_offering, gr_validto },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_openinghoursspecification, gr_closes },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_openinghoursspecification, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_openinghoursspecification, gr_hasopeninghoursdayofweek },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_openinghoursspecification, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_openinghoursspecification, gr_opens },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_openinghoursspecification, gr_validfrom },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_openinghoursspecification, gr_validto },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_appliestopaymentmethod },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_eligibletransactionvolume },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_hascurrency },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_hascurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_haseligiblequantity },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_hasmaxcurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_hasmincurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_validfrom },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_validto },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentchargespecification, gr_valueaddedtaxincluded },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_paymentmethod, gr_name },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_eligibletransactionvolume },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_hascurrency },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_hascurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_haseligiblequantity },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_hasmaxcurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_hasmincurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_validfrom },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_validto },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_pricespecification, gr_valueaddedtaxincluded },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_category },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_colour },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_condition },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_datatypeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_depth },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_hasbrand },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_hasean_ucc_13 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_hasgtin_14 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_hasgtin_8 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_hasmpn },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_hasmanufacturer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_hasstockkeepingunit },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_height },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_isaccessoryorsparepartfor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_isconsumablefor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_issimilarto },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_qualitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_quantitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_weight },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservice, gr_width },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_category },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_colour },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_condition },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_datatypeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_depth },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_hasbrand },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_hasean_ucc_13 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_hasgtin_14 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_hasgtin_8 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_hasmpn },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_hasmakeandmodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_hasmanufacturer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_hasstockkeepingunit },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_height },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_isaccessoryorsparepartfor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_isconsumablefor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_issimilarto },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_isvariantof },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_predecessorof },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_qualitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_quantitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_serialnumber },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_successorof },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_weight },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_productorservicemodel, gr_width },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_category },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_colour },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_condition },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_datatypeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_depth },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_hasbrand },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_hasean_ucc_13 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_hasgtin_14 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_hasgtin_8 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_hasmpn },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_hasmanufacturer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_hasstockkeepingunit },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_height },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_isaccessoryorsparepartfor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_isconsumablefor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_issimilarto },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_qualitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_quantitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_weight },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_locationofsalesorserviceprovisioning, gr_width },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_qualitativevalue, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_qualitativevalue, gr_equal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_qualitativevalue, gr_greater },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_qualitativevalue, gr_greaterorequal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_qualitativevalue, gr_lesser },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_qualitativevalue, gr_lesserorequal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_qualitativevalue, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_qualitativevalue, gr_nonequal },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_qualitativevalue, gr_valuereference },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalue, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalue, gr_hasmaxvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalue, gr_hasminvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalue, gr_hasunitofmeasurement },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalue, gr_hasvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalue, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalue, gr_valuereference },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevaluefloat, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevaluefloat, gr_hasmaxvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevaluefloat, gr_hasmaxvaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevaluefloat, gr_hasminvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevaluefloat, gr_hasminvaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevaluefloat, gr_hasunitofmeasurement },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevaluefloat, gr_hasvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevaluefloat, gr_hasvaluefloat },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevaluefloat, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevaluefloat, gr_valuereference },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalueinteger, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalueinteger, gr_hasmaxvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalueinteger, gr_hasmaxvalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalueinteger, gr_hasminvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalueinteger, gr_hasminvalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalueinteger, gr_hasunitofmeasurement },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalueinteger, gr_hasvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalueinteger, gr_hasvalueinteger },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalueinteger, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_quantitativevalueinteger, gr_valuereference },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_category },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_colour },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_condition },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_datatypeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_depth },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_hasbrand },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_hasean_ucc_13 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_hasgtin_14 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_hasgtin_8 },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_hasinventorylevel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_hasmpn },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_hasmakeandmodel },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_hasmanufacturer },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_hasstockkeepingunit },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_height },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_isaccessoryorsparepartfor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_isconsumablefor },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_issimilarto },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_isvariantof },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_qualitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_quantitativeproductorserviceproperty },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_serialnumber },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_weight },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_someitems, gr_width },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_typeandquantitynode, gr_amountofgoods },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_typeandquantitynode, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_typeandquantitynode, gr_hasbusinessfunction },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_typeandquantitynode, gr_hasunitofmeasurement },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_typeandquantitynode, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_typeandquantitynode, gr_typeofgoods },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_billingincrement },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_eligibletransactionvolume },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_hascurrency },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_hascurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_haseligiblequantity },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_hasmaxcurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_hasmincurrencyvalue },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_hasunitofmeasurement },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_name },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_pricetype },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_validfrom },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_validto },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_unitpricespecification, gr_valueaddedtaxincluded },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_warrantypromise, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_warrantypromise, gr_durationofwarrantyinmonths },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_warrantypromise, gr_haswarrantyscope },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_warrantypromise, gr_name },

    { { s_gr, 1, 0 }, { 0, 0 }, gr_warrantyscope, gr_description },
    { { s_gr, 1, 0 }, { 0, 0 }, gr_warrantyscope, gr_name },

    // grddl
    { { s_grddl, 1, 0 }, { 0, 0 }, grddl_transformation, grddlp_transformationproperty },

    // gs1
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_additionalproductclassificationdetails, gs1p_additionalproductclassificationcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_additionalproductclassificationdetails, gs1p_additionalproductclassificationcodedescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_additionalproductclassificationdetails, gs1p_additionalproductclassificationvalue },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_additivedetails, gs1p_additivelevelofcontainment },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_additivedetails, gs1p_additivename },

    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_city },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_countrycode },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_countycode },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_crossstreet },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_latitude },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_longitude },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_poboxnumber },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_partyname },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_postalcode },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_provincestatecode },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_streetaddress1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_streetaddress2 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, gs1s_address, gs1p_streetaddress3 },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergendetails, gs1p_allergenlevelofcontainmentcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_allergendetails, gs1p_allergentype },

    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_authenticitydetails, gs1p_authenticitysecurityfeatureinstructions },
    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_authenticitydetails, gs1p_authenticitysecurityfeatureinstructionsurl },
    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_authenticitydetails, gs1p_authenticitysecurityfeatureregularexpression },
    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_authenticitydetails, gs1p_authenticitysecurityfeaturetype },
    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_authenticitydetails, gs1p_authenticitysecurityfeaturevalue },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_awardprizedetails, gs1p_awardprizecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_awardprizedetails, gs1p_awardprizecountrycode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_awardprizedetails, gs1p_awardprizedescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_awardprizedetails, gs1p_awardprizejury },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_awardprizedetails, gs1p_awardprizename },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_awardprizedetails, gs1p_awardprizeyear },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_beverage, gs1p_alcoholicbeveragesubregion },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_beverage, gs1p_beveragevintage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_beverage, gs1p_iscarbonated },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_beverage, gs1p_isdecaffeinated },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_beverage, gs1p_isfromconcentrate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_beverage, gs1p_isvintage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_beverage, gs1p_percentageofalcoholbyvolume },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_beverage, gs1p_vintner },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_brand, gs1p_brandname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_brand, gs1p_subbrandname },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_certificationdetails, gs1p_certificationagency },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_certificationdetails, gs1p_certificationstandard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_certificationdetails, gs1p_certificationvalue },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_clothing, gs1p_clothingcut },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_clothing, gs1p_collartype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_clothing, gs1p_ismaternity },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_clothing, gs1p_textilematerial },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodedetails, gs1p_colourcodelist },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_colourcodedetails, gs1p_colourcodevalue },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_contactpoint, gs1p_availablelanguage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_contactpoint, gs1p_contacttitle },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_contactpoint, gs1p_contacttype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_contactpoint, gs1p_email },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_contactpoint, gs1p_faxnumber },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_contactpoint, gs1p_responsibility },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_contactpoint, gs1p_telephone },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_country, gs1p_countrycode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_country, gs1p_countrysubdivisioncode },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecodedetails, gs1p_diettype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_diettypecodedetails, gs1p_diettypesubcode },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_discountrepeatspermultipleminimum },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_discounttype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_eligibletradechannel },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_exactdiscountamount },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_exactdiscountpercentage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_exclusiondescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_isonlyavailablethroughretailer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_isonlywithmailinglistsignup },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_isonlywithpaymentcard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_isonlywithretailerloyaltycard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_isonlywithretailerpaymentcard },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_ispromoterexclusiveoffer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_maximumdiscountamount },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_maximumdiscountpercentage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_maximumqualifyingitems },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_maximumqualifyingspend },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_minimumdiscountamount },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_minimumdiscountpercentage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_minimumqualifyingitems },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_minimumqualifyingspend },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_offerredemptiontype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_offerrestrictiondescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_qualifyingbrandname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_qualifyinggpcs },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_qualifyingproductcategorydescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_qualifyingproductclassificationcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_qualifyingproductgtins },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_qualifyingsubbrandname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_discount, gs1p_selectedproductsonly },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodandbeveragepreparationinformation, gs1p_conveniencelevelpercent },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodandbeveragepreparationinformation, gs1p_manufacturerpreparationcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodandbeveragepreparationinformation, gs1p_maximumoptimumconsumptiontemperature },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodandbeveragepreparationinformation, gs1p_minimumoptimumconsumptiontemperature },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodandbeveragepreparationinformation, gs1p_preparationcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodandbeveragepreparationinformation, gs1p_preparationconsumptionprecautions },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodandbeveragepreparationinformation, gs1p_preparationinstructions },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodandbeveragepreparationinformation, gs1p_yield },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoingredientdetails, gs1p_ingredientcontentpercentage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoingredientdetails, gs1p_ingredientname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoingredientdetails, gs1p_ingredientsequence },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_allergenspecificationagency },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_allergenspecificationname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_allergenstatement },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_biotinpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_calciumpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_carbohydratespernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_chloridepernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_cholesterolpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_chromiumpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_copperpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_dietcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_diettypedescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_drainedweight },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_energyfromfatpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_energypernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_fatpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_fibrepernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_fluoridepernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_folicacidpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_foodbeveragerefrigerationclaim },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_foodbeveragetargetuse },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_geneticallymodifieddeclaration },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_hasallergen },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_ingredientofconcern },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_ingredientstatement },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_iodinepernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_ironpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_irradiatedcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_isinstant },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_issliced },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_juicecontentpercent },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_magnesiumpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_manganesepernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_molybdenumpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_monounsaturatedfatpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_niacinpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_numberofservingsperpackage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_numberofservingsperpackagemeasurementprecision },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_numberofservingsrangedescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_nutrientbasisquantity },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_nutrientbasisquantitytype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_nutrientmeasurementprecision },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_nutritionalclaim },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_nutritionalclaimstatement },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_packagingmarkeddietallergentype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_packagingmarkedfreefrom },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_pantothenicacidpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_phosphoruspernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_polyolspernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_polyunsaturatedfatpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_potassiumpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_preparationinformation },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_preservationtechnique },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_proteinpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_reheatingclaim },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_riboflavinpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_saltpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_saturatedfatpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_seleniumpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_servingsize },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_servingsizedescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_servingsuggestion },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_sodiumpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_sourceanimal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_starchpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_sugarspernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_thiaminpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_transfatpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_vitaminapernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_vitaminb12pernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_vitaminb6pernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_vitamincpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_vitamindpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_vitaminepernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_vitaminkpernutrientbasis },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_foodbeveragetobaccoproduct, gs1p_zincpernutrientbasis },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwear, gs1p_footwearfasteningtype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwear, gs1p_footwearuppertype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_footwear, gs1p_uppermaterialtype },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_fruitsvegetables, gs1p_ispittedstoned },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_fruitsvegetables, gs1p_isseedless },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_fruitsvegetables, gs1p_isshelledpeeled },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_fruitsvegetables, gs1p_iswashedreadytoeat },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_fruitsvegetables, gs1p_maturationmethod },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_geocoordinates, gs1p_latitude },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_geocoordinates, gs1p_longitude },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_meatpoultry, gs1p_anatomicalform },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_meatpoultry, gs1p_bonelessclaim },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_meatpoultry, gs1p_meatpoultrytype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_meatpoultry, gs1p_minimummeatpoultrycontent },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_milkbuttercreamyogurtcheeseeggssubstitutes, gs1p_cheesefirmness },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_milkbuttercreamyogurtcheeseeggssubstitutes, gs1p_cheesematurationperioddescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_milkbuttercreamyogurtcheeseeggssubstitutes, gs1p_fatinmilkcontent },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_milkbuttercreamyogurtcheeseeggssubstitutes, gs1p_fatpercentageindrymatter },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_milkbuttercreamyogurtcheeseeggssubstitutes, gs1p_ishomogenised },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_milkbuttercreamyogurtcheeseeggssubstitutes, gs1p_isrindedible },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_milkbuttercreamyogurtcheeseeggssubstitutes, gs1p_sharpnessofcheese },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_nutritionmeasurementtype, gs1p_dailyvalueintakepercent },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_acceptedpaymentmethod },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_availabilitystarts },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_availabilityends },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_availableatorfrom },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_eligiblequantity },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_eligiblequantitymaximum },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_eligiblequantityminimum },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_itemoffered },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_offerdescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_offerdiscount },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_offerredemptionurl },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_paymentterms },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_pricespecification },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_seeker },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_seller },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_offer, gs1p_warranty },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimdetails, gs1p_organicclaimagency },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organicclaimdetails, gs1p_organicpercentclaim },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisation, gs1p_additionalorganizationidentificationtypevalue },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisation, gs1p_address },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisation, gs1p_afterhourscontact },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisation, gs1p_contactpoint },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisation, gs1p_department },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisation, gs1p_globallocationnumber },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisation, gs1p_location },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisation, gs1p_makesoffer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisation, gs1p_organisationname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_organisation, gs1p_organisationrole },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingdetails, gs1p_hasreturnablepackagedeposit },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingdetails, gs1p_packagingfeature  },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingdetails, gs1p_packagingfunction },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingdetails, gs1p_packagingmaterial },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingdetails, gs1p_packagingrecyclingprocesstype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingdetails, gs1p_packagingrecyclingscheme },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingdetails, gs1p_packagingshape },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingdetails, gs1p_packagingtype },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialdetails, gs1p_packagingmaterialcompositionquantity },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialdetails, gs1p_packagingmaterialthickness },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_packagingmaterialdetails, gs1p_packagingmaterialtype },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_place, gs1p_address },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_place, gs1p_geo },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_place, gs1p_globallocationnumber },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_postaladdress, gs1p_addresscountry },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_postaladdress, gs1p_addresslocality },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_postaladdress, gs1p_addressregion },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_postaladdress, gs1p_countycode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_postaladdress, gs1p_crossstreet },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_postaladdress, gs1p_organisationname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_postaladdress, gs1p_postofficeboxnumber },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_postaladdress, gs1p_postalcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_postaladdress, gs1p_streetaddress },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_dutyfeetaxamount },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_dutyfeetaxdescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_dutyfeetaxrate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_eligiblequantity },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_eligiblequantitymaximum },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_eligiblequantityminimum },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_maxprice },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_minprice },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_price },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_pricecurrency },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_validfrom },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_pricespecification, gs1p_validto },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_additionalproductclassificationcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_additionalproductclassificationcodedescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_additive },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_audiofile },
    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_product, gs1p_authenticity },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_awardprize },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_bestbeforedate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_brand },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_brandowner },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_certification },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_colourcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_colourdescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_consumerhandlingstorage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_consumerpackagedisclaimer },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_consumerproductvariant },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_consumersafetyinformation },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_consumersalescondition },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_consumerstorageinstructions },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_consumerusageinstructions },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_countryofassembly },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_countryoflastprocessing },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_countryoforigin },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_countryoforiginstatement },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_customersupportcentre },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_dependentproprietaryproduct },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_descriptivesize },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_duedate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_equivalentproduct },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_expirationdate },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_expirationdatetime },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_firstfreezedate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_functionalname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_gpccategorycode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_gpccategorydescription },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_grossarea },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_grossvolume },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_grossweight },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_growingmethod },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_gtin },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_harvestdate },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_harvestdateend },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_harvestdatestart },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_hasbatchlotnumber },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_hasserialnumber },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_hasthirdpartycontrolledserialnumber },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_healthclaimdescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_image },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_includedaccessories },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_inpackagedepth },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_inpackagediameter },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_inpackageheight },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_inpackagewidth },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_instructionsforuse },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_isproductrecalled },
    { { s_gs1, 1, 51 }, { 0, 0 }, gs1s_product, gs1p_jws },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_manufacturer },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_manufacturerswarranty },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_manufacturingplant },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_massperunitarea },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_netarea },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_netcontent },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_netweight },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_organicclaim },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_outofpackagedepth },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_outofpackagediameter },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_outofpackageheight },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_outofpackagewidth },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_packaging },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_packagingdate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_packagingmarkedlabelaccreditation },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_primaryalternateproduct },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_productdescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_productfeaturebenefit },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_productformdescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_productid },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_productmarketingmessage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_productname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_productrange },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_productsustainabilityinfo },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_productiondate },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_productiondatetime },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_productionvariantdescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_productionvarianteffectivedatetime },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_provenancestatement },
    { { s_gs1, 1, 1 }, { s_gs1, 1, 2 }, gs1s_product, gs1p_recipewebsite },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_referencedfile },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_regulatedproductname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_replacedbyproduct },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_replacedproduct },
    { { s_gs1, 1, 3 }, { 0, 0 }, gs1s_product, gs1p_sellbydate },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_sizecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_supplierspecifiedminimumconsumerstoragedays },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_targetmarket },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_variantdescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_product, gs1p_warningcopydescription },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_productyielddetails, gs1p_productyield },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_productyielddetails, gs1p_productyieldtype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_productyielddetails, gs1p_productyieldvariationpercentage },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_quantitativevalue, gs1p_unitcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_quantitativevalue, gs1p_value },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiledetails, gs1p_filelanguagecode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiledetails, gs1p_filepixelheight },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiledetails, gs1p_filepixelwidth },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiledetails, gs1p_referencedfileeffectiveenddatetime },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiledetails, gs1p_referencedfileeffectivestartdatetime },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiledetails, gs1p_referencedfilesize },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiledetails, gs1p_referencedfiletype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_referencedfiledetails, gs1p_referencedfileurl },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_returnablepackagedepositdetails, gs1p_returnablepackagedepositamount },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_returnablepackagedepositdetails, gs1p_returnablepackagedepositregion },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seafood, gs1p_catchzone },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seafood, gs1p_fishtype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seafood, gs1p_freshorseawaterfarmed },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_seafood, gs1p_minimumfishcontent },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizecodedetails, gs1p_sizecodelistcode },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizecodedetails, gs1p_sizecodevalue },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizedetails, gs1p_sizedimension },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizedetails, gs1p_sizegroup },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizedetails, gs1p_sizesystem },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_sizedetails, gs1p_sizetype },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_targetmarketdetails, gs1p_consumerfirstavailabilitydatetime },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_targetmarketdetails, gs1p_targetmarketcountries },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_textilematerialdetails, gs1p_textilematerialdescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_textilematerialdetails, gs1p_textilematerialpercentage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_textilematerialdetails, gs1p_textilematerialthreadcount },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_textilematerialdetails, gs1p_textilematerialweight },

    { { s_gs1, 1, 5 }, { 0, 0 }, gs1s_typecode, gs1p_originalcodevalue },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_warrantypromise, gs1p_durationofwarranty },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_warrantypromise, gs1p_warrantyscopedescription },

    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_consumerlifestage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_ispatterned },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_isthermal },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_iswaterproof },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_iswearableitemdisposable },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_seasoncalendaryear },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_seasonname },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_seasonparameter },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_size },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_sportingactivitytype },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_styledescription },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_targetconsumerage },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_targetconsumergender },
    { { s_gs1, 1, 1 }, { 0, 0 }, gs1s_wearableproduct, gs1p_textilematerialcontent },

    // ical
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_action },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_attendee },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_byday },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_bymonth },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_caladdress },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_calscale },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_categories },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_class },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_cn },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_comment },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_component },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_cutype },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_date },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_datetime },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_daylight },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_description },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_dtend },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_dstamp },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_dtstart },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_duration },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_freq },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_interval },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_language },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_location },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_organiser },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_partstat },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_prodid },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_related },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_role },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_rrule },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_rsvp },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_sequence },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_standard },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_summary },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_transp },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_trigger },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_tzid },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_tzname },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_tzoffsetfrom },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_tzoffsetto },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_uid },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_valarm },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vevent, ical_version },

    { { s_ical, 1, 0 }, { 0, 0 }, ical_vtimezone, ical_tzid },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vtimezone, ical_tzname },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vtimezone, ical_tzoffsetfrom },
    { { s_ical, 1, 0 }, { 0, 0 }, ical_vtimezone, ical_tzoffsetto },

    // jsonld
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_context, jsonld_base },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_context, jsonld_definition },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_context, jsonld_direction },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_context, jsonld_import },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_context, jsonld_language },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_context, jsonld_version },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_context, jsonld_vocab },

    { { s_jsonld, 1, 0 }, { 0, 0 }, json_prefixdefinition, jsonld_id },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_prefixdefinition, jsonld_term },

    { { s_jsonld, 1, 0 }, { 0, 0 }, json_termdefinition, jsonld_container },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_termdefinition, jsonld_direction },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_termdefinition, jsonld_id },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_termdefinition, jsonld_language },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_termdefinition, jsonld_nest },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_termdefinition, jsonld_prefix },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_termdefinition, jsonld_propagate },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_termdefinition, jsonld_protected },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_termdefinition, jsonld_reverse },
    { { s_jsonld, 1, 0 }, { 0, 0 }, json_termdefinition, jsonld_term },

    // ldp
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_container, ldp_hasmemberrelation },
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_container, ldp_ismemberrelationof },
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_container, ldp_membershipresource },
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_container, ldp_insertedcontentrelation },
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_container, ldp_contains },

    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_resource, ldp_member },
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_resource, ldp_constrainedby },

    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_page, ldp_member },

    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_pagesortcriterion, ldp_pagesortpredicate },
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_pagesortcriterion, ldp_pagesortorder },
    { { s_ldp, 1, 0 }, { 0, 0 }, ldp_pagesortcriterion, ldp_pagesortcollation },

    // locn
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_addressarea },
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_addressid },
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_adminunitlevel1 },
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_adminunitlevel2 },
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_fulladdress },
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_locatordesignator },
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_locatorname },
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_pobox },
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_postname },
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_thoroughfare },
    { { s_locn, 1, 0 }, { 0, 0 }, loc_address, locn_postcode },

    // media resource
    { { s_ma, 1, 0 }, { 0, 0 }, ma_collection, ma_collectionname },

    { { s_ma, 1, 0 }, { 0, 0 }, ma_location, ma_haslocationcoordinatesystem },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_location, ma_locationaltitude },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_location, ma_locationlatitude },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_location, ma_locationlongitude },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_location, ma_locationname },

    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_alternativetitle },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_averagebitrate },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_createdin },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_copyright },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_creationdate },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_depictsfictionallocation },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_description },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_duration },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_editdate },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_features },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_frameheight },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_framerate },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_framesizeunit },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hasaccessconditions },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hasaudiodescription },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_haschapter },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hasclassification },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hascompression },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hascontributor },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hascreator },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hasformat },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hasfragment },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hasgenre },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_haskeyword },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_haslanguage },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_haspolicy },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hasrating },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hasrelatedlocation },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hasrelatedresource },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_hastargetaudience },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_iscopyrightedby },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_ismemberof },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_locator },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_numberoftracks },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_mediaresource, ma_samplingrate },

    { { s_ma, 1, 0 }, { 0, 0 }, ma_rating, ma_hasratingsystem },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_rating, ma_isprovidedby },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_rating, ma_ratingscalemax },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_rating, ma_ratingscalemin },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_rating, ma_ratingvalue },

    { { s_ma, 1, 0 }, { 0, 0 }, ma_targetaudience, ma_hasclassification },
    { { s_ma, 1, 0 }, { 0, 0 }, ma_targetaudience, ma_hasclassificationsystem },

    { { s_ma, 1, 0 }, { 0, 0 }, ma_track, ma_trackname },

    // microformats v1
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_adr, mp_country_name },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_adr, mp_extended_address, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_adr, mp_locality },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_adr, mp_post_office_box, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_adr, mp_postal_code },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_adr, mp_region },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_adr, mp_street_address, SS_REPEATABLE },

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_geo, mp_latitude }, // looks like MF only, not living standard, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_geo, mp_longitude }, // looks like MF only, not living standard, but ...

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haggregate, mp_best },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haggregate, mp_count },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haggregate, mp_item, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haggregate, mp_rating, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haggregate, mp_summary },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haggregate, mp_votes },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haggregate, mp_worst },

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_author, SS_REQUIRED | SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_bookmark, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_category, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_entry_content, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_entry_summary, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_entry_title, SS_REQUIRED | SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_feed_category },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_hentry, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_label, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_published, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_tag, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_tags },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_term, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_updated, SS_REQUIRED | SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_value, SS_REPEATABLE }, // XMDP
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hatom, mp_value_title, SS_REPEATABLE }, // XMDP

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_album },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_amount },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_category, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_contributor },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_currency },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_description },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_duration },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_enclosure },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_fn },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_h },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_item },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_min },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_payment, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_position },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_photo, SS_REPEATABLE  },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_published },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_price },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_s },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_haudio, mp_sample, SS_REPEATABLE },

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_attach },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_attendee },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_category },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_class },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_comment },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_contact },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_created },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_description },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_dtstart, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_dtend },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_duration },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_exdate },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_geo },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_last_modified },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_latitude },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_location },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_longitude },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_method }, // XMDP
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_organiser },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_partstat },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_rdate },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_role },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_resources },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_rrule },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_sequence },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_status },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_summary, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_transp },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_uid },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_value }, // XMDP
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcalendar, mp_value_title }, // XMDP

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hentry, mp_author, SS_REQUIRED | SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hentry, mp_bookmark, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hentry, mp_entry_content, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hentry, mp_entry_summary, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hentry, mp_entry_title, SS_REQUIRED | SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hentry, mp_published, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hentry, mp_tag, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hentry, mp_tags },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hentry, mp_updated, SS_REQUIRED | SS_REPEATABLE },

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_amount },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_currency },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_description, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_dtlisted },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_dtexpired },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_info, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_item },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_lister, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_listing_action },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_location },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_permalink },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_price },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_summary },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_tag },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_tags, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_type },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hlisting, mp_version },

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hmedia, mp_contributor, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hmedia, mp_enclosure, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hmedia, mp_fn, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hmedia, mp_photo },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hmedia, mp_player },

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hnews, mp_dateline },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hnews, mp_entry },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hnews, mp_geo },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hnews, mp_hentry },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hnews, mp_item_licence },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hnews, mp_principles },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hnews, mp_source_org, SS_REQUIRED },

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_availability },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_brand },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_category, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_description },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_fn, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_identifier },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_listing, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_photo, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_price },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_review, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hproduct, mp_url },

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_author, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_duration, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_fn, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_h, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_ingredient, SS_REQUIRED | SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_instructions },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_licence, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_min, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_nutrition, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_photo, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_published },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_s, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_summary },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_tag, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_tags },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_type },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_value },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hrecipe, mp_yield },

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hresume, mp_affliation, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hresume, mp_contact, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hresume, mp_education, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hresume, mp_experience, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hresume, mp_include },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hresume, mp_publications, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hresume, mp_skill, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hresume, mp_summary },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hresume, mp_value },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hresume, mp_value_title },

    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_best },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_description },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_dtreviewed },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_fn },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_item, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_licence },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_reviewer },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_permalink },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_photo },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_rating },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_self },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_summary },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_tag, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_tags },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_type },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_url },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_version },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_worst },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_value },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hreview, mp_value_title },

    // living standard / microformats
    // Note that the MF and Living Standard docs differ for a number of fields, but since the living standard
    // explicitly reference the MF definition (as http://microformats.org/profile/hcard), I've explicitly included
    // the MF fields.
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_additional_name, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_adr, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_agent, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_anniversary },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_bday },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_categories, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_category }, // looks like MF only, not living standard, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_class }, // looks like MF only, not living standard, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_country_name },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_email, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_extended_address, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_family_name, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_fn, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_gender_identity },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_geo, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_given_name, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_honourific_prefix, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_honourific_suffix, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_impp, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_key }, // looks like MF only, not living standard, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_kind },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_lang, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_latitude }, // looks like MF only, not living standard, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_locality },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_logo, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_longitude }, // looks like MF only, not living standard, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_mailer }, // looks like MF only, not living standard, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_member, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_n, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_nickname, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_note, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_org, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_organisation_name },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_organisation_unit, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_photo, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_post_office_box, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_postal_code },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_region },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_rel },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_related, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_rev, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_role, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_sex },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_sort_string },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_sound, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_street_address, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_tel, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_title, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_type, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_tz, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_uid },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_url, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_value },
    { { s_microformats, 1, 0 }, { 0, 0 }, mft_hcard, mp_value_title },

    // living standard
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_attach, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_categories, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_class },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_comment, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_contact, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_created },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_description },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_dtstart, SS_REQUIRED },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_dtend },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_duration },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_exdate, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_geo },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_last_modified },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_location },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_rdate, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_resources, SS_REPEATABLE },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_rrule },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_sequence },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_status },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_summary },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_transp },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_uid },
    { { s_microformats, 1, 0 }, { 0, 0 }, mf_vevent, mp_url },

    // microformats v2
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_altitude },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_country_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_extended_address },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_region },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_geo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_label },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_latitude },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_locality },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_longitude },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_post_office_box },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_postal_code },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hadr, mp2_p_street_address },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hbreadcrumb, mp2_h_breadcrumb },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hbreadcrumb, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hbreadcrumb, mp2_u_url },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_dt_anniversary },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_dt_bday },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_dt_rev },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_additional_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_adr },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_altitude },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_category },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_country_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_extended_address },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_family_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_region },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_gender_identity },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_geo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_given_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_honourific_prefix },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_honourific_suffix },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_job_title },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_label },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_latitude },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_locality },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_longitude },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_nickname },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_note },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_post_office_box },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_postal_code },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_org },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_organisation_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_organisation_unit },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_role },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_sex },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_sort_string },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_street_address },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_tel },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_p_tz },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_u_email },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_u_geo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_u_impp },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_u_logo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_u_key },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_u_photo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_u_sound },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_u_uid },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcard, mp2_u_url },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcite, mp2_dt_accessed },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcite, mp2_p_author },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcite, mp2_p_content },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcite, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcite, mp2_p_publication },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcite, mp2_dt_published },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcite, mp2_u_uid },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hcite, mp2_u_url },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_audio },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_author },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_bookmark_of },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_category },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_comment },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_e_content },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_duration },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_featured },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_in_reply_to },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_like },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_like_of },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_listen_of },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_latitude },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_location },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_longitude },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_photo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_dt_published },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_repost },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_repost_of },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_read_of },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_read_of },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_rsvp },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_size },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_p_summary },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_syndication },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_translation_of },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_uid },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_dt_updated },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_video },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hentry, mp2_u_watch_of },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_p_attendee },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_p_category },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_p_description },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_dt_duration },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_e_content },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_dt_end },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_p_location },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_dt_start },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_p_summary },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hevent, mp2_u_url },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hfeed, mp2_p_author },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hfeed, mp2_h_entry },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hfeed, mp2_p_entry },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hfeed, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hfeed, mp2_u_photo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hfeed, mp2_p_summary },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hfeed, mp2_u_url },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hgeo, mp2_p_altitude },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hgeo, mp2_p_latitude },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hgeo, mp2_p_longitude },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hitem, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hitem, mp2_u_photo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hitem, mp2_u_url },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hlisting, mp2_p_action },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hlisting, mp2_p_category },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hlisting, mp2_dt_expired },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hlisting, mp2_dt_listed },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hlisting, mp2_p_lister },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hlisting, mp2_p_price },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hlisting, mp2_u_url },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hproduct, mp2_p_brand },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hproduct, mp2_p_category },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hproduct, mp2_e_description },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hproduct, mp2_u_identifier },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hproduct, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hproduct, mp2_u_photo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hproduct, mp2_p_price },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hproduct, mp2_p_review },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hproduct, mp2_u_url },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_p_author },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_p_category },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_dt_duration },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_p_ingredient },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_e_instructions },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_p_nutrition },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_u_photo },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_dt_published },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_p_summary },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hrecipe, mp2_p_yield },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hresume, mp2_p_affiliation },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hresume, mp2_p_contact },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hresume, mp2_p_education },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hresume, mp2_p_experience },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hresume, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hresume, mp2_p_skill },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hresume, mp2_p_summary },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_p_author },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_p_best },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_p_category },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_e_content },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_dt_published },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_p_item },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_p_location },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_p_rating },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_p_summary },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_u_syndication },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_u_url },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_hreview, mp2_p_worst },

    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_haggregate, mp2_p_average },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_haggregate, mp2_p_best },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_haggregate, mp2_p_count },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_haggregate, mp2_p_item },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_haggregate, mp2_p_name },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_haggregate, mp2_p_votes },
    { { s_microformats, 2, 0 }, { 0, 0 }, mft2_haggregate, mp2_p_worst },

    // web annotation
    { { s_oa, 1, 0 }, { 0, 0 }, oa_annotation, oa_bodyvalue },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_annotation, oa_hasbody },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_annotation, oa_hastarget },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_annotation, oa_motivatedby },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_annotation, oa_styledby },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_rangeselector, oa_hasendselector },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_rangeselector, oa_hasstartselector },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_resourceselection, oa_hasselector },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_resourceselection, oa_hassource },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_resourceselection, oa_hasstate },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_selector, oa_refinedby },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_specificresource, oa_hasscope },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_specificresource, oa_renderedvia },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_specificresource, oa_styleclass },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_state, oa_refinedby },

    { { s_oa, 1, 0 }, { 0, 0 }, oa_timestate, oa_cachedsource },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_timestate, oa_sourcedate },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_timestate, oa_sourcedateend },
    { { s_oa, 1, 0 }, { 0, 0 }, oa_timestate, oa_sourcedatestart },

    // odrl
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrlp_implies },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrlp_includedin },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_action, odrlp_refinement },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_asset, odrlp_haspolicy },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_asset, odrlp_partof },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_asset, odrlp_uid },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_assetcollection, odrlp_refinement },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_assetcollection, odrlp_source },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_and },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_andsequence},
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_consequence },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_datatype },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_leftoperand },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_operator },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_or },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_rightoperand },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_rightoperandreference },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_status },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_unit },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_constraint, odrlp_xone },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_duty, odrlp_consequence },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_logicalconstraint, odrlp_operand },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_logicalconstraint, odrlp_uid },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_party, odrlp_assigneeof },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_party, odrlp_assignerof },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_party, odrlp_partof },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_party, odrlp_uid },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_partycollection, odrlp_refinement },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_partycollection, odrlp_source },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_permission, odrlp_duty },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_action },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_assignee },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_assigner },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_conflict },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_constraint },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_function },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_inheritfrom },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_obligation },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_permission },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_profile },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_prohibition },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_relation },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_target },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_policy, odrlp_uid },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_prohibition, odrlp_remedy },

    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrlp_action },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrlp_assignee },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrlp_assigner },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrlp_constraint },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrlp_failure },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrlp_function },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrlp_output },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrlp_relation },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrlp_target },
    { { s_odrl, 1, 0 }, { 0, 0 }, odrl_rule, odrlp_uid },

    // org
    { { s_org, 1, 0 }, { 0, 0 }, org_changeevent, orgp_originalorganisation },
    { { s_org, 1, 0 }, { 0, 0 }, org_changeevent, orgp_resultingorganisation },

    { { s_org, 1, 0 }, { 0, 0 }, org_formalorganisation, orgp_hasunit },

    { { s_org, 1, 0 }, { 0, 0 }, org_membership, orgp_member },
    { { s_org, 1, 0 }, { 0, 0 }, org_membership, orgp_memberduring },
    { { s_org, 1, 0 }, { 0, 0 }, org_membership, orgp_organisation },
    { { s_org, 1, 0 }, { 0, 0 }, org_membership, orgp_role },

    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_classification },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_changedby },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_hasmember },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_haspost },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_hasprimarysite },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_hasregisteredsite },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_hassite },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_hassuborganisation },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_identifier },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_linkedto },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_purpose },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_resultedfrom },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_suborganisationof },
    { { s_org, 1, 0 }, { 0, 0 }, org_organisation, orgp_transitivesuborganisationof },

    { { s_org, 1, 0 }, { 0, 0 }, org_organisationalunit, orgp_unitof },

    { { s_org, 1, 0 }, { 0, 0 }, org_post, orgp_heldby },
    { { s_org, 1, 0 }, { 0, 0 }, org_post, orgp_postin },
    { { s_org, 1, 0 }, { 0, 0 }, org_post, orgp_reportsto },
    { { s_org, 1, 0 }, { 0, 0 }, org_post, orgp_role },

    { { s_org, 1, 0 }, { 0, 0 }, org_role, orgp_roleproperty },
    { { s_org, 1, 0 }, { 0, 0 }, org_role, orgp_headof },
    { { s_org, 1, 0 }, { 0, 0 }, org_role, orgp_remuneration },

    { { s_org, 1, 0 }, { 0, 0 }, org_site, orgp_siteaddress },
    { { s_org, 1, 0 }, { 0, 0 }, org_site, orgp_siteof },

    // owl
    { { s_owl, 1, 0 }, { 0, 0 }, owl_alldifferent, owl_distinctmembers },

    { { s_owl, 1, 0 }, { 0, 0 }, owl_class, owl_complementof },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_class, owl_disjointunionof },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_class, owl_disjointwith },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_class, owl_haskey },

    { { s_owl, 1, 0 }, { 0, 0 }, owl_negativepropertyassertion, owl_sourceindividual },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_negativepropertyassertion, owl_targetindividual },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_negativepropertyassertion, owl_targetvalue },

    { { s_owl, 1, 0 }, { 0, 0 }, owl_objectproperty, owl_inverseof },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_objectproperty, owl_propertychainaxiom },

    { { s_owl, 1, 0 }, { 0, 0 }, owl_ontology, owl_backwardcompatiblewith },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_ontology, owl_imports },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_ontology, owl_incompatiblewith },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_ontology, owl_priorversion },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_ontology, owl_versioniri },
    { { s_vann, 1, 0 }, { s_vann, 1, 0 }, owl_ontology, vann_changes },
    { { s_vann, 1, 0 }, { s_vann, 1, 0 }, owl_ontology, vann_example },
    { { s_vann, 1, 0 }, { s_vann, 1, 0 }, owl_ontology, vann_pnp },
    { { s_vann, 1, 0 }, { s_vann, 1, 0 }, owl_ontology, vann_pnu },
    { { s_vann, 1, 0 }, { s_vann, 1, 0 }, owl_ontology, vann_usagenote },

    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_allvaluesfrom },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_cardinality },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_maxcardinality },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_maxqualifiedcardinality },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_mincardinality },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_minqualifiedcardinality },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_onclass },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_ondatarange },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_onproperties },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_onproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_qualifiedcardinality },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_restriction, owl_somevaluesfrom },

    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_activityideas },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_allergeninfo },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_thing, owl_bottomdataproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_thing, owl_bottomobjectproperty },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_brandhomepageclinical },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_brandhomepagepatient },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_careersinfo },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_certificationinfo },
    { { s_gs1, 1, 5 }, { 0, 0 }, owl_thing, gs1p_defaultlink },
    { { s_gs1, 1, 5 }, { 0, 0 }, owl_thing, gs1p_defaultlinkmulti },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_thing, owl_differentfrom },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_epcis },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_epil },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_eventsinfo },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_faqs },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_handledby },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_hasretailers },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_homepage },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_ingredientsinfo },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_instructions },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_leavereview },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_linktype },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_locationinfo },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_logisticsinfo },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_masterdata },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_nutritionalinfo },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_openinghoursinfo },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_paymentlink },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_promotion },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_pip },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_purchasesuppliesoraccessories },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_quickstartguide },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_recallstatus },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_recipeinfo },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_registerproduct },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_registryentry },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_relatedvideo },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_review },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_safetyinfo },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_thing, owl_sameas },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_scheduletime },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_serviceinfo },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_smartlabel },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_smpc },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_socialmedia },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_statisticinfo },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_support },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_sustainabilityinfo },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_thing, owl_topdataproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, owl_thing, owl_topobjectproperty },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_traceability },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_tutorial },
    { { s_gs1, 1, 51 }, { 0, 0 }, owl_thing, gs1p_useragreement },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_verificationservice },
    { { s_gs1, 1, 3 }, { 0, 0 }, owl_thing, gs1p_whatsinthebox },

    // provincial
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_atlocation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_endedattime },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_generated },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_invalidated },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_qualifiedassociation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_qualifiedcommunication },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_qualifiedend },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_qualifiedinfluence },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_qualifiedstart },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_qualifiedusage },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_startedattime },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_used },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_wasassociatedwith },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_wasendedby },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_wasinfluencedby },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_wasinformedby },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activity, pp_wasstartedby },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_activityinfluence, pp_activity },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activityinfluence, pp_hadactivity },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activityinfluence, pp_hadrole },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_activityinfluence, pp_influencer },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_agent, pp_actedonbehalfof },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agent, pp_atlocation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agent, pp_qualifieddelegation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agent, pp_qualifiedinfluence },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agent, pp_wasinfluencedby },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_agentinfluence, pp_agent },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agentinfluence, pp_hadactivity },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agentinfluence, pp_hadrole },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_agentinfluence, pp_influencer },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_association, pp_agent },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_association, pp_hadplan },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_association, pp_hadrole },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_attribution, pp_agent },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_collection, pp_hadmember },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_communication, pp_activity },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_delegation, pp_agent },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_delegation, pp_hadactivity },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_derivation, pp_entity },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_derivation, pp_hadactivity },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_derivation, pp_hadgeneration },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_derivation, pp_hadusage },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_emptycollection, pp_hadmember },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_end, pp_attime },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_end, pp_entity },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_end, pp_hadactivity },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_alternativeto },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_atlocation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_generatedattime },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_hadprimarysource },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_invalidatedattime },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_qualifiedattribution },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_qualifiedderivation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_qualifiedgeneration },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_qualifiedinfluence },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_qualifiedinvalidation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_qualifiedprimarysource },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_qualifiedquotation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_qualifiedrevision },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_specialisationof },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_value },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_wasattributedto },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_wasderivedfrom },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_wasgeneratedby },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_wasinfluencedby },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_wasinvalidatedby },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_wasquotedfrom },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_entity, pp_wasrevisionof },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_entityinfluence, pp_entity },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_generation, pp_activity },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_generation, pp_attime },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_influence, pp_hadactivity },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_influence, pp_hadrole },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_influence, pp_influencer },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_instantaneousevent, pp_atlocation },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_instantaneousevent, pp_attime },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_instantaneousevent, pp_hadrole },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_invalidation, pp_activity },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_invalidation, pp_attime },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_organisation, pp_actedonbehalfof },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_organisation, pp_qualifieddelegation },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_person, pp_actedonbehalfof },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_person, pp_qualifieddelegation },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_primarysource, pp_hadgeneration },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_primarysource, pp_hadusage },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_quotation, pp_hadgeneration },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_quotation, pp_hadusage },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_revision, pp_hadgeneration },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_revision, pp_hadusage },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_softwareagent, pp_actedonbehalfof },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_softwareagent, pp_qualifieddelegation },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_start, pp_attime },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_start, pp_entity },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_start, pp_hadactivity },

    { { s_prov, 1, 0 }, { 0, 0 }, prov_usage, pp_attime },
    { { s_prov, 1, 0 }, { 0, 0 }, prov_usage, pp_entity },

    // ptr
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_content, ptr_characterencoding },

    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_contentasbase64, ptr_bytes },

    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_contentastext, ptr_chars },

    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_contentasxml, ptr_declaredencoding },
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_contentasxml, ptr_dtdecl },
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_contentasxml, ptr_leadingmisc },
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_contentasxml, ptr_rest },
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_contentasxml, ptr_standalone },
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_contentasxml, ptr_version },

    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_doctypedecl, ptr_doctypename },
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_doctypedecl, ptr_internalsubset },
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_doctypedecl, ptr_publicid },
    { { s_ptr, 1, 0 }, { 0, 0 }, ptr_doctypedecl, ptr_systemid },

    // data cube
    { { s_qb, 1, 0 }, { 0, 0 }, anything, qbp_observationgroup },
    { { s_qb, 1, 0 }, { 0, 0 }, anything, qbp_measuretype },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentproperty, qbp_concept },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentset, qbp_attribute },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentset, qbp_componentproperty },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentset, qbp_dimension },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentset, qbp_measure },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentset, qbp_measuredimension },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentspecification, qbp_order },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentspecification, qbp_componentattachment },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_componentspecification, qbp_componentrequired },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_codedproperty, qbp_codelist },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_datastructuredefinition, qbp_component },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_datastructuredefinition, qbp_slicekey },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_dataset, qbp_slice },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_dataset, qbp_structure },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_hierarchicalcodelist, qbp_hierarchyroot },
    { { s_qb, 1, 0 }, { 0, 0 }, qb_hierarchicalcodelist, qbp_parentchildproperty },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_observation, qbp_dataset },
    { { s_dqv, 1, 0 }, { 0, 0 }, qb_observation, dqv_ismeasurementof },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_observationgroup, qbp_observation },

    { { s_qb, 1, 0 }, { 0, 0 }, qb_slice, qbp_slicestructure },

    // RDF / RDFs
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_class, owl_equivalentclass },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_class, owl_oneof },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_class, rdfs_subclassof },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdfs_class, rdf_type },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_class, owl_unionof },

    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_datatype, owl_datatypecomplementof },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_datatype, owl_ondatatype },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_datatype, owl_withrestrictions },

    { { s_rdf, 1, 0 }, { 0, 0 }, rdf_list, rdf_rest },
    { { s_owl, 1, 0 }, { 0, 0 }, rdf_list, owl_intersectionof },

    { { s_rdfs, 1, 0 }, { 0, 0 }, rdf_property, rdfs_domain },
    { { s_owl, 1, 0 }, { 0, 0 }, rdf_property, owl_assertionproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, rdf_property, owl_equivalentproperty },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdf_property, rdfs_range },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdf_property, rdfs_subpropertyof },

    { { s_bibo, 1, 3, }, { 0, 0 }, rdfs_resource, bibo_abstract },
    { { s_locn, 1, 0 }, { 0, 0 }, rdfs_resource, locn_address },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_annotatedproperty },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_annotatedsource },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_annotatedtarget },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdfs_comment },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_deprecated },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdfs_resource, rdf_first },
    { { s_locn, 1, 0 }, { 0, 0 }, rdfs_resource, locn_geometry },
    { { s_wdrs, 1, 0 }, { 0, 0 }, rdfs_resource, wdrs_hasiri },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_hasself },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_hasvalue },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdfs_isdefinedby },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdfs_label },
    { { s_locn, 1, 0 }, { 0, 0 }, rdfs_resource, locn_location },
    { { s_wdrs, 1, 0 }, { 0, 0 }, rdfs_resource, wdrs_matchesregex },
    { { s_wdrs, 1, 0 }, { 0, 0 }, rdfs_resource, wdrs_notmatchesregex },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdfs_member },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_members },
    { { s_skos, 1, 0 }, { 0, 0 }, rdfs_resource, skos_notation },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdfs_resource, rdf_object },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdfs_resource, rdf_predicate },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_propertydisjointwith },
    { { s_rdfs, 1, 0 }, { 0, 0 }, rdfs_resource, rdfs_seealso },
    { { s_vann, 1, 0 }, { 0, 0 }, rdfs_resource, vann_changes },
    { { s_vann, 1, 0 }, { 0, 0 }, rdfs_resource, vann_example },
    { { s_vann, 1, 0 }, { 0, 0 }, rdfs_resource, vann_usagenote },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdfs_resource, rdf_subject },
    { { s_rdf, 1, 0 }, { 0, 0 }, rdfs_resource, rdf_value },
    { { s_owl, 1, 0 }, { 0, 0 }, rdfs_resource, owl_versioninfo },

    // RDF Reviews
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_commenter },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_hascomment },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_hasfeedback },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_hasreview },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_maxrating },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_minrating },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_positivevotes },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_rating },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_reviewer },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_text },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_totalvotes },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_comment, rev_type },

    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_commenter },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_hascomment },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_hasfeedback },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_hasreview },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_maxrating },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_minrating },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_positivevotes },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_rating },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_reviewer },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_text },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_totalvotes },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_feedback, rev_type },

    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_commenter },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_hascomment },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_hasfeedback },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_hasreview },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_maxrating },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_minrating },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_positivevotes },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_rating },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_reviewer },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_text },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_totalvotes },
    { { s_rev, 1, 0 }, { 0, 0 }, rev_review, rev_type },

    // rdb to rdf
    { { s_rr, 1, 0 }, { 0, 0 }, rr_basetableorview, rrp_tablename },

    { { s_rr, 1, 0 }, { 0, 0 }, rr_join, rrp_child },
    { { s_rr, 1, 0 }, { 0, 0 }, rr_join, rrp_parent },

    { { s_rr, 1, 0 }, { 0, 0 }, rr_objectmap, rrp_datatype },
    { { s_rr, 1, 0 }, { 0, 0 }, rr_objectmap, rrp_language },

    { { s_rr, 1, 0 }, { 0, 0 }, rr_predicateobjectmap, rrp_objectmap },
    { { s_rr, 1, 0 }, { 0, 0 }, rr_predicateobjectmap, rrp_predicatemap },

    { { s_rr, 1, 0 }, { 0, 0 }, rr_r2rmlview, rrp_sqlquery },
    { { s_rr, 1, 0 }, { 0, 0 }, rr_r2rmlview, rrp_sqlversion },

    { { s_rr, 1, 0 }, { 0, 0 }, rr_refobjectmap, rrp_joincondition },
    { { s_rr, 1, 0 }, { 0, 0 }, rr_refobjectmap, rrp_parenttriplesmap },

    { { s_rr, 1, 0 }, { 0, 0 }, rr_subjectmap, rrp_class },

    { { s_rr, 1, 0 }, { 0, 0 }, rr_termmap, rrp_column },
    { { s_rr, 1, 0 }, { 0, 0 }, rr_termmap, rrp_inverseexpression },
    { { s_rr, 1, 0 }, { 0, 0 }, rr_termmap, rrp_template },
    { { s_rr, 1, 0 }, { 0, 0 }, rr_termmap, rrp_termtype },

    { { s_rr, 1, 0 }, { 0, 0 }, rr_triplesmap, rrp_logicaltable },
    { { s_rr, 1, 0 }, { 0, 0 }, rr_triplesmap, rrp_subjectmap },

    // schema.org
    { { 6, 0 }, { 0, 0 }, sch_3dmodel, sp_isresizable },

    { { 5, 0 }, { 0, 0 }, sch_accommodation, sp_accommodationcategory },
    { { 6, 0 }, { 0, 0 }, sch_accommodation, sp_accommodationfloorplan },
    { { 3, 1 }, { 0, 0 }, sch_accommodation, sp_amenityfeature },
    { { 5, 0 }, { 0, 0 }, sch_accommodation, sp_floorlevel },
    { { 3, 1 }, { 0, 0 }, sch_accommodation, sp_floorsize },
    { { 5, 0 }, { 0, 0 }, sch_accommodation, sp_leaselength },
    { { 5, 0 }, { 0, 0 }, sch_accommodation, sp_numberofbathroomstotal },
    { { 7, 0 }, { 0, 0 }, sch_accommodation, sp_numberofbedrooms },
    { { 5, 0 }, { 0, 0 }, sch_accommodation, sp_numberoffullbathrooms },
    { { 6, 0 }, { 0, 0 }, sch_accommodation, sp_numberofpartialbathrooms },
    { { 3, 1 }, { 0, 0 }, sch_accommodation, sp_numberofrooms },
    { { 3, 1 }, { 0, 0 }, sch_accommodation, sp_permittedusage },
    { { 3, 1 }, { 0, 0 }, sch_accommodation, sp_petsallowed },
    { { 7, 0 }, { 0, 0 }, sch_accommodation, sp_tourbookingpage },
    { { 6, 0 }, { 0, 0 }, sch_accommodation, sp_yearbuilt },

    { { 1, 20 }, { 0, 0 }, sch_action, sp_actionstatus },
    { { 1, 2 }, { 0, 0 }, sch_action, sp_agent },
    { { 1, 2 }, { 0, 0 }, sch_action, sp_endtime },
    { { 1, 91 }, { 0, 0 }, sch_action, sp_error },
    { { 1, 2 }, { 0, 0 }, sch_action, sp_instrument },
    { { 1, 2 }, { 0, 0 }, sch_action, sp_location },
    { { 1, 2 }, { 0, 0 }, sch_action, sp_object },
    { { 1, 2 }, { 0, 0 }, sch_action, sp_participant },
    { { 14, 0 }, { 0, 0 }, sch_action, sp_provider },
    { { 1, 2 }, { 0, 0 }, sch_action, sp_result },
    { { 1, 2 }, { 0, 0 }, sch_action, sp_starttime },
    { { 1, 91 }, { 0, 0 }, sch_action, sp_target },

    { { 3, 5 }, { 0, 0 }, sch_actionaccessspecification, sp_availabilityends },
    { { 3, 5 }, { 0, 0 }, sch_actionaccessspecification, sp_availabilitystarts },
    { { 3, 5 }, { 0, 0 }, sch_actionaccessspecification, sp_category },
    { { 3, 5 }, { 0, 0 }, sch_actionaccessspecification, sp_eligibleregion },
    { { 3, 5 }, { 0, 0 }, sch_actionaccessspecification, sp_expectsacceptanceof },
    { { 3, 9 }, { 0, 0 }, sch_actionaccessspecification, sp_ineligibleregion },
    { { 3, 5 }, { 0, 0 }, sch_actionaccessspecification, sp_requiressubscription },

    { { 0, 10 }, { 0, 0 }, sch_aggregateoffer, sp_highprice },
    { { 0, 10 }, { 0, 0 }, sch_aggregateoffer, sp_lowprice },
    { { 0, 10 }, { 0, 0 }, sch_aggregateoffer, sp_offercount },
    { { 2, 0 }, { 0, 0 }, sch_aggregateoffer, sp_offers },

    { { 0, 10 }, { 0, 0 }, sch_aggregaterating, sp_itemreviewed },
    { { 0, 10 }, { 0, 0 }, sch_aggregaterating, sp_ratingcount },
    { { 0, 10 }, { 0, 0 }, sch_aggregaterating, sp_reviewcount },

    { { 2, 0 }, { 0, 0 }, sch_airline, sp_boardingpolicy },
    { { 1, 20 }, { 0, 0 }, sch_airline, sp_iatacode },

    { { 1, 20 }, { 0, 0 }, sch_airport, sp_iatacode },
    { { 2, 0 }, { 0, 0 }, sch_airport, sp_ocaocode },

    { { 1, 0 }, { 0, 0 }, sch_alignmentobject, sp_alignmenttype },
    { { 1, 0 }, { 0, 0 }, sch_alignmentobject, sp_educationalframework },
    { { 1, 0 }, { 0, 0 }, sch_alignmentobject, sp_targetdescription },
    { { 1, 0 }, { 0, 0 }, sch_alignmentobject, sp_targetname },
    { { 1, 0 }, { 0, 0 }, sch_alignmentobject, sp_targeturl },

    { { 1, 2 }, { 6, 0 }, sch_allocateaction, sp_purpose },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalstructure, sp_associatedpathophysiology },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalstructure, sp_bodylocation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalstructure, sp_connectedto },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalstructure, sp_diagram },
    { { 0, 95, SV_NOT_3034 }, { 6, 0 }, sch_anatomicalstructure, sp_function },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalstructure, sp_partofsystem },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalstructure, sp_relatedcondition },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalstructure, sp_relatedtherapy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalstructure, sp_substructure },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalsystem, sp_associatedpathophysiology },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalsystem, sp_comprisedof },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalsystem, sp_relatedcondition },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalsystem, sp_relatedstructure },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_anatomicalsystem, sp_relatedtherapy },

    { { 1, 20 }, { 1, 93 }, sch_answer, sp_downvotecount },
    { { 1, 20 }, { 1, 93 }, sch_answer, sp_parentitem },
    { { 1, 20 }, { 1, 93 }, sch_answer, sp_upvotecount },

    { { 3, 1 }, { 0, 0 }, sch_apartment, sp_numberofrooms },
    { { 3, 1 }, { 0, 0 }, sch_apartment, sp_occupancy },

    { { 6, 0 }, { 0, 0 }, sch_apartmentcomplex, sp_numberofaccommodationunits },
    { { 6, 0 }, { 0, 0 }, sch_apartmentcomplex, sp_numberofavailableaccommodationunits },
    { { 7, 0 }, { 0, 0 }, sch_apartmentcomplex, sp_numberofbedrooms },
    { { 7, 0 }, { 0, 0 }, sch_apartmentcomplex, sp_petsallowed },
    { { 7, 0 }, { 0, 0 }, sch_apartmentcomplex, sp_tourbookingpage },

    { { 1, 0 }, { 0, 0 }, sch_apireference, sp_assemblyversion },
    { { 1, 0 }, { 0, 0 }, sch_apireference, sp_executablelibraryname },
    { { 1, 0 }, { 0, 0 }, sch_apireference, sp_programmingmodel },
    { { 1, 0 }, { 0, 0 }, sch_apireference, sp_targetplatform },

    { { 3, 5 }, { 0, 0 }, sch_archivecomponent, sp_holdingarchive },
    { { 3, 5 }, { 0, 0 }, sch_archivecomponent, sp_itemlocation },

    { { 3, 5 }, { 0, 0 }, sch_archiveorganisation, sp_archiveheld },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_artery, sp_arterialbranch },
    { { 0, 95, SV_NOT_3034 }, { 6, 0 }, sch_artery, sp_source },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_artery, sp_supplyto },

    { { 0, 10 }, { 0, 0 }, sch_article, sp_articlebody },
    { { 0, 10 }, { 0, 0 }, sch_article, sp_articlesection },
    { { 3, 5 }, { 0, 0 }, sch_article, sp_backstory },
    { { 1, 90 }, { 0, 0 }, sch_article, sp_pageend },
    { { 1, 90 }, { 0, 0 }, sch_article, sp_pagestart },
    { { 1, 90 }, { 0, 0 }, sch_article, sp_pagination },
    { { 3, 5 }, { 0, 0 }, sch_article, sp_speakable },
    { { 0, 30 }, { 0, 0 }, sch_article, sp_wordcount },

    { { 2, 0 }, { 0, 0 }, sch_askaction, sp_question },

    { { 1, 3 }, { 0, 0 }, sch_audience, sp_audiencetype },
    { { 1, 3 }, { 0, 0 }, sch_audience, sp_geographicarea },

    { { 3, 5 }, { 0, 0 }, sch_audiobook, sp_duration },
    { { 3, 5 }, { 0, 0 }, sch_audiobook, sp_readby },

    { { 3, 5 }, { 0, 0 }, sch_audioobject, sp_caption },
    { { 13, 0 }, { 0, 0 }, sch_audioobject, sp_embeddedtextcaption },
    { { 0, 30 }, { 0, 0 }, sch_audioobject, sp_transcript },

    { { 1, 2 }, { 0, 0 }, sch_authoriseaction, sp_recipient },

    { { 3, 5 }, { 0, 0 }, sch_bankaccount, sp_accountminimuminflow },
    { { 3, 5 }, { 0, 0 }, sch_bankaccount, sp_accountoverdraftlimit },
    { { 3, 5 }, { 0, 0 }, sch_bankaccount, sp_bankaccounttype },

    { { 3, 1 }, { 0, 0 }, sch_beddetails, sp_numberofbeds },
    { { 3, 1 }, { 0, 0 }, sch_beddetails, sp_typeofbed },

    { { 3, 5 }, { 0, 0 }, sch_bedtype, sp_bed },
    { { 3, 5 }, { 0, 0 }, sch_bedtype, sp_typeofbed },

    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_associateddisease },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_biocheminteraction },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_biochemsimilarity },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_biologicalrole },
    { { 14, 0 }, { 0, 0 }, sch_biochementity, sp_funding },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_hasbiochementitypart },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_hasmolecularfunction },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_hasrepresentation },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_isencodedbybiochementity },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_isinvolvedinbiologicalprocess },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_islocatedinsubcellularlocation },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_ispartofbiochementity },
    { { 13, 0 }, { 0, 0 }, sch_biochementity, sp_taxonomicrange },

    { { 0, 91 }, { 0, 0 }, sch_blog, sp_blogpost },
    { { 0, 10 }, { 1, 90 }, sch_blog, sp_blogposts },
    { { 3, 5 }, { 0, 0 }, sch_blog, sp_issn },

    { { 9, 0 }, { 0, 0 }, sch_boattrip, sp_arrivalboatterminal },
    { { 9, 0 }, { 0, 0 }, sch_boattrip, sp_departureboatterminal },

    { { 3, 5 }, { 0, 0 }, sch_book, sp_abridged },
    { { 0, 10 }, { 0, 0 }, sch_book, sp_bookedition },
    { { 0, 10 }, { 0, 0 }, sch_book, sp_bookformat },
    { { 0, 10 }, { 0, 0 }, sch_book, sp_illustrator },
    { { 0, 10 }, { 0, 0 }, sch_book, sp_isbn },
    { { 0, 10 }, { 0, 0 }, sch_book, sp_numberofpages },

    { { 1, 2 }, { 0, 0 }, sch_borrowaction, sp_lender },

    { { 2, 0 }, { 0, 0 }, sch_brand, sp_aggregaterating },
    { { 0, 99 }, { 0, 0 }, sch_brand, sp_logo },
    { { 2, 0 }, { 0, 0 }, sch_brand, sp_review },
    { { 3, 5 }, { 0, 0 }, sch_brand, sp_slogan },

    { { 2, 0 }, { 0, 0 }, sch_broadcastchannel, sp_broadcastchannelid },
    { { 3, 5 }, { 0, 0 }, sch_broadcastchannel, sp_broadcastfrequency },
    { { 2, 0 }, { 0, 0 }, sch_broadcastchannel, sp_broadcastservicetier },
    { { 3, 2 }, { 0, 0 }, sch_broadcastchannel, sp_genre },
    { { 2, 0 }, { 0, 0 }, sch_broadcastchannel, sp_inbroadcastlineup },
    { { 2, 0 }, { 0, 0 }, sch_broadcastchannel, sp_providesbroadcastservice },

    { { 2, 2 }, { 0, 0 }, sch_broadcastevent, sp_broadcastofevent },
    { { 2, 0 }, { 0, 0 }, sch_broadcastevent, sp_islivebroadcast },
    { { 4, 0 }, { 0, 0 }, sch_broadcastevent, sp_subtitlelanguage },
    { { 2, 1, SV_NOT_30 }, { 0, 0 }, sch_broadcastevent, sp_videoformat },

    { { 3, 5 }, { 0, 0 }, sch_broadcastfrequencyspecification, sp_broadcastfrequencyvalue },
    { { 3, 5 }, { 0, 0 }, sch_broadcastfrequencyspecification, sp_broadcastsignalmodulation },
    { { 3, 5 }, { 0, 0 }, sch_broadcastfrequencyspecification, sp_broadcastsubchannel },

    { { 1, 3 }, { 1, 93 }, sch_broadcastservice, sp_area },
    { { 2, 0 }, { 0, 0 }, sch_broadcastservice, sp_broadcastaffiliateof },
    { { 2, 0 }, { 0, 0 }, sch_broadcastservice, sp_broadcastdisplayname },
    { { 2, 0 }, { 0, 0 }, sch_broadcastservice, sp_broadcasttimezone },
    { { 1, 3 }, { 0, 0 }, sch_broadcastservice, sp_broadcaster },
    { { 5, 0 }, { 0, 0 }, sch_broadcastservice, sp_callsign },
    { { 3, 5 }, { 0, 0 }, sch_broadcastservice, sp_hasbroadcastchannel },
    { { 4, 0 }, { 0, 0 }, sch_broadcastservice, sp_inlanguage },
    { { 1, 3 }, { 0, 0 }, sch_broadcastservice, sp_parentservice },
    { { 2, 1, SV_NOT_30 }, { 0, 0 }, sch_broadcastservice, sp_videoformat },

    { { 3, 5 }, { 0, 0 }, sch_busorcoach, sp_acrisscode },
    { { 3, 5 }, { 0, 0 }, sch_busorcoach, sp_roofload },

    { { 1, 20 }, { 0, 0 }, sch_bustrip, sp_arrivalbusstop },
    { { 1, 20 }, { 3, 3 }, sch_bustrip, sp_arrivaltime },
    { { 1, 20 }, { 0, 0 }, sch_bustrip, sp_busname },
    { { 1, 20 }, { 0, 0 }, sch_bustrip, sp_busnumber },
    { { 1, 20 }, { 0, 0 }, sch_bustrip, sp_departurebusstop },
    { { 1, 20 }, { 3, 3 }, sch_bustrip, sp_departuretime },
    { { 1, 20 }, { 3, 3 }, sch_bustrip, sp_provider },

    { { 1, 3 }, { 0, 0 }, sch_businessaudience, sp_numberofemployees },
    { { 1, 3 }, { 0, 0 }, sch_businessaudience, sp_yearlyrevenue },
    { { 1, 3 }, { 0, 0 }, sch_businessaudience, sp_yearsinoperation },

    { { 1, 2 }, { 1, 93 }, sch_buyaction, sp_vendor },
    { { 1, 2 }, { 1, 93 }, sch_buyaction, sp_warrantypromise },
    { { 2, 0 }, { 0, 0 }, sch_buyaction, sp_seller },

    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdcollectiondate },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdfacilitycounty },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdfacilityid },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumbeds },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumbedsocc },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumc19died },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumc19hopats },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumc19hosppats },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumc19mechventpats },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumc19ofmechventpats },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumc19overflowpats },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumicubeds },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumicubedsocc },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumtotbeds },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumvent },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_cvdnumventuse },
    { { 7, 3 }, { 0, 0 }, sch_cdcpmdrecord, sp_dateposted },

    { { 3, 5 }, { 0, 0 }, sch_car, sp_acrisscode },
    { { 3, 5 }, { 0, 0 }, sch_car, sp_roofload },

    { { 3, 5 }, { 0, 0 }, sch_categorycode, sp_codevalue },
    { { 3, 5 }, { 0, 0 }, sch_categorycode, sp_incodeset },

    { { 3, 5 }, { 0, 0 }, sch_categorycodeset, sp_hascategorycode },

    { { 3, 5 }, { 0, 0 }, sch_chapter, sp_pageend },
    { { 3, 5 }, { 0, 0 }, sch_chapter, sp_pagestart },
    { { 3, 5 }, { 0, 0 }, sch_chapter, sp_pagination },

    { { 13, 0 }, { 0, 0 }, sch_chemicalsubstance, sp_chemicalcomposition },
    { { 13, 0 }, { 0, 0 }, sch_chemicalsubstance, sp_chemicalrole },
    { { 13, 0 }, { 0, 0 }, sch_chemicalsubstance, sp_potentialuse },

    { { 1, 2 }, { 0, 0 }, sch_chooseaction, sp_actionoption },

    { { 0, 10 }, { 0, 0 }, sch_civicstructure, sp_openinghours },

    { { 3, 5 }, { 0, 0 }, sch_claim, sp_appearance },
    { { 13, 0 }, { 0, 0 }, sch_claim, sp_claiminterpreter },
    { { 3, 5 }, { 0, 0 }, sch_claim, sp_firstappearance },

    { { 3, 2 }, { 0, 0 }, sch_claimreview, sp_claimreviewed },

    { { 2, 0, SV_NOT_3034 }, { 0, 0 }, sch_class, sp_supersededby },

    { { 2, 0 }, { 0, 0 }, sch_clip, sp_actor },
    { { 1, 3 }, { 0, 0 }, sch_clip, sp_clipnumber },
    { { 2, 0 }, { 0, 0 }, sch_clip, sp_director },
    { { 3, 5 }, { 0, 0 }, sch_clip, sp_endoffset },
    { { 2, 0 }, { 0, 0 }, sch_clip, sp_musicby },
    { { 1, 3 }, { 0, 0 }, sch_clip, sp_partofepisode },
    { { 1, 3 }, { 0, 0 }, sch_clip, sp_partofseason },
    { { 1, 3 }, { 0, 0 }, sch_clip, sp_partofseries },
    { { 1, 3 }, { 1, 93 }, sch_clip, sp_position },
    { { 1, 3 }, { 1, 93 }, sch_clip, sp_publication },
    { { 3, 5 }, { 0, 0 }, sch_clip, sp_startoffset },

    { { 1, 0 }, { 1, 93 }, sch_code, sp_coderepository },
    { { 1, 0 }, { 1, 93 }, sch_code, sp_programminglanguage },
    { { 1, 0 }, { 1, 93 }, sch_code, sp_runtime },
    { { 1, 0 }, { 1, 93 }, sch_code, sp_sampletype },
    { { 1, 0 }, { 1, 93 }, sch_code, sp_targetproduct },

    { { 3, 5 }, { 0, 0 }, sch_collection, sp_collectionsize },

    { { 3, 5 }, { 0, 0 }, sch_comicissue, sp_artist },
    { { 3, 5 }, { 0, 0 }, sch_comicissue, sp_colourist },
    { { 3, 5 }, { 0, 0 }, sch_comicissue, sp_inker },
    { { 3, 5 }, { 0, 0 }, sch_comicissue, sp_letterer },
    { { 3, 5 }, { 0, 0 }, sch_comicissue, sp_penciler },
    { { 3, 5 }, { 0, 0 }, sch_comicissue, sp_variantcover },

    { { 3, 5 }, { 0, 0 }, sch_comicstory, sp_artist },
    { { 3, 5 }, { 0, 0 }, sch_comicstory, sp_colourist },
    { { 3, 5 }, { 0, 0 }, sch_comicstory, sp_inker },
    { { 3, 5 }, { 0, 0 }, sch_comicstory, sp_letterer },
    { { 3, 5 }, { 0, 0 }, sch_comicstory, sp_penciler },

    { { 1, 20 }, { 0, 0 }, sch_comment, sp_downvotecount },
    { { 1, 20 }, { 0, 0 }, sch_comment, sp_parentitem },
    { { 1, 20 }, { 0, 0 }, sch_comment, sp_upvotecount },

    { { 2, 0 }, { 0, 0 }, sch_commentaction, sp_resultcomment },

    { { 1, 2 }, { 0, 0 }, sch_communicateaction, sp_about },
    { { 1, 2 }, { 1, 93 }, sch_communicateaction, sp_language },
    { { 2, 0 }, { 0, 0 }, sch_communicateaction, sp_inlanguage },
    { { 1, 2 }, { 0, 0 }, sch_communicateaction, sp_recipient },

    { { 3, 0 }, { 0, 0 }, sch_compoundpricespecification, sp_pricecomponent },
    { { 11, 0 }, { 0, 0 }, sch_compoundpricespecification, sp_pricetype },

    { { 3, 5 }, { 0, 0 }, sch_consumeaction, sp_actionaccessibilityrequirement },
    { { 2, 0 }, { 0, 0 }, sch_consumeaction, sp_expectsacceptanceof },

    { { 1, 3 }, { 0, 0 }, sch_contactpoint, sp_areaserved },
    { { 1, 3 }, { 0, 0 }, sch_contactpoint, sp_availablelanguage },
    { { 1, 3 }, { 0, 0 }, sch_contactpoint, sp_contactoption },
    { { 0, 10 }, { 0, 0 }, sch_contactpoint, sp_contacttype },
    { { 0, 10 }, { 0, 0 }, sch_contactpoint, sp_email },
    { { 0, 10 }, { 0, 0 }, sch_contactpoint, sp_faxnumber },
    { { 1, 3 }, { 0, 0 }, sch_contactpoint, sp_hoursavailable },
    { { 1, 3 }, { 0, 0 }, sch_contactpoint, sp_productsupported },
    { { 0, 10 }, { 0, 0 }, sch_contactpoint, sp_telephone },

    { { 1, 2 }, { 0, 0 }, sch_cookaction, sp_foodestablishment },
    { { 1, 2 }, { 0, 0 }, sch_cookaction, sp_foodevent },
    { { 1, 2 }, { 0, 0 }, sch_cookaction, sp_recipe },

    { { 0, 30 }, { 0, 0 }, sch_corporation, sp_tickersymbol },

    { { 3, 4 }, { 0, 0 }, sch_course, sp_coursecode },
    { { 3, 4 }, { 0, 0 }, sch_course, sp_coursepresequisites },
    { { 3, 5 }, { 0, 0 }, sch_course, sp_educationcredentialawarded },
    { { 3, 4 }, { 0, 0 }, sch_course, sp_hascourseinstance },
    { { 6, 0 }, { 0, 0 }, sch_course, sp_numberofcredits },
    { { 3, 9 }, { 0, 0 }, sch_course, sp_occupationalcredentialawarded },

    { { 3, 4 }, { 0, 0 }, sch_courseinstance, sp_coursemode },
    { { 3, 6 }, { 0, 0 }, sch_courseinstance, sp_courseworkload },
    { { 3, 4 }, { 0, 0 }, sch_courseinstance, sp_instructor },

    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_about },
    { { 4, 0 }, { 0, 0 }, sch_creativework, sp_abstract },
    { { 3, 0 }, { 0, 0 }, sch_creativework, sp_accessmode },
    { { 3, 0 }, { 0, 0 }, sch_creativework, sp_accessmodesufficient },
    { { 1, 4 }, { 0, 0 }, sch_creativework, sp_accessibilityapi },
    { { 1, 4 }, { 0, 0 }, sch_creativework, sp_accessibilitycontrol },
    { { 1, 4 }, { 0, 0 }, sch_creativework, sp_accessibilityfeature },
    { { 1, 4 }, { 0, 0 }, sch_creativework, sp_accessibilityhazard },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sp_accessibilitysummary },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_accountableperson },
    { { 7, 0 }, { 0, 0 }, sch_creativework, sp_acquirelicensepage },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_aggregaterating },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_alternativeheadline },
    { { 13, 0 }, { 0, 0 }, sch_creativework, sp_archivedat },
    { { 9, 0 }, { 0, 0 }, sch_creativework, sp_assesses },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_associatedmedia },
    { { 1, 4 }, { 0, 0 }, sch_creativework, sp_audience },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_audio },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_author },
    { { 0, 91 }, { 0, 0 }, sch_creativework, sp_award },
    { { 0, 10 }, { 1, 10 }, sch_creativework, sp_awards },
    { { 1, 92 }, { 0, 0 }, sch_creativework, sp_character },
    { { 1, 1 }, { 0, 0 }, sch_creativework, sp_citation },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_comment },
    { { 1, 10 }, { 0, 0 }, sch_creativework, sp_commentcount },
    { { 3, 8 }, { 0, 0 }, sch_creativework, sp_conditionsofaccess },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_contentlocation },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_contentrating },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sp_contentreferencetime },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_contributor },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_copyrightholder },
    { { 11, 0 }, { 0, 0 }, sch_creativework, sp_copyrightnotice },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_copyrightyear },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sp_correction },
    { { 13, 0 }, { 0, 0 }, sch_creativework, sp_countryoforigin },
    { { 3, 8 }, { 0, 0 }, sch_creativework, sp_creativeworkstatus },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_creator },
    { { 11, 0 }, { 0, 0 }, sch_creativework, sp_credittext },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_datecreated },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_datemodified },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_datepublished },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_discussionurl },
    { { 8, 0 }, { 0, 0 }, sch_creativework, sp_editeidr },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_editor },
    { { 1, 0 }, { 0, 0 }, sch_creativework, sp_educationalalignment },
    { { 8, 0 }, { 0, 0 }, sch_creativework, sp_educationallevel },
    { { 1, 0 }, { 0, 0 }, sch_creativework, sp_educationaluse },
    { { 0, 91 }, { 0, 0 }, sch_creativework, sp_encoding },
    { { 0, 10 }, { 1, 90 }, sch_creativework, sp_encodings },
    { { 3, 4 }, { 0, 0 }, sch_creativework, sp_encodingformat },
    { { 1, 90 }, { 0, 0 }, sch_creativework, sp_exampleofwork },
    { { 3, 3 }, { 0, 0 }, sch_creativework, sp_expires },
    { { 2, 2 }, { 3, 3 }, sch_creativework, sp_fileformat },
    { { 3, 1 }, { 0, 0 }, sch_creativework, sp_funder },
    { { 14, 0 }, { 0, 0 }, sch_creativework, sp_funding },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_genre },
    { { 1, 90 }, { 0, 0 }, sch_creativework, sp_haspart },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_headline },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_inlanguage },
    { { 0, 10 }, { 1, 93 }, sch_creativework, sp_interactioncount },
    { { 2, 1 }, { 0, 0 }, sch_creativework, sp_interactionstatistic },
    { { 1, 0 }, { 0, 0 }, sch_creativework, sp_interactivitytype },
    { { 13, 0 }, { 0, 0 }, sch_creativework, sp_interpretedasclaim },
    { { 3, 1 }, { 0, 0 }, sch_creativework, sp_isaccessibleforfree },
    { { 3, 0 }, { 0, 0 }, sch_creativework, sp_isbasedon },
    { { 1, 0 }, { 2, 1 }, sch_creativework, sp_isbasedonurl },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_isfamilyfriendly },
    { { 1, 80 }, { 0, 0 }, sch_creativework, sp_ispartof },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_keywords },
    { { 1, 0 }, { 0, 0 }, sch_creativework, sp_learningresourcetype },
    { { 1, 70 }, { 0, 0 }, sch_creativework, sp_license },
    { { 2, 2 }, { 0, 0 }, sch_creativework, sp_locationcreated },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sp_mainentity },
    { { 6, 0 }, { 0, 0 }, sch_creativework, sp_maintainer },
    { { 3, 2 }, { 0, 0 }, sch_creativework, sp_material },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sp_materialextent },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_mentions },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_offers },
    { { 9, 0 }, { 0, 0 }, sch_creativework, sp_pattern },
    { { 1, 90 }, { 0, 0 }, sch_creativework, sp_position },
    { { 1, 92 }, { 0, 0 }, sch_creativework, sp_producer },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_provider },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sp_publication },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_publisher },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sp_publisherimprint },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_publishingprinciples },
    { { 1, 92 }, { 0, 0 }, sch_creativework, sp_recordedat },
    { { 1, 92 }, { 0, 0 }, sch_creativework, sp_releasedevent },
    { { 0, 91 }, { 0, 0 }, sch_creativework, sp_review },
    { { 0, 10 }, { 1, 90 }, sch_creativework, sp_reviews },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sp_schemaversion },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sp_sddatepublished },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sp_sdlicence },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sp_sdpublisher },
    { { 9, 0 }, { 0, 0 }, sch_creativework, sp_size },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_sourceorganisation },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sp_spatial },
    { { 3, 1 }, { 0, 0 }, sch_creativework, sp_spatialcoverage },
    { { 3, 1 }, { 0, 0 }, sch_creativework, sp_sponsor },
    { { 8, 0 }, { 0, 0 }, sch_creativework, sp_teaches },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sp_temporal },
    { { 3, 1 }, { 0, 0 }, sch_creativework, sp_temporalcoverage },
    { { 0, 93 }, { 0, 0 }, sch_creativework, sp_text },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_thumbnailurl },
    { { 1, 0 }, { 0, 0 }, sch_creativework, sp_timerequired },
    { { 2, 0 }, { 0, 0 }, sch_creativework, sp_translationofwork },
    { { 1, 92 }, { 0, 0 }, sch_creativework, sp_translator },
    { { 1, 0 }, { 0, 0 }, sch_creativework, sp_typicalagerange },
    { { 7, 0 }, { 0, 0 }, sch_creativework, sp_usageinfo },
    { { 0, 30 }, { 0, 0 }, sch_creativework, sp_version },
    { { 0, 10 }, { 0, 0 }, sch_creativework, sp_video },
    { { 1, 90 }, { 0, 0 }, sch_creativework, sp_workexample },
    { { 3, 5 }, { 0, 0 }, sch_creativework, sp_worktranslation },

    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sp_actor },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sp_director },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sp_enddate },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sp_episode },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sp_numberofepisodes },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sp_partofseries },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sp_productioncompany },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sp_seasonnumber },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sp_startdate },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseason, sp_trailer },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseries, sp_enddate },
    { { 3, 3 }, { 0, 0 }, sch_creativeworkseries, sp_issn },
    { { 2, 0 }, { 0, 0 }, sch_creativeworkseries, sp_startdate },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_ddxelement, sp_diagnosis },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_ddxelement, sp_distinguishingsign },

    { { 1, 0 }, { 0, 0 }, sch_datacatalogue, sp_dataset },
    { { 3, 5 }, { 0, 0 }, sch_datacatalogue, sp_measurementtechnique },

    { { 3, 5 }, { 0, 0 }, sch_datadownload, sp_measurementtechnique },

    { { 2, 2 }, { 0, 0 }, sch_datafeed, sp_datafeedelement },

    { { 2, 2 }, { 0, 0 }, sch_datafeeditem, sp_datecreated },
    { { 3, 0 }, { 0, 0 }, sch_datafeeditem, sp_datedeleted },
    { { 2, 2 }, { 0, 0 }, sch_datafeeditem, sp_datemodified },
    { { 2, 2 }, { 0, 0 }, sch_datafeeditem, sp_item },

    { { 1, 0 }, { 1, 93 }, sch_dataset, sp_catalogue },
    { { 2, 0 }, { 3, 0 }, sch_dataset, sp_datasettimeinterval },
    { { 1, 0 }, { 0, 0 }, sch_dataset, sp_distribution },
    { { 2, 0 }, { 0, 0 }, sch_dataset, sp_includeddatacatalog },
    { { 1, 0 }, { 3, 0 }, sch_dataset, sp_spatial },
    { { 3, 3 }, { 0, 0 }, sch_dataset, sp_issn },
    { { 3, 5 }, { 0, 0 }, sch_dataset, sp_measurementtechnique },
    { { 1, 0 }, { 1, 93 }, sch_dataset, sp_temporal },
    { { 3, 5 }, { 0, 0 }, sch_dataset, sp_variablemeasured },
    { { 3, 5 }, { 0, 0 }, sch_dataset, sp_variablesmeasured },

    { { 2, 0 }, { 0, 0 }, sch_datedmoneyspecification, sp_amount },
    { { 2, 0 }, { 0, 0 }, sch_datedmoneyspecification, sp_currency },
    { { 2, 0 }, { 0, 0 }, sch_datedmoneyspecification, sp_enddate },
    { { 2, 0 }, { 0, 0 }, sch_datedmoneyspecification, sp_startdate },

    { { 8, 0 }, { 0, 0 }, sch_definedregion, sp_addresscountry },
    { { 8, 0 }, { 0, 0 }, sch_definedregion, sp_addressregion },
    { { 8, 0 }, { 0, 0 }, sch_definedregion, sp_postalcode },
    { { 8, 0 }, { 0, 0 }, sch_definedregion, sp_postalcodeprefix },
    { { 8, 0 }, { 0, 0 }, sch_definedregion, sp_postalcoderange },

    { { 3, 5 }, { 0, 0 }, sch_definedterm, sp_indefinedtermset },
    { { 3, 5 }, { 0, 0 }, sch_definedterm, sp_termcode },

    { { 3, 5 }, { 0, 0 }, sch_definedtermset, sp_hasdefinedterm },

    { { 0, 99 }, { 0, 0 }, sch_deliverychargespecification, sp_appliestodeliverymethod },
    { { 2, 2 }, { 0, 0 }, sch_deliverychargespecification, sp_areaserved },
    { { 0, 99 }, { 0, 0 }, sch_deliverychargespecification, sp_eligibleregion },
    { { 2, 0 }, { 0, 0 }, sch_deliverychargespecification, sp_ineligibleregion },

    { { 1, 4 }, { 0, 0 }, sch_deliveryevent, sp_accesscode },
    { { 1, 4 }, { 0, 0 }, sch_deliveryevent, sp_availablefrom },
    { { 1, 4 }, { 0, 0 }, sch_deliveryevent, sp_availableuntil },
    { { 1, 4 }, { 0, 0 }, sch_deliveryevent, sp_hasdeliverymethod },

    { { 8, 0 }, { 0, 0 }, sch_deliverytimesettings, sp_deliverytime },
    { { 9, 0 }, { 0, 0 }, sch_deliverytimesettings, sp_isunlabelledfallback },
    { { 8, 0 }, { 0, 0 }, sch_deliverytimesettings, sp_shippingdestination },
    { { 8, 0 }, { 0, 0 }, sch_deliverytimesettings, sp_transittimelabel },

    { { 0, 99 }, { 0, 0 }, sch_demand, sp_acceptedpaymentmethod },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_advancebookingrequirement },
    { { 3, 0 }, { 0, 0 }, sch_demand, sp_areaserved },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_availability },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_availabilityends },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_availabilitystarts },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_availableatorfrom },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_availabledeliverymethod },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_businessfunction },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_deliveryleadtime },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_eligibilecustomertype },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_eligibleduration },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_eligiblequantity },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_eligibleregion },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_eligibletransactionvolume },
    { { 3, 8 }, { 0, 0 }, sch_demand, sp_gtin },
    { { 2, 0 }, { 0, 0 }, sch_demand, sp_gtin12 },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_gtin13 },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_gtin14 },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_gtin8 },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_includesobject },
    { { 2, 0 }, { 0, 0 }, sch_demand, sp_ineligibleregion },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_inventorylevel },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_itemcondition },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_itemoffered },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_mpn },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_pricespecification },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_seller },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_serialnumber },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_sku },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_validfrom },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_validto },
    { { 0, 99 }, { 0, 0 }, sch_demand, sp_warranty },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_diagnosticlab, sp_availabletest },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_diet, sp_dietfeatures },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_diet, sp_endorsers },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_diet, sp_exportconsiderations },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_diet, sp_overview },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_diet, sp_physiologicalbenefits },
    { { 0, 95, SV_NOT_3034 }, { 2, 1 }, sch_diet, sp_proprietaryname },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_diet, sp_risks },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_activeingredient },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_background },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_dosageform },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_isproprietary },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_legalstatus },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_manufacturer },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_maximumintake },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_mechanismofaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_nonproprietaryname },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_recommendedintake },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_safetyconsideration },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_dietarysupplement, sp_targetpopulation },

    { { 3, 0 }, { 0, 0 }, sch_digitaldocument, sp_hasdigitaldocumentpermission },
    { { 3, 0 }, { 0, 0 }, sch_digitaldocumentpermission, sp_grantee },
    { { 3, 0 }, { 0, 0 }, sch_digitaldocumentpermission, sp_permissiontype },

    { { 1, 2 }, { 0, 0 }, sch_donateaction, sp_recipient },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_doseschedule, sp_doseunit },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_doseschedule, sp_dosevalue },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_doseschedule, sp_frequency },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_doseschedule, sp_targetpopulation },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_activeingredient },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_administrationroute },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_alcoholwarning },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_availablestrength },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_breastfeedingwarning },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_clinicalpharamcology },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_cost },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_dosageform },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_doseschedule },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_drugclass },
    { { 3, 5 }, { 0, 0 }, sch_drug, sp_drugunit },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_foodwarning },
    { { 3, 5 }, { 0, 0 }, sch_drug, sp_includedinhealthinsuranceplan },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_interactingdrug },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_isavailablegenerally },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_isproprietary },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_labeldetails },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_legalstatus },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_manufacturer },
    { { 3, 5 }, { 0, 0 }, sch_drug, sp_maximumintake },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_mechanismofaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_nonproprietaryname },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_overdosage },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_pregnancycategory },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_pregnancywarning },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_prescribinginfo },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_prescriptionstatus },
    { { 3, 5 }, { 0, 0 }, sch_drug, sp_proprietaryname },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_relateddrug },
    { { 3, 5 }, { 0, 0 }, sch_drug, sp_rxcui },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drug, sp_warning },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugclass, sp_drug },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugcost, sp_availablelocation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugcost, sp_costcategory },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugcost, sp_costcurrency },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugcost, sp_costorigin },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugcost, sp_costperunit },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugcost, sp_drugunit },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_druglegalstatus, sp_applicablelocation },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugstrength, sp_activeingredient },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugstrength, sp_availablein },
    { { 3, 5 }, { 0, 0 }, sch_drugstrength, sp_maximumintake },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugstrength, sp_strengthunit },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_drugstrength, sp_strengthvalue },

    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthinsuranceplan, sp_benefitssummaryurl },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthinsuranceplan, sp_contactpoint },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthinsuranceplan, sp_healthplandrugoption },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthinsuranceplan, sp_healthplandrugtier },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthinsuranceplan, sp_healthplanid },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthinsuranceplan, sp_healthplanmarketingurl },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthinsuranceplan, sp_includeshealthplanformulary },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthinsuranceplan, sp_includeshealthplannetwork },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthinsuranceplan, sp_useshealthplanidstandard },

    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplancostsharingspecification, sp_healthplancoinsuranceoption },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplancostsharingspecification, sp_healthplancoinsurancerate },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplancostsharingspecification, sp_healthplancopay },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplancostsharingspecification, sp_healthplancopayoption },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplancostsharingspecification, sp_healthplanpharmacycategory },

    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplanformulary, sp_healthplancostsharing },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplanformulary, sp_healthplandrugtier },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplanformulary, sp_offersprescriptionbymail },

    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplannetwork, sp_healthplancostsharing },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplannetwork, sp_healthplannetworkid },
    { { 3, 5 }, { 0, 0 }, sch_dystopianhealthplannetwork, sp_healthplannetworktier },

    { { 8, 0 }, { 0, 0 }, sch_educationevent, sp_assess },
    { { 8, 0 }, { 0, 0 }, sch_educationevent, sp_educationallevel },
    { { 8, 0 }, { 0, 0 }, sch_educationevent, sp_teaches },

    { { 1, 0 }, { 0, 0 }, sch_educationaudience, sp_educationalrole },

    { { 3, 5 }, { 0, 0 }, sch_educationoccupationcredential, sp_competencyrequired },
    { { 3, 5 }, { 0, 0 }, sch_educationoccupationcredential, sp_credentialcategory },
    { { 3, 5 }, { 0, 0 }, sch_educationoccupationcredential, sp_educationallevel },
    { { 3, 5 }, { 0, 0 }, sch_educationoccupationcredential, sp_recognisedby },
    { { 3, 5 }, { 0, 0 }, sch_educationoccupationcredential, sp_validfor },
    { { 3, 5 }, { 0, 0 }, sch_educationoccupationcredential, sp_validin },

    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_applicationdeadline },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_applicationstartdate },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_dayofweek },
    { { 3, 9 }, { 0, 0 }, sch_educationoccupationprogramme, sp_educationalcredentialawarded },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_educationalprogrammode },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_enddate },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_financialaideligible },
    { { 10, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_hascourse },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_maximumenrollment },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_numberofcredits },
    { { 7, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_occupationalcategory },
    { { 3, 9 }, { 0, 0 }, sch_educationoccupationprogramme, sp_occupationalcredentialawarded },
    { { 3, 9 }, { 0, 0 }, sch_educationoccupationprogramme, sp_offers },
    { { 3, 9 }, { 0, 0 }, sch_educationoccupationprogramme, sp_programprequisites },
    { { 7, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_programtype },
    { { 3, 9 }, { 0, 0 }, sch_educationoccupationprogramme, sp_provider },
    { { 3, 9 }, { 0, 0 }, sch_educationoccupationprogramme, sp_salaryuponcompletion },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_startdate },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_termduration },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_termsperyear },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_timeofday },
    { { 3, 9 }, { 0, 0 }, sch_educationoccupationprogramme, sp_timetocomplete },
    { { 7, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_trainingsalary },
    { { 6, 0 }, { 0, 0 }, sch_educationoccupationprogramme, sp_typicalcreditsperterm },

    { { 0, 10 }, { 0, 0 }, sch_educationorganisation, sp_alumni },

    { { 2, 0 }, { 0, 0 }, sch_employeerole, sp_basesalary },
    { { 2, 0 }, { 0, 0 }, sch_employeerole, sp_salarycurrency },

    { { 1, 2 }, { 0, 0 }, sch_endorseaction, sp_endorsee },

    { { 10, 0 }, { 0, 0 }, sch_energyconsumptiondetails, sp_energyefficiencyscalemax },
    { { 10, 0 }, { 0, 0 }, sch_energyconsumptiondetails, sp_energyefficiencyscalemin },
    { { 10, 0 }, { 0, 0 }, sch_energyconsumptiondetails, sp_hasenergyefficiencycategory },

    { { 3, 5 }, { 0, 0 }, sch_enginespecification, sp_enginedisplacement },
    { { 3, 5 }, { 0, 0 }, sch_enginespecification, sp_enginepower },
    { { 3, 5 }, { 0, 0 }, sch_enginespecification, sp_enginetype },
    { { 2, 0 }, { 0, 0 }, sch_enginespecification, sp_fueltype },
    { { 3, 5 }, { 0, 0 }, sch_enginespecification, sp_torque },

    { { 1, 20 }, { 1, 93 }, sch_entrypoint, sp_application },
    { { 2, 0 }, { 0, 0 }, sch_entrypoint, sp_actionapplication },
    { { 2, 2 }, { 0, 0 }, sch_entrypoint, sp_actionplatform },
    { { 1, 20 }, { 0, 0 }, sch_entrypoint, sp_contenttype },
    { { 1, 20 }, { 0, 0 }, sch_entrypoint, sp_encodingtype },
    { { 1, 20 }, { 0, 0 }, sch_entrypoint, sp_httpmethod },
    { { 1, 60 }, { 0, 0 }, sch_entrypoint, sp_urltemplate },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_enumeration, sp_supersededby },

    { { 1, 3 }, { 0, 0 }, sch_episode, sp_actor },
    { { 1, 3 }, { 1, 90 }, sch_episode, sp_actors },
    { { 1, 3 }, { 0, 0 }, sch_episode, sp_director },
    { { 1, 3 }, { 1, 90 }, sch_episode, sp_directors },
    { { 11, 3 }, { 0, 0 }, sch_episode, sp_duration },
    { { 1, 3 }, { 0, 0 }, sch_episode, sp_episodenumber },
    { { 1, 3 }, { 0, 0 }, sch_episode, sp_musicby },
    { { 1, 3 }, { 0, 0 }, sch_episode, sp_partofseason },
    { { 1, 3 }, { 0, 0 }, sch_episode, sp_partofseries },
    { { 1, 3 }, { 0, 0 }, sch_episode, sp_productioncompany },
    { { 1, 3 }, { 1, 93 }, sch_episode, sp_publication },
    { { 1, 3 }, { 0, 0 }, sch_episode, sp_trailer },

    { { 3, 1 }, { 0, 0 }, sch_event, sp_about },
    { { 3, 0 }, { 0, 0 }, sch_event, sp_actor },
    { { 2, 0 }, { 0, 0 }, sch_event, sp_aggregaterating },
    { { 0, 91 }, { 0, 0 }, sch_event, sp_attendee },
    { { 0, 10 }, { 1, 90 }, sch_event, sp_attendees },
    { { 3, 1 }, { 0, 0 }, sch_event, sp_audience },
    { { 3, 0 }, { 0, 0 }, sch_event, sp_composer },
    { { 3, 0 }, { 0, 0 }, sch_event, sp_contributor },
    { { 3, 0 }, { 0, 0 }, sch_event, sp_director },
    { { 1, 4 }, { 0, 0 }, sch_event, sp_doortime },
    { { 0, 10 }, { 0, 0 }, sch_event, sp_duration },
    { { 0, 10 }, { 0, 0 }, sch_event, sp_enddate },
    { { 7, 0 }, { 0, 0 }, sch_event, sp_eventattendancemode },
    { { 6, 0 }, { 0, 0 }, sch_event, sp_eventschedule },
    { { 1, 3 }, { 0, 0 }, sch_event, sp_eventstatus },
    { { 3, 1 }, { 0, 0 }, sch_event, sp_funder },
    { { 14, 0 }, { 0, 0 }, sch_event, sp_funding },
    { { 2, 0 }, { 0, 0 }, sch_event, sp_inlanguage },
    { { 3, 1 }, { 0, 0 }, sch_event, sp_isaccessibleforfree },
    { { 0, 10 }, { 0, 0 }, sch_event, sp_location },
    { { 3, 1 }, { 0, 0 }, sch_event, sp_maximumattendeecapacity },
    { { 7, 0 }, { 0, 0 }, sch_event, sp_maximumphysicalattendeecapacity },
    { { 7, 0 }, { 0, 0 }, sch_event, sp_maximumvirtualattendeecapacity },
    { { 0, 10 }, { 0, 0 }, sch_event, sp_offers },
    { { 2, 0 }, { 0, 0 }, sch_event, sp_organiser },
    { { 0, 91 }, { 0, 0 }, sch_event, sp_performer },
    { { 0, 10 }, { 1, 90 }, sch_event, sp_performers },
    { { 1, 3 }, { 0, 0 }, sch_event, sp_previousstartdate },
    { { 2, 0 }, { 0, 0 }, sch_event, sp_recordedin },
    { { 3, 1 }, { 0, 0 }, sch_event, sp_remainingattendeecapacity },
    { { 2, 0 }, { 0, 0 }, sch_event, sp_review },
    { { 3, 0 }, { 0, 0 }, sch_event, sp_sponsor },
    { { 0, 10 }, { 0, 0 }, sch_event, sp_startdate },
    { { 0, 91 }, { 0, 0 }, sch_event, sp_subevent },
    { { 0, 10 }, { 1, 90 }, sch_event, sp_subevents },
    { { 0, 10 }, { 0, 0 }, sch_event, sp_superevent },
    { { 3, 0 }, { 0, 0 }, sch_event, sp_translator },
    { { 1, 3 }, { 0, 0 }, sch_event, sp_typicalagerange },
    { { 3, 0 }, { 0, 0 }, sch_event, sp_workfeatured },
    { { 2, 0 }, { 0, 0 }, sch_event, sp_workperformed },

    { { 3, 5 }, { 0, 0 }, sch_exchangeratespecification, sp_currency },
    { { 3, 5 }, { 0, 0 }, sch_exchangeratespecification, sp_currentexchangerate },
    { { 3, 5 }, { 0, 0 }, sch_exchangeratespecification, sp_exchangeratespread },

    { { 1, 2 }, { 1, 93 }, sch_exerciseaction, sp_course },
    { { 1, 2 }, { 0, 0 }, sch_exerciseaction, sp_diet },
    { { 1, 2 }, { 0, 0 }, sch_exerciseaction, sp_distance },
    { { 2, 0 }, { 0, 0 }, sch_exerciseaction, sp_exercisecourse },
    { { 1, 2, SV_NOT_3034 }, { 0, 0 }, sch_exerciseaction, sp_exerciseplan },
    { { 1, 2, SV_NOT_3034 }, { 0, 0 }, sch_exerciseaction, sp_exerciserelateddiet },
    { { 1, 2, SV_NOT_3034 }, { 0, 0 }, sch_exerciseaction, sp_exercisetype },
    { { 1, 2 }, { 0, 0 }, sch_exerciseaction, sp_fromlocation },
    { { 1, 2, SV_DEPRECATED }, { 1, 20 }, sch_exerciseaction, sp_oponent },
    { { 1, 2 }, { 0, 0 }, sch_exerciseaction, sp_opponent },
    { { 1, 2 }, { 0, 0 }, sch_exerciseaction, sp_sportsactivitylocation },
    { { 1, 2 }, { 0, 0 }, sch_exerciseaction, sp_sportsevent },
    { { 1, 2 }, { 0, 0 }, sch_exerciseaction, sp_sportsteam },
    { { 1, 2 }, { 0, 0 }, sch_exerciseaction, sp_tolocation },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_exerciseplan, sp_activityduration },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_exerciseplan, sp_activityfrequency },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_exerciseplan, sp_additionalvariable },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_exerciseplan, sp_exercisetype },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_exerciseplan, sp_intensity },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_exerciseplan, sp_repetitions },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_exerciseplan, sp_restperiods },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_exerciseplan, sp_workload },

    { { 3, 0 }, { 0, 0 }, sch_financialproduct, sp_annualpercentagerate },
    { { 3, 0 }, { 0, 0 }, sch_financialproduct, sp_feesandcommissionsspecification },
    { { 3, 0 }, { 0, 0 }, sch_financialproduct, sp_interestrate },
    { { 3, 0 }, { 0, 0 }, sch_financialservice, sp_feesandcommissionsspecification },

    { { 1, 20 }, { 0, 0 }, sch_flight, sp_aircraft },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_arrivalairport },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_arrivalgate },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_arrivalterminal },
    { { 1, 20 }, { 3, 3 }, sch_flight, sp_arrivaltime },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_boardingpolicy },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_departureairport },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_departuregate },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_departureterminal },
    { { 1, 20 }, { 3, 3 }, sch_flight, sp_departuretime },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_estimatedflightduration },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_flightdistance },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_flightnumber },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_mealservice },
    { { 1, 20 }, { 3, 3 }, sch_flight, sp_provider },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_seller },
    { { 1, 20 }, { 0, 0 }, sch_flight, sp_webcheckintime },

    { { 1, 10 }, { 0, 0 }, sch_flightreservation, sp_boardinggroup },
    { { 2, 0 }, { 0, 0 }, sch_flightreservation, sp_passengerprioritystatus },
    { { 2, 0 }, { 0, 0 }, sch_flightreservation, sp_passengersequencenumber },
    { { 2, 0 }, { 0, 0 }, sch_flightreservation, sp_securityscreening },

    { { 7, 0 }, { 0, 0 }, sch_floorplan, sp_amenityfeature },
    { { 7, 0 }, { 0, 0 }, sch_floorplan, sp_floorsize },
    { { 6, 0 }, { 0, 0 }, sch_floorplan, sp_isplanforapartment },
    { { 10, 0 }, { 0, 0 }, sch_floorplan, sp_layoutimage },
    { { 6, 0 }, { 0, 0 }, sch_floorplan, sp_numberofaccommodationunits },
    { { 6, 0 }, { 0, 0 }, sch_floorplan, sp_numberofavailableaccommodationunits },
    { { 6, 0 }, { 0, 0 }, sch_floorplan, sp_numberofbathroomstotal },
    { { 7, 0 }, { 0, 0 }, sch_floorplan, sp_numberofbedrooms },
    { { 6, 0 }, { 0, 0 }, sch_floorplan, sp_numberoffullbathrooms },
    { { 6, 0 }, { 0, 0 }, sch_floorplan, sp_numberofpartialbathrooms },
    { { 6, 0 }, { 0, 0 }, sch_floorplan, sp_numberofrooms },
    { { 7, 0 }, { 0, 0 }, sch_floorplan, sp_petsallowed },

    { { 1, 2 }, { 0, 0 }, sch_followaction, sp_followee },

    { { 0, 10 }, { 0, 0 }, sch_foodestablishment, sp_acceptsreservations },
    { { 0, 10 }, { 3, 1 }, sch_foodestablishment, sp_menu },
    { { 3, 2 }, { 0, 0 }, sch_foodestablishment, sp_hasmenu },
    { { 0, 10 }, { 0, 0 }, sch_foodestablishment, sp_servescuisine },
    { { 3, 1 }, { 0, 0 }, sch_foodestablishment, sp_starrating },

    { { 2, 0 }, { 0, 0 }, sch_foodestablishmentreservation, sp_endtime },
    { { 1, 10 }, { 0, 0 }, sch_foodestablishmentreservation, sp_partysize },
    { { 1, 10 }, { 0, 0 }, sch_foodestablishmentreservation, sp_starttime },

    { { 2, 0 }, { 0, 0 }, sch_game, sp_characterattribute },
    { { 2, 0 }, { 0, 0 }, sch_game, sp_gameitem },
    { { 2, 0 }, { 0, 0 }, sch_game, sp_gamelocation },
    { { 2, 0 }, { 0, 0 }, sch_game, sp_numberofplayers },
    { { 2, 0 }, { 0, 0 }, sch_game, sp_quest },

    { { 1, 92 }, { 0, 0 }, sch_gameserver, sp_game },
    { { 1, 92 }, { 0, 0 }, sch_gameserver, sp_playersonline },
    { { 1, 92 }, { 0, 0 }, sch_gameserver, sp_serverstatus },

    { { 13, 0 }, { 0, 0 }, sch_gene, sp_alternativeof },
    { { 13, 0 }, { 0, 0 }, sch_gene, sp_encodesbiochementity },
    { { 13, 0 }, { 0, 0 }, sch_gene, sp_expressedin },
    { { 13, 0 }, { 0, 0 }, sch_gene, sp_hasbiopolymersequence },

    { { 2, 2 }, { 0, 0 }, sch_geocircle, sp_geomidpoint },
    { { 2, 2 }, { 0, 0 }, sch_geocircle, sp_georadius },

    { { 3, 0 }, { 0, 0 }, sch_geocoordinates, sp_address },
    { { 2, 2 }, { 0, 0 }, sch_geocoordinates, sp_addresscountry },
    { { 0, 10 }, { 0, 0 }, sch_geocoordinates, sp_elevation },
    { { 0, 10 }, { 0, 0 }, sch_geocoordinates, sp_latitude },
    { { 0, 10 }, { 0, 0 }, sch_geocoordinates, sp_longitude },
    { { 2, 2 }, { 0, 0 }, sch_geocoordinates, sp_postalcode },

    { { 3, 0 }, { 0, 0 }, sch_geoshape, sp_address },
    { { 2, 2 }, { 0, 0 }, sch_geoshape, sp_addresscountry },
    { { 0, 30 }, { 0, 0 }, sch_geoshape, sp_box },
    { { 0, 30 }, { 0, 0 }, sch_geoshape, sp_circle },
    { { 0, 30 }, { 0, 0 }, sch_geoshape, sp_elevation },
    { { 0, 30 }, { 0, 0 }, sch_geoshape, sp_line },
    { { 0, 30 }, { 0, 0 }, sch_geoshape, sp_polygon },
    { { 2, 2 }, { 0, 0 }, sch_geoshape, sp_postalcode },

    { { 3, 5 }, { 0, 0 }, sch_geospatialgeometry, sp_geocontains },
    { { 3, 5 }, { 0, 0 }, sch_geospatialgeometry, sp_geocoveredby },
    { { 3, 5 }, { 0, 0 }, sch_geospatialgeometry, sp_geocovers },
    { { 3, 5 }, { 0, 0 }, sch_geospatialgeometry, sp_geocrosses },
    { { 3, 5 }, { 0, 0 }, sch_geospatialgeometry, sp_geodisjoint },
    { { 3, 5 }, { 0, 0 }, sch_geospatialgeometry, sp_geoequals },
    { { 3, 5 }, { 0, 0 }, sch_geospatialgeometry, sp_geointersects },
    { { 3, 5 }, { 0, 0 }, sch_geospatialgeometry, sp_geooverlaps },
    { { 3, 5 }, { 0, 0 }, sch_geospatialgeometry, sp_geotouches },
    { { 3, 5 }, { 0, 0 }, sch_geospatialgeometry, sp_geowithin },

    { { 1, 2 }, { 0, 0 }, sch_giveaction, sp_recipient },

    { { 7, 4 }, { 0, 0 }, sch_governmentservice, sp_jurisdiction },
    { { 1, 3 }, { 0, 0 }, sch_governmentservice, sp_serviceoperator },

    { { 3, 5 }, { 0, 0 }, sch_grant, sp_fundeditem },
    { { 3, 5 }, { 0, 0 }, sch_grant, sp_sponsor },

    { { 5, 0 }, { 0, 0 }, sch_guide, sp_reviewaspect },

    { { 5, 0 }, { 0, 0 }, sch_healthtopiccontent, sp_hashealthaspect },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_hospital, sp_availableservice },
    { { 7, 3 }, { 0, 0 }, sch_hospital, sp_healthcarereportingdata },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_hospital, sp_medicalspeciality },

    { { 3, 1 }, { 0, 0 }, sch_hotelroom, sp_bed },
    { { 3, 1 }, { 0, 0 }, sch_hotelroom, sp_occupancy },

    { { 3, 1 }, { 0, 0 }, sch_house, sp_numberofrooms },

    { { 3, 3 }, { 0, 0 }, sch_howto, sp_estimatedcost },
    { { 3, 3 }, { 0, 0 }, sch_howto, sp_performtime },
    { { 3, 3 }, { 0, 0 }, sch_howto, sp_preptime },
    { { 3, 4 }, { 0, 0 }, sch_howto, sp_step },
    { { 3, 3 }, { 3, 3 }, sch_howto, sp_steps },
    { { 3, 3 }, { 0, 0 }, sch_howto, sp_supply },
    { { 3, 3 }, { 0, 0 }, sch_howto, sp_tool },
    { { 3, 3 }, { 0, 0 }, sch_howto, sp_totaltime },
    { { 3, 3 }, { 0, 0 }, sch_howto, sp_yield },

    { { 3, 3 }, { 0, 0 }, sch_howtodirection, sp_aftermedia },
    { { 3, 3 }, { 0, 0 }, sch_howtodirection, sp_beforemedia },
    { { 3, 3 }, { 0, 0 }, sch_howtodirection, sp_duringmedia },
    { { 3, 3 }, { 0, 0 }, sch_howtodirection, sp_performtime },
    { { 3, 3 }, { 0, 0 }, sch_howtodirection, sp_preptime },
    { { 3, 3 }, { 0, 0 }, sch_howtodirection, sp_supply },
    { { 3, 3 }, { 0, 0 }, sch_howtodirection, sp_tool },
    { { 3, 3 }, { 0, 0 }, sch_howtodirection, sp_totaltime },

    { { 3, 3 }, { 0, 0 }, sch_howtoitem, sp_requiredquantity },

    { { 3, 3 }, { 3, 3 }, sch_howtosection, sp_steps },
    { { 3, 4 }, { 0, 0 }, sch_howtosection, sp_step },

    { { 3, 3 }, { 0, 0 }, sch_howtosupply, sp_estimatedcost },

    { { 11, 0 }, { 0, 0 }, sch_hypertoc, sp_associatedmedia },
    { { 11, 0 }, { 0, 0 }, sch_hypertoc, sp_tocentry },

    { { 11, 0 }, { 0, 0 }, sch_hypertocentry, sp_associatedmedia },
    { { 11, 0 }, { 0, 0 }, sch_hypertocentry, sp_toccontinuation },
    { { 11, 0 }, { 0, 0 }, sch_hypertocentry, sp_utterances },

    { { 0, 10 }, { 0, 0 }, sch_imageobject, sp_caption },
    { { 13, 0 }, { 0, 0 }, sch_imageobject, sp_embeddedtextcaption },
    { { 0, 10 }, { 0, 0 }, sch_imageobject, sp_exifdata },
    { { 0, 10 }, { 0, 0 }, sch_imageobject, sp_representativeofpage },
    { { 0, 10 }, { 0, 0 }, sch_imageobject, sp_thumbnail },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_imagingtest, sp_imagingtechnique },

    { { 0, 99 }, { 0, 0 }, sch_individualproduct, sp_serialnumber },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_infectiousdisease, sp_infectiousagent },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_infectiousdisease, sp_infectiousagentclass },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_infectiousdisease, sp_transmissionmethod },

    { { 1, 2 }, { 0, 0 }, sch_informaction, sp_event },

    { { 1, 2 }, { 0, 0 }, sch_insertaction, sp_tolocation },

    { { 13, 0 }, { 0, 0 }, sch_interactioncounter, sp_endtime },
    { { 2, 0 }, { 0, 0 }, sch_interactioncounter, sp_interactionservice },
    { { 2, 0 }, { 0, 0 }, sch_interactioncounter, sp_interactiontype },
    { { 13, 0 }, { 0, 0 }, sch_interactioncounter, sp_location },
    { { 13, 0 }, { 0, 0 }, sch_interactioncounter, sp_starttime },
    { { 2, 0 }, { 0, 0 }, sch_interactioncounter, sp_userinteractioncount },

    { { 3, 0 }, { 0, 0 }, sch_investmentordeposit, sp_amount },

    { { 1, 2 }, { 0, 0 }, sch_inviteaction, sp_event },

    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_accountid },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_billingperiod },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_broker },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_category },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_confirmationnumber },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_customer },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_minimumpaymentdue },
    { { 1, 93 }, { 2, 0 }, sch_invoice, sp_paymentdue },
    { { 2, 1 }, { 0, 0 }, sch_invoice, sp_paymentduedate },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_paymentmethod },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_paymentmethodid },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_paymentstatus },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_provider },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_referencesorder },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_scheduledpaymentdate },
    { { 1, 93 }, { 0, 0 }, sch_invoice, sp_totalpaymentdue },

    { { 0, 10 }, { 0, 0 }, sch_itemlist, sp_itemlistelement },
    { { 0, 30 }, { 0, 0 }, sch_itemlist, sp_itemlistorder },
    { { 1, 93 }, { 0, 0 }, sch_itemlist, sp_numberofitems },

    { { 3, 5 }, { 0, 0 }, sch_jobposting, sp_applicantlocationrequirements },
    { { 6, 0 }, { 0, 0 }, sch_jobposting, sp_applicantcontact },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_basesalary },
    { { 0, 60 }, { 1, 93 }, sch_jobposting, sp_benefits },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_dateposted },
    { { 13, 0 }, { 0, 0 }, sch_jobposting, sp_directapply },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_educationrequirements },
    { { 8, 0 }, { 0, 0 }, sch_jobposting, sp_eligibilitytoworkrequirement },
    { { 6, 0 }, { 0, 0 }, sch_jobposting, sp_employeroverview },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_employmenttype },
    { { 4, 0 }, { 0, 0 }, sch_jobposting, sp_employmentunit },
    { { 3, 6 }, { 0, 0 }, sch_jobposting, sp_estimatedsalary },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_experiencerequirements },
    { { 12, 0 }, { 0, 0 }, sch_jobposting, sp_experienceinplaceofeducation },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_hiringorganisation },
    { { 0, 60 }, { 1, 93 }, sch_jobposting, sp_incentives },
    { { 2, 0 }, { 0, 0 }, sch_jobposting, sp_incentivecompensation },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_industry },
    { { 2, 0 }, { 0, 0 }, sch_jobposting, sp_jobbenefits },
    { { 3, 8 }, { 0, 0 }, sch_jobposting, sp_jobimmediatestart },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_joblocation },
    { { 3, 5 }, { 0, 0 }, sch_jobposting, sp_joblocationtype },
    { { 3, 8 }, { 0, 0 }, sch_jobposting, sp_jobstartdate },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_occupationalcategory },
    { { 6, 0 }, { 0, 0 }, sch_jobposting, sp_physicalrequirement },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_qualifications },
    { { 3, 5 }, { 0, 0 }, sch_jobposting, sp_relevantoccupation },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_responsibilities },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_salarycurrency },
    { { 6, 0 }, { 0, 0 }, sch_jobposting, sp_securityclearancerequirement },
    { { 6, 0 }, { 0, 0 }, sch_jobposting, sp_sensoryclearancerequirement },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_skills },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_specialcommitments },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_title },
    { { 4, 0 }, { 0, 0 }, sch_jobposting, sp_totaljobopenings },
    { { 3, 0 }, { 0, 0 }, sch_jobposting, sp_validto },
    { { 0, 60 }, { 0, 0 }, sch_jobposting, sp_workhours },

    { { 1, 2 }, { 0, 0 }, sch_joinaction, sp_event },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_joint, sp_biomechanicalclass },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_joint, sp_funcionalclass },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_joint, sp_structuralclass },

    { { 9, 0 }, { 0, 0 }, sch_learningresource, sp_assesses },
    { { 11, 0 }, { 0, 0 }, sch_learningresource, sp_competencyrequired },
    { { 9, 0 }, { 0, 0 }, sch_learningresource, sp_educationalalignment },
    { { 9, 0 }, { 0, 0 }, sch_learningresource, sp_educationallevel },
    { { 9, 0 }, { 0, 0 }, sch_learningresource, sp_educationaluse },
    { { 10, 0 }, { 0, 0 }, sch_learningresource, sp_learningresourcetype },
    { { 9, 0 }, { 0, 0 }, sch_learningresource, sp_teaches },

    { { 1, 2 }, { 0, 0 }, sch_leaveaction, sp_event },

    { { 7, 4 }, { 0, 0 }, sch_legislation, sp_jurisdiction },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationapplies },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationchanges },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationconsolidates },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationdate },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationdateversion },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationidentifier },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationjurisdiction },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationlegalforce },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationpassedby },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationresponsible },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationtransposes },
    { { 3, 5 }, { 0, 0 }, sch_legislation, sp_legislationtype },
    { { 3, 5 }, { 0, 0 }, sch_legislationobject, sp_legislationlegalvalue },

    { { 1, 2 }, { 0, 0 }, sch_lendaction, sp_borrower },

    { { 3, 5 }, { 0, 0 }, sch_linkrole, sp_inlanguage },
    { { 3, 5 }, { 0, 0 }, sch_linkrole, sp_linkrelationship },

    { { 1, 92 }, { 0, 0 }, sch_listitem, sp_item },
    { { 1, 92 }, { 0, 0 }, sch_listitem, sp_nextitem },
    { { 1, 92 }, { 0, 0 }, sch_listitem, sp_position },
    { { 1, 92 }, { 0, 0 }, sch_listitem, sp_previousitem },

    { { 2, 1 }, { 0, 0 }, sch_liveblogposting, sp_coverageendtime },
    { { 2, 1 }, { 0, 0 }, sch_liveblogposting, sp_coveragestarttime },
    { { 2, 1 }, { 0, 0 }, sch_liveblogposting, sp_liveblogupdate },

    { { 3, 0 }, { 0, 0 }, sch_loanorcredit, sp_amount },
    { { 3, 5 }, { 0, 0 }, sch_loanorcredit, sp_currency },
    { { 3, 5 }, { 0, 0 }, sch_loanorcredit, sp_graceperiod },
    { { 3, 5 }, { 0, 0 }, sch_loanorcredit, sp_loanrepaymentform },
    { { 3, 0 }, { 0, 0 }, sch_loanorcredit, sp_loanterm },
    { { 3, 5 }, { 0, 0 }, sch_loanorcredit, sp_loantype },
    { { 3, 5 }, { 0, 0 }, sch_loanorcredit, sp_recourseloan },
    { { 3, 5 }, { 0, 0 }, sch_loanorcredit, sp_renegotiableloan },
    { { 3, 0 }, { 0, 0 }, sch_loanorcredit, sp_requiredcollateral },

    { { 0, 10 }, { 1, 93 }, sch_localbusiness, sp_branchof },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sp_currenciesaccepted },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sp_openinghours },
    { { 2, 0 }, { 2, 0 }, sch_localbusiness, sp_parentorganisation },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sp_paymentaccepted },
    { { 0, 10 }, { 0, 0 }, sch_localbusiness, sp_pricerange },

    { { 3, 1 }, { 0, 0 }, sch_locationfeaturespecification, sp_hoursavailable },
    { { 3, 1 }, { 0, 0 }, sch_locationfeaturespecification, sp_validfrom },
    { { 3, 1 }, { 0, 0 }, sch_locationfeaturespecification, sp_validto },

    { { 3, 1 }, { 0, 0 }, sch_lodgingbusiness, sp_amenityfeature },
    { { 3, 1 }, { 0, 0 }, sch_lodgingbusiness, sp_audience },
    { { 3, 1 }, { 0, 0 }, sch_lodgingbusiness, sp_availablelanguage },
    { { 3, 1 }, { 0, 0 }, sch_lodgingbusiness, sp_checkintime },
    { { 3, 1 }, { 0, 0 }, sch_lodgingbusiness, sp_checkouttime },
    { { 3, 6 }, { 0, 0 }, sch_lodgingbusiness, sp_numberofrooms },
    { { 3, 1 }, { 0, 0 }, sch_lodgingbusiness, sp_petsallowed },
    { { 3, 1 }, { 0, 0 }, sch_lodgingbusiness, sp_starrating },

    { { 1, 10 }, { 0, 0 }, sch_lodgingreservation, sp_checkintime },
    { { 1, 10 }, { 0, 0 }, sch_lodgingreservation, sp_checkouttime },
    { { 1, 10 }, { 0, 0 }, sch_lodgingreservation, sp_lodgingunitdescription },
    { { 1, 10 }, { 0, 0 }, sch_lodgingreservation, sp_lodgingunittype },
    { { 1, 10 }, { 0, 0 }, sch_lodgingreservation, sp_numadults },
    { { 1, 10 }, { 0, 0 }, sch_lodgingreservation, sp_numchildren },

    { { 1, 2 }, { 0, 0 }, sch_loseaction, sp_winner },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_lymphaticvessel, sp_originatesfrom },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_lymphaticvessel, sp_regiondrained },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_lymphaticvessel, sp_runsto },

    { { 2, 0 }, { 0, 0 }, sch_map, sp_maptype },

    { { 11, 0 }, { 0, 0 }, sch_mathsolver, sp_mathexpression },

    { { 0, 35 }, { 0, 0 }, sch_mediaobject, sp_associatedarticle },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_bitrate },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_contentsize },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_contenturl },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_duration, },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_embedurl },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_encodescreativework },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_encodingformat },
    { { 3, 5 }, { 0, 0 }, sch_mediaobject, sp_endtime },
    { { 0, 10 }, { 3, 2 }, sch_mediaobject, sp_expires },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_height },
    { { 12, 0 }, { 0, 0 }, sch_mediaobject, sp_ineligibleregion },
    { { 0, 10 }, { 1, 93 }, sch_mediaobject, sp_interactioncount },
    { { 13, 0 }, { 0, 0 }, sch_mediaobject, sp_interpretedasclaim },
    { { 0, 10 }, { 1, 93 }, sch_mediaobject, sp_offers },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_playertype },
    { { 1, 3 }, { 0, 0 }, sch_mediaobject, sp_productioncompany },
    { { 1, 3 }, { 1, 93 }, sch_mediaobject, sp_publication },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_regionsallowed },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_requiressubscription },
    { { 13, 0 }, { 0, 0 }, sch_mediaobject, sp_sha256 },
    { { 3, 5 }, { 0, 0 }, sch_mediaobject, sp_starttime },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_uploaddate },
    { { 0, 10 }, { 0, 0 }, sch_mediaobject, sp_width },

    { { 7, 0 }, { 0, 0 }, sch_mediareview, sp_mediaauthenticitycategory },
    { { 13, 0 }, { 0, 0 }, sch_mediareview, sp_originalmediacontentdescription },
    { { 13, 0 }, { 0, 0 }, sch_mediareview, sp_originalmedialink },

    { { 13, 0 }, { 0, 0 }, sch_mediareviewitem, sp_mediaitemappearance },

    { { 3, 5 }, { 0, 0 }, sch_mediasubscription, sp_authenticator },
    { { 3, 5 }, { 0, 0 }, sch_mediasubscription, sp_expectsacceptanceof },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcause, sp_causeof },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalclinic, sp_availableservice },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalclinic, sp_medicalspeciality },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcode, sp_codevalue },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcode, sp_codingsystem },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_associatedanatomy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_cause },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_differentialdiagnosis },
    { { 3, 5 }, { 0, 0 }, sch_medicalcondition, sp_drug },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_epidemiology },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_expectedprognosis },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_naturalprogression },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_pathophysiology },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_possiblecomplication },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_possibletreatment },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_primaryprevention },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_riskfactor },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_secondaryprevention },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_signorsymptom },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_stage },
    { { 3, 5 }, { 0, 0 }, sch_medicalcondition, sp_status },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_subtype },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalcondition, sp_typicaltest },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalconditionstage, sp_stageasnumber },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalconditionstage, sp_substagesuffix },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaldevice, sp_adverseoutcome },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaldevice, sp_contraindication },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaldevice, sp_indication },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaldevice, sp_postop },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaldevice, sp_preop },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaldevice, sp_procedure },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaldevice, sp_purpose },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaldevice, sp_seriousadverseoutcome },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalentity, sp_code },
    { { 14, 0 }, { 0, 0 }, sch_medicalentity, sp_funding },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalentity, sp_guideline },
    { { 3, 5 }, { 0, 0 }, sch_medicalentity, sp_legalstatus },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalentity, sp_medicinesystem },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalentity, sp_recognisingauthority },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalentity, sp_relevantspeciality },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalentity, sp_study },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalguideline, sp_evidencelevel },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalguideline, sp_evidenceorigin },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalguideline, sp_guidelinedate },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalguideline, sp_guidelinesubject },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalguidelinerecommendation, sp_recommendationstrength },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalobservationalstudy, sp_studydesign },

    { { 3, 5 }, { 0, 0 }, sch_medicalorganisation, sp_healthplannetworkid },
    { { 3, 5 }, { 0, 0 }, sch_medicalorganisation, sp_isacceptingnewpatients },
    { { 3, 5 }, { 0, 0 }, sch_medicalorganisation, sp_medicalspeciality },

    { { 3, 5 }, { 0, 0 }, sch_medicalprocedure, sp_bodylocation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalprocedure, sp_followup },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalprocedure, sp_howperformed },
    { { 3, 5 }, { 6, 0 }, sch_medicalprocedure, sp_indication },
    { { 3, 5 }, { 6, 0 }, sch_medicalprocedure, sp_outcome },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalprocedure, sp_preparation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalprocedure, sp_proceduretype },
    { { 3, 5 }, { 0, 0 }, sch_medicalprocedure, sp_status },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalriskestimator, sp_estimatesriskof },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalriskestimator, sp_includedriskfactor },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalriskfactor, sp_increasesriskof },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalriskscore, sp_algorithm },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalscholarlyarticle, sp_publicationtype },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalsign, sp_identifyingexam },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalsign, sp_identifyingtest },

    { { 0, 95, SV_NOT_3034 }, { 6, 0 }, sch_medicalsignorsymptom, sp_cause },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalsignorsymptom, sp_possibletreatment },

    { { 3, 5 }, { 0, 0 }, sch_medicalstudy, sp_healthcondition },
    { { 0, 95, SV_NOT_3034 }, { 6, 0 }, sch_medicalstudy, sp_outcome },
    { { 0, 95, SV_NOT_3034 }, { 6, 0 }, sch_medicalstudy, sp_population },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalstudy, sp_sponsor },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalstudy, sp_status },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalstudy, sp_studylocation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicalstudy, sp_studysubject },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaltest, sp_affectedby },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaltest, sp_normalrange },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaltest, sp_signdetected },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaltest, sp_usedtodiagnose },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaltest, sp_usesdevice },

    { { 0, 95 }, { 2, 1 }, sch_medicaltherapy, sp_adverseoutcome },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaltherapy, sp_contraindication },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaltherapy, sp_duplicatetherapy },
    { { 0, 95 }, { 2, 1 }, sch_medicaltherapy, sp_indication },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaltherapy, sp_seriousadverseoutcome },

    { { 0, 95, SV_NOT_3034 }, { 6, 0 }, sch_medicaltrial, sp_phase },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_medicaltrial, sp_trialdesign },

    { { 0, 95 }, { 2, 1 }, sch_medicalwebpage, sp_aspect },
    { { 9, 0 }, { 0, 0 }, sch_medicalwebpage, sp_medicalaudience },

    { { 3, 3 }, { 0, 0 }, sch_menu, sp_hasmenuitem },
    { { 3, 3 }, { 0, 0 }, sch_menu, sp_hasmenusection },

    { { 3, 5 }, { 0, 0 }, sch_menuitem, sp_menuaddon },
    { { 3, 3 }, { 0, 0 }, sch_menuitem, sp_nutrition },
    { { 3, 3 }, { 0, 0 }, sch_menuitem, sp_offers },
    { { 3, 3 }, { 0, 0 }, sch_menuitem, sp_suitablefordiet },

    { { 3, 3 }, { 0, 0 }, sch_menusection, sp_hasmenuitem },
    { { 3, 3 }, { 0, 0 }, sch_menusection, sp_hasmenusection },

    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_additionalproperty },
    { { 14, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_applicablecountry },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_customerremorsereturnfees },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_customerremorsereturnlabelsource },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_customerremorsereturnshippingfeesamount },
    { { 6, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_instorereturnsoffered },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_itemcondition },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_itemdefectreturnfees },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_itemdefectreturnlabelsource },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_itemdefectreturnshippingfeesamount },
    { { 6, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_merchantreturndays },
    { { 6, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_merchantreturnlink },
    { { 6, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_refundtype },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_restockingfee },
    { { 6, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_returnfees },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_returnlabelsource },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_returnmethod },
    { { 6, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_returnpolicycategory },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_returnpolicycountry },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_returnpolicyseasonaloverride },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicy, sp_returnshippingfeesamount },

    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicyseasonaloverride, sp_enddate },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicyseasonaloverride, sp_merchantreturndays },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicyseasonaloverride, sp_returnpolicycategory },
    { { 13, 0 }, { 0, 0 }, sch_merchantreturnpolicyseasonaloverride, sp_startdate },

    { { 0, 95 }, { 0, 0 }, sch_mobileapplication, sp_carrierrequirements },

    { { 3, 3 }, { 0, 0 }, sch_message, sp_bccrecipient },
    { { 3, 3 }, { 0, 0 }, sch_message, sp_ccrecipient },
    { { 3, 0 }, { 0, 0 }, sch_message, sp_dateread },
    { { 3, 0 }, { 0, 0 }, sch_message, sp_datereceived },
    { { 3, 0 }, { 0, 0 }, sch_message, sp_datesent },
    { { 3, 0 }, { 0, 0 }, sch_message, sp_messageattachment },
    { { 3, 0 }, { 0, 0 }, sch_message, sp_recipient },
    { { 3, 0 }, { 0, 0 }, sch_message, sp_sender },
    { { 3, 3 }, { 0, 0 }, sch_message, sp_torecipient },

    { { 13, 0 }, { 0, 0 }, sch_molecularentity, sp_chemicalrole },
    { { 13, 0 }, { 0, 0 }, sch_molecularentity, sp_inchi },
    { { 13, 0 }, { 0, 0 }, sch_molecularentity, sp_inchikey },
    { { 13, 0 }, { 0, 0 }, sch_molecularentity, sp_iupacname },
    { { 13, 0 }, { 0, 0 }, sch_molecularentity, sp_molecularformula },
    { { 13, 0 }, { 0, 0 }, sch_molecularentity, sp_molecularweight },
    { { 13, 0 }, { 0, 0 }, sch_molecularentity, sp_monoistopicmolecularweight },
    { { 13, 0 }, { 0, 0 }, sch_molecularentity, sp_potentialuse },
    { { 13, 0 }, { 0, 0 }, sch_molecularentity, sp_smiles },

    { { 3, 0 }, { 0, 0 }, sch_monetaryamount, sp_currency },
    { { 3, 0 }, { 0, 0 }, sch_monetaryamount, sp_maxvalue },
    { { 3, 0 }, { 0, 0 }, sch_monetaryamount, sp_minvalue },
    { { 3, 0 }, { 0, 0 }, sch_monetaryamount, sp_validfrom },
    { { 3, 0 }, { 0, 0 }, sch_monetaryamount, sp_validto },
    { { 3, 0 }, { 0, 0 }, sch_monetaryamount, sp_value },

    { { 3, 5 }, { 0, 0 }, sch_monetaryamountdistribution, sp_currency },

    { { 3, 5 }, { 0, 0 }, sch_monetarygrant, sp_amount },
    { { 3, 5 }, { 0, 0 }, sch_monetarygrant, sp_funder },

    { { 3, 5 }, { 0, 0 }, sch_moneytransfer, sp_amount },
    { { 3, 5 }, { 0, 0 }, sch_moneytransfer, sp_beneficiarybank },

    { { 3, 5 }, { 0, 0 }, sch_mortgageloan, sp_domiciledmortgage },
    { { 3, 5 }, { 0, 0 }, sch_mortgageloan, sp_loanmortgagamandateamount },

    { { 1, 2 }, { 0, 0 }, sch_moveaction, sp_fromlocation },
    { { 1, 2 }, { 0, 0 }, sch_moveaction, sp_tolocation },

    { { 0, 91 }, { 0, 0 }, sch_movie, sp_actor },
    { { 0, 10 }, { 1, 90 }, sch_movie, sp_actors },
    { { 2, 2 }, { 12, 0 }, sch_movie, sp_countryoforigin },
    { { 0, 10 }, { 0, 0 }, sch_movie, sp_director },
    { { 1, 3 }, { 1, 90 }, sch_movie, sp_directors },
    { { 0, 10 }, { 0, 0 }, sch_movie, sp_duration },
    { { 0, 10 }, { 0, 0 }, sch_movie, sp_musicby },
    { { 0, 10 }, { 0, 0 }, sch_movie, sp_producer },
    { { 0, 10 }, { 0, 0 }, sch_movie, sp_productioncompany },
    { { 2, 0 }, { 0, 0 }, sch_movie, sp_subtitlelanguage },
    { { 8, 0 }, { 0, 0 }, sch_movie, sp_titleeidr },
    { { 0, 10 }, { 0, 0 }, sch_movie, sp_trailer },

    { { 2, 0 }, { 0, 0 }, sch_movieseries, sp_actor },
    { { 2, 0 }, { 0, 0 }, sch_movieseries, sp_director },
    { { 2, 0 }, { 0, 0 }, sch_movieseries, sp_musicby },
    { { 2, 0 }, { 0, 0 }, sch_movieseries, sp_productioncompany },
    { { 2, 0 }, { 0, 0 }, sch_movieseries, sp_trailer },

    { { 2, 0 }, { 0, 0 }, sch_movietheatre, sp_screencount },

    { { 3, 5 }, { 0, 0 }, sch_muscle, sp_action },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_muscle, sp_antagonist },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_muscle, sp_bloodsupply },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_muscle, sp_insertion },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_muscle, sp_muscleaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_muscle, sp_nerve, },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_muscle, sp_origin },

    { { 2, 0 }, { 0, 0 }, sch_musicalbum, sp_albumproductiontype },
    { { 2, 0 }, { 0, 0 }, sch_musicalbum, sp_albumrelease },
    { { 2, 0 }, { 0, 0 }, sch_musicalbum, sp_albumreleasetype },
    { { 0, 10 }, { 0, 0 }, sch_musicalbum, sp_byartist },

    { { 0, 91 }, { 0, 0 }, sch_musicgroup, sp_album },
    { { 0, 10 }, { 1, 90 }, sch_musicgroup, sp_albums },
    { { 1, 92 }, { 0, 0 }, sch_musicgroup, sp_genre },
    { { 0, 10 }, { 1, 93 }, sch_musicgroup, sp_musicgroupmember },
    { { 0, 91 }, { 0, 0 }, sch_musicgroup, sp_track },
    { { 0, 10 }, { 1, 90 }, sch_musicgroup, sp_tracks },

    { { 0, 10 }, { 0, 0 }, sch_musicplaylist, sp_numtracks },
    { { 0, 91 }, { 0, 0 }, sch_musicplaylist, sp_track },
    { { 0, 10 }, { 1, 90 }, sch_musicplaylist, sp_tracks },

    { { 0, 10 }, { 0, 0 }, sch_musicrecording, sp_byartist },
    { { 0, 10 }, { 0, 0 }, sch_musicrecording, sp_duration },
    { { 0, 10 }, { 0, 0 }, sch_musicrecording, sp_inalbum },
    { { 0, 10 }, { 0, 0 }, sch_musicrecording, sp_inplaylist },
    { { 2, 0 }, { 0, 0 }, sch_musicrecording, sp_isrccode },
    { { 2, 0 }, { 0, 0 }, sch_musicrecording, sp_recordingof },

    { { 2, 0 }, { 0, 0 }, sch_musicrelease, sp_cataloguenumber },
    { { 2, 0 }, { 0, 0 }, sch_musicrelease, sp_creditedto },
    { { 2, 0 }, { 0, 0 }, sch_musicrelease, sp_duration },
    { { 2, 0 }, { 0, 0 }, sch_musicrelease, sp_musicreleaseformat },
    { { 2, 0 }, { 0, 0 }, sch_musicrelease, sp_recordlabel },
    { { 2, 0 }, { 0, 0 }, sch_musicrelease, sp_releaseof },

    { { 2, 0 }, { 0, 0 }, sch_musiccomposition, sp_composer },
    { { 2, 0 }, { 0, 0 }, sch_musiccomposition, sp_firstperformance },
    { { 2, 0 }, { 0, 0 }, sch_musiccomposition, sp_includedcomposition },
    { { 2, 0 }, { 0, 0 }, sch_musiccomposition, sp_iswccode },
    { { 2, 0 }, { 0, 0 }, sch_musiccomposition, sp_lyricist },
    { { 2, 1 }, { 0, 0 }, sch_musiccomposition, sp_lyrics },
    { { 2, 0 }, { 0, 0 }, sch_musiccomposition, sp_musicarrangement },
    { { 2, 0 }, { 0, 0 }, sch_musiccomposition, sp_musiccompositionform },
    { { 2, 0 }, { 0, 0 }, sch_musiccomposition, sp_musicalkey },
    { { 2, 0 }, { 0, 0 }, sch_musiccomposition, sp_recordedas },

    { { 2, 0 }, { 0, 0 }, sch_nerve, sp_branch },
    { { 2, 0 }, { 0, 0 }, sch_nerve, sp_nervemotor },
    { { 2, 0 }, { 0, 0 }, sch_nerve, sp_sensoryunit },
    { { 2, 0 }, { 0, 0 }, sch_nerve, sp_sourcedfrom },

    { { 0, 30 }, { 0, 0 }, sch_newsarticle, sp_dateline },
    { { 0, 30 }, { 0, 0 }, sch_newsarticle, sp_printcolumn },
    { { 0, 30 }, { 0, 0 }, sch_newsarticle, sp_printedition },
    { { 0, 30 }, { 0, 0 }, sch_newsarticle, sp_printpage },
    { { 0, 30 }, { 0, 0 }, sch_newsarticle, sp_printsection },

    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_actionablefeedbackpolicy },
    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_correctionspolicy },
    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_diversitypolicy },
    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_diversitystaffingreport },
    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_ethicspolicy },
    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_masthead },
    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_missioncoverageprioritiespolicy },
    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_nobylinespolicy },
    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_ownershipfundinginfo },
    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_unnamedsourcespolicy },
    { { 3, 5 }, { 0, 0 }, sch_newsmediaorganisation, sp_verificationfactcheckingpolicy },

    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_calories },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_carbohydratecontent },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_cholesterolcontent },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_fatcontent },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_fibrecontent },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_proteincontent },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_saturatedfatcontent },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_servingsize },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_sodiumcontent },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_sugarcontent },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_transfatcontent },
    { { 0, 10 }, { 0, 0 }, sch_nutritioninformation, sp_unsaturatedfatcontent },

    { { 3, 9 }, { 0, 0 }, sch_observation, sp_marginoferror },
    { { 3, 9 }, { 0, 0 }, sch_observation, sp_measuredproperty },
    { { 3, 9 }, { 0, 0 }, sch_observation, sp_measuredvalue },
    { { 3, 9 }, { 0, 0 }, sch_observation, sp_observationdate },
    { { 3, 9 }, { 0, 0 }, sch_observation, sp_observednode },

    { { 3, 5 }, { 0, 0 }, sch_occupation, sp_educationrequirements },
    { { 3, 5 }, { 0, 0 }, sch_occupation, sp_estimatedsalary },
    { { 3, 5 }, { 0, 0 }, sch_occupation, sp_experiencerequirements },
    { { 3, 5 }, { 0, 0 }, sch_occupation, sp_occupationlocation },
    { { 3, 5 }, { 0, 0 }, sch_occupation, sp_occupationalcategory },
    { { 3, 5 }, { 0, 0 }, sch_occupation, sp_qualifications },
    { { 3, 5 }, { 0, 0 }, sch_occupation, sp_responsibilities },
    { { 3, 5 }, { 0, 0 }, sch_occupation, sp_skills },

    { { 12, 0 }, { 0, 0 }, sch_occupationalexperiencerequirements, sp_monthsofexperience },

    { { 0, 99 }, { 0, 0 }, sch_offer, sp_acceptedpaymentmethod },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_acceptedpaymentmethods },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_addon },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_addon },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_advancebookingrequirement },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_advancebookingrequirement },
    { { 0, 10 }, { 0, 0 }, sch_offer, sp_aggregaterating },
    { { 2, 2 }, { 0, 0 }, sch_offer, sp_areaserved },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_availableatorfrom },
    { { 0, 10 }, { 0, 0 }, sch_offer, sp_availability },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_availabilityends },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_availabilityends },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_availabilitystarts },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_availabilitystarts },
    { { 2, 0 }, { 0, 0 }, sch_offer, sp_availableatorfrom },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_availabledeliverymethod },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_availabledeliverymethods },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_businessfunction },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_category },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_deliveryleadtime },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_deliveryleadtime },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_eligiblecustomertype },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_eligiblecustomertypes },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_eligibleduration },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_eligibleduration },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_eligiblequantity },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_eligibleregion },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_eligibletransactionvolume },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_eligibletransactionvolume },
    { { 3, 8 }, { 0, 0 }, sch_offer, sp_gtin },
    { { 2, 0 }, { 0, 0 }, sch_offer, sp_gtin12 },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_gtin13 },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_gtin14 },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_gtin8 },
    { { 14, 0 }, { 0, 0 }, sch_offer, sp_hasadultconsideration },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_hasbusinessfunction },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_haseligiblequantity },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_hasinventorylevel },
    { { 13, 0 }, { 0, 0 }, sch_offer, sp_hasmerchantreturnpolicy },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_haspricespecification },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_haswarrantypromise },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_includes },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_includesobject },
    { { 2, 0 }, { 0, 0 }, sch_offer, gr_includesobject },
    { { 2, 0 }, { 0, 0 }, sch_offer, sp_ineligibleregion },
    { { 2, 0 }, { 0, 0 }, sch_offer, sp_inventorylevel },
    { { 14, 0 }, { 0, 0 }, sch_offer, sp_isfamilyfriendly },
    { { 0, 10 }, { 0, 0 }, sch_offer, sp_itemcondition },
    { { 0, 10 }, { 0, 0 }, sch_offer, sp_itemoffered },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_mpn },
    { { 2, 2 }, { 0, 0 }, sch_offer, sp_offeredby },
    { { 0, 10 }, { 0, 0 }, sch_offer, sp_price },
    { { 0, 10 }, { 0, 0 }, sch_offer, sp_pricecurrency },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_pricespecification },
    { { 0, 10 }, { 0, 0 }, sch_offer, sp_pricevaliduntil },
    { { 0, 91 }, { 0, 0 }, sch_offer, sp_review },
    { { 0, 10 }, { 1, 90 }, sch_offer, sp_reviews },
    { { 0, 10 }, { 0, 0 }, sch_offer, sp_seller },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_serialnumber },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_sku },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_validfrom },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_validto },
    { { 0, 99 }, { 0, 0 }, sch_offer, sp_warranty },

    { { 8, 0 }, { 0, 0 }, sch_offershippingdetails, sp_deliverytime },
    { { 8, 0 }, { 0, 0 }, sch_offershippingdetails, sp_doesnotship },
    { { 7, 2 }, { 0, 0 }, sch_offershippingdetails, sp_shippingdestination },
    { { 8, 0 }, { 0, 0 }, sch_offershippingdetails, sp_shippinglabel },
    { { 8, 0 }, { 0, 0 }, sch_offershippingdetails, sp_shippingrate },
    { { 8, 0 }, { 0, 0 }, sch_offershippingdetails, sp_shippingsettingslink },
    { { 8, 0 }, { 0, 0 }, sch_offershippingdetails, sp_transittimelabel },

    { { 0, 99 }, { 0, 0 }, sch_openinghoursspecification, sp_closes },
    { { 0, 99 }, { 0, 0 }, sch_openinghoursspecification, sp_dayofweek },
    { { 0, 99 }, { 0, 0 }, sch_openinghoursspecification, sp_opens },
    { { 0, 99 }, { 0, 0 }, sch_openinghoursspecification, sp_validfrom },
    { { 0, 99 }, { 0, 0 }, sch_openinghoursspecification, sp_validto },

    { { 1, 4 }, { 0, 0 }, sch_order, sp_acceptedoffer },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_billingaddress },
    { { 2, 0 }, { 0, 0 }, sch_order, sp_broker },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_confirmationnumber },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_customer },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_discount },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_discountcode },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_discountcurrency },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_isgift },
    { { 1, 4 }, { 1, 93 }, sch_order, sp_merchant },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_orderdate },
    { { 2, 0 }, { 0, 0 }, sch_order, sp_orderdelivery },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_ordernumber },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_orderstatus },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_ordereditem },
    { { 2, 0 }, { 0, 0 }, sch_order, sp_partofinvoice },
    { { 1, 4 }, { 2, 0 }, sch_order, sp_paymentdue },
    { { 2, 1 }, { 0, 0 }, sch_order, sp_paymentduedate },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_paymentmethod },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_paymentmethodid },
    { { 1, 4 }, { 0, 0 }, sch_order, sp_paymenturl },
    { { 2, 0 }, { 0, 0 }, sch_order, sp_seller },

    { { 2, 0 }, { 0, 0 }, sch_orderaction, sp_deliverymethod },

    { { 2, 0 }, { 0, 0 }, sch_orderitem, sp_orderdelivery },
    { { 2, 0 }, { 0, 0 }, sch_orderitem, sp_orderitemnumber },
    { { 2, 0 }, { 0, 0 }, sch_orderitem, sp_orderitemstatus  },
    { { 2, 0 }, { 0, 0 }, sch_orderitem, sp_orderquantity  },
    { { 2, 0 }, { 0, 0 }, sch_orderitem, sp_ordereditem  },

    { { 3, 5 }, { 0, 0 }, sch_organisation, sp_actionablefeedbackpolicy },
    { { 0, 10 }, { 0, 0 }, sch_organisation, sp_address },
    { { 0, 10 }, { 0, 0 }, sch_organisation, sp_aggregaterating },
    { { 2, 2 }, { 0, 0 }, sch_organisation, sp_alumni },
    { { 3, 0 }, { 0, 0 }, sch_organisation, sp_areaserved },
    { { 2, 0 }, { 0, 0 }, sch_organisation, sp_award },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_brand },
    { { 0, 91 }, { 0, 0 }, sch_organisation, sp_contactpoint },
    { { 0, 10 }, { 1, 90 }, sch_organisation, sp_contactpoints },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sp_correctionspolicy },
    { { 1, 3 }, { 0, 0 }, sch_organisation, sp_department },
    { { 2, 0 }, { 0, 0 }, sch_organisation, sp_dissolutiondate },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sp_diversitypolicy },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sp_diversitystaffingreport },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_duns },
    { { 0, 10 }, { 0, 0 }, sch_organisation, sp_email },
    { { 0, 91 }, { 0, 0 }, sch_organisation, sp_employee },
    { { 0, 10 }, { 1, 90 }, sch_organisation, sp_employees },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sp_ethicspolicy },
    { { 0, 91 }, { 0, 0 }, sch_organisation, sp_event },
    { { 0, 10 }, { 1, 90 }, sch_organisation, sp_events },
    { { 0, 10 }, { 0, 0 }, sch_organisation, sp_faxnumber },
    { { 0, 91 }, { 0, 0 }, sch_organisation, sp_founder },
    { { 0, 10 }, { 1, 90 }, sch_organisation, sp_founders },
    { { 0, 10 }, { 0, 0 }, sch_organisation, sp_foundingdate },
    { { 2, 0 }, { 0, 0 }, sch_organisation, sp_foundinglocation },
    { { 3, 1 }, { 0, 0 }, sch_organisation, sp_funder },
    { { 14, 0 }, { 0, 0 }, sch_organisation, sp_funding },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_globallocationnumber },
    { { 2, 0 }, { 0, 0 }, sch_organisation, gr_hasbrand },
    { { 3, 9 }, { 0, 0 }, sch_organisation, sp_hascredential },
    { { 6, 0 }, { 0, 0 }, sch_organisation, sp_hasmerchantreturnpolicy },
    { { 2, 2 }, { 0, 0 }, sch_organisation, sp_hasoffercatalogue },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_haspos },
    { { 2, 0 }, { 0, 0 }, sch_organisation, gr_haspos },
    { { 3, 8 }, { 5, 0 }, sch_organisation, sp_hasproductreturnpolicy },
    { { 0, 10 }, { 1, 93 }, sch_organisation, sp_interactioncount },
    { { 6, 0 }, { 0, 0 }, sch_organisation, sp_interactionstatistic },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_isicv4 },
    { { 14, 0 }, { 0, 0 }, sch_organisation, sp_iso6523code },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sp_knowsabout },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sp_knowslanguage },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_legalname },
    { { 3, 0 }, { 0, 0 }, sch_organisation, sp_leicode },
    { { 0, 10 }, { 0, 0 }, sch_organisation, sp_location },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_logo },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_makesoffer },
    { { 0, 91 }, { 0, 0 }, sch_organisation, sp_member },
    { { 2, 0 }, { 0, 0 }, sch_organisation, sp_memberof },
    { { 0, 10 }, { 1, 90 }, sch_organisation, sp_members },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_naics },
    { { 8, 0 }, { 0, 0 }, sch_organisation, sp_nonprofitstatus },
    { { 2, 0 }, { 0, 0 }, sch_organisation, sp_numberofemployees },
    { { 2, 0 }, { 0, 0 }, sch_organisation, gr_offers },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sp_ownershipfundinginfo },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_owns },
    { { 2, 0 }, { 0, 0 }, sch_organisation, gr_owns },
    { { 2, 1 }, { 0, 0 }, sch_organisation, sp_parentorganisation },
    { { 3, 3 }, { 0, 0 }, sch_organisation, sp_publishingprinciples },
    { { 0, 91 }, { 0, 0 }, sch_organisation, sp_review },
    { { 0, 10 }, { 1, 90 }, sch_organisation, sp_reviews },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_seeks },
    { { 2, 0 }, { 0, 0 }, sch_organisation, gr_seeks },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sp_slogan },
    { { 3, 0 }, { 0, 0 }, sch_organisation, sp_sponsor },
    { { 1, 3 }, { 0, 0 }, sch_organisation, sp_suborganisation },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_taxid },
    { { 0, 10 }, { 0, 0 }, sch_organisation, sp_telephone },
    { { 3, 5 }, { 0, 0 }, sch_organisation, sp_unnamedsourcespolicy },
    { { 0, 99 }, { 0, 0 }, sch_organisation, sp_vatid },

    { { 2, 0 }, { 0, 0 }, sch_organisationrole, sp_numberedposition },

    { { 0, 99 }, { 0, 0 }, sch_ownershipinfo, sp_acquiredfrom },
    { { 0, 99 }, { 0, 0 }, sch_ownershipinfo, sp_ownedfrom },
    { { 0, 99 }, { 0, 0 }, sch_ownershipinfo, sp_ownedto },
    { { 0, 99 }, { 0, 0 }, sch_ownershipinfo, sp_typeofgood },

    { { 1, 4 }, { 0, 0 }, sch_parceldelivery, sp_deliveryaddress },
    { { 1, 4 }, { 0, 0 }, sch_parceldelivery, sp_deliverystatus },
    { { 1, 4 }, { 0, 0 }, sch_parceldelivery, sp_expectedarrivalfrom },
    { { 1, 4 }, { 0, 0 }, sch_parceldelivery, sp_expectedarrivaluntil },
    { { 1, 4 }, { 0, 0 }, sch_parceldelivery, sp_hasdeliverymethod },
    { { 1, 4 }, { 0, 0 }, sch_parceldelivery, sp_itemshipped },
    { { 1, 4 }, { 0, 0 }, sch_parceldelivery, sp_originaddress },
    { { 1, 4 }, { 0, 0 }, sch_parceldelivery, sp_partoforder },
    { { 2, 0 }, { 0, 0 }, sch_parceldelivery, sp_provider },
    { { 1, 4 }, { 0, 0 }, sch_parceldelivery, sp_trackingnumber },
    { { 1, 4 }, { 0, 0 }, sch_parceldelivery, sp_trackingurl },

    { { 1, 0 }, { 0, 0 }, sch_parentaudience, sp_childminage },
    { { 1, 0 }, { 0, 0 }, sch_parentaudience, sp_childmaxage  },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_pathologytest, sp_tissuesample },

    { { 3, 5 }, { 0, 0 }, sch_patient, sp_diagnosis },
    { { 3, 5 }, { 0, 0 }, sch_patient, sp_drug },
    { { 3, 5 }, { 0, 0 }, sch_patient, sp_healthcondition },

    { { 1, 2 }, { 6, 0 }, sch_payaction, sp_purpose },
    { { 1, 2 }, { 0, 0 }, sch_payaction, sp_recipient },

    { { 3, 5 }, { 0, 0 }, sch_paymentcard, sp_cashback },
    { { 3, 5 }, { 0, 0 }, sch_paymentcard, sp_contactlesspayment },
    { { 3, 5 }, { 0, 0 }, sch_paymentcard, sp_floorlimit },
    { { 9, 0 }, { 0, 0 }, sch_paymentcard, sp_minimummonthlyrepaymentamount },

    { { 0, 99 }, { 0, 0 }, sch_paymentchargespecification, sp_appliestodeliverymethod },
    { { 0, 99 }, { 0, 0 }, sch_paymentchargespecification, sp_appliestopaymentmethod },

    { { 1, 0 }, { 0, 0 }, sch_peopleaudience, sp_healthcondition },
    { { 1, 4 }, { 0, 0 }, sch_peopleaudience, sp_requiredgender },
    { { 1, 4 }, { 0, 0 }, sch_peopleaudience, sp_requiredmaxage },
    { { 1, 4 }, { 0, 0 }, sch_peopleaudience, sp_requiredminage },
    { { 1, 0 }, { 0, 0 }, sch_peopleaudience, sp_suggestedgender },
    { { 1, 0 }, { 0, 0 }, sch_peopleaudience, sp_suggestedmaxage },
    { { 1, 0 }, { 0, 0 }, sch_peopleaudience, sp_suggestedminage },

    { { 1, 2 }, { 0, 0 }, sch_performaction, sp_entertainmentbusiness },

    { { 2, 0 }, { 0, 0 }, sch_performancerole, sp_charactername },

    { { 1, 90 }, { 3, 2 }, sch_periodical, sp_issn },

    { { 1, 4 }, { 0, 0 }, sch_permit, sp_issuedby },
    { { 1, 4 }, { 0, 0 }, sch_permit, sp_issuedthrough },
    { { 1, 4 }, { 0, 0 }, sch_permit, sp_permitaudience },
    { { 1, 4 }, { 0, 0 }, sch_permit, sp_validfor },
    { { 1, 4 }, { 0, 0 }, sch_permit, sp_validfrom },
    { { 1, 4 }, { 0, 0 }, sch_permit, sp_validin },
    { { 1, 4 }, { 0, 0 }, sch_permit, sp_validuntil },

    { { 0, 30 }, { 0, 0 }, sch_person, sp_additionalname },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_address },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_affiliation },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_alumniof },
    { { 0, 91 }, { 0, 0 }, sch_person, sp_award },
    { { 0, 10 }, { 1, 90 }, sch_person, sp_awards },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_birthdate },
    { { 1, 93 }, { 0, 0 }, sch_person, sp_birthplace },
    { { 0, 99 }, { 0, 0 }, sch_person, sp_brand },
    { { 5, 0 }, { 0, 0 }, sch_person, sp_callsign },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_children },
    { { 0, 91 }, { 0, 0 }, sch_person, sp_colleague },
    { { 0, 10 }, { 1, 90 }, sch_person, sp_colleagues },
    { { 0, 91 }, { 0, 0 }, sch_person, sp_contactpoint },
    { { 0, 10 }, { 1, 90 }, sch_person, sp_contactpoints },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_deathdate },
    { { 1, 93 }, { 0, 0 }, sch_person, sp_deathplace },
    { { 0, 99 }, { 0, 0 }, sch_person, sp_duns },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_email },
    { { 0, 30 }, { 0, 0 }, sch_person, sp_familyname },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_faxnumber },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_follows },
    { { 3, 1 }, { 0, 0 }, sch_person, sp_funder },
    { { 14, 0 }, { 0, 0 }, sch_person, sp_funding },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_gender },
    { { 0, 30 }, { 0, 0 }, sch_person, sp_givenname },
    { { 0, 99 }, { 0, 0 }, sch_person, sp_globallocationnumber },
    { { 2, 0 }, { 0, 0 }, sch_person, gr_hasbrand },
    { { 3, 9 }, { 0, 0 }, sch_person, sp_hascredential },
    { { 3, 5 }, { 0, 0 }, sch_person, sp_hasoccupation },
    { { 2, 0 }, { 0, 0 }, sch_person, gr_haspos },
    { { 2, 2 }, { 0, 0 }, sch_person, sp_hasoffercatalogue },
    { { 0, 99 }, { 0, 0 }, sch_person, sp_haspos },
    { { 1, 93 }, { 0, 0 }, sch_person, sp_height },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_homelocation },
    { { 0, 30 }, { 0, 0 }, sch_person, sp_honourificprefix },
    { { 0, 30 }, { 0, 0 }, sch_person, sp_honourificsuffix },
    { { 0, 10 }, { 1, 93 }, sch_person, sp_interactioncount },
    { { 6, 0 }, { 0, 0 }, sch_person, sp_interactionstatistic },
    { { 1, 93 }, { 0, 0 }, sch_person, sp_isicv4 },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_jobtitle },
    { { 0, 30 }, { 0, 0 }, sch_person, sp_knows },
    { { 3, 5 }, { 0, 0 }, sch_person, sp_knowsabout },
    { { 3, 5 }, { 0, 0 }, sch_person, sp_knowslanguage },
    { { 0, 99 }, { 0, 0 }, sch_person, sp_makesoffer },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_memberof },
    { { 0, 99 }, { 0, 0 }, sch_person, sp_naics },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_nationality },
    { { 1, 93 }, { 0, 0 }, sch_person, sp_networth },
    { { 2, 0 }, { 0, 0 }, sch_person, gr_offers },
    { { 0, 99 }, { 0, 0 }, sch_person, sp_owns },
    { { 2, 0 }, { 0, 0 }, sch_person, gr_owns },
    { { 0, 91 }, { 0, 0 }, sch_person, sp_parent },
    { { 0, 10 }, { 1, 90 }, sch_person, sp_parents },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_performerin },
    { { 3, 3 }, { 0, 0 }, sch_person, sp_publishingprinciples },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_relatedto },
    { { 0, 99 }, { 0, 0 }, sch_person, sp_seeks },
    { { 2, 0 }, { 0, 0 }, sch_person, gr_seeks },
    { { 0, 91 }, { 0, 0 }, sch_person, sp_sibling },
    { { 0, 10 }, { 1, 90 }, sch_person, sp_siblings },
    { { 3, 0 }, { 0, 0 }, sch_person, sp_sponsor },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_spouse },
    { { 0, 99 }, { 0, 0 }, sch_person, sp_taxid },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_telephone },
    { { 0, 99 }, { 0, 0 }, sch_person, sp_vatid },
    { { 1, 93 }, { 0, 0 }, sch_person, sp_weight },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_worklocation },
    { { 0, 10 }, { 0, 0 }, sch_person, sp_worksfor },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_physicalactivity, sp_associatedanatomy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_physicalactivity, sp_category },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_physicalactivity, sp_epidemiology },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_physicalactivity, sp_pathophysiology },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_physician, sp_availableservice },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_physician, sp_hospitalaffiliation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_physician, sp_medicalspeciality },

    { { 2, 0 }, { 0, 0 }, sch_place, sp_additionalproperty },
    { { 0, 10 }, { 0, 0 }, sch_place, sp_address },
    { { 0, 10 }, { 0, 0 }, sch_place, sp_aggregaterating },
    { { 3, 1 }, { 0, 0 }, sch_place, sp_amenityfeature },
    { { 2, 2 }, { 0, 0 }, sch_place, sp_branchcode },
    { { 0, 10 }, { 2, 1 }, sch_place, sp_containedin },
    { { 2, 2 }, { 0, 0 }, sch_place, sp_containedinplace },
    { { 3, 0 }, { 0, 0 }, sch_place, sp_containsplace },
    { { 0, 91 }, { 0, 0 }, sch_place, sp_event },
    { { 0, 10 }, { 1, 90 }, sch_place, sp_events },
    { { 0, 10 }, { 0, 0 }, sch_place, sp_faxnumber },
    { { 0, 10 }, { 0, 0 }, sch_place, sp_geo },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_geocontains },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_geocoveredby },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_geocovers },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_geocrosses },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_geodisjoint },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_geoequals },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_geointersects },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_geooverlaps },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_geotouches },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_geowithin },
    { { 0, 99 }, { 0, 0 }, sch_place, sp_globallocationnumber },
    { { 7, 0 }, { 0, 0 }, sch_place, sp_hasdrivethroughservice },
    { { 2, 0 }, { 0, 0 }, sch_place, gr_hasopeninghoursspecification },
    { { 1, 93 }, { 0, 0 }, sch_place, sp_hasmap },
    { { 0, 10 }, { 1, 93 }, sch_place, sp_interactioncount },
    { { 3, 3 }, { 0, 0 }, sch_place, sp_isaccessibleforfree },
    { { 0, 99 }, { 0, 0 }, sch_place, sp_isicv4 },
    { { 4, 0 }, { 0, 0 }, sch_place, sp_latitude },
    { { 2, 0 }, { 0, 0 }, sch_place, sp_logo },
    { { 4, 0 }, { 0, 0 }, sch_place, sp_longitude },
    { { 0, 91 }, { 1, 92 }, sch_place, sp_map },
    { { 0, 10 }, { 1, 90 }, sch_place, sp_maps },
    { { 3, 2 }, { 0, 0 }, sch_place, sp_maximumattendeecapacity },
    { { 0, 99 }, { 0, 0 }, sch_place, sp_openinghoursspecification },
    { { 0, 91 }, { 0, 0 }, sch_place, sp_photo },
    { { 0, 10 }, { 1, 90 }, sch_place, sp_photos },
    { { 3, 3 }, { 0, 0 }, sch_place, sp_publicaccess },
    { { 0, 91 }, { 0, 0 }, sch_place, sp_review },
    { { 0, 10 }, { 1, 90 }, sch_place, sp_reviews },
    { { 3, 5 }, { 0, 0 }, sch_place, sp_slogan },
    { { 3, 1 }, { 0, 0 }, sch_place, sp_smokingallowed },
    { { 3, 0 }, { 0, 0 }, sch_place, sp_specialopeninghoursspecification },
    { { 0, 10 }, { 0, 0 }, sch_place, sp_telephone },
    { { 7, 0 }, { 0, 0 }, sch_place, sp_tourbookingpage },

    { { 1, 2 }, { 0, 0 }, sch_planaction, sp_scheduledtime },

    { { 1, 2 }, { 0, 0 }, sch_playaction, sp_audience },
    { { 1, 2 }, { 0, 0 }, sch_playaction, sp_event },

    { { 14, 0 }, { 0, 0 }, sch_playgameaction, sp_gameavailabilitytype },

    { { 13, 0 }, { 0, 0 }, sch_podcastseries, sp_actor },
    { { 4, 0 }, { 0, 0 }, sch_podcastseries, sp_webfeed },

    { { 0, 10 }, { 0, 0 }, sch_postaladdress, sp_addresscountry },
    { { 0, 10 }, { 0, 0 }, sch_postaladdress, sp_addresslocality },
    { { 0, 10 }, { 0, 0 }, sch_postaladdress, sp_addressregion },
    { { 0, 10 }, { 0, 0 }, sch_postaladdress, sp_postofficeboxnumber },
    { { 0, 10 }, { 0, 0 }, sch_postaladdress, sp_postalcode },
    { { 0, 10 }, { 0, 0 }, sch_postaladdress, sp_streetaddress },

    { { 8, 0 }, { 0, 0 }, sch_postalcoderangespecification, sp_postalcodebegin },
    { { 8, 0 }, { 0, 0 }, sch_postalcoderangespecification, sp_postalcodeend },

    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sp_eligiblequantity },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sp_eligibletransactionvolume },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sp_maxprice },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sp_minprice },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sp_price },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sp_pricecurrency },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sp_validfrom },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sp_validto },
    { { 0, 99 }, { 0, 0 }, sch_pricespecification, sp_valueaddedtaxincluded },

    { { 2, 0 }, { 0, 0 }, sch_product, sp_additionalproperty },
    { { 0, 10 }, { 0, 0 }, sch_product, sp_aggregaterating },
    { { 2, 0 }, { 0, 0 }, sch_product, sp_audience },
    { { 2, 0 }, { 0, 0 }, sch_product, sp_award },
    { { 0, 10 }, { 0, 0 }, sch_product, sp_brand },
    { { 2, 0 }, { 0, 0 }, sch_product, sp_category },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_colour },
    { { 13, 0 }, { 0, 0 }, sch_product, sp_countryofassembly },
    { { 13, 0 }, { 0, 0 }, sch_product, sp_countryoflastprocessing },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_depth },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_depth },
    { { 14, 0 }, { 0, 0 }, sch_product, sp_funding },
    { { 3, 8 }, { 0, 0 }, sch_product, sp_gtin },
    { { 2, 0 }, { 0, 0 }, sch_product, sp_gtin12 },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_gtin13 },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_gtin14 },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_gtin8 },
    { { 14, 0 }, { 0, 0 }, sch_product, sp_hasadultconsideration },
    { { 10, 0 }, { 0, 0 }, sch_product, sp_hasenergyconsumptiondetails },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_hasbrand },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_hasmakeandmodel },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_hasmanufacturer },
    { { 6, 0 }, { 0, 0 }, sch_product, sp_hasmerchantreturnpolicy },
    { { 3, 8 }, { 5, 0 }, sch_product, sp_hasproductreturnpolicy },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_height },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_height },
    { { 9, 0 }, { 0, 0 }, sch_product, sp_inproductgroupwithid },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_isaccessoryorsparepartfor },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_isaccessoryorsparepartfor },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_isconsumablefor },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_isconsumablefor },
    { { 14, 0 }, { 0, 0 }, sch_product, sp_isfamilyfriendly },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_isrelatedto },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_issimilarto },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_issimilarto },
    { { 9, 0 }, { 0, 0 }, sch_product, sp_isvariantof },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_itemcondition },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_logo },
    { { 0, 10 }, { 0, 0 }, sch_product, sp_manufacturer },
    { { 3, 2 }, { 0, 0 }, sch_product, sp_material },
    { { 0, 10 }, { 0, 0 }, sch_product, sp_model },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_mpn },
    { { 3, 5 }, { 0, 0 }, sch_product, sp_nsn },
    { { 0, 10 }, { 0, 0 }, sch_product, sp_offers },
    { { 9, 0 }, { 0, 0 }, sch_product, sp_pattern },
    { { 0, 10 }, { 0, 0 }, sch_product, sp_productid },
    { { 2, 0 }, { 0, 0 }, sch_product, sp_productiondate },
    { { 2, 0 }, { 0, 0 }, sch_product, sp_purchasedate },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_qualitativeproductorserviceproperty },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_quantitativeproductorserviceproperty },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_releasedate },
    { { 0, 91 }, { 0, 0 }, sch_product, sp_review },
    { { 0, 10 }, { 1, 90 }, sch_product, sp_reviews },
    { { 9, 0 }, { 0, 0 }, sch_product, sp_size },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_sku },
    { { 3, 5 }, { 0, 0 }, sch_product, sp_slogan },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_weight },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_weight },
    { { 0, 99 }, { 0, 0 }, sch_product, sp_width },
    { { 2, 0 }, { 0, 0 }, sch_product, gr_width },

    { { 9, 0 }, { 0, 0 }, sch_productcollection, sp_includesobject },

    { { 9, 0 }, { 0, 0 }, sch_productgroup, sp_hasvariant },
    { { 9, 0 }, { 0, 0 }, sch_productgroup, sp_productgroupid },
    { { 9, 0 }, { 0, 0 }, sch_productgroup, sp_variesby },

    { { 0, 99 }, { 0, 0 }, sch_productmodel, sp_isvariantof },
    { { 0, 99 }, { 0, 0 }, sch_productmodel, sp_predecessorof },
    { { 0, 99 }, { 0, 0 }, sch_productmodel, sp_successorof },

    { { 1, 20 }, { 0, 0 }, sch_programmemembership, sp_hostingorganisation },
    { { 1, 20 }, { 0, 0 }, sch_programmemembership, sp_member },
    { { 1, 20 }, { 0, 0 }, sch_programmemembership, sp_membershipnumber },
    { { 3, 5 }, { 0, 0 }, sch_programmemembership, sp_membershippointsearned },
    { { 1, 20 }, { 0, 0 }, sch_programmemembership, sp_programmename },

    { { 6, 0 }, { 0, 0 }, sch_pronounceabletext, sp_inlanguage },
    { { 6, 0 }, { 0, 0 }, sch_pronounceabletext, sp_phonetictext },
    { { 6, 0 }, { 0, 0 }, sch_pronounceabletext, sp_speechtotextmarkup },
    { { 6, 0 }, { 0, 0 }, sch_pronounceabletext, sp_textvalue },

    { { 1, 0 }, { 0, 0 }, sch_property, sp_domainincludes },
    { { 2, 0 }, { 0, 0 }, sch_property, sp_inverseof },
    { { 1, 0 }, { 0, 0 }, sch_property, sp_rangeincludes },
    { { 2, 0 }, { 0, 0 }, sch_property, sp_supersededby },

    { { 2, 0 }, { 0, 0 }, sch_propertyvalue, sp_maxvalue },
    { { 3, 5 }, { 0, 0 }, sch_propertyvalue, sp_measurementtechnique },
    { { 2, 0 }, { 0, 0 }, sch_propertyvalue, sp_minvalue },
    { { 2, 0 }, { 0, 0 }, sch_propertyvalue, sp_propertyid },
    { { 2, 0 }, { 0, 0 }, sch_propertyvalue, sp_unitcode },
    { { 2, 0 }, { 0, 0 }, sch_propertyvalue, sp_unittext },
    { { 2, 0 }, { 0, 0 }, sch_propertyvalue, sp_value },
    { { 2, 0 }, { 0, 0 }, sch_propertyvalue, sp_valuereference },

    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_defaultvalue },
    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_maxvalue },
    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_minvalue },
    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_multiplevalues },
    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_readonlyvalue },
    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_stepvalue },
    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_valuemaxlength },
    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_valueminlength },
    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_valuename },
    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_valuepattern },
    { { 2, 0 }, { 0, 0 }, sch_propertyvaluespecification, sp_valuerequired },

    { { 1, 3 }, { 1, 93 }, sch_publicationevent, sp_free },
    { { 2, 0 }, { 8, 0 }, sch_publicationevent, sp_isaccessibleforfree },
    { { 3, 5 }, { 0, 0 }, sch_publicationevent, sp_publishedby },
    { { 1, 3 }, { 0, 0 }, sch_publicationevent, sp_publishedon },

    { { 1, 90 }, { 0, 0 }, sch_publicationissue, sp_issuenumber },
    { { 1, 90 }, { 0, 0 }, sch_publicationissue, sp_pageend },
    { { 1, 90 }, { 0, 0 }, sch_publicationissue, sp_pagestart },
    { { 1, 90 }, { 0, 0 }, sch_publicationissue, sp_pagination },

    { { 1, 90 }, { 0, 0 }, sch_publicationvolume, sp_pageend },
    { { 1, 90 }, { 0, 0 }, sch_publicationvolume, sp_pagestart },
    { { 1, 90 }, { 0, 0 }, sch_publicationvolume, sp_pagination },
    { { 1, 90 }, { 0, 0 }, sch_publicationvolume, sp_volumenumber },

    { { 13, 0 }, { 0, 0 }, sch_protein, sp_hasbiopolymersequence },

    { { 2, 0 }, { 0, 0 }, sch_qualitativevalue, sp_additionalproperty },
    { { 0, 99 }, { 0, 0 }, sch_qualitativevalue, sp_equal },
    { { 0, 99 }, { 0, 0 }, sch_qualitativevalue, sp_greater },
    { { 0, 99 }, { 0, 0 }, sch_qualitativevalue, sp_greaterorequal },
    { { 0, 99 }, { 0, 0 }, sch_qualitativevalue, sp_lesser },
    { { 0, 99 }, { 0, 0 }, sch_qualitativevalue, sp_lesserorequal },
    { { 0, 99 }, { 0, 0 }, sch_qualitativevalue, sp_nonequal },
    { { 0, 99 }, { 0, 0 }, sch_qualitativevalue, sp_valuereference },

    { { 2, 0 }, { 0, 0 }, sch_quantitativevalue, sp_additionalproperty },
    { { 0, 99 }, { 0, 0 }, sch_quantitativevalue, sp_maxvalue },
    { { 0, 99 }, { 0, 0 }, sch_quantitativevalue, sp_minvalue },
    { { 0, 99 }, { 0, 0 }, sch_quantitativevalue, sp_unitcode },
    { { 2, 0 }, { 0, 0 }, sch_quantitativevalue, sp_unittext },
    { { 0, 99 }, { 0, 0 }, sch_quantitativevalue, sp_value },
    { { 0, 99 }, { 0, 0 }, sch_quantitativevalue, sp_valuereference },

    { { 3, 5 }, { 0, 0 }, sch_quantitativevaluedistribution, sp_duration },
    { { 3, 5 }, { 0, 0 }, sch_quantitativevaluedistribution, sp_median },
    { { 3, 5 }, { 0, 0 }, sch_quantitativevaluedistribution, sp_percentile10 },
    { { 3, 5 }, { 0, 0 }, sch_quantitativevaluedistribution, sp_percentile25 },
    { { 3, 5 }, { 0, 0 }, sch_quantitativevaluedistribution, sp_percentile75 },
    { { 3, 5 }, { 0, 0 }, sch_quantitativevaluedistribution, sp_percentile90 },

    { { 1, 20 }, { 0, 0 }, sch_question, sp_acceptedanswer },
    { { 1, 20 }, { 0, 0 }, sch_question, sp_answercount },
    { { 1, 20 }, { 0, 0 }, sch_question, sp_downvotecount },
    { { 10, 0 }, { 0, 0 }, sch_question, sp_eduquestiontype },
    { { 1, 20 }, { 0, 0 }, sch_question, sp_suggestedanswer },
    { { 1, 20 }, { 0, 0 }, sch_question, sp_upvotecount },

    { { 3, 5 }, { 0, 0 }, sch_quotation, sp_spokenbycharacter },

    { { 1, 3 }, { 1, 93 }, sch_radioepisode, sp_actor },
    { { 1, 3 }, { 1, 90 }, sch_radioepisode, sp_actors },
    { { 1, 3 }, { 1, 93 }, sch_radioepisode, sp_director },
    { { 1, 3 }, { 1, 90 }, sch_radioepisode, sp_directors },
    { { 1, 3 }, { 1, 93 }, sch_radioepisode, sp_musicby },
    { { 1, 3 }, { 1, 93 }, sch_radioepisode, sp_partofseason },
    { { 1, 3 }, { 1, 93 }, sch_radioepisode, sp_partofseries },
    { { 1, 3 }, { 1, 93 }, sch_radioepisode, sp_productioncompany },
    { { 1, 3 }, { 1, 93 }, sch_radioepisode, sp_trailer },

    { { 1, 3 }, { 1, 93 }, sch_radioseason, sp_episode },
    { { 1, 3 }, { 1, 93 }, sch_radioseason, sp_episodes },
    { { 1, 3 }, { 1, 93 }, sch_radioseason, sp_numberofepisodes },
    { { 1, 3 }, { 1, 93 }, sch_radioseason, sp_partofseries },
    { { 1, 3 }, { 1, 93 }, sch_radioseason, sp_trailer },

    { { 1, 3 }, { 0, 0 }, sch_radioseries, sp_actor },
    { { 1, 3 }, { 1, 90 }, sch_radioseries, sp_actors },
    { { 2, 0 }, { 0, 0 }, sch_radioseries, sp_containsseason },
    { { 1, 3 }, { 0, 0 }, sch_radioseries, sp_director },
    { { 1, 3 }, { 1, 90 }, sch_radioseries, sp_directors },
    { { 1, 3 }, { 0, 0 }, sch_radioseries, sp_episode },
    { { 1, 3 }, { 1, 90 }, sch_radioseries, sp_episodes },
    { { 1, 3 }, { 0, 0 }, sch_radioseries, sp_musicby },
    { { 1, 3 }, { 0, 0 }, sch_radioseries, sp_numberofepisodes },
    { { 2, 0 }, { 0, 0 }, sch_radioseries, sp_numberofseasons },
    { { 1, 3 }, { 1, 93 }, sch_radioseries, sp_producer },
    { { 1, 3 }, { 0, 0 }, sch_radioseries, sp_productioncompany },
    { { 1, 3 }, { 1, 93 }, sch_radioseries, sp_season },
    { { 1, 3 }, { 1, 90 }, sch_radioseries, sp_seasons },
    { { 1, 3 }, { 0, 0 }, sch_radioseries, sp_trailer },

    { { 3, 1 }, { 0, 0 }, sch_rating, sp_author },
    { { 0, 10 }, { 0, 0 }, sch_rating, sp_bestrating },
    { { 3, 9 }, { 0, 0 }, sch_rating, sp_ratingexplanation },
    { { 0, 10 }, { 0, 0 }, sch_rating, sp_ratingvalue },
    { { 3, 5 }, { 0, 0 }, sch_rating, sp_reviewaspect },
    { { 0, 10 }, { 0, 0 }, sch_rating, sp_worstrating },

    { { 5, 0 }, { 0, 0 }, sch_realestatelisting, sp_leaselength },
    { { 6, 0 }, { 0, 0 }, sch_realestatelisting, sp_dateposted },

    { { 1, 2 }, { 0, 0 }, sch_receiveaction, sp_deliverymethod },
    { { 1, 2 }, { 0, 0 }, sch_receiveaction, sp_sender },

    { { 0, 10 }, { 0, 0 }, sch_recipe, sp_cookingmethod },
    { { 0, 10 }, { 0, 0 }, sch_recipe, sp_cooktime },
    { { 0, 10 }, { 1, 90 }, sch_recipe, sp_ingredients },
    { { 0, 10 }, { 0, 0 }, sch_recipe, sp_nutrition },
    { { 0, 10 }, { 3, 2 }, sch_recipe, sp_preptime },
    { { 0, 10 }, { 0, 0 }, sch_recipe, sp_recipecategory },
    { { 0, 10 }, { 0, 0 }, sch_recipe, sp_recipecuisine },
    { { 2, 0 }, { 0, 0 }, sch_recipe, sp_recipeingredient },
    { { 0, 10 }, { 0, 0 }, sch_recipe, sp_recipeinstructions },
    { { 0, 10 }, { 0, 0 }, sch_recipe, sp_recipeyield },
    { { 3, 0 }, { 0, 0 }, sch_recipe, sp_suitablefordiet },
    { { 0, 10 }, { 3, 2 }, sch_recipe, sp_totaltime },

    { { 1, 2 }, { 0, 0 }, sch_rentaction, sp_landlord },
    { { 1, 2 }, { 0, 0 }, sch_rentaction, sp_realestateagent },

    { { 1, 10 }, { 0, 0 }, sch_rentalcarreservation, sp_dropofflocation },
    { { 1, 10 }, { 0, 0 }, sch_rentalcarreservation, sp_dropofftime },
    { { 1, 10 }, { 0, 0 }, sch_rentalcarreservation, sp_pickuplocation },
    { { 1, 10 }, { 0, 0 }, sch_rentalcarreservation, sp_pickuptime },

    { { 3, 5 }, { 0, 0 }, sch_repaymentspecification, sp_downpayment },
    { { 3, 5 }, { 0, 0 }, sch_repaymentspecification, sp_earlyprepaymentpenalty },
    { { 3, 5 }, { 0, 0 }, sch_repaymentspecification, sp_loanpaymentamount },
    { { 3, 5 }, { 0, 0 }, sch_repaymentspecification, sp_loanpaymentfrequency },
    { { 3, 5 }, { 0, 0 }, sch_repaymentspecification, sp_numberofloanpayments },

    { { 1, 2 }, { 0, 0 }, sch_replaceaction, sp_replacee },
    { { 1, 2 }, { 0, 0 }, sch_replaceaction, sp_replacer },

    { { 2, 0 }, { 0, 0 }, sch_replyaction, sp_resultcomment },

    { { 2, 1 }, { 0, 0 }, sch_report, sp_reportnumber },

    { { 1, 20 }, { 1, 93 }, sch_reservation, sp_bookingtime },
    { { 2, 0 }, { 0, 0 }, sch_reservation, sp_broker },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sp_modifiedtime },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sp_pricecurrency },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sp_programmemembershipused },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sp_provider },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sp_reservationfor },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sp_reservationid },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sp_reservationstatus },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sp_reservedticket },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sp_totalprice },
    { { 1, 20 }, { 0, 0 }, sch_reservation, sp_undername },

    { { 1, 20 }, { 0, 0 }, sch_reservationpackage, sp_subreservation },

    { { 1, 2 }, { 0, 0 }, sch_reserveaction, sp_scheduledtime },

    { { 6, 0 }, { 0, 0 }, sch_residence, sp_accommodationfloorplan },

    { { 1, 2 }, { 0, 0 }, sch_returnaction, sp_recipient },

    { { 13, 0 }, { 0, 0 }, sch_review, sp_associatedclaimreview },
    { { 13, 0 }, { 0, 0 }, sch_review, sp_associatedmediareview },
    { { 13, 0 }, { 0, 0 }, sch_review, sp_associatedreview },
    { { 0, 10 }, { 0, 0 }, sch_review, sp_itemreviewed },
    { { 13, 0 }, { 0, 0 }, sch_review, sp_negativevotes },
    { { 13, 0 }, { 0, 0 }, sch_review, sp_positivevotes },
    { { 3, 5 }, { 0, 0 }, sch_review, sp_reviewaspect },
    { { 0, 10 }, { 0, 0 }, sch_review, sp_reviewbody },
    { { 0, 10 }, { 0, 0 }, sch_review, sp_reviewrating },

    { { 1, 2 }, { 0, 0 }, sch_reviewaction, sp_resultreview },

    { { 2, 0 }, { 0, 0 }, sch_role, sp_enddate },
    { { 2, 0 }, { 0, 0 }, sch_role, sp_rolename },
    { { 2, 0 }, { 0, 0 }, sch_role, sp_startdate },

    { { 2, 0 }, { 0, 0 }, sch_rsvpaction, sp_additionalnumberofguests },
    { { 2, 0 }, { 0, 0 }, sch_rsvpaction, sp_comment },
    { { 2, 0 }, { 0, 0 }, sch_rsvpaction, sp_rsvpresponse },

    { { 3, 5 }, { 0, 0 }, sch_schedule, sp_byday },
    { { 3, 5 }, { 0, 0 }, sch_schedule, sp_bymonth },
    { { 3, 5 }, { 0, 0 }, sch_schedule, sp_bymonthday },
    { { 9, 0 }, { 0, 0 }, sch_schedule, sp_bymonthweek },
    { { 6, 0 }, { 0, 0 }, sch_schedule, sp_duration },
    { { 7, 0 }, { 0, 0 }, sch_schedule, sp_enddate },
    { { 7, 4 }, { 0, 0 }, sch_schedule, sp_endtime },
    { { 3, 5 }, { 5, 0 }, sch_schedule, sp_eventschedule },
    { { 3, 5 }, { 0, 0 }, sch_schedule, sp_exceptdate },
    { { 3, 5 }, { 0, 0 }, sch_schedule, sp_repeatcount },
    { { 3, 5 }, { 0, 0 }, sch_schedule, sp_repeatfrequency },
    { { 6, 0 }, { 0, 0 }, sch_schedule, sp_scheduletimezone },
    { { 7, 0 }, { 0, 0 }, sch_schedule, sp_startdate },
    { { 7, 4 }, { 0, 0 }, sch_schedule, sp_starttime },

    { { 2, 0 }, { 0, 0 }, sch_screeningevent, sp_subtitlelanguage },
    { { 2, 0 }, { 0, 0 }, sch_screeningevent, sp_videoformat },
    { { 2, 0 }, { 0, 0 }, sch_screeningevent, sp_workpresented },

    { { 1, 2 }, { 0, 0 }, sch_searchaction, sp_query },

    { { 1, 3 }, { 1, 93 }, sch_season, sp_enddate },
    { { 1, 3 }, { 1, 93 }, sch_season, sp_episode },
    { { 1, 3 }, { 1, 90 }, sch_season, sp_episodes },
    { { 1, 3 }, { 1, 93 }, sch_season, sp_numberofepisodes },
    { { 1, 3 }, { 1, 93 }, sch_season, sp_partofseries },
    { { 1, 3 }, { 1, 93 }, sch_season, sp_position },
    { { 1, 3 }, { 1, 93 }, sch_season, sp_producer },
    { { 1, 3 }, { 1, 93 }, sch_season, sp_productioncompany },
    { { 1, 3 }, { 1, 93 }, sch_season, sp_seasonnumber },
    { { 1, 3 }, { 1, 93 }, sch_season, sp_startdate },
    { { 1, 3 }, { 1, 93 }, sch_season, sp_trailer },

    { { 1, 20 }, { 0, 0 }, sch_seat, sp_seatingtype },
    { { 1, 20 }, { 0, 0 }, sch_seat, sp_seatnumber },
    { { 1, 20 }, { 0, 0 }, sch_seat, sp_seatrow },
    { { 1, 20 }, { 0, 0 }, sch_seat, sp_seatsection },

    { { 13, 0 }, { 0, 0 }, sch_seektoaction, sp_startoffset },

    { { 1, 2 }, { 0, 0 }, sch_sellaction, sp_buyer },

    { { 1, 2 }, { 0, 0 }, sch_sendaction, sp_deliverymethod },
    { { 1, 2 }, { 0, 0 }, sch_sendaction, sp_recipient },

    { { 1, 3 }, { 1, 93 }, sch_series, sp_actor },
    { { 1, 3 }, { 1, 90 }, sch_series, sp_actors },
    { { 1, 3 }, { 1, 93 }, sch_series, sp_director },
    { { 1, 3 }, { 1, 90 }, sch_series, sp_directors },
    { { 1, 3 }, { 1, 93 }, sch_series, sp_enddate },
    { { 1, 3 }, { 1, 93 }, sch_series, sp_episode },
    { { 1, 3 }, { 1, 90 }, sch_series, sp_episodes },
    { { 1, 3 }, { 1, 93 }, sch_series, sp_musicby },
    { { 1, 3 }, { 1, 93 }, sch_series, sp_numberofepisodes },
    { { 1, 3 }, { 1, 93 }, sch_series, sp_producer },
    { { 1, 3 }, { 1, 93 }, sch_series, sp_productioncompany },
    { { 1, 3 }, { 1, 93 }, sch_series, sp_season },
    { { 1, 3 }, { 1, 90 }, sch_series, sp_seasons },
    { { 1, 3 }, { 1, 93 }, sch_series, sp_startdate },
    { { 1, 3 }, { 1, 93 }, sch_series, sp_trailer },

    { { 2, 0 }, { 0, 0 }, sch_service, sp_aggregaterating },
    { { 2, 2 }, { 0, 0 }, sch_service, sp_areaserved },
    { { 3, 0 }, { 0, 0 }, sch_service, sp_audience },
    { { 1, 3 }, { 0, 0 }, sch_service, sp_availablechannel },
    { { 2, 2 }, { 0, 0 }, sch_service, sp_award },
    { { 3, 0 }, { 0, 0 }, sch_service, sp_brand },
    { { 3, 2 }, { 0, 0 }, sch_service, sp_broker },
    { { 2, 2 }, { 0, 0 }, sch_service, sp_category },
    { { 2, 2 }, { 0, 0 }, sch_service, sp_hasoffercatalogue },
    { { 2, 2 }, { 0, 0 }, sch_service, sp_hoursavailable },
    { { 3, 0 }, { 0, 0 }, sch_service, sp_isrelatedto },
    { { 3, 0 }, { 0, 0 }, sch_service, sp_issimilarto },
    { { 3, 0 }, { 0, 0 }, sch_service, sp_logo },
    { { 2, 2 }, { 0, 0 }, sch_service, sp_offers },
    { { 1, 3 }, { 1, 93 }, sch_service, sp_produces },
    { { 1, 3 }, { 0, 0 }, sch_service, sp_provider },
    { { 2, 2 }, { 0, 0 }, sch_service, sp_providermobility },
    { { 2, 0 }, { 0, 0 }, sch_service, sp_review },
    { { 1, 3 }, { 2, 1 }, sch_service, sp_servicearea },
    { { 1, 3 }, { 1, 93 }, sch_service, sp_serviceaudience },
    { { 2, 0 }, { 0, 0 }, sch_service, sp_serviceoutput },
    { { 1, 3 }, { 0, 0 }, sch_service, sp_servicetype },
    { { 3, 5 }, { 0, 0 }, sch_service, sp_slogan },
    { { 3, 5 }, { 0, 0 }, sch_service, sp_termsofservice },

    { { 1, 3 }, { 0, 0 }, sch_servicechannel, sp_availablelanguage },
    { { 1, 3 }, { 0, 0 }, sch_servicechannel, sp_processingtime },
    { { 1, 3 }, { 0, 0 }, sch_servicechannel, sp_providesservice },
    { { 1, 3 }, { 0, 0 }, sch_servicechannel, sp_servicelocation },
    { { 1, 3 }, { 0, 0 }, sch_servicechannel, sp_servicephone },
    { { 1, 3 }, { 0, 0 }, sch_servicechannel, sp_servicepostaladdress },
    { { 1, 3 }, { 0, 0 }, sch_servicechannel, sp_servicesmsnumber },
    { { 1, 3 }, { 0, 0 }, sch_servicechannel, sp_serviceurl },

    { { 8, 0 }, { 0, 0 }, sch_shippingdeliverytime, sp_businessdays },
    { { 8, 0 }, { 0, 0 }, sch_shippingdeliverytime, sp_cutofftime },
    { { 8, 0 }, { 0, 0 }, sch_shippingdeliverytime, sp_handlingtime },
    { { 8, 0 }, { 0, 0 }, sch_shippingdeliverytime, sp_transittime },

    { { 8, 0 }, { 0, 0 }, sch_shippingratesettings, sp_doesnotship },
    { { 8, 0 }, { 0, 0 }, sch_shippingratesettings, sp_freeshippingthreshold },
    { { 8, 0 }, { 0, 0 }, sch_shippingratesettings, sp_isunlabelledfallback },
    { { 8, 0 }, { 0, 0 }, sch_shippingratesettings, sp_shippingdestination },
    { { 8, 0 }, { 0, 0 }, sch_shippingratesettings, sp_shippinglabel },
    { { 8, 0 }, { 0, 0 }, sch_shippingratesettings, sp_shippingrate },

    { { 3, 1 }, { 0, 0 }, sch_singlefamilyresidence, sp_numberofrooms },
    { { 3, 1 }, { 0, 0 }, sch_singlefamilyresidence, sp_occupancy },

    { { 12, 0 }, { 0, 0 }, sch_sizespecification, sp_hasmeasurement },
    { { 12, 0 }, { 0, 0 }, sch_sizespecification, sp_sizegroup },
    { { 12, 0 }, { 0, 0 }, sch_sizespecification, sp_sizesystem },
    { { 12, 0 }, { 0, 0 }, sch_sizespecification, sp_suggestedage },
    { { 12, 0 }, { 0, 0 }, sch_sizespecification, sp_suggestedgender },
    { { 12, 0 }, { 0, 0 }, sch_sizespecification, sp_suggestedmeasurement },

    { { 2, 1 }, { 0, 0 }, sch_socialmediaposting, sp_sharedcontent },

    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_applicationcategory },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_applicationsubcategory },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_applicationsuite },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_availableondevice },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_countriesnotsupported },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_countriessupported },
    { { 0, 95 }, { 1, 93 }, sch_softwareapplication, sp_device },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_downloadurl },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_featurelist },
    { { 0, 95 }, { 2, 1 }, sch_softwareapplication, sp_fileformat },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_filesize },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_installurl },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_memoryrequirements },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_operatingsystem },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_permissions },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_processorrequirements },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_releasenotes },
    { { 0, 95 }, { 1, 93 }, sch_softwareapplication, sp_requirements },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_screenshot },
    { { 2, 0 }, { 0, 0 }, sch_softwareapplication, sp_softwareaddon },
    { { 2, 0 }, { 0, 0 }, sch_softwareapplication, sp_softwarehelp },
    { { 2, 0 }, { 0, 0 }, sch_softwareapplication, sp_softwarerequirements },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_softwareversion },
    { { 0, 95 }, { 0, 0 }, sch_softwareapplication, sp_storagerequirements },
    { { 2, 2 }, { 0, 0 }, sch_softwareapplication, sp_supportingdata },

    { { 2, 0 }, { 0, 0 }, sch_softwaresourcecode, sp_coderepository },
    { { 2, 0 }, { 0, 0 }, sch_softwaresourcecode, sp_codesampletype },
    { { 2, 0 }, { 0, 0 }, sch_softwaresourcecode, sp_programminglanguage },
    { { 2, 0 }, { 0, 0 }, sch_softwaresourcecode, sp_runtimeplatform },
    { { 2, 0 }, { 0, 0 }, sch_softwaresourcecode, sp_targetplatform },

    { { 0, 99 }, { 0, 0 }, sch_someproducts, sp_inventorylevel },

    { { 11, 0 }, { 0, 0 }, sch_solvemathaction, sp_eduquestiontype },

    { { 3, 5 }, { 0, 0 }, sch_speakablespecification, sp_cssselector },
    { { 3, 5 }, { 0, 0 }, sch_speakablespecification, sp_xpath },

    { { 7, 2 }, { 0, 0 }, sch_specialannouncement, sp_announcementlocation },
    { { 7, 0 }, { 0, 0 }, sch_specialannouncement, sp_category },
    { { 7, 1 }, { 0, 0 }, sch_specialannouncement, sp_dateposted },
    { { 7, 0 }, { 0, 0 }, sch_specialannouncement, sp_diseasepreventioninfo },
    { { 7, 0 }, { 0, 0 }, sch_specialannouncement, sp_diseasespreadstatistics },
    { { 7, 0 }, { 0, 0 }, sch_specialannouncement, sp_gettingtestedinfo },
    { { 7, 4 }, { 0, 0 }, sch_specialannouncement, sp_governmentbenefitsinfo },
    { { 7, 0 }, { 0, 0 }, sch_specialannouncement, sp_newupdatesandguidelines },
    { { 7, 0 }, { 0, 0 }, sch_specialannouncement, sp_publictransportclosuresinfo },
    { { 7, 0 }, { 0, 0 }, sch_specialannouncement, sp_quarantineguidelines },
    { { 7, 0 }, { 0, 0 }, sch_specialannouncement, sp_schoolclosuresinfo },
    { { 7, 0 }, { 0, 0 }, sch_specialannouncement, sp_travelbans },
    { { 7, 3 }, { 0, 0 }, sch_specialannouncement, sp_webfeed },

    { { 2, 0 }, { 0, 0 }, sch_sportsevent, sp_awayteam },
    { { 2, 0 }, { 0, 0 }, sch_sportsevent, sp_competitor },
    { { 2, 0 }, { 0, 0 }, sch_sportsevent, sp_hometeam },
    { { 6, 0 }, { 0, 0 }, sch_sportsevent, sp_sport },

    { { 2, 0 }, { 0, 0 }, sch_sportsorganisation, sp_sport },

    { { 2, 0 }, { 0, 0 }, sch_sportsteam, sp_athlete },
    { { 2, 0 }, { 0, 0 }, sch_sportsteam, sp_coach },
    { { 4, 0 }, { 0, 0 }, sch_sportsteam, sp_gender },

    { { 3, 9 }, { 0, 0 }, sch_statisticalpopulation, sp_constrainingproperty },
    { { 3, 9 }, { 0, 0 }, sch_statisticalpopulation, sp_numconstraints },
    { { 3, 9 }, { 0, 0 }, sch_statisticalpopulation, sp_populationtype },

    { { 3, 5 }, { 0, 0 }, sch_stupidtype, sp_stupidproperty },

    { { 3, 5 }, { 0, 0 }, sch_substance, sp_activeingredient },
    { { 3, 5 }, { 0, 0 }, sch_substance, sp_maximumintake },

    { { 3, 1 }, { 0, 0 }, sch_suite, sp_bed },
    { { 3, 1 }, { 0, 0 }, sch_suite, sp_numberofrooms },
    { { 3, 1 }, { 0, 0 }, sch_suite, sp_occupancy },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_superficialanatomy, sp_associatedpathophysiology },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_superficialanatomy, sp_relatedanatomy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_superficialanatomy, sp_relatedcondition },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_superficialanatomy, sp_relatedtherapy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_superficialanatomy, sp_significance },

    { { 13, 0 }, { 0, 0 }, sch_taxon, sp_childtaxon },
    { { 13, 0 }, { 0, 0 }, sch_taxon, sp_hasdefinedterm },
    { { 13, 0 }, { 0, 0 }, sch_taxon, sp_parenttaxon },
    { { 13, 0 }, { 0, 0 }, sch_taxon, sp_taxonrank },

    { { 1, 10 }, { 0, 0 }, sch_taxireservation, sp_partysize },
    { { 1, 10 }, { 0, 0 }, sch_taxireservation, sp_pickuplocation },
    { { 1, 10 }, { 0, 0 }, sch_taxireservation, sp_pickuptime },

    { { 1, 0 }, { 0, 0 }, sch_techarticle, sp_dependencies },
    { { 1, 0 }, { 0, 0 }, sch_techarticle, sp_proficiencylevel },

    { { 3, 5 }, { 0, 0 }, sch_therapeuticprocedure, sp_adverseoutcome },
    { { 3, 5 }, { 0, 0 }, sch_therapeuticprocedure, sp_doseschedule },
    { { 3, 5 }, { 0, 0 }, sch_therapeuticprocedure, sp_drug },
    { { 3, 5 }, { 6, 0 }, sch_therapeuticprocedure, sp_indication },

    { { 3, 5 }, { 0, 0 }, sch_thesis, sp_insupportof },

    { { 0, 97 }, { 0, 0 }, sch_thing, sp_additionaltype },
    { { 1, 3 }, { 0, 0 }, sch_thing, sp_alternativename },
    { { 0, 10 }, { 0, 0 }, sch_thing, sp_description },
    { { 3, 0 }, { 0, 0 }, sch_thing, sp_disambiguatingdescription },
    { { 3, 2 }, { 0, 0 }, sch_thing, sp_identifier },
    { { 0, 10 }, { 0, 0 }, sch_thing, sp_image },
    { { 2, 0 }, { 0, 0 }, sch_thing, sp_mainentityofpage },
    { { 0, 10 }, { 0, 0 }, sch_thing, sp_name },
    { { 1, 20 }, { 0, 0 }, sch_thing, sp_potentialaction },
    { { 1, 1 }, { 0, 0 }, sch_thing, sp_sameas },
    { { 3, 5 }, { 0, 0 }, sch_thing, sp_subjectof },
    { { 0, 10 }, { 0, 0 }, sch_thing, sp_url },

    { { 1, 20 }, { 0, 0 }, sch_ticket, sp_dateissued },
    { { 1, 20 }, { 0, 0 }, sch_ticket, sp_issuedby },
    { { 1, 20 }, { 0, 0 }, sch_ticket, sp_pricecurrency },
    { { 1, 20 }, { 0, 0 }, sch_ticket, sp_ticketnumber },
    { { 1, 20 }, { 0, 0 }, sch_ticket, sp_tickettoken },
    { { 1, 20 }, { 0, 0 }, sch_ticket, sp_ticketedseat },
    { { 1, 20 }, { 0, 0 }, sch_ticket, sp_totalprice },
    { { 1, 20 }, { 0, 0 }, sch_ticket, sp_undername },

    { { 1, 2 }, { 0, 0 }, sch_tipaction, sp_recipient },

    { { 3, 3 }, { 0, 0 }, sch_touristattraction, sp_availablelanguage },
    { { 3, 3 }, { 0, 0 }, sch_touristattraction, sp_touristtype },

    { { 3, 5 }, { 0, 0 }, sch_touristdestination, sp_includesattraction },
    { { 3, 5 }, { 0, 0 }, sch_touristdestination, sp_touristtype },

    { { 3, 5 }, { 0, 0 }, sch_touristtrip, sp_touristtype },

    { { 1, 2 }, { 0, 0 }, sch_trackaction, sp_deliverymethod },

    { { 1, 2 }, { 0, 0 }, sch_tradeaction, sp_price },
    { { 3, 5 }, { 0, 0 }, sch_tradeaction, sp_pricecurrency },
    { { 2, 0 }, { 0, 0 }, sch_tradeaction, sp_pricespecification },

    { { 1, 20 }, { 0, 0 }, sch_traintrip, sp_arrivalplatform },
    { { 1, 20 }, { 0, 0 }, sch_traintrip, sp_arrivalstation },
    { { 1, 20 }, { 3, 3 }, sch_traintrip, sp_arrivaltime },
    { { 1, 20 }, { 0, 0 }, sch_traintrip, sp_departureplatform },
    { { 1, 20 }, { 0, 0 }, sch_traintrip, sp_departurestation },
    { { 1, 20 }, { 3, 3 }, sch_traintrip, sp_departuretime },
    { { 1, 20 }, { 3, 3 }, sch_traintrip, sp_provider },
    { { 1, 20 }, { 0, 0 }, sch_traintrip, sp_trainname },
    { { 1, 20 }, { 0, 0 }, sch_traintrip, sp_trainnumber },

    { { 1, 2 }, { 0, 0 }, sch_transferaction, sp_fromlocation },
    { { 1, 2 }, { 0, 0 }, sch_transferaction, sp_tolocation },

    { { 1, 2 }, { 0, 0 }, sch_travelaction, sp_distance },

    { { 3, 4 }, { 0, 0 }, sch_trip, sp_arrivaltime },
    { { 3, 4 }, { 0, 0 }, sch_trip, sp_departuretime },
    { { 3, 5 }, { 0, 0 }, sch_trip, sp_itinerary },
    { { 3, 4 }, { 0, 0 }, sch_trip, sp_offers },
    { { 3, 5 }, { 0, 0 }, sch_trip, sp_partoftrip },
    { { 3, 4 }, { 0, 0 }, sch_trip, sp_provider },
    { { 3, 5 }, { 0, 0 }, sch_trip, sp_subtrip },

    { { 1, 3 }, { 1, 93 }, sch_tvclip, sp_partoftvseries },

    { { 0, 91 }, { 1, 93 }, sch_tvepisode, sp_actor },
    { { 0, 10 }, { 1, 90 }, sch_tvepisode, sp_actors },
    { { 2, 0 }, { 12, 0 }, sch_tvepisode, sp_countryoforigin },
    { { 0, 10 }, { 1, 93 }, sch_tvepisode, sp_director },
    { { 1, 3 }, { 1, 90 }, sch_tvepisode, sp_directors },
    { { 0, 10 }, { 1, 10 }, sch_tvepisode, sp_episodenumber },
    { { 0, 10 }, { 1, 93 }, sch_tvepisode, sp_musicby },
    { { 0, 10 }, { 1, 93 }, sch_tvepisode, sp_partofseason },
    { { 0, 10 }, { 1, 93 }, sch_tvepisode, sp_partoftvseries },
    { { 0, 10 }, { 1, 93 }, sch_tvepisode, sp_producer },
    { { 0, 10 }, { 1, 93 }, sch_tvepisode, sp_productioncompany },
    { { 2, 0 }, { 0, 0 }, sch_tvepisode, sp_subtitlelanguage },
    { { 8, 0 }, { 0, 0 }, sch_tvepisode, sp_titleeidr },
    { { 0, 35 }, { 1, 93 }, sch_tvepisode, sp_trailer },

    { { 2, 2 }, { 12, 0 }, sch_tvseason, sp_countryoforigin },
    { { 0, 10 }, { 1, 93 }, sch_tvseason, sp_enddate },
    { { 1, 3 }, { 1, 93 }, sch_tvseason, sp_episode },
    { { 0, 10 }, { 1, 93 }, sch_tvseason, sp_episodes },
    { { 0, 10 }, { 1, 93 }, sch_tvseason, sp_numberofepisodes },
    { { 1, 3 }, { 1, 93 }, sch_tvseason, sp_partofseries },
    { { 0, 10 }, { 1, 93 }, sch_tvseason, sp_partoftvseries },
    { { 0, 10 }, { 1, 93 }, sch_tvseason, sp_seasonnumber },
    { { 0, 10 }, { 1, 93 }, sch_tvseason, sp_startdate },
    { { 0, 10 }, { 1, 93 }, sch_tvseason, sp_trailer },

    { { 1, 3 }, { 0, 0 }, sch_tvseries, sp_actor },
    { { 0, 10 }, { 1, 90 }, sch_tvseries, sp_actors },
    { { 2, 0 }, { 0, 0 }, sch_tvseries, sp_containsseason },
    { { 2, 2 }, { 12, 0 }, sch_tvseries, sp_countryoforigin },
    { { 0, 10 }, { 0, 0 }, sch_tvseries, sp_director },
    { { 1, 3 }, { 1, 90 }, sch_tvseries, sp_directors },
    { { 0, 10 }, { 1, 10 }, sch_tvseries, sp_enddate },
    { { 1, 3 }, { 0, 0 }, sch_tvseries, sp_episode },
    { { 0, 10 }, { 1, 90 }, sch_tvseries, sp_episodes },
    { { 0, 10 }, { 0, 0 }, sch_tvseries, sp_musicby },
    { { 0, 10 }, { 0, 0 }, sch_tvseries, sp_numberofepisodes },
    { { 2, 0 }, { 0, 0 }, sch_tvseries, sp_numberofseasons },
    { { 0, 10 }, { 1, 10 }, sch_tvseries, sp_producer },
    { { 0, 10 }, { 0, 0 }, sch_tvseries, sp_productioncompany },
    { { 1, 3 }, { 1, 93 }, sch_tvseries, sp_season },
    { { 1, 3 }, { 1, 90 }, sch_tvseries, sp_seasons },
    { { 0, 10 }, { 1, 10 }, sch_tvseries, sp_startdate },
    { { 0, 10 }, { 0, 0 }, sch_tvseries, sp_trailer },

    { { 0, 99 }, { 0, 0 }, sch_typeandquantitynode, sp_amountofthisgood },
    { { 0, 99 }, { 0, 0 }, sch_typeandquantitynode, sp_businessfunction },
    { { 0, 99 }, { 0, 0 }, sch_typeandquantitynode, sp_typeofgood },
    { { 0, 99 }, { 0, 0 }, sch_typeandquantitynode, sp_unitcode },
    { { 2, 0 }, { 0, 0 }, sch_typeandquantitynode, sp_unittext },

    { { 11, 0 }, { 0, 0 }, sch_unitpricespecification, sp_billingduration },
    { { 0, 99 }, { 0, 0 }, sch_unitpricespecification, sp_billingincrement },
    { { 11, 0 }, { 0, 0 }, sch_unitpricespecification, sp_billingstart },
    { { 11, 0 }, { 0, 0 }, sch_unitpricespecification, sp_pricecomponenttype },
    { { 0, 99 }, { 0, 0 }, sch_unitpricespecification, sp_pricetype },
    { { 3, 0 }, { 0, 0 }, sch_unitpricespecification, sp_referencequantity },
    { { 0, 99 }, { 0, 0 }, sch_unitpricespecification, sp_unitcode },
    { { 2, 0 }, { 0, 0 }, sch_unitpricespecification, sp_unittext },

    { { 1, 2 }, { 1, 93 }, sch_updateaction, sp_collection },
    { { 2, 0 }, { 0, 0 }, sch_updateaction, sp_targetcollection },

    { { 0, 30 }, { 3, 4 }, sch_usercomments, sp_commenttext },
    { { 0, 30 }, { 3, 4 }, sch_usercomments, sp_commenttime },
    { { 0, 30 }, { 3, 4 }, sch_usercomments, sp_creator },
    { { 0, 30 }, { 3, 4 }, sch_usercomments, sp_discusses },
    { { 0, 30 }, { 3, 4 }, sch_usercomments, sp_replytourl },

    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_accelerationtime },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_bodytype },
    { { 5, 0 }, { 0, 0 }, sch_vehicle, sp_callsign },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_cargovolume },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_datevehiclefirstregistered },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_drivewheelconfiguration },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_emmissionsco2 },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_fuelcapacity },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_fuelconsumption },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_fuelefficiency },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_fueltype },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_knownvehicledamages },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_meetsemissionstandard },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_mileagefromodometer },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_modeldate },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_numberofairbags },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_numberofaxles },
    { { 2, 1 }, { 0, 0 }, sch_vehicle, sp_numberofdoors },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_numberofforwardgears },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_numberofpreviousowners },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_payload },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_productiondate },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_purchasedate },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_seatingcapacity },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_speed },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_steeringposition },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_stupidproperty }, // ?!
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_tongueweight },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_trailerweight },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_vehicleconfiguration },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_vehicleengine },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_vehicleidentificationnumber },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_vehicleinteriorcolour },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_vehicleinteriortype },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_vehiclemodeldate },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_vehicleseatingcapacity },
    { { 3, 0 }, { 0, 0 }, sch_vehicle, sp_vehiclespecialusage },
    { { 2, 0 }, { 0, 0 }, sch_vehicle, sp_vehicletransmission },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_weighttotal },
    { { 3, 5 }, { 0, 0 }, sch_vehicle, sp_wheelbase },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_vein, sp_drainsto },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_vein, sp_regiondrained },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_vein, sp_tributary },

    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_vessel, sp_bloodsupply },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_vessel, sp_drainsto },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_vessel, sp_originatesfrom },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, sch_vessel, sp_runsto },

    { { 2, 0 }, { 0, 0 }, sch_videogame, sp_actor },
    { { 2, 0 }, { 0, 0 }, sch_videogame, sp_cheatcode },
    { { 2, 0 }, { 0, 0 }, sch_videogame, sp_director },
    { { 14, 0 }, { 0, 0 }, sch_videogame, sp_gameedition },
    { { 2, 0 }, { 0, 0 }, sch_videogame, sp_gameplatform },
    { { 2, 0 }, { 0, 0 }, sch_videogame, sp_gameserver },
    { { 2, 0 }, { 0, 0 }, sch_videogame, sp_gametip },
    { { 2, 0 }, { 0, 0 }, sch_videogame, sp_musicby },
    { { 2, 0 }, { 0, 0 }, sch_videogame, sp_playmode },
    { { 2, 0 }, { 0, 0 }, sch_videogame, sp_trailer },

    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_actor },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_characterattribute },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_cheatcode },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_containsseason },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_director },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_episode },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_gameitem },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_gamelocation },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_gameplatform },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_musicby },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_numberofepisodes },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_numberofplayers },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_numberofseasons },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_playmode },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_productioncompany },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_quest },
    { { 2, 0 }, { 0, 0 }, sch_videogameseries, sp_trailer },

    { { 2, 0 }, { 0, 0 }, sch_videoobject, sp_actor },
    { { 0, 10 }, { 0, 0 }, sch_videoobject, sp_caption },
    { { 2, 0 }, { 0, 0 }, sch_videoobject, sp_director },
    { { 13, 0 }, { 0, 0 }, sch_videoobject, sp_embeddedtextcaption },
    { { 2, 0 }, { 0, 0 }, sch_videoobject, sp_musicby },
    { { 0, 15 }, { 1, 93 }, sch_videoobject, sp_productioncompany },
    { { 0, 10 }, { 0, 0 }, sch_videoobject, sp_thumbnail },
    { { 0, 30 }, { 0, 0 }, sch_videoobject, sp_transcript },
    { { 0, 15 }, { 0, 0 }, sch_videoobject, sp_videoframesize },
    { { 0, 15 }, { 0, 0 }, sch_videoobject, sp_videoquality },

    { { 2, 0 }, { 0, 0 }, sch_visualartwork, sp_artedition },
    { { 2, 0 }, { 0, 0 }, sch_visualartwork, sp_artmedium },
    { { 2, 0 }, { 0, 0 }, sch_visualartwork, sp_artform },
    { { 3, 5 }, { 0, 0 }, sch_visualartwork, sp_artist },
    { { 2, 0 }, { 0, 0 }, sch_visualartwork, sp_artworksurface },
    { { 3, 5 }, { 0, 0 }, sch_visualartwork, sp_colourist },
    { { 2, 0 }, { 0, 0 }, sch_visualartwork, sp_depth },
    { { 2, 0 }, { 0, 0 }, sch_visualartwork, sp_height },
    { { 3, 5 }, { 0, 0 }, sch_visualartwork, sp_inker },
    { { 3, 5 }, { 0, 0 }, sch_visualartwork, sp_letterer },
    { { 3, 5 }, { 0, 0 }, sch_visualartwork, sp_penciler },
    { { 2, 0 }, { 0, 0 }, sch_visualartwork, sp_width },

    { { 1, 2 }, { 0, 0 }, sch_voteaction, sp_candidate },

    { { 0, 99 }, { 0, 0 }, sch_warrantypromise, sp_durationofwarranty },
    { { 0, 99 }, { 0, 0 }, sch_warrantypromise, sp_warrantyscope },

    { { 3, 5 }, { 0, 0 }, sch_webapi, sp_documentation },

    { { 0, 95 }, { 0, 0 }, sch_webapplication, sp_browserrequirements },

    { { 0, 10 }, { 0, 0 }, sch_webpage, sp_breadcrumb },
    { { 0, 10 }, { 1, 10 }, sch_webpage, sp_ispartof },
    { { 0, 95 }, { 0, 0 }, sch_webpage, sp_lastreviewed },
    { { 0, 10 }, { 0, 0 }, sch_webpage, sp_maincontentofpage },
    { { 0, 10 }, { 0, 0 }, sch_webpage, sp_primaryimageofpage },
    { { 0, 95 }, { 0, 0 }, sch_webpage, sp_relatedlink },
    { { 0, 95 }, { 0, 0 }, sch_webpage, sp_reviewedby },
    { { 0, 91 }, { 0, 0 }, sch_webpage, sp_significantlink },
    { { 0, 10 }, { 1, 93 }, sch_webpage, sp_significantlinks },
    { { 3, 5 }, { 0, 0 }, sch_webpage, sp_speakable },
    { { 0, 95 }, { 0, 0 }, sch_webpage, sp_speciality },

    { { 3, 5 }, { 0, 0 }, sch_webpageelement, sp_cssselector },
    { { 3, 5 }, { 0, 0 }, sch_webpageelement, sp_xpath },

    { { 3, 3 }, { 0, 0 }, sch_website, sp_issn },

    { { 1, 2 }, { 0, 0 }, sch_winaction, sp_loser },

    { { 3, 9 }, { 0, 0 }, sch_workbasedprogramme, sp_occupationalcategory },
    { { 3, 9 }, { 0, 0 }, sch_workbasedprogramme, sp_trainingsalary },

    { { 1, 2 }, { 1, 93 }, sch_writeaction, sp_language },
    { { 2, 0 }, { 0, 0 }, sch_writeaction, sp_inlanguage },

    // sd
    { { s_sd, 1, 0 }, { 0, 0 }, sd_dataset, sdp_defaultgraph },

    { { s_sd, 1, 0 }, { 0, 0 }, sd_graphcollection, sdp_namedgraph },

    { { s_sd, 1, 0 }, { 0, 0 }, sd_namedgraph, sdp_entailmentregime },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_namedgraph, sdp_graph },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_namedgraph, sdp_name },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_namedgraph, sdp_supportedentailmentprofile },

    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_availablegraphs },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_defaultdataset },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_defaultentailmentregime },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_defaultsupportedentailmentprofile },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_extensionaggregate },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_extensionfunction },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_feature },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_inputformat },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_languageextension },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_propertyfeature },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_resultformat },
    { { s_sd, 1, 0 }, { 0, 0 }, sd_service, sdp_supportedlanguage },

    // sioc
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_community, dct_haspart },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_community, dct_ispartof },

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_container, sioc_container_of },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_container, sioc_has_owner },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_container, sioc_has_parent },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_container, sioc_has_subscriber },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_container, sioc_parent_of },

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_forum, sioc_has_host },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_forum, sioc_has_moderator },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_forum, sioc_scope_of },

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_about },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, dct_created },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_has_container },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_has_creator },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_has_modifier },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, dct_haspart },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_has_reply },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_ip_address },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, dct_ispartof },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, dct_modified },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_next_by_date },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_next_version },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_previous_by_date },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_previous_version },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, sioc_reply_of },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, dc_subject },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_item, dc_title },

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_post, sioc_attachment },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_post, sioc_content },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_post, dct_created },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_post, dct_modified },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_post, sioc_note },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_post, sioc_num_replies },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_post, sioc_related_to },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_post, sioc_sibling },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_post, dc_subject },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_post, dc_title },
// sioc documents mention, but don't specify, a property called content:encoded

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_role, sioc_function_of },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_role, sioc_has_scope },

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_site, sioc_has_administrator },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_site, sioc_host_of },

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_space, sioc_has_usergroup },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_space, sioc_space_of },

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_account_of },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_administrator_of },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_avatar },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_creator_of },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_email },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_email_sha1 },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_has_function },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_member_of },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_moderator_of },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_modifier_of },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_owner_of },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_user, sioc_subscriber_of },

    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_usergroup, sioc_has_member },
    { { s_sioc, 1, 0 }, { 0, 0 }, sioc_usergroup, sioc_usergroup_of },

    // skos
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_altlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skosxl_altlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_broadmatch },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_broader },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_broadertransitive },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_closematch },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_exactmatch },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_hiddenlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skosxl_hiddenlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_inscheme },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_narrowmatch },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_narrower },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_narrowertransitive },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_preflabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skosxl_preflabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_related },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_relatedmatch },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_semanticrelation },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_concept, skos_topconceptof },

    { { s_skos, 1, 0 }, { 0, 0 }, skos_conceptscheme, skos_altlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_conceptscheme, skosxl_altlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_conceptscheme, skos_hiddenlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_conceptscheme, skosxl_hiddenlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_conceptscheme, skos_hastopconcept },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_conceptscheme, skos_preflabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_conceptscheme, skosxl_preflabel },

    { { s_skos, 1, 0 }, { 0, 0 }, skos_collection, skos_altlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_collection, skosxl_altlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_collection, skos_hiddenlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_collection, skosxl_hiddenlabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_collection, skos_member },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_collection, skos_preflabel },
    { { s_skos, 1, 0 }, { 0, 0 }, skos_collection, skosxl_preflabel },

    { { s_skos, 1, 0 }, { 0, 0 }, skos_orderedcollection, skos_memberlist },

    // skosxl
    { { s_skosxl, 1, 0 }, { 0, 0 }, skosxl_label, skosxl_labelrelation },
    { { s_skosxl, 1, 0 }, { 0, 0 }, skosxl_label, skosxl_literalform },

    // sosa
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_actuatableproperty, sosa_isacteduponby },

    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_actuation, sosa_hasfeatureofinterest },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_actuation, sosa_hasresult },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_actuation, sosa_hassimpleresult },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_actuation, sosa_madebyactuator },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_actuation, sosa_usedprocedure },

    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_actuator, sosa_actsonproperty },
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_actuator, ssn_forproperty },
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_actuator, ssn_isimplementedby },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_actuator, sosa_ishostedby },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_actuator, sosa_madeactuation },

    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_featureofinterest, ssn_hasproperty },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_featureofinterest, sosa_hassample },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_featureofinterest, sosa_isfeatureofinterest },

    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_observableproperty, sosa_isobservedby },

    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_observation, sosa_hasfeatureofinterest },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_observation, sosa_isobservedby },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_observation, sosa_madebysensor },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_observation, sosa_observedproperty },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_observation, sosa_hasresult },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_observation, sosa_hassimpleresult },
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_observation, ssn_wasoriginatedby },

    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_platform, sosa_hosts },
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_platform, ssn_indeployment },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_platform, sosa_ishostedby },

    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_procedure, ssn_hasinput },
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_procedure, ssn_hasoutput },
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_procedure, ssn_implements },

    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_result, sosa_isresultof },

    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sample, sosa_isfeatureofinterest },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sample, sosa_isresultof },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sample, sosa_issampleof },

    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sampler, sosa_ishostedby },
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_sampler, ssn_isimplementedby },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sampler, sosa_madesampling },

    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sampling, sosa_hasfeatureofinterest },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sampling, sosa_isobservedby },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sampling, sosa_madebysampler },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sampling, sosa_hasresult },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sampling, sosa_hassimpleresult },

    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_sensor, ssn_detects },
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_sensor, ssn_forproperty },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sensor, sosa_ishostedby },
    { { s_ssn, 1, 0 }, { 0, 0 }, sosa_sensor, ssn_isimplementedby },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sensor, sosa_madeobservation },
    { { s_sosa, 1, 0 }, { 0, 0 }, sosa_sensor, sosa_observes },

    // ssn
    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_deployment, ssn_deployedplatform },
    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_deployment, ssn_deployedsystem },
    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_deployment, ssn_forproperty },

    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_property, ssn_ispropertyof },

    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_stimulus, ssn_isproxyfor },

    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_system, ssn_hasdeployment },
    { { s_ssn, 1, 0 }, { 0, 0 }, ssn_system, ssn_hassubsystem },

    // time
    { { s_time, 1, 0 }, { 0, 0 }, time_datetimeinterval, tp_hasdatetimedescription },

    { { s_time, 1, 0 }, { 0, 0 }, time_duration, tp_numericduration },

    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_day },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_dayofweek },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_dayofyear },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_hour },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_minute },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_month },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_monthofyear },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_second },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_timezone },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_week },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldatetimedescription, tp_year },

    { { s_time, 1, 0 }, { 0, 0 }, time_generaldurationdescription, tp_days },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldurationdescription, tp_hastrs },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldurationdescription, tp_hours },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldurationdescription, tp_minutes },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldurationdescription, tp_months },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldurationdescription, tp_seconds },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldurationdescription, tp_weeks },
    { { s_time, 1, 0 }, { 0, 0 }, time_generaldurationdescription, tp_years },

    { { s_time, 1, 0 }, { 0, 0 }, time_instant, tp_indatetime },
    { { s_time, 1, 0 }, { 0, 0 }, time_instant, tp_intemporalposition },
    { { s_time, 1, 0 }, { 0, 0 }, time_instant, tp_intimeposition },
    { { s_time, 1, 0 }, { 0, 0 }, time_instant, tp_inxsddate },
    { { s_time, 1, 0 }, { 0, 0 }, time_instant, tp_inxsddatetime },
    { { s_time, 1, 0 }, { 0, 0 }, time_instant, tp_inxsddatetimestamp },
    { { s_time, 1, 0 }, { 0, 0 }, time_instant, tp_inxsdgyear },
    { { s_time, 1, 0 }, { 0, 0 }, time_instant, tp_inxsdgyearmonth },

    { { s_time, 1, 0 }, { 0, 0 }, time_interval, tp_inside },

    { { s_time, 1, 0 }, { 0, 0 }, time_position, tp_nominalposition },

    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalafter },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalbefore },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalcontains },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervaldisjoint },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalduring },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalequals },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalfinishedby },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalfinishes },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalin },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalmeets },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalmetby },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervaloverlappedby },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervaloverlaps },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalstartedby },
    { { s_time, 1, 0 }, { 0, 0 }, time_properinterval, tp_intervalstarts },

    { { s_time, 1, 0 }, { 0, 0 }, time_temporalposition, tp_hastrs },

    { { s_time, 1, 0 }, { 0, 0 }, time_temporalentity, tp_after },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalentity, tp_before },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalentity, tp_hasbeginning },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalentity, tp_hasduration },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalentity, tp_hasend },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalentity, tp_hastemporalduration },
    { { s_time, 1, 0 }, { 0, 0 }, time_temporalentity, tp_hasxsduration },

    { { s_time, 1, 0 }, { 0, 0 }, time_timeposition, tp_numericposition },

    // data vocabulary
    { { s_v, 1, 0 }, { 0, 0 }, v_breadcrumb, v_child },
    { { s_v, 1, 0 }, { 0, 0 }, v_breadcrumb, v_title },
    { { s_v, 1, 0 }, { 0, 0 }, v_breadcrumb, v_url },

    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_description },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_duration },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_enddate },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_eventtype },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_geo },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_latitude },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_location },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_longitude },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_photo },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_startdate },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_summary },
    { { s_v, 1, 0 }, { 0, 0 }, v_event, v_url },

    { { s_v, 1, 0 }, { 0, 0 }, v_offer, v_availability },
    { { s_v, 1, 0 }, { 0, 0 }, v_offer, v_condition },
    { { s_v, 1, 0 }, { 0, 0 }, v_offer, v_currency },
    { { s_v, 1, 0 }, { 0, 0 }, v_offer, v_identifier },
    { { s_v, 1, 0 }, { 0, 0 }, v_offer, v_itemoffered },
    { { s_v, 1, 0 }, { 0, 0 }, v_offer, v_offerurl },
    { { s_v, 1, 0 }, { 0, 0 }, v_offer, v_price },
    { { s_v, 1, 0 }, { 0, 0 }, v_offer, v_pricevaliduntil },
    { { s_v, 1, 0 }, { 0, 0 }, v_offer, v_seller },
    { { s_v, 1, 0 }, { 0, 0 }, v_offer, v_url },

    { { s_v, 1, 0 }, { 0, 0 }, v_offeraggregate, v_availability },
    { { s_v, 1, 0 }, { 0, 0 }, v_offeraggregate, v_condition },
    { { s_v, 1, 0 }, { 0, 0 }, v_offeraggregate, v_currency },
    { { s_v, 1, 0 }, { 0, 0 }, v_offeraggregate, v_highprice },
    { { s_v, 1, 0 }, { 0, 0 }, v_offeraggregate, v_identifier },
    { { s_v, 1, 0 }, { 0, 0 }, v_offeraggregate, v_itemoffered },
    { { s_v, 1, 0 }, { 0, 0 }, v_offeraggregate, v_lowprice },
    { { s_v, 1, 0 }, { 0, 0 }, v_offeraggregate, v_offercount },
    { { s_v, 1, 0 }, { 0, 0 }, v_offeraggregate, v_offerurl },

    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_acquaintance },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_address },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_adr },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_affiliation },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_contact },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_countryname },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_fn },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_friend },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_locality },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_name },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_org },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_photo },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_postcode },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_region },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_streetaddress },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_title },
    { { s_v, 1, 0 }, { 0, 0 }, v_person, v_url },

    { { s_v, 1, 0 }, { 0, 0 }, v_product, v_brand },
    { { s_v, 1, 0 }, { 0, 0 }, v_product, v_category },
    { { s_v, 1, 0 }, { 0, 0 }, v_product, v_description },
    { { s_v, 1, 0 }, { 0, 0 }, v_product, v_fn },
    { { s_v, 1, 0 }, { 0, 0 }, v_product, v_identifier },
    { { s_v, 1, 0 }, { 0, 0 }, v_product, v_image },
    { { s_v, 1, 0 }, { 0, 0 }, v_product, v_name },
    { { s_v, 1, 0 }, { 0, 0 }, v_product, v_offerdetails },
    { { s_v, 1, 0 }, { 0, 0 }, v_product, v_reviewed },

    { { s_v, 1, 0 }, { 0, 0 }, v_review, v_best },
    { { s_v, 1, 0 }, { 0, 0 }, v_review, v_dtreviewed },
    { { s_v, 1, 0 }, { 0, 0 }, v_review, v_description },
    { { s_v, 1, 0 }, { 0, 0 }, v_review, v_fn },
    { { s_v, 1, 0 }, { 0, 0 }, v_review, v_item },
    { { s_v, 1, 0 }, { 0, 0 }, v_review, v_itemreviewed },
    { { s_v, 1, 0 }, { 0, 0 }, v_review, v_summary },
    { { s_v, 1, 0 }, { 0, 0 }, v_review, v_rating },
    { { s_v, 1, 0 }, { 0, 0 }, v_review, v_reviewer },
    { { s_v, 1, 0 }, { 0, 0 }, v_review, v_worst },

    { { s_v, 1, 0 }, { 0, 0 }, v_reviewaggregate, v_count },
    { { s_v, 1, 0 }, { 0, 0 }, v_reviewaggregate, v_item },
    { { s_v, 1, 0 }, { 0, 0 }, v_reviewaggregate, v_itemreviewed },
    { { s_v, 1, 0 }, { 0, 0 }, v_reviewaggregate, v_rating },
    { { s_v, 1, 0 }, { 0, 0 }, v_reviewaggregate, v_summary },
    { { s_v, 1, 0 }, { 0, 0 }, v_reviewaggregate, v_votes },

    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrproperties, vcard_country },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrproperties, vcard_extadd },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrproperties, vcard_labelp },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrproperties, vcard_locality },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrproperties, vcard_pobox },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrproperties, vcard_pcode },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrproperties, vcard_region },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_adrproperties, vcard_street },

    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_country_name },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_extended_address },
    { { s_vcard, 3, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_labelp },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_locality },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_post_office_box },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_postcode },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_region },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_address, vcard_street_address },

    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_location, vcard_latitude },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_location, vcard_longitude },

    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_name, vcard_additional_name },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_name, vcard_family_name },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_name, vcard_given_name },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_name, vcard_honourific_prefix },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_name, vcard_honourific_suffix },

    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_nproperties, vcard_family },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_nproperties, vcard_given },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_nproperties, vcard_other },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_nproperties, vcard_prefix },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_nproperties, vcard_suffix },

    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_organisation, vcard_organisation_name },
    { { s_vcard, 2, 0 }, { s_vcard, 3, 0 }, vcard_organisation, vcard_organisation_unit },

    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_orgproperties, vcard_orgname },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, vcard_orgproperties, vcard_orgunit },

    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_tel, rdf_value },

    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_adr },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_agent },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_bday },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_category },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_class },
    { { s_vcard, 3, 0 }, { 0, 0 }, vcard_vcard, vcard_hasfax },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_fn },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_geo },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_pemail },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_homeadr },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_hometel },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_key },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_logo },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_mailer },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_mobileemail },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_mobiletel },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_n },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_nickname },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_note },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_org },
    { { s_vcard, 3, 0 }, { 0, 0 }, vcard_vcard, vcard_organisation_name },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_personalemail },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_phone },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_photo },
    { { s_vcard, 3, 0 }, { 0, 0 }, vcard_vcard, vcard_prodid },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_role },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_sort_string },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_sound },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_title },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_tz },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_uid },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_unlabeledadr },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_unlabeledemail },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_unlabeledtel },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_url },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_workadr },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_workemail },
    { { s_vcard, 2, 0 }, { 0, 0 }, vcard_vcard, vcard_worktel },

    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_adr },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_category },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_hasfax },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_fn },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_geo },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_hasemail },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_key },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_n },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_nickname },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_note },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_organisation_name },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_phone },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_photo },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_prodid },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_sound },
    { { s_vcard, 4, 0 }, { 0, 0 }, vcard_vkind, vcard_url },

    // void
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_class },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_classes },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_classpartition },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_datadump },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_distinctobjects },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_distinctsubjects },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_documents },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_entities },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_exampleresource },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_feature },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_opensearchdescription },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_properties },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_property },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_propertypartition },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_rootresource },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_sparqlendpoint },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_subset },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_triples },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_urilookupendpoint },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_uriregexpattern },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_urispace },
    { { s_void, 1, 0 }, { 0, 0 }, void_dataset, void_vocabulary },

    { { s_void, 1, 0 }, { 0, 0 }, void_linkset, void_linkpredicate },
    { { s_void, 1, 0 }, { 0, 0 }, void_linkset, void_objectstarget },
    { { s_void, 1, 0 }, { 0, 0 }, void_linkset, void_subjectstarget },
    { { s_void, 1, 0 }, { 0, 0 }, void_linkset, void_subset },

    // wdr
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_attributiontype, wdr_abouthosts },

    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_excludeexactpaths },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_excludehosts },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_excludeiripattern },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_excludepathcontains },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_excludepathendswith },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_excludepathstartswith },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_excludequerycontains },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_excludeports },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_excluderesources },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_excludeschemes },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_includeexactpaths },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_includehosts },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_includeiripattern },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_includepathcontains },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_includepathendswith },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_includepathstartswith },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_includequerycontains },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_includeports },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_includeresources },
    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_irisettype, wdr_includeschemes },

    { { s_wdr, 1, 0 }, { 0, 0 }, wdr_querycontainstype, wdr_delimiter },

    // living standard
    { { s_whatwg, 1, 0 }, { 0, 0 }, wwg_work, wp_author },
    { { s_whatwg, 1, 0 }, { 0, 0 }, wwg_work, wp_license },
    { { s_whatwg, 1, 0 }, { 0, 0 }, wwg_work, wp_title },
    { { s_whatwg, 1, 0 }, { 0, 0 }, wwg_work, wp_work },

    // xpath (required, vaguely, by grddl)
//    { { s_xpath, 1, 0 }, { 0, 0 }, xpath_rootnode, grddlp_transformation },

    { { 0, 0 }, { 0, 0 }, sty_illegal, sp_illegal } };

typedef ::std::pair < e_schema_type, e_schema_property > mmd_key;
typedef ::std::multimap < mmd_key, microdata_structure* > mmd_t;
mmd_t micromap;

void microdata_init (nitpick& )
{   PRESUME (micromap.empty (), __FILE__, __LINE__);
#ifdef _MSC_VER
#pragma warning (push, 3)
#pragma warning (disable : 26481)
#endif // _MSC_VER
    for (microdata_structure* p = &schema_structure [0]; p -> record_ != sty_illegal; ++p)
#ifdef _MSC_VER
#pragma warning (pop)
#endif // _MSC_VER
    {   VERIFY_NOT_NULL (p, __FILE__, __LINE__);
        if (micromap.find (::std::pair < e_schema_type, e_schema_property > (p -> record_, p -> property_)) != micromap.end ())
            ::std::cerr << "microdata_init reports " << sch::name (p -> record_) << " (" << p -> record_ << "), " << schema_property_name (p -> property_) << " (" << p -> property_ << ") repeated\n";
        else
        {   mmd_key k (p -> record_, p -> property_);
            micromap.insert (mmd_t::value_type (k, p));
#if defined (_DEBUG)
            bool found = false;
            if (! check_schema_property_version (p -> from_, p -> to_, p -> property_, found))
                if (found)
                    ::std::cerr << "microdata_init reports " << sch::name (p -> record_) << " (" << p -> record_ << "), " <<
                                                                schema_property_name (p -> property_) << " (" << p -> property_ << ") is incompatible with version in property table\n";
                else
                    ::std::cerr << "microdata_init reports " << sch::name (p -> record_) << " (" << p -> record_ << "), " <<
                                                                schema_property_name (p -> property_) << " (" << p -> property_ << ") is not found in property table\n";
#endif // _DEBUG
} } }

bool is_schema_property (const e_schema_type s, const e_schema_property p)
{   mmd_t::const_iterator i = micromap.find (mmd_key (s, p));
    if (i == micromap.cend ()) return false;
    VERIFY_NOT_NULL (i -> second, __FILE__, __LINE__);
    return does_apply < schema_version > (context.schema_ver (sch::root (s)), i -> second -> from_, i -> second -> to_); };
