/*
ssc (static site checker)
Copyright (c) 2020-2022 Dylan Harris
https://dylanharris.org/

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public Licence as published by
the Free Software Foundation, either version 3 of the Licence, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public Licence for more details.

You should have received a copy of the GNU General Public
Licence along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "main/standard.h"
#include "main/context.h"
#include "schema/schema_type.h"
#include "utility/quote.h"
#include "microformat/def.h"

#define MAKE_SIMPLE_TYPE(T)     static_cast < flags_t > (T)

struct symbol_entry < schema_version, e_schema_type, e_schema, s_schema > schema_type_symbol_table [] =
{
    { { s_faux, 1, 0 }, { 0, 0 }, "__FAUX__", anything, s_faux },

    // activity streams etc.
    { { s_as, 2, 0 }, { 0, 0 }, "Accept", asc_accept, s_as },
    { { s_as, 1, 0 }, { 0, 0 }, "Activity", asc_activity, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Add", asc_add, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Announce", asc_announce, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Application", asc_application, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Arrive", asc_arrive, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Article", asc_article, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Audio", asc_audio, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Block", asc_block, s_as },
    { { s_as, 1, 0 }, { 0, 0 }, "Collection", asc_collection, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "CollectionPage", asc_collectionpage, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Create", asc_create, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Delete", asc_delete, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Dislike", asc_dislike, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Document", asc_document, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Event", asc_event, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Flag", asc_flag, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Follow", asc_follow, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Group", asc_group, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Ignore", asc_ignore, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Image", asc_image, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "IntransitiveActivity", asc_intransitiveactivity, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Invite", asc_invite, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "IsContact", asc_iscontact, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "IsFollowedBy", asc_isfollowedby, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "IsFollowing", asc_isfollowing, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "IsMember", asc_ismember, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Join", asc_join, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Leave", asc_leave, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Like", asc_like, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Link", asc_link, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Listen", asc_listen, s_as },
    { { s_as, 1, 0 }, { 1, 0 }, "MediaLink", asc_medialink, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Mention", asc_mention, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Move", asc_move, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Note", asc_note, s_as },
    { { s_as, 1, 0 }, { 0, 0 }, "Object", asc_object, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Offer", asc_offer, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "OrderedCollection", asc_orderedcollection, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "OrderedCollectionPage", asc_orderedcollectionpage, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Organization", asc_organisation, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Page", asc_page, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Person", asc_person, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Place", asc_place, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Profile", asc_profile, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Question", asc_question, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Read", asc_read, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Reject", asc_reject, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Relationship", asc_relationship, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Remove", asc_remove, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Service", asc_service, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "TentativeAccept", asc_tentativeaccept, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "TentativeReject", asc_tentativereject, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Tombstone", asc_tombstone, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Travel", asc_travel, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Undo", asc_undo, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Update", asc_update, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "Video", asc_video, s_as },
    { { s_as, 2, 0 }, { 0, 0 }, "View", asc_view, s_as },

    // bibo
    { { s_bibo, 1, 3 }, { 0, 0 }, "AcademicArticle", bibo_academicarticle, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Article", bibo_article, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "AudioDocument", bibo_audiodocument, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "AudioVisualDocument", bibo_audiovisualdocument, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "bdarcus", bibo_bdarcus, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Bill", bibo_bill, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Book", bibo_book, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "BookSection", bibo_booksection, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Brief", bibo_brief, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Chapter", bibo_chapter, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Code", bibo_code, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "CollectedDocument", bibo_collecteddocument, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Collection", bibo_collection, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Conference", bibo_conference, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "CourtReporter", bibo_courtreporter, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "degrees/ma", bibo_degrees_ma, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "degrees/ms", bibo_degrees_ms, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "degrees/phd", bibo_degrees_phd, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Document", bibo_document, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "DocumentPart", bibo_documentpart, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "DocumentStatus", bibo_documentstatus, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "EditedBook", bibo_editedbook, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Email", bibo_email, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Event", bibo_event, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Excerpt", bibo_excerpt, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "fgiasson", bibo_fgiasson, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Film", bibo_film, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Hearing", bibo_hearing, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Image", bibo_image, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Interview", bibo_interview, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Issue", bibo_issue, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Journal", bibo_journal, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "LegalCaseDocument", bibo_legalcasedocument, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "LegalDecision", bibo_legaldecision, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "LegalDocument", bibo_legaldocument, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Legislation", bibo_legislation, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Letter", bibo_letter, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Magazine", bibo_magazine, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Manual", bibo_manual, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Manuscript", bibo_manuscript, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Map", bibo_map, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "MultiVolumeBook", bibo_multivolumebook, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Newspaper", bibo_newspaper, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Note", bibo_note, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Patent", bibo_patent, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Performance", bibo_performance, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Periodical", bibo_periodical, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "PersonalCommunication", bibo_personalcommunication, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "PersonalCommunicationDocument", bibo_personalcommunicationdocument, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Proceedings", bibo_proceedings, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Quote", bibo_quote, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "ReferenceSource", bibo_referencesource, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Report", bibo_report, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Series", bibo_series, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Slide", bibo_slide, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Slideshow", bibo_slideshow, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Standard", bibo_standard, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "status/accepted", bibo_status_accepted, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "status/draft", bibo_status_draft, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "status/forthcoming", bibo_status_forthcoming, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "status/nonPeerReviewed", bibo_status_nonpeerreviewed, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "status/peerReviewed", bibo_status_peerreviewed, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "status/published", bibo_status_published, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "status/rejected", bibo_status_rejected, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "status/unpublished", bibo_status_unpublished, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Statute", bibo_statute, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Thesis", bibo_thesis, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "ThesisDegree", bibo_thesisdegree, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Webpage", bibo_webpage, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Website", bibo_website, s_bibo },
    { { s_bibo, 1, 3 }, { 0, 0 }, "Workshop", bibo_workshop, s_bibo },

    // common tag
    { { s_ctag, 1, 0 }, { 0, 0 }, "AuthorTag", ctag_author, s_ctag },
    { { s_ctag, 1, 0 }, { 0, 0 }, "AutoTag", ctag_auto, s_ctag },
    { { s_ctag, 1, 0 }, { 0, 0 }, "ReaderTag", ctag_reader, s_ctag },
    { { s_ctag, 1, 0 }, { 0, 0 }, "Tag", ctag_tag, s_ctag },

    // contrary sausages
    { { s_csvw, 1, 0 }, { 0, 0 }, "Cell", csv_cell, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "Column", csv_column, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "Datatype", csv_datatype, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "Dialect", csv_dialect, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "Direction", csv_direction, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "ForeignKey", csv_foreignkey, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "NumericFormat", csv_numericformat, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "Row", csv_row, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "Schema", csv_schema, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "Table", csv_table, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "TableGroup", csv_tablegroup, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "TableReference", csv_tablereference, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "Transformation", csv_transformation, s_csvw },
    { { s_csvw, 1, 0 }, { 0, 0 }, "uriTemplate", csv_uritemplate, s_csvw },

    // data quality
    { { s_daq, 1, 0 }, { 0, 0 }, "Metric", daq_metric, s_daq },
    { { s_daq, 1, 0 }, { 0, 0 }, "Category", daq_category, s_daq },
    { { s_daq, 1, 0 }, { 0, 0 }, "Dimension", daq_dimension, s_daq },
    { { s_daq, 1, 0 }, { 0, 0 }, "Observation", daq_observation, s_daq },
    { { s_daq, 1, 0 }, { 0, 0 }, "QualityGraph", daq_qualitygraph, s_daq },

    // DCAM
    { { s_dcam, 1, 0 }, { 0, 0 }, "VocabularyEncodingScheme", dcam_vocabularyencodingscheme, s_dcam },

    // data catalogue
    { { s_dcat, 1, 0 }, { 0, 0 }, "Catalog", dca_catalogue, s_dcat },
    { { s_dcat, 1, 0 }, { 0, 0 }, "CatalogRecord", dca_cataloguerecord, s_dcat },
    { { s_dcat, 1, 0 }, { 0, 0 }, "DataService", dca_dataservice, s_dcat },
    { { s_dcat, 1, 0 }, { 0, 0 }, "Dataset", dca_dataset, s_dcat },
    { { s_dcat, 1, 0 }, { 0, 0 }, "Distribution", dca_distribution, s_dcat },
    { { s_dcat, 2, 0 }, { 0, 0 }, "Relationship", dca_relationship, s_dcat },
    { { s_dcat, 2, 0 }, { 0, 0 }, "Resource", dca_resource, s_dcat },
    { { s_dcat, 2, 0 }, { 0, 0 }, "Role", dca_role, s_dcat },

    // DCMI
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Collection", dcmi_collection, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Dataset", dcmi_dataset, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Event", dcmi_event, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Image", dcmi_image, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "InteractiveResource", dcmi_interactiveresource, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "MovingImage", dcmi_movingimage, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "PhysicalObject", dcmi_physicalobject, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Service", dcmi_service, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Software", dcmi_software, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Sound", dcmi_sound, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "StillImage", dcmi_stillimage, s_dcmi },
    { { s_dcmi, 1, 0 }, { 0, 0 }, "Text", dcmi_text, s_dcmi },

    // dublin core terms
    { { s_dct, 1, 0 }, { 0, 0 }, "Agent", dct_agent, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "AgentClass", dct_agentclass, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "BibliographicResource", dct_bibliographicresource, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "FileFormat", dct_fileformat, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "Frequency", dct_frequency, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "Jurisdiction", dct_jurisdiction, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "LicenseDocument", dct_licencedocument, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "LinguisticSystem", dct_linguisticsystem, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "Location", dct_location, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "LocationPeriodOrJurisdiction", dct_locationperiodorjurisdiction, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "MediaType", dct_mediatype, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "MediaTypeOrExtent", dct_mediatypeorextent, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "MethodOfAccrual", dct_methodofaccrual, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "MethodOfInstruction", dct_methodofinstruction, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "PeriodOfTime", dct_periodoftime, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "PhysicalMedium", dct_physicalmedium, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "PhysicalResource", dct_physicalresource, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "Policy", dct_policy, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "ProvenanceStatement", dct_provenancestatement, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "RightsStatement", dct_rightsstatement, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "SizeOrDuration", dct_sizeorduration, s_dct },
    { { s_dct, 1, 0 }, { 0, 0 }, "Standard", dct_standard, s_dct },

    // doap
    { { s_doap, 1, 0 }, { 0, 0 }, "ArchRepository", doap_archrepository, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "BazaarBranch", doap_bazaarbranch, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "BKRepository", doap_bkrepository, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "CVSRepository", doap_cvsrepository, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "DarcsRepository", doap_darcsrepository, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "GitBranch", doap_gitbranch, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "GitRepository", doap_gitrepository, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "HgRepository", doap_hgrepository, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "Project", doap_project, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "Repository", doap_repository, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "Specification", doap_specification, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "SVNRepository", doap_svnrepository, s_doap },
    { { s_doap, 1, 0 }, { 0, 0 }, "Version", doap_version, s_doap },

    // data quality
    { { s_dqv, 1, 0 }, { 0, 0 }, "Category", dqv_category, s_dqv },
    { { s_dqv, 1, 0 }, { 0, 0 }, "Dimension", dqv_dimension, s_dqv },
    { { s_dqv, 1, 0 }, { 0, 0 }, "Metric", dqv_metric, s_dqv },
    { { s_dqv, 1, 0 }, { 0, 0 }, "QualityAnnotation", dqv_qualityannotation, s_dqv },
    { { s_dqv, 1, 0 }, { 0, 0 }, "QualityCertificate", dqv_qualitycertificate, s_dqv },
    { { s_dqv, 1, 0 }, { 0, 0 }, "QualityMetadata", dqv_qualitymetadata, s_dqv },
    { { s_dqv, 1, 0 }, { 0, 0 }, "QualityMeasurement", dqv_qualitymeasurement, s_dqv },
    { { s_dqv, 1, 0 }, { 0, 0 }, "QualityMeasurementDataset", dqv_qualitymeasurementdataset, s_dqv },
    { { s_dqv, 1, 0 }, { 0, 0 }, "QualityPolicy", dqv_qualitypolicy, s_dqv },
    { { s_dqv, 1, 0 }, { 0, 0 }, "UserQualityFeedback", dqv_userqualityfeedback, s_dqv },

    // duv
    { { s_duv, 1, 0 }, { 0, 0 }, "RatingFeedback", duv_ratingfeedback, s_duv },
    { { s_duv, 1, 0 }, { 0, 0 }, "Usage", duv_usage, s_duv },
    { { s_duv, 1, 0 }, { 0, 0 }, "UsageTool", duv_usagetool, s_duv },
    { { s_duv, 1, 0 }, { 0, 0 }, "UserFeedback", duv_userfeedback, s_duv },

    // earl
    { { s_earl, 1, 0 }, { 0, 0 }, "Assertion", earl_assertion, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "Assertor", earl_assertor, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "automatic", earl_automatic, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "CannotTell", earl_cannottell, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "cantTell", earl_canttell, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "Fail", earl_fail, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "failed", earl_failed, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "inapplicable", earl_inapplicable, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "manual", earl_manual, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "NotApplicable", earl_notapplicable, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "NotTested", earl_nottested, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "OutcomeValue", earl_outcomevalue, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "Pass", earl_pass, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "passed", earl_passed, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "semiAuto", earl_semiauto, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "Software", earl_software, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "TestCase", earl_testcase, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "TestCriterion", earl_testcriterion, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "TestMode", earl_testmode, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "TestRequirement", earl_testrequirement, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "TestResult", earl_testresult, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "TestSubject", earl_testsubject, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "undisclosed", earl_undisclosed, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "unknownMode", earl_unknownmode, s_earl },
    { { s_earl, 1, 0 }, { 0, 0 }, "untested", earl_untested, s_earl },

    // event
    { { s_event, 1, 0 }, { 0, 0 }, "Event", event_event, s_event },
    { { s_event, 1, 0 }, { 0, 0 }, "Factor", event_factor, s_event },
    { { s_event, 1, 0 }, { 0, 0 }, "Product", event_product, s_event },

    // foaf
    { { s_foaf, 0, 1 }, { 0, 0 }, "Agent", foaf_agent, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Document", foaf_document, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Group", foaf_group, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Image", foaf_image, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "LabelProperty", foaf_labelproperty, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "OnlineAccount", foaf_onlineaccount, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "OnlineChatAccount", foaf_onlinechataccount, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "OnlineEcommerceAccount", foaf_onlineecommerceaccount, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "OnlineGamingAccount", foaf_onlinegamingaccount, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Organization", foaf_organisation, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Person", foaf_person, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "PersonalProfileDocument", foaf_personalprofiledocument, s_foaf },
    { { s_foaf, 0, 1 }, { 0, 0 }, "Project", foaf_project, s_foaf },

    // frbr core
    { { s_frbr, 1, 0 }, { 0, 0 }, "Concept", fs_concept, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "CorporateBody", fs_corporatebody, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "Endeavour", fs_endeavour, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "Event", fs_event, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "Expression", fs_expression, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "Item", fs_item, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "Manifestation", fs_manifestation, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "Object", fs_object, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "Person", fs_person, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "Place", fs_place, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "ResponsibleEntity", fs_responsibleentity, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "Subject", fs_subject, s_frbr },
    { { s_frbr, 1, 0 }, { 0, 0 }, "Work", fs_work, s_frbr },

    // good relations
    { { s_gr, 1, 0, SV_DEPRECATED }, { 0, 0 }, "ActualProductOrServiceInstance", gr_actualproductorserviceinstance, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Brand", gr_brand, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "BusinessEntity", gr_businessentity, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "BusinessEntityType", gr_businessentitytype, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "BusinessFunction", gr_businessfunction, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DayOfWeek", gr_dayofweek, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryChargeSpecification", gr_deliverychargespecification, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryMethod", gr_deliverymethod, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModeParcelService", gr_deliverymodeparcelservice, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Individual", gr_individual, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "License", gr_licence, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Location", gr_location, s_gr },
    { { s_gr, 1, 0, SV_DEPRECATED }, { 0, 0 }, "LocationOfSalesOrServiceProvisioning", gr_locationofsalesorserviceprovisioning, s_gr },
    { { s_gr, 1, 0, SV_DEPRECATED }, { 0, 0 }, "N-Ary-Relations", gr_n_ary_relations, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Offering", gr_offering, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "OpeningHoursSpecification", gr_openinghoursspecification, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PaymentChargeSpecification", gr_paymentchargespecification, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PaymentMethod", gr_paymentmethod, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PaymentMethodCreditCard", gr_paymentmethodcreditcard, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PriceSpecification", gr_pricespecification, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ProductOrService", gr_productorservice, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ProductOrServiceModel", gr_productorservicemodel, s_gr },
    { { s_gr, 1, 0, SV_DEPRECATED }, { 0, 0 }, "ProductOrServicesSomeInstancesPlaceholder", gr_productorservicessomeinstancesplaceholder, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "QualitativeValue", gr_qualitativevalue, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "QuantitativeValue", gr_quantitativevalue, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "QuantitativeValueFloat", gr_quantitativevaluefloat, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "QuantitativeValueInteger", gr_quantitativevalueinteger, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "SomeItems", gr_someitems, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "TypeAndQuantityNode", gr_typeandquantitynode, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "UnitPriceSpecification", gr_unitpricespecification, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "WarrantyPromise", gr_warrantypromise, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "WarrantyScope", gr_warrantyscope, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "AmericanExpress", gr_americanexpress, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Business", gr_business, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ByBankTransferInAdvance", gr_bybanktransferinadvance, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ByInvoice", gr_byinvoice, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Cash", gr_cash, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "CheckInAdvance", gr_checkinadvance, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "COD", gr_cod, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ConstructionInstallation", gr_constructioninstallation, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModeDirectDownload", gr_deliverymodedirectdownload, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModeFreight", gr_deliverymodefreight, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModeMail", gr_deliverymodemail, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModeOwnFleet", gr_deliverymodeownfleet, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DeliveryModePickUp", gr_deliverymodepickup, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DHL", gr_dhl, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DinersClub", gr_dinersclub, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "DirectDebit", gr_directdebit, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Discover", gr_discover, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Dispose", gr_dispose, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Enduser", gr_enduser, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "FederalExpress", gr_federalexpress, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Friday", gr_friday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "GoogleCheckout", gr_googlecheckout, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "JCB", gr_jcb, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Labor-BringIn", gr_labour_bringin, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "LeaseOut", gr_leaseout, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Maintain", gr_maintain, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "MasterCard", gr_mastercard, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Monday", gr_monday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PartsAndLabor-BringIn", gr_partsandlabour_bringin, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PartsAndLabor-PickUp", gr_partsandlabour_pickup, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PayPal", gr_paypal, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PaySwarm", gr_payswarm, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "ProvideService", gr_provideservice, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PublicHolidays", gr_publicholidays, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "PublicInstitution", gr_publicinstitution, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Repair", gr_repair, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Reseller", gr_reseller, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Saturday", gr_saturday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Sell", gr_sell, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Sunday", gr_sunday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Thursday", gr_thursday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Tuesday", gr_tuesday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "UPS", gr_ups, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "VISA", gr_visa, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Wednesday", gr_wednesday, s_gr },
    { { s_gr, 1, 0 }, { 0, 0 }, "Buy", gr_buy, s_gr },

    // grddl
    { { s_grddl, 1, 0 }, { 0, 0 }, "Transformation", grddl_transformation, s_grddl },
    { { s_grddl, 1, 0 }, { 0, 0 }, "TransformationProperty", grddl_transformationproperty, s_grddl },

    // gs1
    { { s_gs1, 1, 1 }, { 0, 0 }, "AdditionalProductClassificationDetails", gs1s_additionalproductclassificationdetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AdditiveDetails", gs1s_additivedetails, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "Address", gs1s_address, s_gs1 },
    { { s_gs1, 1, 3 }, { 0, 0 }, "AI", gs1s_ai, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenDetails", gs1s_allergendetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode", gs1s_allergentypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-1%2C3-BIS-%282%2C4-DIAMINOPHENOXY%29PROPANE", gs1s_allergentypecode_13_bis_24_diaminophenoxypropane, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-1-NAPHTHOL", gs1s_allergentypecode_1_naphthol, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-2%2C6-DIMETHOXY-3%2C5-PYRIDINEDIAMINE_HCL", gs1s_allergentypecode_26_dimethoxy_35_pyridinediamine_hcl, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-2-HYDROXYETHYL-PICRAMIC_ACID", gs1s_allergentypecode_2_hydroxyethyl_picramic_acid, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-2-METHYL-5-HYDROXYETHYLAMINOPHENOL", gs1s_allergentypecode_2_methyl_5_hydroxyethylaminophenol, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-3-AMINO-2%2C4-DICHLOROPHENOL", gs1s_allergentypecode_3_amino_24_dichlorophenol, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-3-AMINOPHENOL", gs1s_allergentypecode_3_aminophenol, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-4-AMINO-3-NITROPHENOL", gs1s_allergentypecode_4_amino_3_nitrophenol, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-4-HYDROXY-PROPYLAMINO-3-NITROPHENOL", gs1s_allergentypecode_4_hydroxy_propylamino_3_nitrophenol, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-ABALONE", gs1s_allergentypecode_abalone, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-ALMONDS", gs1s_allergentypecode_almonds, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-ALPHA_ISOMETHYL_IONONE", gs1s_allergentypecode_alpha_isomethyl_ionone, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-AMYLCINNAMYL_ALCOHOL", gs1s_allergentypecode_amylcinnamyl_alcohol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-AMYL_CINNAMAL", gs1s_allergentypecode_amyl_cinnamal, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-ANCHOVY", gs1s_allergentypecode_anchovy, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-ANISE_ALCOHOL", gs1s_allergentypecode_anise_alcohol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-BARLEY", gs1s_allergentypecode_barley, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-BARNACLE", gs1s_allergentypecode_barnacle, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-BASA", gs1s_allergentypecode_basa, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-BASS", gs1s_allergentypecode_bass, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-BEECH_NUTS", gs1s_allergentypecode_beech_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-BEEF", gs1s_allergentypecode_beef, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-BENZYL_ALCOHOL", gs1s_allergentypecode_benzyl_alcohol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-BENZYL_BENZOATE", gs1s_allergentypecode_benzyl_benzoate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-BENZYL_CINNAMATE", gs1s_allergentypecode_benzyl_cinnamate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-BENZYL_SALICYLATE", gs1s_allergentypecode_benzyl_salicylate, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-BLUEFISH", gs1s_allergentypecode_bluefish, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-BRAZIL_NUTS", gs1s_allergentypecode_brazil_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-BREAM", gs1s_allergentypecode_bream, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-BUTTERNUTS", gs1s_allergentypecode_butternuts, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-BUTYLPHENYL_METHYLPROPIONATE", gs1s_allergentypecode_butylphenyl_methylpropionate, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CARP", gs1s_allergentypecode_carp, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CARROTS", gs1s_allergentypecode_carrots, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CASHEW_NUTS", gs1s_allergentypecode_cashew_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CATFISH", gs1s_allergentypecode_catfish, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CELERY", gs1s_allergentypecode_celery, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CEREALS_CONTAINING_GLUTEN", gs1s_allergentypecode_cereals_containing_gluten, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CHAR", gs1s_allergentypecode_char, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CHESTNUTS", gs1s_allergentypecode_chestnuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CHICKEN_MEAT", gs1s_allergentypecode_chicken_meat, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CHINQUAPINS", gs1s_allergentypecode_chinquapins, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CHUB", gs1s_allergentypecode_chub, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CINNAMAL", gs1s_allergentypecode_cinnamal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CINNAMYL_ALCOHOL", gs1s_allergentypecode_cinnamyl_alcohol, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CISCO", gs1s_allergentypecode_cisco, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CITRAL", gs1s_allergentypecode_citral, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CITRONELLOL", gs1s_allergentypecode_citronellol, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CLAM", gs1s_allergentypecode_clam, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-COCKLE", gs1s_allergentypecode_cockle, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-COCOA", gs1s_allergentypecode_cocoa, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-COCONUTS", gs1s_allergentypecode_coconuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-COD", gs1s_allergentypecode_cod, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CONCH", gs1s_allergentypecode_conch, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CORIANDER", gs1s_allergentypecode_coriander, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CORN", gs1s_allergentypecode_corn, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-COTTON_SEEDS", gs1s_allergentypecode_cotton_seeds, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-COUMARIN", gs1s_allergentypecode_coumarin, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CRAB", gs1s_allergentypecode_crab, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-CRAWFISH", gs1s_allergentypecode_crawfish, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-CRUSTACEANS", gs1s_allergentypecode_crustaceans, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-D-LIMONENE", gs1s_allergentypecode_d_limonene, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-DIAMINOPHENOLS", gs1s_allergentypecode_diaminophenols, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-EEL", gs1s_allergentypecode_eel, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-EGGS", gs1s_allergentypecode_eggs, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-EUGENOL", gs1s_allergentypecode_eugenol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-EVERNIA_FURFURACEA", gs1s_allergentypecode_evernia_furfuracea, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-EVERNIA_PRUNASTRI", gs1s_allergentypecode_evernia_prunastri, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-FARNESOL", gs1s_allergentypecode_farnesol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-FISH", gs1s_allergentypecode_fish, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-FLOUNDER", gs1s_allergentypecode_flounder, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-GERANIOL", gs1s_allergentypecode_geraniol, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-GINKGO_NUTS", gs1s_allergentypecode_ginkgo_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-GLUTAMATE", gs1s_allergentypecode_glutamate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-GLUTEN", gs1s_allergentypecode_gluten, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-GROUPER", gs1s_allergentypecode_grouper, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-HADDOCK", gs1s_allergentypecode_haddock, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-HAKE", gs1s_allergentypecode_hake, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-HALIBUT", gs1s_allergentypecode_halibut, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-HAZELNUTS", gs1s_allergentypecode_hazelnuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-HC_BLUE_NO_11", gs1s_allergentypecode_hc_blue_no_11, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-HC_BLUE_NO_12", gs1s_allergentypecode_hc_blue_no_12, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-HERRING", gs1s_allergentypecode_herring, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-HEXYL_CINNAMAL", gs1s_allergentypecode_hexyl_cinnamal, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-HICKORY_NUTS", gs1s_allergentypecode_hickory_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-HYDROXYBENZOMORPHOLINE", gs1s_allergentypecode_hydroxybenzomorpholine, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-HYDROXYCITRONELLAL", gs1s_allergentypecode_hydroxycitronellal, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-HYDROXYETHYL-2-NITRO-P-TOLUIDINE", gs1s_allergentypecode_hydroxyethyl_2_nitro_p_toluidine, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-HYDROXYISOHEXYL_3-CYCLOHEXENE_CARBOXALDEHYDE_ISOEUGENOL_LIMONENE_LINAL", gs1s_allergentypecode_hydroxyisohexyl_3_cyclohexene_carboxaldehyde_isoeugenol_limonene_linal, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-HYDROXYPROPYL_BIS%28N-HYDROXYETHYL-P-PHENYLDIAMINE%29_HCL", gs1s_allergentypecode_hydroxypropyl_bisn_hydroxyethyl_p_phenyldiamine_hcl, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-ISOEUGENOL", gs1s_allergentypecode_isoeugenol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-KAMUT", gs1s_allergentypecode_kamut, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-KRILL", gs1s_allergentypecode_krill, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-LACTOSE", gs1s_allergentypecode_lactose, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-LAND", gs1s_allergentypecode_land, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-LICHEE_NUTS", gs1s_allergentypecode_lichee_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-LIMPETS", gs1s_allergentypecode_limpets, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-LINALOOL", gs1s_allergentypecode_linalool, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-LOBSTER", gs1s_allergentypecode_lobster, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-LUPINE", gs1s_allergentypecode_lupine, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-MACADAMIA_NUTS", gs1s_allergentypecode_macadamia_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-MACKEREL", gs1s_allergentypecode_mackerel, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-MAHI_MAHI", gs1s_allergentypecode_mahi_mahi, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-MARLIN", gs1s_allergentypecode_marlin, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-MELATONIN", gs1s_allergentypecode_melatonin, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-METHYL_2_OCTYNOATE", gs1s_allergentypecode_methyl_2_octynoate, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-METHYL_HEPTIN_CARBONATE", gs1s_allergentypecode_methyl_heptin_carbonate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-MILK", gs1s_allergentypecode_milk, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-MOLLUSCS", gs1s_allergentypecode_molluscs, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-MONKFISH_%28ANGLERFISH%2C_LOTTE%29", gs1s_allergentypecode_monkfish_anglerfish_lotte, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-MUSSELS", gs1s_allergentypecode_mussels, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-MUSTARD", gs1s_allergentypecode_mustard, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-NO_DECLARED_ALLERGENS", gs1s_allergentypecode_no_declared_allergens, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-OAT", gs1s_allergentypecode_oat, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-OCTOPUS", gs1s_allergentypecode_octopus, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-ORANGE_ROUGHY", gs1s_allergentypecode_orange_roughy, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-OYSTERS", gs1s_allergentypecode_oysters, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-P-METHYLAMINOPHENOL", gs1s_allergentypecode_p_methylaminophenol, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-P-PHENYLENEDIAMINE", gs1s_allergentypecode_p_phenylenediamine, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-PEANUTS", gs1s_allergentypecode_peanuts, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-PEAS", gs1s_allergentypecode_peas, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-PECAN_NUTS", gs1s_allergentypecode_pecan_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-PERCH", gs1s_allergentypecode_perch, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-PERIWINKLE", gs1s_allergentypecode_periwinkle, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-PIKE", gs1s_allergentypecode_pike, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-PILI_NUTS", gs1s_allergentypecode_pili_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-PINE_NUTS", gs1s_allergentypecode_pine_nuts, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-PISTACHIOS", gs1s_allergentypecode_pistachios, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-PLAICE", gs1s_allergentypecode_plaice, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-POD_FRUITS", gs1s_allergentypecode_pod_fruits, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-POLLOCK", gs1s_allergentypecode_pollock, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-POMPANO", gs1s_allergentypecode_pompano, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-POPPY_SEEDS", gs1s_allergentypecode_poppy_seeds, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-PORGY", gs1s_allergentypecode_porgy, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-PORK", gs1s_allergentypecode_pork, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-PRAWNS", gs1s_allergentypecode_prawns, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-PULSES", gs1s_allergentypecode_pulses, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-QUAHAUGS", gs1s_allergentypecode_quahaugs, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-QUEENSLAND_NUTS", gs1s_allergentypecode_queensland_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-ROCKFISH", gs1s_allergentypecode_rockfish, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-RYE", gs1s_allergentypecode_rye, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SALICYLATE", gs1s_allergentypecode_salicylate, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SALMON", gs1s_allergentypecode_salmon, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SARDINE", gs1s_allergentypecode_sardine, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SCALLOPS", gs1s_allergentypecode_scallops, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SEED_PRODUCTS", gs1s_allergentypecode_seed_products, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-SESAME_SEEDS", gs1s_allergentypecode_sesame_seeds, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SHARK", gs1s_allergentypecode_shark, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SHEA_NUTS", gs1s_allergentypecode_shea_nuts, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SHELLFISH", gs1s_allergentypecode_shellfish, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SHRIMP", gs1s_allergentypecode_shrimp, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SMELT", gs1s_allergentypecode_smelt, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SNAPPER", gs1s_allergentypecode_snapper, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SOLE", gs1s_allergentypecode_sole, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-SOYBEANS", gs1s_allergentypecode_soybeans, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-SPELT", gs1s_allergentypecode_spelt, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SQUID_%28CALAMARI%29", gs1s_allergentypecode_squid_calamari, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-STURGEON", gs1s_allergentypecode_sturgeon, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-SULPHUR_DIOXIDE", gs1s_allergentypecode_sulphur_dioxide, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SUNFLOWER_SEEDS", gs1s_allergentypecode_sunflower_seeds, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-SWORDFISH", gs1s_allergentypecode_swordfish, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-TILAPIA", gs1s_allergentypecode_tilapia, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-TOLUENE-2%2C5-DIAMINE", gs1s_allergentypecode_toluene_25_diamine, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-TREE_NUTS", gs1s_allergentypecode_tree_nuts, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-TREE_NUT_TRACES", gs1s_allergentypecode_tree_nut_traces, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-TRITICALE", gs1s_allergentypecode_triticale, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-TROUT", gs1s_allergentypecode_trout, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-TUNA", gs1s_allergentypecode_tuna, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-TURBOT", gs1s_allergentypecode_turbot, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-WALLEYE", gs1s_allergentypecode_walleye, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-WALNUTS", gs1s_allergentypecode_walnuts, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AllergenTypeCode-WHEAT", gs1s_allergentypecode_wheat, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-WHELKS", gs1s_allergentypecode_whelks, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-WHITEFISH", gs1s_allergentypecode_whitefish, s_gs1 },
    { { s_gs1, 1, 4 }, { 0, 0 }, "AllergenTypeCode-WHITING", gs1s_allergentypecode_whiting, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode", gs1s_anatomicalformcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-BELLY", gs1s_anatomicalformcode_belly, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-BLOOD", gs1s_anatomicalformcode_blood, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-BONE", gs1s_anatomicalformcode_bone, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-BRAIN", gs1s_anatomicalformcode_brain, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-CARCASS_NONPOULTRY", gs1s_anatomicalformcode_carcass_nonpoultry, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-DIAPHRAM", gs1s_anatomicalformcode_diaphram, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-ESOPHAGUS", gs1s_anatomicalformcode_esophagus, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-FAT", gs1s_anatomicalformcode_fat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-FEET", gs1s_anatomicalformcode_feet, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-FOREQUARTER", gs1s_anatomicalformcode_forequarter, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-GIZZARD", gs1s_anatomicalformcode_gizzard, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-HEAD", gs1s_anatomicalformcode_head, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-HEADMEAT_CHEEKMEAT", gs1s_anatomicalformcode_headmeat_cheekmeat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-HEART", gs1s_anatomicalformcode_heart, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-HINDQUARTER", gs1s_anatomicalformcode_hindquarter, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-HORN", gs1s_anatomicalformcode_horn, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-INTESTINE_LARGE", gs1s_anatomicalformcode_intestine_large, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-INTESTINE_SMALL", gs1s_anatomicalformcode_intestine_small, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-KIDNEY", gs1s_anatomicalformcode_kidney, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-LIPS", gs1s_anatomicalformcode_lips, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-LIVER", gs1s_anatomicalformcode_liver, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-LUNGS", gs1s_anatomicalformcode_lungs, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-MIXED_OFFAL", gs1s_anatomicalformcode_mixed_offal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-MIXED_PARTS_FOR_GROUNDING_MINCING", gs1s_anatomicalformcode_mixed_parts_for_grounding_mincing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-NECK", gs1s_anatomicalformcode_neck, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-PANCREAS", gs1s_anatomicalformcode_pancreas, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-SKIN", gs1s_anatomicalformcode_skin, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-STOMACH", gs1s_anatomicalformcode_stomach, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-TAIL", gs1s_anatomicalformcode_tail, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-TENDONS", gs1s_anatomicalformcode_tendons, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-TESTICLES", gs1s_anatomicalformcode_testicles, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-THYMUS", gs1s_anatomicalformcode_thymus, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-TONGUE", gs1s_anatomicalformcode_tongue, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-TRIPE", gs1s_anatomicalformcode_tripe, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-UNCLASSIFIED", gs1s_anatomicalformcode_unclassified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-UNIDENTIFIED", gs1s_anatomicalformcode_unidentified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-WHOLE_BIRD", gs1s_anatomicalformcode_whole_bird, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-WHOLE_MUSCLE_NONPOULTRY_MEATPART_PIECE", gs1s_anatomicalformcode_whole_muscle_nonpoultry_meatpart_piece, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-WHOLE_MUSCLE_POULTRY_PART", gs1s_anatomicalformcode_whole_muscle_poultry_part, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-WHOLE_MUSCLE_PRIMAL", gs1s_anatomicalformcode_whole_muscle_primal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-WHOLE_MUSCLE_RIBCUT", gs1s_anatomicalformcode_whole_muscle_ribcut, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-WHOLE_MUSCLE_ROASTCUT", gs1s_anatomicalformcode_whole_muscle_roastcut, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-WHOLE_MUSCLE_SLICE_CHOPCUT", gs1s_anatomicalformcode_whole_muscle_slice_chopcut, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-WHOLE_MUSCLE_SLICE_CUTLET", gs1s_anatomicalformcode_whole_muscle_slice_cutlet, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-WHOLE_MUSCLE_STEAK_CHOPCUT", gs1s_anatomicalformcode_whole_muscle_steak_chopcut, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AnatomicalFormCode-WHOLE_MUSCLE_SUBPRIMAL", gs1s_anatomicalformcode_whole_muscle_subprimal, s_gs1 },
    { { s_gs1, 1, 51 }, { 0, 0 }, "AuthenticityDetails", gs1s_authenticitydetails, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "AwardPrize", gs1s_awardprize, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "AwardPrizeDetails", gs1s_awardprizedetails, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "Beverage", gs1s_beverage, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "Brand", gs1s_brand, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "Certification", gs1s_certification, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "CertificationDetails", gs1s_certificationdetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "CheeseFirmnessCode", gs1s_cheesefirmnesscode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "CheeseFirmnessCode-EXTRA_HARD", gs1s_cheesefirmnesscode_extra_hard, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "CheeseFirmnessCode-FIRM_SEMIHARD", gs1s_cheesefirmnesscode_firm_semihard, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "CheeseFirmnessCode-HARD", gs1s_cheesefirmnesscode_hard, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "CheeseFirmnessCode-SOFT", gs1s_cheesefirmnesscode_soft, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "CheeseFirmnessCode-UNIDENTIFIED", gs1s_cheesefirmnesscode_unidentified, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "Clothing", gs1s_clothing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeDetails", gs1s_colourcodedetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeListCode", gs1s_colourcodelistcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-BUYER", gs1s_colourcodelist_buyer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-IFPS", gs1s_colourcodelist_ifps, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-NCS", gs1s_colourcodelist_ncs, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-NRF", gs1s_colourcodelist_nrf, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-PANTONE_HEXACHROME", gs1s_colourcodelist_pantone_hexachrome, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-PANTONE_MATCHING", gs1s_colourcodelist_pantone_matching, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-PANTONE_PROCESS_COLOUR", gs1s_colourcodelist_pantone_process_colour, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-PANTONE_TEXTILE", gs1s_colourcodelist_pantone_textile, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-RAL", gs1s_colourcodelist_ral, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-SELLER", gs1s_colourcodelist_seller, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ColourCodeList-WWS", gs1s_colourcodelist_wws, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerLifestageCode", gs1s_consumerlifestagecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerLifestageCode-ADULT", gs1s_consumerlifestagecode_adult, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerLifestageCode-ALL_AGES", gs1s_consumerlifestagecode_all_ages, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerLifestageCode-BABY_INFANT", gs1s_consumerlifestagecode_baby_infant, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerLifestageCode-CHILD_1_To_2_YEARS", gs1s_consumerlifestagecode_child_1_to_2_years, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerLifestageCode-CHILD_2_YEARS_ONWARDS", gs1s_consumerlifestagecode_child_2_years_onwards, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerLifestageCode-UNCLASSIFIED", gs1s_consumerlifestagecode_unclassified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerLifestageCode-UNIDENTIFIED", gs1s_consumerlifestagecode_unidentified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerSalesConditionsCode", gs1s_consumersalesconditionscode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerSalesConditionsCode-BTC", gs1s_consumersalesconditionscode_btc, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerSalesConditionsCode-OTC", gs1s_consumersalesconditionscode_otc, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerSalesConditionsCode-PRESCRIPTION_REQUIRED", gs1s_consumersalesconditionscode_prescription_required, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerSalesConditionsCode-RESTRICTED_TO_SELL_16", gs1s_consumersalesconditionscode_restricted_to_sell_16, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerSalesConditionsCode-RESTRICTED_TO_SELL_18", gs1s_consumersalesconditionscode_restricted_to_sell_18, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ConsumerSalesConditionsCode-RESTRICTED_TO_SELL_21", gs1s_consumersalesconditionscode_restricted_to_sell_21, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "Contact", gs1s_contact, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ContactPoint", gs1s_contactpoint, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "Country", gs1s_country, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode", gs1s_diettypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode-COELIAC", gs1s_diettypecode_coeliac, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode-DIETETIC", gs1s_diettypecode_dietetic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode-FREE_FROM_GLUTEN", gs1s_diettypecode_free_from_gluten, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode-HALAL", gs1s_diettypecode_halal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode-KOSHER", gs1s_diettypecode_kosher, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode-ORGANIC", gs1s_diettypecode_organic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode-VEGAN", gs1s_diettypecode_vegan, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode-VEGETARIAN", gs1s_diettypecode_vegetarian, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode-WITHOUT_BEEF", gs1s_diettypecode_without_beef, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCode-WITHOUT_PORK", gs1s_diettypecode_without_pork, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DietTypeCodeDetails", gs1s_diettypecodedetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "Discount", gs1s_discount, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DiscountTypeCode", gs1s_discounttypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DiscountTypeCode-BOGO", gs1s_discounttypecode_bogo, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DiscountTypeCode-DISCOUNTED_ITEM", gs1s_discounttypecode_discounted_item, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DiscountTypeCode-FREE_GIFT", gs1s_discounttypecode_free_gift, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DiscountTypeCode-FREE_SHIPPING", gs1s_discounttypecode_free_shipping, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DiscountTypeCode-OTHER", gs1s_discounttypecode_other, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "DiscountTypeCode-PERCENTAGE_OFF", gs1s_discounttypecode_percentage_off, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "FoodAndBeveragePreparationInformation", gs1s_foodandbeveragepreparationinformation, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageRefrigerationClaimCode", gs1s_foodbeveragerefrigerationclaimcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageRefrigerationClaimCode-CAN_BE_REFRIGERATED", gs1s_foodbeveragerefrigerationclaimcode_can_be_refrigerated, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageRefrigerationClaimCode-MUST_BE_REFRIGERATED", gs1s_foodbeveragerefrigerationclaimcode_must_be_refrigerated, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageRefrigerationClaimCode-SHELF_STABLE", gs1s_foodbeveragerefrigerationclaimcode_shelf_stable, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageRefrigerationClaimCode-UNIDENTIFIED", gs1s_foodbeveragerefrigerationclaimcode_unidentified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageTargetUseCode", gs1s_foodbeveragetargetusecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageTargetUseCode-ANY_MEAL", gs1s_foodbeveragetargetusecode_any_meal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageTargetUseCode-BREAKFAST", gs1s_foodbeveragetargetusecode_breakfast, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageTargetUseCode-MAIN_MEAL", gs1s_foodbeveragetargetusecode_main_meal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageTargetUseCode-PORTABLE_MEAL", gs1s_foodbeveragetargetusecode_portable_meal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageTargetUseCode-SNACK", gs1s_foodbeveragetargetusecode_snack, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageTargetUseCode-UNCLASSIFIED", gs1s_foodbeveragetargetusecode_unclassified, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "FoodBeverageTobaccoIngredient", gs1s_foodbeveragetobaccoingredient, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageTobaccoIngredientDetails", gs1s_foodbeveragetobaccoingredientdetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FoodBeverageTobaccoProduct", gs1s_foodbeveragetobaccoproduct, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "FoodBeverageTobaccoTradeIten", gs1s_foodbeveragetobaccotradeitem, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "Footwear", gs1s_footwear, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FootwearFasteningTypeCode", gs1s_footwearfasteningtypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FootwearFasteningTypeCode-MULTIPLE_FASTENING", gs1s_footwearfasteningtypecode_multiple_fastening, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FootwearFasteningTypeCode-SHOE_LACE", gs1s_footwearfasteningtypecode_shoe_lace, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FootwearFasteningTypeCode-SLIP_ON_WITHOUT_ELASTIC", gs1s_footwearfasteningtypecode_slip_on_without_elastic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FootwearFasteningTypeCode-SLIP_ON_WITH_ELASTIC", gs1s_footwearfasteningtypecode_slip_on_with_elastic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FootwearFasteningTypeCode-STRAP", gs1s_footwearfasteningtypecode_strap, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FootwearFasteningTypeCode-UNCLASSIFIED", gs1s_footwearfasteningtypecode_unclassified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FootwearFasteningTypeCode-UNIDENTIFIED", gs1s_footwearfasteningtypecode_unidentified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FootwearFasteningTypeCode-VELCRO", gs1s_footwearfasteningtypecode_velcro, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FreshOrSeawaterFarmedCode", gs1s_freshorseawaterfarmedcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FreshOrSeawaterFarmedCode-FRESHWATER_FARMED", gs1s_freshorseawaterfarmedcode_freshwater_farmed, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FreshOrSeawaterFarmedCode-SEAWATER_FARMED", gs1s_freshorseawaterfarmedcode_seawater_farmed, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FreshOrSeawaterFarmedCode-UNCLASSIFIED", gs1s_freshorseawaterfarmedcode_unclassified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "FreshOrSeawaterFarmedCode-UNIDENTIFIED", gs1s_freshorseawaterfarmedcode_unidentified, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "FruitsVegetables", gs1s_fruitsvegetables, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GeoCoordinates", gs1s_geocoordinates, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode", gs1s_growingmethodcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-CAGE_FREE", gs1s_growingmethodcode_cage_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-CLONED_FOODS", gs1s_growingmethodcode_cloned_foods, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-CONVENTIONAL", gs1s_growingmethodcode_conventional, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-FARM_RAISED", gs1s_growingmethodcode_farm_raised, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-FIELD_GROWN", gs1s_growingmethodcode_field_grown, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-FLY_FREE", gs1s_growingmethodcode_fly_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-FREE_RANGE", gs1s_growingmethodcode_free_range, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-GRASS_FED", gs1s_growingmethodcode_grass_fed, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-GREENHOUSE", gs1s_growingmethodcode_greenhouse, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-HUMANELY_RAISED", gs1s_growingmethodcode_humanely_raised, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-HYDROPONIC", gs1s_growingmethodcode_hydroponic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-INTEGRATED_PEST_MANAGEMENT", gs1s_growingmethodcode_integrated_pest_management, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-ORGANIC", gs1s_growingmethodcode_organic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-SHADE_GROWN", gs1s_growingmethodcode_shade_grown, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-SUSTAINABLE", gs1s_growingmethodcode_sustainable, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "GrowingMethodCode-WILD", gs1s_growingmethodcode_wild, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "LevelOfContainmentCode", gs1s_levelofcontainmentcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "LevelOfContainmentCode-CONTAINS", gs1s_levelofcontainmentcodecontains, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "LevelOfContainmentCode-FREE_FROM", gs1s_levelofcontainmentcodefreefrom, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "LevelOfContainmentCode-MAY_CONTAIN", gs1s_levelofcontainmentcodemaycontain, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "MaturationMethodCode", gs1s_maturationmethodcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "MaturationMethodCode-JET_FRESH", gs1s_maturationmethodcode_jet_fresh, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "MaturationMethodCode-PRECONDITIONED", gs1s_maturationmethodcode_preconditioned, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "MaturationMethodCode-TREE_VINE_RIPE", gs1s_maturationmethodcode_tree_vine_ripe, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "MeasurementPrecisionCode", gs1s_measurementprecisioncode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "MeasurementPrecisionCode-APPROXIMATELY", gs1s_measurementprecisioncode_approximately, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "MeasurementPrecisionCode-EXACT", gs1s_measurementprecisioncode_exact, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "MeasurementPrecisionCode-LESS_THAN", gs1s_measurementprecisioncode_less_than, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "Meat", gs1s_meat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "MeatPoultry", gs1s_meatpoultry, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "MilkButterCreamYogurtCheeseEggsSubstitutes", gs1s_milkbuttercreamyogurtcheeseeggssubstitutes, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "NonbinaryLogicCode", gs1s_nonbinarylogiccode, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "NonbinaryLogicCode-FALSE", gs1s_nonbinarylogiccode_false, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "NonbinaryLogicCode-NOT_APPLICABLE", gs1s_nonbinarylogiccode_na, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "NonbinaryLogicCode-TRUE", gs1s_nonbinarylogiccode_true, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "NonbinaryLogicCode-UNSPECIFIED", gs1s_nonbinarylogiccode_unspecified, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "NutrientBasisQuantityCode", gs1s_nutrientbasisquantitycode, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutrientBasisQuantityCode-BY_MEASURE", gs1s_nutrientbasisquantitycode_by_measure, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutrientBasisQuantityCode-BY_SERVING", gs1s_nutrientbasisquantitycode_by_serving, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionMeasurementType", gs1s_nutritionmeasurementtype, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode", gs1s_nutritionalclaimtypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-ADDITIVE_FREE", gs1s_nutritionalclaimtypecode_additive_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-ARTIFICIALLY_SWEETENED", gs1s_nutritionalclaimtypecode_artificially_sweetened, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-CHOLESTEROL_FREE", gs1s_nutritionalclaimtypecode_cholesterol_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-COLOURING_AGENT_FREE", gs1s_nutritionalclaimtypecode_colouring_agent_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-CONTAINS_GLYZYRRHIZIN", gs1s_nutritionalclaimtypecode_contains_glyzyrrhizin, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-CONTAINS_LIQUORICE", gs1s_nutritionalclaimtypecode_contains_liquorice, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-CONTAINS_SOY", gs1s_nutritionalclaimtypecode_contains_soy, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-EGG_FREE", gs1s_nutritionalclaimtypecode_egg_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-ENERGY_FREE", gs1s_nutritionalclaimtypecode_energy_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-ENERGY_REDUCED", gs1s_nutritionalclaimtypecode_energy_reduced, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-ENRICHED_OR_FORTIFIED_IN_VITAMINS_AND_OR_MINERALS", gs1s_nutritionalclaimtypecode_enriched_or_fortified_in_vitamins_and_or_minerals, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-FAT_FREE", gs1s_nutritionalclaimtypecode_fat_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-FREE_FROM_GLUTEN", gs1s_nutritionalclaimtypecode_free_from_gluten, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-GUARANTEED_LACTOSE_FREE", gs1s_nutritionalclaimtypecode_guaranteed_lactose_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-HIGH_FIBRE", gs1s_nutritionalclaimtypecode_high_fibre, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-HIGH_PROTEIN", gs1s_nutritionalclaimtypecode_high_protein, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-HIGH_VITAMINS_AND_OR_MINERALS", gs1s_nutritionalclaimtypecode_high_vitamins_and_or_minerals, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-LACTOSE_FREE", gs1s_nutritionalclaimtypecode_lactose_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-LIGHT_LITE", gs1s_nutritionalclaimtypecode_light_lite, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-LOW_ENERGY", gs1s_nutritionalclaimtypecode_low_energy, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-LOW_FAT", gs1s_nutritionalclaimtypecode_low_fat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-LOW_LACTOSE", gs1s_nutritionalclaimtypecode_low_lactose, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-LOW_PROTEIN", gs1s_nutritionalclaimtypecode_low_protein, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-LOW_SATURATED_FAT", gs1s_nutritionalclaimtypecode_low_saturated_fat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-LOW_SODIUM_SALT", gs1s_nutritionalclaimtypecode_low_sodium_salt, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-LOW_SUGARS", gs1s_nutritionalclaimtypecode_low_sugars, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-MILK_FREE", gs1s_nutritionalclaimtypecode_milk_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-MILK_PROTEIN_FREE", gs1s_nutritionalclaimtypecode_milk_protein_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-NATURAL_SOURCE_OF_VITAMINS_AND_OR_MINERALS", gs1s_nutritionalclaimtypecode_natural_source_of_vitamins_and_or_minerals, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-NON_ALCOHOLIC", gs1s_nutritionalclaimtypecode_non_alcoholic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-NUT_FREE", gs1s_nutritionalclaimtypecode_nut_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-PEANUT_FREE", gs1s_nutritionalclaimtypecode_peanut_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-PRESERVATIVE_FREE", gs1s_nutritionalclaimtypecode_preservative_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SATURATED_FAT_FREE", gs1s_nutritionalclaimtypecode_saturated_fat_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SODIUM_FREE_OR_SALT_FREE", gs1s_nutritionalclaimtypecode_sodium_free_or_salt_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SOURCE_OF_FIBRE", gs1s_nutritionalclaimtypecode_source_of_fibre, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SOURCE_OF_PROTEIN", gs1s_nutritionalclaimtypecode_source_of_protein, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SOY_FREE", gs1s_nutritionalclaimtypecode_soy_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-STRONGLY_SALTED", gs1s_nutritionalclaimtypecode_strongly_salted, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SUGARS_FREE", gs1s_nutritionalclaimtypecode_sugars_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SWEETENED_WITH_AGAVE_SYRUP", gs1s_nutritionalclaimtypecode_sweetened_with_agave_syrup, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SWEETENED_WITH_CANE_SUGAR", gs1s_nutritionalclaimtypecode_sweetened_with_cane_sugar, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SWEETENED_WITH_CORN_SYRUP", gs1s_nutritionalclaimtypecode_sweetened_with_corn_syrup, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SWEETENED_WITH_FRUCTOSE", gs1s_nutritionalclaimtypecode_sweetened_with_fructose, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SWEETENED_WITH_FRUIT_JUICE", gs1s_nutritionalclaimtypecode_sweetened_with_fruit_juice, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SWEETENED_WITH_FRUIT_SYRUP", gs1s_nutritionalclaimtypecode_sweetened_with_fruit_syrup, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SWEETENED_WITH_HONEY", gs1s_nutritionalclaimtypecode_sweetened_with_honey, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SWEETENED_WITH_MALT", gs1s_nutritionalclaimtypecode_sweetened_with_malt, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SWEETENED_WITH_RAW_BEET_SUGAR", gs1s_nutritionalclaimtypecode_sweetened_with_raw_beet_sugar, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-SWEETENED_WITH_WHITE_SUGAR", gs1s_nutritionalclaimtypecode_sweetened_with_white_sugar, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-VERY_LOW_GLUTEN", gs1s_nutritionalclaimtypecode_very_low_gluten, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-VERY_LOW_SODIUM_SALT", gs1s_nutritionalclaimtypecode_very_low_sodium_salt, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-WHEAT_FREE", gs1s_nutritionalclaimtypecode_wheat_free, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "NutritionalClaimTypeCode-WITH_NO_ADDED_SUGARS", gs1s_nutritionalclaimtypecode_with_no_added_sugars, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "Offer", gs1s_offer, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "Offering", gs1s_offering, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OfferRedemptionTypeCode", gs1s_offerredemptiontypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OfferRedemptionTypeCode-COUPON_CODE", gs1s_offerredemptiontypecode_coupon_code, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OfferRedemptionTypeCode-GIFT_CARD", gs1s_offerredemptiontypecode_gift_card, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OfferRedemptionTypeCode-REBATE", gs1s_offerredemptiontypecode_rebate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OfferRedemptionTypeCode-SALE", gs1s_offerredemptiontypecode_sale, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OfferRedemptionTypeCode-SINGLE_USE_CODE", gs1s_offerredemptiontypecode_single_use_code, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode", gs1s_organicclaimagencycode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-BFA", gs1s_organicclaimagencycode_bfa, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-BIOLAND", gs1s_organicclaimagencycode_bioland, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-DEMETER_BUND", gs1s_organicclaimagencycode_demeter_bund, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-DEMETER_INTERNATIONAL", gs1s_organicclaimagencycode_demeter_international, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-ECOLAND", gs1s_organicclaimagencycode_ecoland, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-ECOVIN", gs1s_organicclaimagencycode_ecovin, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-EPA", gs1s_organicclaimagencycode_epa, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-FDA", gs1s_organicclaimagencycode_fda, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-FSA", gs1s_organicclaimagencycode_fsa, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-GAA", gs1s_organicclaimagencycode_gaa, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-IFOA", gs1s_organicclaimagencycode_ifoa, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-IFOAM", gs1s_organicclaimagencycode_ifoam, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-NATURLAND", gs1s_organicclaimagencycode_naturland, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-QAI", gs1s_organicclaimagencycode_qai, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-SUDITIROL", gs1s_organicclaimagencycode_suditirol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimAgencyCode-USDA", gs1s_organicclaimagencycode_usda, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganicClaimDetails", gs1s_organicclaimdetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "Organization", gs1s_organisation, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType", gs1s_organisationroletype, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-ACCEPTING_PARTY", gs1s_organisationroletype_accepting_party, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-BILL_OF_LADING_RECIPIENT", gs1s_organisationroletype_bill_of_lading_recipient, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-BILL_TO", gs1s_organisationroletype_bill_to, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-BRAND_OWNER", gs1s_organisationroletype_brand_owner, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-BREEDER", gs1s_organisationroletype_breeder, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-BROKER_AGENT", gs1s_organisationroletype_broker_agent, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-BUYER", gs1s_organisationroletype_buyer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-BUYERS_AGENT_REPRESENTATIVE", gs1s_organisationroletype_buyers_agent_representative, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CARRIER", gs1s_organisationroletype_carrier, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CENTRAL_PAYMENT_SERVICE", gs1s_organisationroletype_central_payment_service, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CHECKING_PARTY", gs1s_organisationroletype_checking_party, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CHEQUE_ORDER", gs1s_organisationroletype_cheque_order, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CONSIGNEE", gs1s_organisationroletype_consignee, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CONSOLIDATOR", gs1s_organisationroletype_consolidator, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CONSUMER", gs1s_organisationroletype_consumer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CORPORATE_IDENTITY", gs1s_organisationroletype_corporate_identity, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CUSTOMS", gs1s_organisationroletype_customs, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CUSTOMS_BROKER", gs1s_organisationroletype_customs_broker, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-CUTTER", gs1s_organisationroletype_cutter, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-DECLARANTS_AGENT_REPRESENTATIVE", gs1s_organisationroletype_declarants_agent_representative, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-DELIVERY_PARTY", gs1s_organisationroletype_delivery_party, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-DESIGNER", gs1s_organisationroletype_designer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-DESPATCH_PARTY", gs1s_organisationroletype_despatch_party, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-DISTRIBUTOR", gs1s_organisationroletype_distributor, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-DOCK_DOOR", gs1s_organisationroletype_dock_door, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-EMPTY_EQUIPMENT_DESPATCH_PARTY", gs1s_organisationroletype_empty_equipment_despatch_party, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-EMPTY_EQUIPMENT_RETURN_PARTY", gs1s_organisationroletype_empty_equipment_return_party, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-EQUIPMENT_OWNER", gs1s_organisationroletype_equipment_owner, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-EXPORTER", gs1s_organisationroletype_exporter, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-FACTOR", gs1s_organisationroletype_factor, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-FATTENER", gs1s_organisationroletype_fattener, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-FREIGHT_FORWARDER", gs1s_organisationroletype_freight_forwarder, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-GOODS_OWNER", gs1s_organisationroletype_goods_owner, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-GROWER", gs1s_organisationroletype_grower, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-HEAD_OFFICE", gs1s_organisationroletype_head_office, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-IMPORTER", gs1s_organisationroletype_importer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-INFORMATION_PROVIDER", gs1s_organisationroletype_information_provider, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-INSURER", gs1s_organisationroletype_insurer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-INTERMEDIARY_BANK_1", gs1s_organisationroletype_intermediary_bank_1, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-INTERMEDIARY_BANK_2", gs1s_organisationroletype_intermediary_bank_2, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-INVENTORY_CONTROLLER", gs1s_organisationroletype_inventory_controller, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-INVENTORY_REPORTING_PARTY", gs1s_organisationroletype_inventory_reporting_party, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-INVOICEE", gs1s_organisationroletype_invoicee, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-ISSUER_OF_INVOICE", gs1s_organisationroletype_issuer_of_invoice, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-LOGISTICS_SERVICE_PROVIDER", gs1s_organisationroletype_logistics_service_provider, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-MANUFACTURER_OF_GOODS", gs1s_organisationroletype_manufacturer_of_goods, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-MANUFACTURING_PLANT", gs1s_organisationroletype_manufacturing_plant, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-MARK_FOR", gs1s_organisationroletype_mark_for, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-MESSAGE_FROM", gs1s_organisationroletype_message_from, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-MESSAGE_RECIPIENT", gs1s_organisationroletype_message_recipient, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-MINCER", gs1s_organisationroletype_mincer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-OPERATING_DIVISION", gs1s_organisationroletype_operating_division, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-OPERATOR", gs1s_organisationroletype_operator, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-ORDERING_PARTY", gs1s_organisationroletype_ordering_party, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-OWNER_OF_EQUIPMENT", gs1s_organisationroletype_owner_of_equipment, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-OWNER_OF_MEANS_OF_TRANSPORT", gs1s_organisationroletype_owner_of_means_of_transport, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PARTY_DECLARING_THE_VALUE_ADDED_TAX", gs1s_organisationroletype_party_declaring_the_value_added_tax, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PARTY_FOR_WHOM_ITEM_IS_ULTIMATELY_INTENDED", gs1s_organisationroletype_party_for_whom_item_is_ultimately_intended, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PARTY_RECEIVING_PRIVATE_DATA", gs1s_organisationroletype_party_receiving_private_data, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PARTY_RECOVERING_THE_VALUE_ADDED_TAX", gs1s_organisationroletype_party_recovering_the_value_added_tax, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PARTY_TO_RECEIVE_ALL_DOCUMENTS", gs1s_organisationroletype_party_to_receive_all_documents, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PARTY_TO_RECEIVE_COMMERCIAL_INVOICE", gs1s_organisationroletype_party_to_receive_commercial_invoice, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PARTY_TO_RECEIVE_ELECTRONIC_MEMO_OF_INVOICE", gs1s_organisationroletype_party_to_receive_electronic_memo_of_invoice, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PARTY_TO_RECEIVE_FREIGHT_BILL", gs1s_organisationroletype_party_to_receive_freight_bill, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PARTY_TO_RECEIVE_REFUND", gs1s_organisationroletype_party_to_receive_refund, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PAYEE", gs1s_organisationroletype_payee, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PAYER", gs1s_organisationroletype_payer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-POINT_OF_SALE", gs1s_organisationroletype_point_of_sale, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PRICE_LOCATION_PARTY", gs1s_organisationroletype_price_location_party, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PROXY", gs1s_organisationroletype_proxy, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PUBLIC_WAREHOUSE", gs1s_organisationroletype_public_warehouse, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-PURCHASE_ORDER_RECEIVER", gs1s_organisationroletype_purchase_order_receiver, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-REGISTERED_AGENT", gs1s_organisationroletype_registered_agent, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-REMIT_TO", gs1s_organisationroletype_remit_to, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-RETURNABLE_ASSET_SERVICE_CENTRE", gs1s_organisationroletype_returnable_asset_service_centre, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-SELLER", gs1s_organisationroletype_seller, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-SERVICE_PROVIDER", gs1s_organisationroletype_service_provider, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-SHIP_FROM", gs1s_organisationroletype_ship_from, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-SHIP_TO", gs1s_organisationroletype_ship_to, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-SLAUGHTERER", gs1s_organisationroletype_slaughterer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-STORE", gs1s_organisationroletype_store, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-SUBSTITUTE_SUPPLIER", gs1s_organisationroletype_substitute_supplier, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-SUPPLIER", gs1s_organisationroletype_supplier, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-TRANSPORTATION_CARRIER", gs1s_organisationroletype_transportation_carrier, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-VISITING_ADDRESS", gs1s_organisationroletype_visiting_address, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-WAREHOUSE_AND_OR_DEPOT", gs1s_organisationroletype_warehouse_and_or_depot, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-WAREHOUSE_KEEPER", gs1s_organisationroletype_warehouse_keeper, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "OrganizationRoleType-WHOLESALER", gs1s_organisationroletype_wholesaler, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "Packaging", gs1s_packaging, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingDetails", gs1s_packagingdetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode", gs1s_packagingfeaturecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-BASE", gs1s_packagingfeaturecode_base, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-BEAM", gs1s_packagingfeaturecode_beam, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-BUNG_SEAL", gs1s_packagingfeaturecode_bung_seal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-CAP", gs1s_packagingfeaturecode_cap, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-CARRIER", gs1s_packagingfeaturecode_carrier, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-CONSUMPTION_UTENSIL", gs1s_packagingfeaturecode_consumption_utensil, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-CORE", gs1s_packagingfeaturecode_core, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-CREEL", gs1s_packagingfeaturecode_creel, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-EDGE_PROTECTION", gs1s_packagingfeaturecode_edge_protection, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-HANDLE", gs1s_packagingfeaturecode_handle, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-INNER_CONTAINER", gs1s_packagingfeaturecode_inner_container, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-INTERNAL_DIVIDER", gs1s_packagingfeaturecode_internal_divider, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-LABEL", gs1s_packagingfeaturecode_label, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-LID", gs1s_packagingfeaturecode_lid, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-LINER", gs1s_packagingfeaturecode_liner, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-LUG", gs1s_packagingfeaturecode_lug, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-NESTING_EDGE", gs1s_packagingfeaturecode_nesting_edge, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-PEG", gs1s_packagingfeaturecode_peg, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-PULL_OFF_TAB", gs1s_packagingfeaturecode_pull_off_tab, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-RING_HOLDER", gs1s_packagingfeaturecode_ring_holder, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-RIVET", gs1s_packagingfeaturecode_rivet, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-SLEEVE", gs1s_packagingfeaturecode_sleeve, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-SPOUT", gs1s_packagingfeaturecode_spout, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-TAG", gs1s_packagingfeaturecode_tag, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-WICKER_OUTER_CONTAINER", gs1s_packagingfeaturecode_wicker_outer_container, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFeatureCode-WRAP", gs1s_packagingfeaturecode_wrap, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode", gs1s_packagingfunctioncode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-ANTISEPTIC", gs1s_packagingfunctioncode_antiseptic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-ANTI_TAMPERING", gs1s_packagingfunctioncode_anti_tampering, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-COATED", gs1s_packagingfunctioncode_coated, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-COMPRESSED", gs1s_packagingfunctioncode_compressed, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-DISPENSER", gs1s_packagingfunctioncode_dispenser, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-GIFT_WRAPPED", gs1s_packagingfunctioncode_gift_wrapped, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-ISOTHERMIC", gs1s_packagingfunctioncode_isothermic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-MODIFIED_ATMOSPHERE", gs1s_packagingfunctioncode_modified_atmosphere, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-OXYGEN_INFUSED", gs1s_packagingfunctioncode_oxygen_infused, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-PEEL_OFF", gs1s_packagingfunctioncode_peel_off, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-PINPACK", gs1s_packagingfunctioncode_pinpack, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-PROTECTED", gs1s_packagingfunctioncode_protected, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-REINFORCED", gs1s_packagingfunctioncode_reinforced, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-SIFT_PROOF", gs1s_packagingfunctioncode_sift_proof, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-TAMPER_EVIDENT", gs1s_packagingfunctioncode_tamper_evident, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingFunctionCode-WATER_RESISTANT", gs1s_packagingfunctioncode_water_resistant, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode", gs1s_packagingmarkeddietallergencode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-APPROVED_BY_ASTHMA_AND_ALLERGY_ASSOC", gs1s_packagingmarkeddietallergencode_approved_by_asthma_and_allergy_assoc, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-APPROVED_FOR_TUBE_FEEDING", gs1s_packagingmarkeddietallergencode_approved_for_tube_feeding, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-BIOLOGICAL", gs1s_packagingmarkeddietallergencode_biological, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-CALORIES_PER_PORTION", gs1s_packagingmarkeddietallergencode_calories_per_portion, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-CONTAINS_LATEX", gs1s_packagingmarkeddietallergencode_contains_latex, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-CONTAINS_PVC", gs1s_packagingmarkeddietallergencode_contains_pvc, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-CONTAINS_PVC_WITHOUT_PHTHALATES", gs1s_packagingmarkeddietallergencode_contains_pvc_without_phthalates, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-CONTAINS_PVC_WITH_PHTHALATES", gs1s_packagingmarkeddietallergencode_contains_pvc_with_phthalates, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-DIET_PRODUCT_450_800_KCAL_PER_DAY", gs1s_packagingmarkeddietallergencode_diet_product_450_800_kcal_per_day, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-DIET_PRODUCT_800_1200_KCAL_PER_DAY", gs1s_packagingmarkeddietallergencode_diet_product_800_1200_kcal_per_day, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-GEZONDE_KEUZE_KLAVERTJE", gs1s_packagingmarkeddietallergencode_gezonde_keuze_klavertje, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-HALAL", gs1s_packagingmarkeddietallergencode_halal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-IK_KIES_BEWUST", gs1s_packagingmarkeddietallergencode_ik_kies_bewust, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-KOSHER", gs1s_packagingmarkeddietallergencode_kosher, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-LACTASE_ENZYME", gs1s_packagingmarkeddietallergencode_lactase_enzyme, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-LOW_ON_PHENYLALANINE", gs1s_packagingmarkeddietallergencode_low_on_phenylalanine, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-LOW_ON_SUGAR", gs1s_packagingmarkeddietallergencode_low_on_sugar, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-MOTHERS_MILK_SUBSTITUTE", gs1s_packagingmarkeddietallergencode_mothers_milk_substitute, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-NUTRITION_SUPPLEMENT", gs1s_packagingmarkeddietallergencode_nutrition_supplement, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-NYCKELHAL_MARK", gs1s_packagingmarkeddietallergencode_nyckelhal_mark, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedDietAllergenCode-VEGETARIAN", gs1s_packagingmarkeddietallergencode_vegetarian, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode", gs1s_packagingmarkedfreefromcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_ARTIFICIAL_COLOURING", gs1s_packagingmarkedfreefromcode_free_from_artificial_colouring, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_ARTIFICIAL_FLAVOURING", gs1s_packagingmarkedfreefromcode_free_from_artificial_flavouring, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_ARTIFICIAL_PRESERVATIVES", gs1s_packagingmarkedfreefromcode_free_from_artificial_preservatives, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_CHOLESTEROL", gs1s_packagingmarkedfreefromcode_free_from_cholesterol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_CLONED_FOOD", gs1s_packagingmarkedfreefromcode_free_from_cloned_food, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_DAIRY", gs1s_packagingmarkedfreefromcode_free_from_dairy, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_EGG", gs1s_packagingmarkedfreefromcode_free_from_egg, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_FISH", gs1s_packagingmarkedfreefromcode_free_from_fish, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_GLUTEN", gs1s_packagingmarkedfreefromcode_free_from_gluten, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_LACTOSE", gs1s_packagingmarkedfreefromcode_free_from_lactose, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_LATEX", gs1s_packagingmarkedfreefromcode_free_from_latex, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_LEGUME_PROTEIN", gs1s_packagingmarkedfreefromcode_free_from_legume_protein, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_MILK", gs1s_packagingmarkedfreefromcode_free_from_milk, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_MILK_PROTEIN", gs1s_packagingmarkedfreefromcode_free_from_milk_protein, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_NATURAL_GLUTEN", gs1s_packagingmarkedfreefromcode_free_from_natural_gluten, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_PEANUTS", gs1s_packagingmarkedfreefromcode_free_from_peanuts, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_PROTEIN", gs1s_packagingmarkedfreefromcode_free_from_protein, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_PVC", gs1s_packagingmarkedfreefromcode_free_from_pvc, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_SOYA", gs1s_packagingmarkedfreefromcode_free_from_soya, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_SUGAR", gs1s_packagingmarkedfreefromcode_free_from_sugar, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-FREE_FROM_TRANSFAT", gs1s_packagingmarkedfreefromcode_free_from_transfat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-REDUCED_LACTOSE", gs1s_packagingmarkedfreefromcode_reduced_lactose, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-REDUCED_PROTEIN", gs1s_packagingmarkedfreefromcode_reduced_protein, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-REDUCED_TRANSFAT", gs1s_packagingmarkedfreefromcode_reduced_transfat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-VERY_LOW_GLUTEN", gs1s_packagingmarkedfreefromcode_very_low_gluten, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-WITHOUT_ADDED_SALT", gs1s_packagingmarkedfreefromcode_without_added_salt, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-WITHOUT_ADDED_SUGAR", gs1s_packagingmarkedfreefromcode_without_added_sugar, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedFreeFromCode-WITHOUT_ADDED_SWEETENER", gs1s_packagingmarkedfreefromcode_without_added_sweetener, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode", gs1s_packagingmarkedlabelaccreditationcode, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-AMA_ORGANIC_SEAL", gs1s_packagingmarkedlabelaccreditationcode_ama_organic_seal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-AUS_KAUP_ESTONIA", gs1s_packagingmarkedlabelaccreditationcode_aus_kaup_estonia, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BDIH_LOGO", gs1s_packagingmarkedlabelaccreditationcode_bdih_logo, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BETER_LEVEN_1_STER", gs1s_packagingmarkedlabelaccreditationcode_beter_leven_1_ster, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BETER_LEVEN_2_STER", gs1s_packagingmarkedlabelaccreditationcode_beter_leven_2_ster, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BETER_LEVEN_3_STER", gs1s_packagingmarkedlabelaccreditationcode_beter_leven_3_ster, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BIO_AUSTRIA_LABEL", gs1s_packagingmarkedlabelaccreditationcode_bio_austria_label, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BIO_LABEL_GERMAN", gs1s_packagingmarkedlabelaccreditationcode_bio_label_german, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BIO_SUISSE_BUD_SEAL", gs1s_packagingmarkedlabelaccreditationcode_bio_suisse_bud_seal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BLUE_ANGEL", gs1s_packagingmarkedlabelaccreditationcode_blue_angel, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BORD_BIA_APPROVED", gs1s_packagingmarkedlabelaccreditationcode_bord_bia_approved, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BORD_BIA_APPROVED_MEAT", gs1s_packagingmarkedlabelaccreditationcode_bord_bia_approved_meat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-BRA_MILJOVAL_LABEL_SWEDISH", gs1s_packagingmarkedlabelaccreditationcode_bra_miljoval_label_swedish, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-CROSSED_GRAIN_SYMBOL", gs1s_packagingmarkedlabelaccreditationcode_crossed_grain_symbol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-DEMETER_LABEL", gs1s_packagingmarkedlabelaccreditationcode_demeter_label, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-ECOCERT_CERTIFICATE", gs1s_packagingmarkedlabelaccreditationcode_ecocert_certificate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-ECO_LABEL_LADYBUG", gs1s_packagingmarkedlabelaccreditationcode_eco_label_ladybug, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-EC_NATIONAL_HEALTH_MARK", gs1s_packagingmarkedlabelaccreditationcode_ec_national_health_mark, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-EESTI_OKOMARK", gs1s_packagingmarkedlabelaccreditationcode_eesti_okomark, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-EESTI_PARIM_TOIDUAINE", gs1s_packagingmarkedlabelaccreditationcode_eesti_parim_toiduaine, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-EKO", gs1s_packagingmarkedlabelaccreditationcode_eko, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-EU_ECO_LABEL", gs1s_packagingmarkedlabelaccreditationcode_eu_eco_label, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-EU_ORGANIC_FARMING", gs1s_packagingmarkedlabelaccreditationcode_eu_organic_farming, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-FAIR_TRADE_MARK", gs1s_packagingmarkedlabelaccreditationcode_fair_trade_mark, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-FALKEN", gs1s_packagingmarkedlabelaccreditationcode_falken, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-FINNISH_HEART_SYMBOL", gs1s_packagingmarkedlabelaccreditationcode_finnish_heart_symbol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-FOREST_STEWARDSHIP_COUNCIL_LABEL", gs1s_packagingmarkedlabelaccreditationcode_forest_stewardship_council_label, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-GMO_MARKED", gs1s_packagingmarkedlabelaccreditationcode_gmo_marked, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-GOODS_FROM_FINLAND_BLUE_SWAN", gs1s_packagingmarkedlabelaccreditationcode_goods_from_finland_blue_swan, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-GREEN_DOT", gs1s_packagingmarkedlabelaccreditationcode_green_dot, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-GREEN_RESTAURANT_ASSOCIATION_ENDORSED", gs1s_packagingmarkedlabelaccreditationcode_green_restaurant_association_endorsed, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-GREEN_SEAL_CERTIFIED", gs1s_packagingmarkedlabelaccreditationcode_green_seal_certified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-GUARANTEED_IRISH", gs1s_packagingmarkedlabelaccreditationcode_guaranteed_irish, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-KRAV_MARK", gs1s_packagingmarkedlabelaccreditationcode_krav_mark, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-LABEL_OF_THE_ALLERGY_AND_ASTHMA_FEDERATION", gs1s_packagingmarkedlabelaccreditationcode_label_of_the_allergy_and_asthma_federation, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-LEAPING_BUNNY", gs1s_packagingmarkedlabelaccreditationcode_leaping_bunny, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-LOODUSSOBRALIK_TOODE_ESTONIA", gs1s_packagingmarkedlabelaccreditationcode_loodussobralik_toode_estonia, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-LOVE_IRISH_FOOD", gs1s_packagingmarkedlabelaccreditationcode_love_irish_food, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-MADE_IN_FINLAND_FLAG_WITH_KEY", gs1s_packagingmarkedlabelaccreditationcode_made_in_finland_flag_with_key, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-MARINE_STEWARDSHIP_COUNCIL_LABEL", gs1s_packagingmarkedlabelaccreditationcode_marine_stewardship_council_label, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-MAX_HAVELAAR", gs1s_packagingmarkedlabelaccreditationcode_max_havelaar, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-NATRUE_LABEL", gs1s_packagingmarkedlabelaccreditationcode_natrue_label, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-NYCKELHALET", gs1s_packagingmarkedlabelaccreditationcode_nyckelhalet, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-OEKO_TEX_LABEL", gs1s_packagingmarkedlabelaccreditationcode_oeko_tex_label, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-OFFICIAL_ECO_LABEL_SUN", gs1s_packagingmarkedlabelaccreditationcode_official_eco_label_sun, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-PEFC", gs1s_packagingmarkedlabelaccreditationcode_pefc, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-PROTECTED_DESIGNATION_OF_ORIGIN", gs1s_packagingmarkedlabelaccreditationcode_protected_designation_of_origin, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-PROTECTED_GEOGRAPHICAL_INDICATION", gs1s_packagingmarkedlabelaccreditationcode_protected_geographical_indication, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-PROTECTED_HARVEST_CERTIFIED", gs1s_packagingmarkedlabelaccreditationcode_protected_harvest_certified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-QUALITY_MARK_IRELAND", gs1s_packagingmarkedlabelaccreditationcode_quality_mark_ireland, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-RAINFOREST_ALLIANCE", gs1s_packagingmarkedlabelaccreditationcode_rainforest_alliance, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-SCHARRELVLEES", gs1s_packagingmarkedlabelaccreditationcode_scharrelvlees, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-SOIL_ASSOCIATION_ORGANIC_SYMBOL", gs1s_packagingmarkedlabelaccreditationcode_soil_association_organic_symbol, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-SUSTAINABLE_PALM_OIL_RSPO", gs1s_packagingmarkedlabelaccreditationcode_sustainable_palm_oil_rspo, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-SVANEN", gs1s_packagingmarkedlabelaccreditationcode_svanen, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-SWEDISH_SEAL_OF_QUALITY", gs1s_packagingmarkedlabelaccreditationcode_swedish_seal_of_quality, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-TRADITIONAL_SPECIALTY_GUARANTEED", gs1s_packagingmarkedlabelaccreditationcode_traditional_specialty_guaranteed, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-TUNNUSTATUD_EESTI_MAITSE", gs1s_packagingmarkedlabelaccreditationcode_tunnustatud_eesti_maitse, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-TUNNUSTATUD_MAITSE", gs1s_packagingmarkedlabelaccreditationcode_tunnustatud_maitse, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-UNIQUELY_FINNISH", gs1s_packagingmarkedlabelaccreditationcode_uniquely_finnish, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-USDA", gs1s_packagingmarkedlabelaccreditationcode_usda, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-UTZ_CERTIFIED", gs1s_packagingmarkedlabelaccreditationcode_utz_certified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-VAELG_FULDKORN_FORST", gs1s_packagingmarkedlabelaccreditationcode_vaelg_fuldkorn_forst, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMarkedLabelAccreditationCode-WWF_PANDA_LABEL", gs1s_packagingmarkedlabelaccreditationcode_wwf_panda_label, s_gs1 },    { { s_gs1, 1, 0 }, { 0, 0 }, "PackagingMaterialDetails", gs1s_packagingmaterialdetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode", gs1s_packagingmaterialtypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-CERAMIC", gs1s_packagingmaterialtypecode_ceramic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-CLOTH_OR_FABRIC", gs1s_packagingmaterialtypecode_cloth_or_fabric, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-COMPOSITE", gs1s_packagingmaterialtypecode_composite, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-CORRUGATED_BOARD_DOUBLE_WALL", gs1s_packagingmaterialtypecode_corrugated_board_double_wall, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-CORRUGATED_BOARD_OTHER", gs1s_packagingmaterialtypecode_corrugated_board_other, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-CORRUGATED_BOARD_SINGLE_WALL", gs1s_packagingmaterialtypecode_corrugated_board_single_wall, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-CORRUGATED_BOARD_TRIPLE_WALL", gs1s_packagingmaterialtypecode_corrugated_board_triple_wall, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-FIBRE_BURLAP", gs1s_packagingmaterialtypecode_fibre_burlap, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-FIBRE_COTTON", gs1s_packagingmaterialtypecode_fibre_cotton, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-FIBRE_FLAX", gs1s_packagingmaterialtypecode_fibre_flax, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-FIBRE_HEMP", gs1s_packagingmaterialtypecode_fibre_hemp, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-FIBRE_JUTE", gs1s_packagingmaterialtypecode_fibre_jute, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-FIBRE_OTHER", gs1s_packagingmaterialtypecode_fibre_other, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-FOAM", gs1s_packagingmaterialtypecode_foam, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-GLASS", gs1s_packagingmaterialtypecode_glass, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-GLASS_COLOURED", gs1s_packagingmaterialtypecode_glass_coloured, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-LAMINATED_CARTON", gs1s_packagingmaterialtypecode_laminated_carton, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-METAL_ALUMINUM", gs1s_packagingmaterialtypecode_metal_aluminum, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-METAL_BRASS", gs1s_packagingmaterialtypecode_metal_brass, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-METAL_IRON", gs1s_packagingmaterialtypecode_metal_iron, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-METAL_LEAD", gs1s_packagingmaterialtypecode_metal_lead, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-METAL_OTHER", gs1s_packagingmaterialtypecode_metal_other, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-METAL_STAINLESS_STEEL", gs1s_packagingmaterialtypecode_metal_stainless_steel, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-METAL_STEEL", gs1s_packagingmaterialtypecode_metal_steel, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-METAL_TIN", gs1s_packagingmaterialtypecode_metal_tin, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-MINERAL_CALCIUM_CARBONATE", gs1s_packagingmaterialtypecode_mineral_calcium_carbonate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-MINERAL_OTHER", gs1s_packagingmaterialtypecode_mineral_other, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-MINERAL_TALC", gs1s_packagingmaterialtypecode_mineral_talc, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-NATURAL_RUBBER", gs1s_packagingmaterialtypecode_natural_rubber, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-OTHER", gs1s_packagingmaterialtypecode_other, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-PAPER_CORRUGATED", gs1s_packagingmaterialtypecode_paper_corrugated, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-PAPER_MOLDED_PULP", gs1s_packagingmaterialtypecode_paper_molded_pulp, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-PAPER_OTHER", gs1s_packagingmaterialtypecode_paper_other, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-PAPER_PAPER", gs1s_packagingmaterialtypecode_paper_paper, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-PAPER_PAPERBOARD", gs1s_packagingmaterialtypecode_paper_paperboard, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-PAPER_RAYON", gs1s_packagingmaterialtypecode_paper_rayon, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-PLASTIC_BIOPLASTIC", gs1s_packagingmaterialtypecode_plastic_bioplastic, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-PLASTIC_OTHER", gs1s_packagingmaterialtypecode_plastic_other, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-PLASTIC_THERMOPLASTICS", gs1s_packagingmaterialtypecode_plastic_thermoplastics, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_CELLULOSE_ACETATE", gs1s_packagingmaterialtypecode_polymer_cellulose_acetate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_EPOXY", gs1s_packagingmaterialtypecode_polymer_epoxy, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_EVA", gs1s_packagingmaterialtypecode_polymer_eva, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_EVOH", gs1s_packagingmaterialtypecode_polymer_evoh, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_HDPE", gs1s_packagingmaterialtypecode_polymer_hdpe, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_LDPE", gs1s_packagingmaterialtypecode_polymer_ldpe, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_LLDPE", gs1s_packagingmaterialtypecode_polymer_lldpe, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_MDPE", gs1s_packagingmaterialtypecode_polymer_mdpe, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_NYLON", gs1s_packagingmaterialtypecode_polymer_nylon, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_OTHER", gs1s_packagingmaterialtypecode_polymer_other, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PAN", gs1s_packagingmaterialtypecode_polymer_pan, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PC", gs1s_packagingmaterialtypecode_polymer_pc, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PCL", gs1s_packagingmaterialtypecode_polymer_pcl, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PE", gs1s_packagingmaterialtypecode_polymer_pe, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PEN", gs1s_packagingmaterialtypecode_polymer_pen, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PET", gs1s_packagingmaterialtypecode_polymer_pet, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PHA", gs1s_packagingmaterialtypecode_polymer_pha, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PLA", gs1s_packagingmaterialtypecode_polymer_pla, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PP", gs1s_packagingmaterialtypecode_polymer_pp, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PS", gs1s_packagingmaterialtypecode_polymer_ps, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PU", gs1s_packagingmaterialtypecode_polymer_pu, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PVA", gs1s_packagingmaterialtypecode_polymer_pva, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PVC", gs1s_packagingmaterialtypecode_polymer_pvc, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_PVDC", gs1s_packagingmaterialtypecode_polymer_pvdc, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingMaterialTypeCode-POLYMER_TPS", gs1s_packagingmaterialtypecode_polymer_tps, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingRecyclingProcessTypeCode", gs1s_packagingrecyclingprocesstypecode, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingRecyclingProcessTypeCode-COMPOSTABLE", gs1s_packagingrecyclingprocesstypecode_compostable, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingRecyclingProcessTypeCode-ENERGY_RECOVERABLE", gs1s_packagingrecyclingprocesstypecode_energy_recoverable, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingRecyclingProcessTypeCode-RECYCLABLE", gs1s_packagingrecyclingprocesstypecode_recyclable, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingRecyclingProcessTypeCode-REUSABLE", gs1s_packagingrecyclingprocesstypecode_reusable, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingRecyclingSchemeCode", gs1s_packagingrecyclingschemecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingRecyclingSchemeCode-ALKO", gs1s_packagingrecyclingschemecode_alko, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingRecyclingSchemeCode-A_PULLO", gs1s_packagingrecyclingschemecode_a_pullo, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingRecyclingSchemeCode-EKO_PULLO", gs1s_packagingrecyclingschemecode_eko_pullo, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingRecyclingSchemeCode-PALPA", gs1s_packagingrecyclingschemecode_palpa, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingShapeCode", gs1s_packagingshapecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingShapeCode-BAR", gs1s_packagingshapecode_bar, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingShapeCode-COIL", gs1s_packagingshapecode_coil, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingShapeCode-CONE", gs1s_packagingshapecode_cone, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingShapeCode-CYLINDRICAL", gs1s_packagingshapecode_cylindrical, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingShapeCode-POLYGON", gs1s_packagingshapecode_polygon, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingShapeCode-RECTANGULAR", gs1s_packagingshapecode_rectangular, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingShapeCode-SPHERICAL", gs1s_packagingshapecode_spherical, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingShapeCode-TABLET", gs1s_packagingshapecode_tablet, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PackagingShapeCode-UNSPECIFIED", gs1s_packagingshapecode_unspecified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod", gs1s_paymentmethod, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-BANKERS_DRAFT", gs1s_paymentmethod_bankers_draft, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-BANK_CHEQUE", gs1s_paymentmethod_bank_cheque, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-BANK_GIRO", gs1s_paymentmethod_bank_giro, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-BOOKENTRY_CREDIT", gs1s_paymentmethod_bookentry_credit, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-BOOKENTRY_DEBIT", gs1s_paymentmethod_bookentry_debit, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-BOP", gs1s_paymentmethod_bop, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-CASH", gs1s_paymentmethod_cash, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-CERTIFIED_CHEQUE", gs1s_paymentmethod_certified_cheque, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-CHEQUE", gs1s_paymentmethod_cheque, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-CREDIT_CARD", gs1s_paymentmethod_credit_card, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-CREDIT_CARD_AMEX", gs1s_paymentmethod_credit_card_amex, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-CREDIT_CARD_DINERS_CLUB", gs1s_paymentmethod_credit_card_diners_club, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-CREDIT_CARD_DISCOVER", gs1s_paymentmethod_credit_card_discover, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-CREDIT_CARD_MASTERCARD", gs1s_paymentmethod_credit_card_mastercard, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-CREDIT_CARD_VISA", gs1s_paymentmethod_credit_card_visa, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-DEBIT_CARD", gs1s_paymentmethod_debit_card, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-DEBIT_CARD_MAESTRO", gs1s_paymentmethod_debit_card_maestro, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-DEBIT_CARD_MASTERCARD", gs1s_paymentmethod_debit_card_mastercard, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-DEBIT_CARD_VISA", gs1s_paymentmethod_debit_card_visa, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-DEBIT_CARD_VISA_ELECTRON", gs1s_paymentmethod_debit_card_visa_electron, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-ELECTRONIC_CREDIT_ACH", gs1s_paymentmethod_electronic_credit_ach, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-ELECTRONIC_DEBIT_ACH", gs1s_paymentmethod_electronic_debit_ach, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-FED_WIRE_NON_REPETITIVE", gs1s_paymentmethod_fed_wire_non_repetitive, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-FED_WIRE_REPETITIVE", gs1s_paymentmethod_fed_wire_repetitive, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-INTERNATIONAL_WIRE", gs1s_paymentmethod_international_wire, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-LETTER_OF_CREDIT", gs1s_paymentmethod_letter_of_credit, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-ONLINE_PAYMENT", gs1s_paymentmethod_online_payment, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-ONLINE_PAYMENT_APPLE_PAY", gs1s_paymentmethod_online_payment_apple_pay, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-ONLINE_PAYMENT_GOOGLE_WALLET", gs1s_paymentmethod_online_payment_google_wallet, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-ONLINE_PAYMENT_PAYPAL", gs1s_paymentmethod_online_payment_paypal, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-POSTGIRO", gs1s_paymentmethod_postgiro, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-WIRE_TRANSFER_CREDIT", gs1s_paymentmethod_wire_transfer_credit, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PaymentMethod-WIRE_TRANSFER_DEBIT", gs1s_paymentmethod_wire_transfer_debit, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "Place", gs1s_place, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PostalAddress", gs1s_postaladdress, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode", gs1s_preparationtypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-AS_DRAINED", gs1s_preparationtypecode_as_drained, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-BAKE", gs1s_preparationtypecode_bake, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-BARBECUE", gs1s_preparationtypecode_barbecue, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-BLANCH", gs1s_preparationtypecode_blanch, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-BLIND_BAKE", gs1s_preparationtypecode_blind_bake, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-BOIL", gs1s_preparationtypecode_boil, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-BRAISE", gs1s_preparationtypecode_braise, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-BROIL", gs1s_preparationtypecode_broil, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-DEEP_FRY", gs1s_preparationtypecode_deep_fry, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-DOUBLE_STEAM", gs1s_preparationtypecode_double_steam, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-FREEZE", gs1s_preparationtypecode_freeze, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-FRY", gs1s_preparationtypecode_fry, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-GRIDDLE_FRY", gs1s_preparationtypecode_griddle_fry, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-GRILL", gs1s_preparationtypecode_grill, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-HEAT_AND_SERVE", gs1s_preparationtypecode_heat_and_serve, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-MICROWAVE", gs1s_preparationtypecode_microwave, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-PAN_FRY", gs1s_preparationtypecode_pan_fry, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-POACH", gs1s_preparationtypecode_poach, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-PREPARED", gs1s_preparationtypecode_prepared, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-PRESSURE_COOK", gs1s_preparationtypecode_pressure_cook, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-PRESSURE_FRY", gs1s_preparationtypecode_pressure_fry, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-READY_TO_DRINK", gs1s_preparationtypecode_ready_to_drink, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-READY_TO_EAT", gs1s_preparationtypecode_ready_to_eat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-RECONSTITUTE", gs1s_preparationtypecode_reconstitute, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-REFRIGERATE", gs1s_preparationtypecode_refrigerate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-ROAST", gs1s_preparationtypecode_roast, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-ROTISSERIE", gs1s_preparationtypecode_rotisserie, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-SAUTE", gs1s_preparationtypecode_saute, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-SEAR", gs1s_preparationtypecode_sear, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-SIMMER", gs1s_preparationtypecode_simmer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-SMOKE", gs1s_preparationtypecode_smoke, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-STEAM", gs1s_preparationtypecode_steam, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-STEW", gs1s_preparationtypecode_stew, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-STIR_FRY", gs1s_preparationtypecode_stir_fry, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-THAW", gs1s_preparationtypecode_thaw, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-UNPREPARED", gs1s_preparationtypecode_unprepared, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreparationTypeCode-UNSPECIFIED", gs1s_preparationtypecode_unspecified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode", gs1s_preservationtechniquecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-ACIDIFICATION", gs1s_preservationtechniquecode_acidification, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-ALCOHOL_CURING", gs1s_preservationtechniquecode_alcohol_curing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-ATTESTED_MILK", gs1s_preservationtechniquecode_attested_milk, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-BOILING", gs1s_preservationtechniquecode_boiling, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-BRINING", gs1s_preservationtechniquecode_brining, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-CANNING", gs1s_preservationtechniquecode_canning, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-CONSERVE", gs1s_preservationtechniquecode_conserve, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-DEHYDRATION", gs1s_preservationtechniquecode_dehydration, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-DRYING", gs1s_preservationtechniquecode_drying, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-FERMENTATION", gs1s_preservationtechniquecode_fermentation, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-FREEZE_DRYING", gs1s_preservationtechniquecode_freeze_drying, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-FREEZING", gs1s_preservationtechniquecode_freezing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-HIGH_TEMPERATURE_TREATED_MILK", gs1s_preservationtechniquecode_high_temperature_treated_milk, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-HOT_SMOKE_CURING", gs1s_preservationtechniquecode_hot_smoke_curing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-IONISATION", gs1s_preservationtechniquecode_ionisation, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-IRRADIATION", gs1s_preservationtechniquecode_irradiation, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-PASTEURISATION", gs1s_preservationtechniquecode_pasteurisation, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-QUICK_FREEZING", gs1s_preservationtechniquecode_quick_freezing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-SALT_CURING", gs1s_preservationtechniquecode_salt_curing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-SOUS_VIDE", gs1s_preservationtechniquecode_sous_vide, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-STERILISATION", gs1s_preservationtechniquecode_sterilisation, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-SUGAR_CURING", gs1s_preservationtechniquecode_sugar_curing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-ULTRA_HIGH_TEMPERATURE", gs1s_preservationtechniquecode_ultra_high_temperature, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-UNDER_MODIFIED_ATMOSPHERE", gs1s_preservationtechniquecode_under_modified_atmosphere, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PreservationTechniqueCode-VACUUM_PACKED", gs1s_preservationtechniquecode_vacuum_packed, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "Price", gs1s_price, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "PriceSpecification", gs1s_pricespecification, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "Product", gs1s_product, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ProductYieldDetails", gs1s_productyielddetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ProductYieldTypeCode", gs1s_productyieldtypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ProductYieldTypeCode-AFTER_COOKING", gs1s_productyieldtypecode_after_cooking, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ProductYieldTypeCode-AFTER_DILUTION", gs1s_productyieldtypecode_after_dilution, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ProductYieldTypeCode-DRAINED_WEIGHT", gs1s_productyieldtypecode_drained_weight, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ProductYieldTypeCode-UNSPECIFIED", gs1s_productyieldtypecode_unspecified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "QuantitativeValue", gs1s_quantitativevalue, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileDetails", gs1s_referencedfiledetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode", gs1s_referencedfiletypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-DIET_CERTIFICATE", gs1s_referencedfiletypecode_diet_certificate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-DOCUMENT", gs1s_referencedfiletypecode_document, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-GROUP_CHARACTERISTIC_SHEET", gs1s_referencedfiletypecode_group_characteristic_sheet, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-LOGO", gs1s_referencedfiletypecode_logo, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-MARKETING_INFORMATION", gs1s_referencedfiletypecode_marketing_information, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-OUT_OF_PACKAGE_IMAGE", gs1s_referencedfiletypecode_out_of_package_image, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-PLANOGRAM", gs1s_referencedfiletypecode_planogram, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-PRODUCT_LABEL_IMAGE", gs1s_referencedfiletypecode_product_label_image, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-SAFETY_DATA_SHEET", gs1s_referencedfiletypecode_safety_data_sheet, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-SAFETY_SUMMARY_SHEET", gs1s_referencedfiletypecode_safety_summary_sheet, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-TRADE_ITEM_DESCRIPTION", gs1s_referencedfiletypecode_trade_item_description, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-VIDEO", gs1s_referencedfiletypecode_video, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-WARRANTY_INFORMATION", gs1s_referencedfiletypecode_warranty_information, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReferencedFileTypeCode-WEBSITE", gs1s_referencedfiletypecode_website, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "ReferencedResource", gs1s_referencedresource, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "ReturnablePackageDepositDetails", gs1s_returnablepackagedepositdetails, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "Seafood", gs1s_seafood, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SeasonParameterCode", gs1s_seasonparametercode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SeasonParameterCode-ALL_YEAR", gs1s_seasonparametercode_all_year, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SeasonParameterCode-AUTUMN", gs1s_seasonparametercode_autumn, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SeasonParameterCode-AUTUMN_WINTER", gs1s_seasonparametercode_autumn_winter, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SeasonParameterCode-SPRING", gs1s_seasonparametercode_spring, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SeasonParameterCode-SPRING_SUMMER", gs1s_seasonparametercode_spring_summer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SeasonParameterCode-SUMMER", gs1s_seasonparametercode_summer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SeasonParameterCode-WINTER", gs1s_seasonparametercode_winter, s_gs1 },
    { { s_gs1, 1, 51 }, { 0, 0 }, "SecurityMarking", gs1s_securitymarking, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 51 }, { 0, 0 }, "SecurityMarking-HOLOGRAM", gs1s_securitymarking_hologram, s_gs1 },
    { { s_gs1, 1, 51 }, { 0, 0 }, "SecurityMarking-MICROPRINTING", gs1s_securitymarking_microprinting, s_gs1 },
    { { s_gs1, 1, 51 }, { 0, 0 }, "SecurityMarking-UVINK", gs1s_securitymarking_uvink, s_gs1 },
    { { s_gs1, 1, 51 }, { 0, 0 }, "SecurityMarking-WATERMARK", gs1s_securitymarking_watermark, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SharpnessOfCheeseCode", gs1s_sharpnessofcheesecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SharpnessOfCheeseCode-EXTRA_EXTRA_SHARP", gs1s_sharpnessofcheesecode_extra_extra_sharp, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SharpnessOfCheeseCode-EXTRA_SHARP", gs1s_sharpnessofcheesecode_extra_sharp, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SharpnessOfCheeseCode-MILD", gs1s_sharpnessofcheesecode_mild, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SharpnessOfCheeseCode-REGULAR", gs1s_sharpnessofcheesecode_regular, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SharpnessOfCheeseCode-SHARP", gs1s_sharpnessofcheesecode_sharp, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SharpnessOfCheeseCode-UNCLASSIFIED", gs1s_sharpnessofcheesecode_unclassified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeCodeDetails", gs1s_sizecodedetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeDetails", gs1s_sizedetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode", gs1s_sizegroupcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-BOYS", gs1s_sizegroupcode_boys, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-GIRLS", gs1s_sizegroupcode_girls, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-INFANTS", gs1s_sizegroupcode_infants, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-JUNIORS", gs1s_sizegroupcode_juniors, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-MATERNITY", gs1s_sizegroupcode_maternity, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-MENS", gs1s_sizegroupcode_mens, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-MENS_TALL", gs1s_sizegroupcode_mens_tall, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-MISSES", gs1s_sizegroupcode_misses, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-PETITE", gs1s_sizegroupcode_petite, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-WOMENS", gs1s_sizegroupcode_womens, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeGroupCode-WOMENS_TALL", gs1s_sizegroupcode_womens_tall, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeSystemCode", gs1s_sizesystemcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeSystemCode-AUSTRALIA", gs1s_sizesystemcode_australia, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeSystemCode-CONTINENTAL", gs1s_sizesystemcode_continental, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeSystemCode-EUROPE", gs1s_sizesystemcode_europe, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeSystemCode-JAPAN", gs1s_sizesystemcode_japan, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeSystemCode-MEXICO", gs1s_sizesystemcode_mexico, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeSystemCode-UK", gs1s_sizesystemcode_uk, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeSystemCode-US", gs1s_sizesystemcode_us, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode", gs1s_sizetypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-BACK", gs1s_sizetypecode_back, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-CHEST_BUST", gs1s_sizetypecode_chest_bust, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-COLLAR", gs1s_sizetypecode_collar, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-CUP", gs1s_sizetypecode_cup, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-HIPS", gs1s_sizetypecode_hips, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-INSEAM", gs1s_sizetypecode_inseam, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-LENGTH", gs1s_sizetypecode_length, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-ONE_DIMENSION", gs1s_sizetypecode_one_dimension, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-OUTSIDE_LEG", gs1s_sizetypecode_outside_leg, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-SLEEVE", gs1s_sizetypecode_sleeve, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-WAIST", gs1s_sizetypecode_waist, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SizeTypeCode-WIDTH", gs1s_sizetypecode_width, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode", gs1s_sourceanimalcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-ALLIGATOR", gs1s_sourceanimalcode_alligator, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-ASS", gs1s_sourceanimalcode_ass, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-BEAR", gs1s_sourceanimalcode_bear, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-BEEF_COW", gs1s_sourceanimalcode_beef_cow, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-BOAR", gs1s_sourceanimalcode_boar, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-BUFFALO", gs1s_sourceanimalcode_buffalo, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-CALF", gs1s_sourceanimalcode_calf, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-CAMEL", gs1s_sourceanimalcode_camel, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-CHICKEN", gs1s_sourceanimalcode_chicken, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-COCKEREL", gs1s_sourceanimalcode_cockerel, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-COW", gs1s_sourceanimalcode_cow, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-CROCODILE", gs1s_sourceanimalcode_crocodile, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-DEER", gs1s_sourceanimalcode_deer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-DOG", gs1s_sourceanimalcode_dog, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-DUCK", gs1s_sourceanimalcode_duck, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-DUCKLING", gs1s_sourceanimalcode_duckling, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-FISH", gs1s_sourceanimalcode_fish, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-FOWL", gs1s_sourceanimalcode_fowl, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-FROG", gs1s_sourceanimalcode_frog, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-GOAT", gs1s_sourceanimalcode_goat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-GOOSE", gs1s_sourceanimalcode_goose, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-GROUSE", gs1s_sourceanimalcode_grouse, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-GUINEAFOWL", gs1s_sourceanimalcode_guineafowl, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-HARE", gs1s_sourceanimalcode_hare, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-HORSE", gs1s_sourceanimalcode_horse, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-INVERTEBRATE", gs1s_sourceanimalcode_invertebrate, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-LAMB", gs1s_sourceanimalcode_lamb, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-LAND_SNAIL", gs1s_sourceanimalcode_land_snail, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-MILK_COW", gs1s_sourceanimalcode_milk_cow, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-MOOSE", gs1s_sourceanimalcode_moose, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-OSTRICH", gs1s_sourceanimalcode_ostrich, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-OTHER", gs1s_sourceanimalcode_other, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-PARTRIDGE", gs1s_sourceanimalcode_partridge, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-PHEASANT", gs1s_sourceanimalcode_pheasant, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-PIGEON", gs1s_sourceanimalcode_pigeon, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-PORK", gs1s_sourceanimalcode_pork, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-POUSSIN", gs1s_sourceanimalcode_poussin, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-QUAIL", gs1s_sourceanimalcode_quail, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-RABBIT", gs1s_sourceanimalcode_rabbit, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-RAT", gs1s_sourceanimalcode_rat, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-REINDEER", gs1s_sourceanimalcode_reindeer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-ROE_DEER", gs1s_sourceanimalcode_roe_deer, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-SHEEP", gs1s_sourceanimalcode_sheep, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-SHELLFISH", gs1s_sourceanimalcode_shellfish, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-SNAKE", gs1s_sourceanimalcode_snake, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-SNIPE", gs1s_sourceanimalcode_snipe, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-SPARROW", gs1s_sourceanimalcode_sparrow, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-THRUSH", gs1s_sourceanimalcode_thrush, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-TURKEY", gs1s_sourceanimalcode_turkey, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-TURTLE", gs1s_sourceanimalcode_turtle, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-WOODCOCK", gs1s_sourceanimalcode_woodcock, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SourceAnimalCode-YAK", gs1s_sourceanimalcode_yak, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode", gs1s_sportingactivitytypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-AMERICAN_FOOTBALL", gs1s_sportingactivitytypecode_american_football, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-ARCHERY", gs1s_sportingactivitytypecode_archery, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-BADMINTON", gs1s_sportingactivitytypecode_badminton, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-BASEBALL", gs1s_sportingactivitytypecode_baseball, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-BASKETBALL", gs1s_sportingactivitytypecode_basketball, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-BOXING", gs1s_sportingactivitytypecode_boxing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-CLIMBING", gs1s_sportingactivitytypecode_climbing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-CRICKET", gs1s_sportingactivitytypecode_cricket, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-CYCLING", gs1s_sportingactivitytypecode_cycling, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-DIVING", gs1s_sportingactivitytypecode_diving, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-EQUESTRIAN", gs1s_sportingactivitytypecode_equestrian, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-FENCING", gs1s_sportingactivitytypecode_fencing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-FISHING", gs1s_sportingactivitytypecode_fishing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-FOOTBALL", gs1s_sportingactivitytypecode_football, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-GOLF", gs1s_sportingactivitytypecode_golf, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-GYMNASTICS", gs1s_sportingactivitytypecode_gymnastics, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-HOCKEY", gs1s_sportingactivitytypecode_hockey, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-ICE_HOCKEY", gs1s_sportingactivitytypecode_ice_hockey, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-ICE_SKATING", gs1s_sportingactivitytypecode_ice_skating, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-MOTOR_CYCLING", gs1s_sportingactivitytypecode_motor_cycling, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-RUGBY", gs1s_sportingactivitytypecode_rugby, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-RUNNING", gs1s_sportingactivitytypecode_running, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-SKIING", gs1s_sportingactivitytypecode_skiing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-SQUASH", gs1s_sportingactivitytypecode_squash, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-SURFING", gs1s_sportingactivitytypecode_surfing, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-SWIMMING", gs1s_sportingactivitytypecode_swimming, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-TENNIS", gs1s_sportingactivitytypecode_tennis, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-UNCLASSIFIED", gs1s_sportingactivitytypecode_unclassified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "SportingActivityTypeCode-UNIDENTIFIED", gs1s_sportingactivitytypecode_unidentified, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TargetConsumerGenderCode", gs1s_targetconsumergendercode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TargetConsumerGenderCode-FEMALE", gs1s_targetconsumergendercode_female, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TargetConsumerGenderCode-MALE", gs1s_targetconsumergendercode_male, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TargetConsumerGenderCode-UNISEX", gs1s_targetconsumergendercode_unisex, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TargetMarketDetails", gs1s_targetmarketdetails, s_gs1 },
    { { s_gs1, 1, 0 }, { 0, 0 }, "TextileMaterial", gs1s_textilematerial, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TextileMaterialDetails", gs1s_textilematerialdetails, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TradeChannelCode", gs1s_tradechannelcode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TradeChannelCode-INSTORE_ONLY", gs1s_tradechannelcode_instore_only, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TradeChannelCode-OMNICHANNEL", gs1s_tradechannelcode_omnichannel, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TradeChannelCode-ONLINE_ONLY", gs1s_tradechannelcode_online_only, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "TradeItem", gs1s_tradeitem, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "TypeCode", gs1s_typecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "UpperTypeCode", gs1s_uppertypecode, s_gs1, SF_ENUMERATION },
    { { s_gs1, 1, 1 }, { 0, 0 }, "UpperTypeCode-CLOSED_TOE_CLOSED_BACK_OPEN_INSTEP", gs1s_uppertypecode_closed_toe_closed_back_open_instep, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "UpperTypeCode-CLOSED_TOE_OPEN_BACK", gs1s_uppertypecode_closed_toe_open_back, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "UpperTypeCode-CLOSED_TOE_STRAP_BACK", gs1s_uppertypecode_closed_toe_strap_back, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "UpperTypeCode-FULLY_CLOSED", gs1s_uppertypecode_fully_closed, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "UpperTypeCode-OPEN_TOE_FULLY_CLOSED_BACK", gs1s_uppertypecode_open_toe_fully_closed_back, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "UpperTypeCode-OPEN_TOE_OPEN_BACK", gs1s_uppertypecode_open_toe_open_back, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "UpperTypeCode-OPEN_TOE_STRAP_BACK", gs1s_uppertypecode_open_toe_strap_back, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "UpperTypeCode-UNCLASSIFIED", gs1s_uppertypecode_unclassified, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "Warranty", gs1s_warranty, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "WarrantyPromise", gs1s_warrantypromise, s_gs1 },
    { { s_gs1, 1, 1 }, { 0, 0 }, "WearableProduct", gs1s_wearableproduct, s_gs1 },
    { { s_gs1, 1, 0 }, { s_gs1, 1, 0 }, "WearableTradeItem", gs1s_wearabletradeitem, s_gs1 },

    // ical
    { { s_ical, 1, 0 }, { 0, 0 }, "Vevent", ical_vevent, s_ical },
    { { s_ical, 1, 0 }, { 0, 0 }, "Vtimezone", ical_vtimezone, s_ical },

    // jsonld
    { { s_jsonld, 1, 0 }, { 0, 0 }, "Context", json_context, s_jsonld },
    { { s_jsonld, 1, 0 }, { 0, 0 }, "PrefixDefinition", json_prefixdefinition, s_jsonld },
    { { s_jsonld, 1, 0 }, { 0, 0 }, "TermDefinition", json_termdefinition, s_jsonld },

    // ldp
    { { s_ldp, 1, 0 }, { 0, 0 }, "Ascending", ldp_ascending, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "BasicContainer", ldp_basiccontainer, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "Container", ldp_container, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "Descending", ldp_descending, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "DirectContainer", ldp_directcontainer, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "IndirectContainer", ldp_indirectcontainer, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "MemberSubject", ldp_membersubject, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "NonRDFSource", ldp_nonrdfsource, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "Page", ldp_page, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "PageSortCriterion", ldp_pagesortcriterion, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "PreferContainment", ldp_prefercontainment, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "PreferEmptyContainer", ldp_preferemptycontainer, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "PreferMembership", ldp_prefermembership, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "PreferMinimalContainer", ldp_preferminimalcontainer, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "RDFSource", ldp_rdfsource, s_ldp },
    { { s_ldp, 1, 0 }, { 0, 0 }, "Resource", ldp_resource, s_ldp },

    // locn
    { { s_locn, 1, 0 }, { 0, 0 }, "Address", loc_address, s_locn },
    { { s_locn, 1, 0 }, { 0, 0 }, "Geometry", loc_geometry, s_locn },

    // media resources
    { { s_ma, 1, 0 }, { 0, 0 }, "Agent", ma_agent, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "AudioTrack", ma_audiotrack, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "Collection", ma_collection, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "DataTrack", ma_datatrack, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "Image", ma_image, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "Location", ma_location, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "MediaFragment", ma_mediafragment, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "MediaResource", ma_mediaresource, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "Organisation", ma_organisation, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "Person", ma_person, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "Rating", ma_rating, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "TargetAudience", ma_targetaudience, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "Track", ma_track, s_ma },
    { { s_ma, 1, 0 }, { 0, 0 }, "VideoTrack", ma_videotrack, s_ma },

    // https://web.archive.org/web/20101126105442/http://microformats.org/wiki/microdata
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_ADR, mft_adr, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_GEO, mft_geo, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_ATOM, mft_hatom, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_AUDIO, mft_haudio, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/hcalendar", mft_hcalendar, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_ENTRY, mft_hentry, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_LISTING, mft_hlisting, s_microformats, SF_NO_ITEMID | SF_DEPRECATED }, // no XMDP, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_MEDIA, mft_hmedia, s_microformats, SF_NO_ITEMID | SF_DEPRECATED }, // no XMDP, but ...
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_NEWS, mft_hnews, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_PRODUCT, mft_hproduct, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_RECIPE, mft_hrecipe, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_RESUME, mft_hresume, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_REVIEW, mft_hreview, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/" H1_REVIEW_AGGREGATE, mft_haggregate, s_microformats, SF_NO_ITEMID | SF_DEPRECATED },

    // https://microformats.org/wiki/microdata (April 2021)
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_ADR, mft2_hadr, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_BREADCRUMB, mft2_hbreadcrumb, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_CARD, mft2_hcard, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_CITE, mft2_hcite, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_ENTRY, mft2_hentry, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_EVENT, mft2_hevent, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_FEED, mft2_hfeed, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_GEO, mft2_hgeo, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_ITEM, mft2_hitem, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_LISTING, mft2_hlisting, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_PRODUCT, mft2_hproduct, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_RECIPE, mft2_hrecipe, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_RESUME, mft2_hresume, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_REVIEW, mft2_hreview, s_microformats },
    { { s_microformats, 2, 0 }, { 0, 0 }, "profile/" H_AGGREGATE, mft2_haggregate, s_microformats },

    // web annotation
    { { s_oa, 1, 0 }, { 0, 0 }, "Annotation", oa_annotation, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "Choice", oa_choice, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "CssSelector", oa_cssselector, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "CssStyle", oa_cssstyle, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "DataPositionSelector", oa_datapositionselector, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "Direction", oa_direction, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "FragmentSelector", oa_fragmentselector, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "HttpRequestState", oa_httprequeststate, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "Motivation", oa_motivation, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "RangeSelector", oa_rangeselector, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "ResourceSelection", oa_resourceselection, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "Selector", oa_selector, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "SpecificResource", oa_specificresource, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "State", oa_state, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "Style", oa_style, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "SvgSelector", oa_svgselector, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "TextPositionSelector", oa_textpositionselector, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "TextQuoteSelector", oa_textquoteselector, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "TextualBody", oa_textualbody, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "TimeState", oa_timestate },
    { { s_oa, 1, 0 }, { 0, 0 }, "XPathSelector", oa_xpathselector, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "assessing", oa_assessing, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "bookmarking", oa_bookmarking, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "classifying", oa_classifying, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "commenting", oa_commenting, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "describing", oa_describing, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "editing", oa_editing, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "highlighting", oa_highlighting, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "identifying", oa_identifying, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "linking", oa_linking, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "moderating", oa_moderating, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "questioning", oa_questioning, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "replying", oa_replying, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "tagging", oa_tagging, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "autoDirection", oa_autodirection, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "ltrDirection", oa_ltrdirection, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "rtlDirection", oa_rtldirection, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "PreferContainedDescriptions", oa_prefercontaineddescriptions, s_oa },
    { { s_oa, 1, 0 }, { 0, 0 }, "PreferContainedIRIs", oa_prefercontainediris, s_oa },

    // odrl
    { { s_odrl, 1, 0 }, { 0, 0 }, "Action", odrl_action, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Agreement", odrl_agreement, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Assertion", odrl_assertion, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Asset", odrl_asset, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "AssetCollection", odrl_assetcollection, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "ConflictTerm", odrl_conflictterm, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Constraint", odrl_constraint, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Duty", odrl_duty, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "LeftOperand", odrl_leftoperand, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "LogicalConstraint", odrl_logicalconstraint, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Offer", odrl_offer, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Operator", odrl_operator, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Party", odrl_party, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "PartyCollection", odrl_partycollection, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Permission", odrl_permission, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Policy", odrl_policy, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Privacy", odrl_privacy, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Prohibition", odrl_prohibition, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Request", odrl_request, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "RightOperand", odrl_rightoperand, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Rule", odrl_rule, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Set", odrl_set, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Ticket", odrl_ticket, s_odrl },

    { { s_odrl, 1, 0 }, { 0, 0 }, "acceptTracking", odrl_accepttracking, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "adHocShare", odrl_adhocshare, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "aggregate", odrl_aggregate, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "annotate", odrl_annotate, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "anonymize", odrl_anonymize, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "append", odrl_append, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "appendTo", odrl_appendto, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "archive", odrl_archive, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "attachPolicy", odrl_attachpolicy, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "attachSource", odrl_attachsource, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "attribute", odrl_attribute, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Attribution", odrl_attribution, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "commercialize", odrl_commercialize, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "CommericalUse", odrl_commericaluse, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "compensate", odrl_compensate, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "concurrentUse", odrl_concurrentuse, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "copy", odrl_copy, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "delete", odrl_delete, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "DerivativeWorks", odrl_derivativeworks, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "derive", odrl_derive, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "digitize", odrl_digitize, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "display", odrl_display, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "distribute", odrl_distribute, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Distribution", odrl_distribution, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "ensureExclusivity", odrl_ensureexclusivity, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "execute", odrl_execute, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "export", odrl_export, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "extract", odrl_extract, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "extractChar", odrl_extractchar, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "extractPage", odrl_extractpage, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "extractWord", odrl_extractword, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "give", odrl_give, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "grantUse", odrl_grantuse, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "include", odrl_include, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "index", odrl_index, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "inform", odrl_inform, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "install", odrl_install, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "lease", odrl_lease, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "lend", odrl_lend, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "license", odrl_license, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "modify", odrl_modify, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "move", odrl_move, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "nextPolicy", odrl_nextpolicy, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Notice", odrl_notice, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "obtainConsent", odrl_obtainconsent, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "pay", odrl_pay, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "play", odrl_play, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "present", odrl_present, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "preview", odrl_preview, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "print", odrl_print, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "read", odrl_read, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "reproduce", odrl_reproduce, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Reproduction", odrl_reproduction, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "reviewPolicy", odrl_reviewpolicy, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "secondaryUse", odrl_secondaryuse, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "sell", odrl_sell, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "share", odrl_share, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "ShareAlike", odrl_sharealike, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "Sharing", odrl_sharing, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "SourceCode", odrl_sourcecode, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "stream", odrl_stream, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "synchronize", odrl_synchronize, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "textToSpeech", odrl_texttospeech, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "transfer", odrl_transfer, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "transform", odrl_transform, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "translate", odrl_translate, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "uninstall", odrl_uninstall, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "use", odrl_use, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "watermark", odrl_watermark, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "write", odrl_write, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "writeTo", odrl_writeto, s_odrl },

    { { s_odrl, 1, 0 }, { 0, 0 }, "eq", odrl_eq, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "gt", odrl_gt, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "gteq", odrl_gteq, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "hasPart", odrl_haspart, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "isA", odrl_isa, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "isAllOf", odrl_isallof, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "isAnyOf", odrl_isanyof, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "isNoneOf", odrl_isnoneof, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "isPartOf", odrl_ispartof, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "lt", odrl_lt, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "lteq", odrl_lteq, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "neq", odrl_neq, s_odrl },

    { { s_odrl, 1, 0 }, { 0, 0 }, "absolutePosition", odrl_absoluteposition, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "absoluteSize", odrl_absolutesize, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "absoluteSpatialPosition", odrl_absolutespatialposition, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "absoluteTemporalPosition", odrl_absolutetemporalposition, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "count", odrl_count, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "dateTime", odrl_datetime, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "delayPeriod", odrl_delayperiod, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "deliveryChannel", odrl_deliverychannel, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "device", odrl_device, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "elapsedTime", odrl_elapsedtime, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "event", odrl_event, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "fileFormat", odrl_fileformat, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "industry", odrl_industry, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "language", odrl_language, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "media", odrl_media, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "meteredTime", odrl_meteredtime, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "payAmount", odrl_payamount, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "percentage", odrl_percentage, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "product", odrl_product, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "purpose", odrl_purpose, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "recipient", odrl_recipient, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "relativePosition", odrl_relativeposition, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "relativeSize", odrl_relativesize, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "relativeSpatialPosition", odrl_relativespatialposition, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "relativeTemporalPosition", odrl_relativetemporalposition, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "resolution", odrl_resolution, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "spatial", odrl_spatial, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "spatialCoordinates", odrl_spatialcoordinates, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "system", odrl_system, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "systemDevice", odrl_systemdevice, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "timeInterval", odrl_timeinterval, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "unitOfCount", odrl_unitofcount, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "version", odrl_version, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "virtualLocation", odrl_virtuallocation, s_odrl },

    { { s_odrl, 1, 0 }, { 0, 0 }, "invalid", odrl_invalid, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "perm", odrl_perm, s_odrl },
    { { s_odrl, 1, 0 }, { 0, 0 }, "prohibit", odrl_prohibit, s_odrl },

    // org
    { { s_org, 1, 0 }, { 0, 0 }, "ChangEevent", org_changeevent, s_org },
    { { s_org, 1, 0 }, { 0, 0 }, "FormalOrganisation", org_formalorganisation, s_org },
    { { s_org, 1, 0 }, { 0, 0 }, "Head", org_head, s_org },
    { { s_org, 1, 0 }, { 0, 0 }, "Membership", org_membership, s_org },
    { { s_org, 1, 0 }, { 0, 0 }, "Organization", org_organisation, s_org },
    { { s_org, 1, 0 }, { 0, 0 }, "OrganizationalCollaboration", org_organisationalcollaboration, s_org },
    { { s_org, 1, 0 }, { 0, 0 }, "OrganizationalUnit", org_organisationalunit, s_org },
    { { s_org, 1, 0 }, { 0, 0 }, "Post", org_post, s_org },
    { { s_org, 1, 0 }, { 0, 0 }, "Role", org_role, s_org },
    { { s_org, 1, 0 }, { 0, 0 }, "Site", org_site, s_org },

    // owl
    { { s_owl, 1, 0 }, { 0, 0 }, "AllDifferent", owl_alldifferent, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "AllDisjointClasses", owl_alldisjointclasses, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "AllDisjointProperties", owl_alldisjointproperties, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "Annotation", owl_annotation, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "AnnotationProperty", owl_annotationproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "AsymmetricProperty", owl_asymmetricproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "Axiom", owl_axiom, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "Class", owl_class, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "DataRange", owl_datarange, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "DatatypeProperty", owl_datatypeproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "DeprecatedClass", owl_deprecatedclass, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "DeprecatedProperty", owl_deprecatedproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "FunctionalProperty", owl_functionalproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "InverseFunctionalProperty", owl_inversefunctionalproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "IrreflexiveProperty", owl_irreflexiveproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "NamedIndividual", owl_namedindividual, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "NegativePropertyAssertion", owl_negativepropertyassertion, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "Nothing", owl_nothing, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "ObjectProperty", owl_objectproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "Ontology", owl_ontology, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "OntologyProperty", owl_ontologyproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "ReflexiveProperty", owl_reflexiveproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "Restriction", owl_restriction, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "SymmetricProperty", owl_symmetricproperty, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "Thing", owl_thing, s_owl },
    { { s_owl, 1, 0 }, { 0, 0 }, "TransitiveProperty", owl_transitiveproperty, s_owl },

    // poetry
    { { s_poetry, 1, 0 }, { 0, 0 }, "Dialect", poetry_dialect, s_poetry },
    { { s_poetry, 1, 0 }, { 0, 0 }, "Form", poetry_form, s_poetry },
    { { s_poetry, 1, 0 }, { 0, 0 }, "Poem", poetry_poem, s_poetry },
    { { s_poetry, 1, 0 }, { 0, 0 }, "Period", poetry_period, s_poetry },
    { { s_poetry, 1, 0 }, { 0, 0 }, "Stanza", poetry_stanza, s_poetry },

    // provincial
    { { s_prov, 1, 0 }, { 0, 0 }, "Activity", prov_activity, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "ActivityInfluence", prov_activityinfluence, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Agent", prov_agent, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "AgentInfluence", prov_agentinfluence, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Association", prov_association, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Attribution", prov_attribution, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Bundle", prov_bundle, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Collection", prov_collection, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Communication", prov_communication, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Delegation", prov_delegation, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Derivation", prov_derivation, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "EmptyCollection", prov_emptycollection, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "End", prov_end, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Entity", prov_entity, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "EntityInfluence", prov_entityinfluence, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Generation", prov_generation, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Influence", prov_influence, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "InstantaneousEvent", prov_instantaneousevent, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Invalidation", prov_invalidation, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Location", prov_location, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Organization", prov_organisation, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Person", prov_person, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Plan", prov_plan, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "PrimarySource", prov_primarysource, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Quotation", prov_quotation, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Revision", prov_revision, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Role", prov_role, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "SoftwareAgent", prov_softwareagent, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Start", prov_start, s_prov },
    { { s_prov, 1, 0 }, { 0, 0 }, "Usage", prov_usage, s_prov },

    // ptr content
    { { s_ptr, 1, 0 }, { 0, 0 }, "Content", ptr_content, s_ptr },
    { { s_ptr, 1, 0 }, { 0, 0 }, "ContentAsBase64", ptr_contentasbase64, s_ptr },
    { { s_ptr, 1, 0 }, { 0, 0 }, "ContentAsText", ptr_contentastext, s_ptr },
    { { s_ptr, 1, 0 }, { 0, 0 }, "ContentAsXML", ptr_contentasxml, s_ptr },
    { { s_ptr, 1, 0 }, { 0, 0 }, "DoctypeDecl", ptr_doctypedecl, s_ptr },

    // data cube
    { { s_qb, 1, 0 }, { 0, 0 }, "Attachable", qb_attachable, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "AttributeProperty", qb_attributeproperty, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "CodedProperty", qb_codedproperty, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "CodeList", qb_codelist, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "ComponentProperty", qb_componentproperty, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "ComponentSet", qb_componentset, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "ComponentSpecification", qb_componentspecification, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "DataSet", qb_dataset, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "DataStructureDefinition", qb_datastructuredefinition, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "DimensionProperty", qb_dimensionproperty, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "HierarchicalCodeList", qb_hierarchicalcodelist, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "MeasureProperty", qb_measureproperty, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "Observation", qb_observation, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "ObservationGroup", qb_observationgroup, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "Slice", qb_slice, s_qb },
    { { s_qb, 1, 0 }, { 0, 0 }, "SliceKey", qb_slicekey },

    // RDF
    { { s_rdf, 1, 0 }, { 0, 0 }, "Alt", rdf_alt, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Bag", rdf_bag, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Compoundliteral", rdf_compoundliteral, s_rdf, MAKE_SIMPLE_TYPE (t_text) },
    { { s_rdf, 1, 0 }, { 0, 0 }, "HTML", rdf_html, s_rdf, MAKE_SIMPLE_TYPE (t_xmlliteral) },
    { { s_rdf, 1, 0 }, { 0, 0 }, "JSON", rdf_json, s_rdf, MAKE_SIMPLE_TYPE (t_xmlliteral) },
    { { s_rdf, 1, 0 }, { 0, 0 }, "langString", rdf_langstring, s_rdf, MAKE_SIMPLE_TYPE (t_lang) },
    { { s_rdf, 1, 0 }, { 0, 0 }, "List", rdf_list, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Plainliteral", rdf_plainliteral, s_rdf, MAKE_SIMPLE_TYPE (t_text) },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Property", rdf_property, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Seq", rdf_seq, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "Statement", rdf_statement, s_rdf },
    { { s_rdf, 1, 0 }, { 0, 0 }, "XMLLiteral", rdf_xmlliteral, s_rdf, MAKE_SIMPLE_TYPE (t_xmlliteral) },

    // RDFg
    { { s_rdfg, 1, 0 }, { 0, 0 }, "Graph", rdfg_graph, s_rdfg },

    // RDFs
    { { s_rdfs, 1, 0 }, { 0, 0 }, "Class", rdfs_class, s_rdfs },
    { { s_rdfs, 1, 0 }, { 0, 0 }, "Container", rdfs_container, s_rdfs },
    { { s_rdfs, 1, 0 }, { 0, 0 }, "ContainerMembershipProperty", rdfs_containermembershipproperty, s_rdfs },
    { { s_rdfs, 1, 0 }, { 0, 0 }, "Datatype", rdfs_datatype, s_rdfs },
    { { s_rdfs, 1, 0 }, { 0, 0 }, "Literal", rdfs_literal, s_rdfs },
    { { s_rdfs, 1, 0 }, { 0, 0 }, "Resource", rdfs_resource, s_rdfs },

    // RDF reviews
    { { s_rev, 1, 0 }, { 0, 0 }, "Comment", rev_comment, s_rev },
    { { s_rev, 1, 0 }, { 0, 0 }, "Feedback", rev_feedback, s_rev },
    { { s_rev, 1, 0 }, { 0, 0 }, "Review", rev_review, s_rev },

    // rif
    // this is certainly incomplete; the documented XML schema has an include containing the actual definitions,
    // and that is a 404 (http://www.w3.org/2010/rif-schema/bld/LC/BLDRule.xsd). The full and detailed documents on RIF
    // itself are as clear as mud, at least in terms of RDFa. There do not appear to be any class and property lists,
    // nor do there seem to be any RDF/a example documents. No doubt I am missing something.
    { { s_rif, 1, 0 }, { 0, 0 }, "local", rev_review, s_rif, MAKE_SIMPLE_TYPE (t_text) },
    { { s_rif, 1, 0 }, { 0, 0 }, "iri", rev_review, s_rif, MAKE_SIMPLE_TYPE (t_url) },

    // rr
    { { s_rr, 1, 0 }, { 0, 0 }, "BaseTableOrView", rr_basetableorview, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "BlankNode", rr_blanknode, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "defaultGraph", rr_defaultgraph, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "GraphMap", rr_graphmap, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "IRI", rr_iri, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "Join", rr_join, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "Literal", rr_literal, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "LogicalTable", rr_logicaltable, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "ObjectMap", rr_objectmap, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "PredicateMap", rr_predicatemap, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "PredicateObjectMap", rr_predicateobjectmap, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "RefObjectMap", rr_refobjectmap, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "R2RMLView", rr_r2rmlview, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "SQL2008", rr_sql2008, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "SubjectMap", rr_subjectmap, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "TriplesMap", rr_triplesmap, s_rr },
    { { s_rr, 1, 0 }, { 0, 0 }, "TermMap", rr_termmap, s_rr },

    // schema.org
    { { 3, 5 }, { 0, 0 }, "3DModel", sch_3dmodel },
    { { 1, 0 }, { 0, 0 }, "APIReference", sch_apireference },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Abdomen", sch_abdomen },
    { { 0, 10 }, { 0, 0 }, "AboutPage", sch_aboutpage },
    { { 1, 2 }, { 0, 0 }, "AcceptAction", sch_acceptaction },
    { { 3, 1 }, { 0, 0 }, "Accommodation", sch_accommodation },
    { { 0, 10 }, { 0, 0 }, "AccountingService", sch_accountingservice },
    { { 1, 2 }, { 0, 0 }, "AchieveAction", sch_achieveaction },
    { { 1, 2 }, { 0, 0 }, "Action", sch_action },
    { { 3, 5 }, { 0, 0 }, "ActionAccessSpecification", sch_actionaccessspecification },
    { { 1, 20 }, { 0, 0 }, "ActionStatusType", sch_actionstatustype, s_schema, SF_ENUMERATION },
    { { 11, 0 }, { 0, 0 }, "ActivationFee", sch_activationfee },
    { { 1, 92 }, { 0, 0 }, "ActivateAction", sch_activateaction },
    { { 1, 20 }, { 0, 0 }, "ActiveActionStatus", sch_activeactionstatus },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "ActiveNotRecruiting", sch_activenotrecruiting },
    { { 1, 2 }, { 0, 0 }, "AddAction", sch_addaction },
    { { 0, 10 }, { 0, 0 }, "AdministrativeArea", sch_administrativearea },
    { { 0, 10 }, { 0, 0 }, "AdultEntertainment", sch_adultentertainment },
    { { 14, 0 }, { 0, 0 }, "AdultOrientatedEnumeration", sch_adultorientatedenumeration, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "AdvertiserContentArticle", sch_advertisercontentarticle },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "AerobicActivity", sch_aerobicactivity },
    { { 0, 10 }, { 0, 0 }, "AggregateOffer", sch_aggregateoffer },
    { { 0, 10 }, { 0, 0 }, "AggregateRating", sch_aggregaterating },
    { { 1, 2 }, { 0, 0 }, "AgreeAction", sch_agreeaction },
    { { 1, 20 }, { 0, 0 }, "Airline", sch_airline },
    { { 0, 10 }, { 0, 0 }, "Airport", sch_airport },
    { { 1, 92 }, { 0, 0 }, "AlbumRelease", sch_albumrelease },
    { { 14, 0 }, { 0, 0 }, "AlcoholConsideration", sch_alcoholconsideration },
    { { 1, 0 }, { 0, 0 }, "AlignmentObject", sch_alignmentobject },
    { { 12, 0 }, { 0, 0 }, "AllergiesHealthAspect", sch_allergieshealthaspect },
    { { 1, 2 }, { 0, 0 }, "AllocateAction", sch_allocateaction },
    { { 3, 6 }, { 6, 0 }, "AllWheelDriveConfiguration", sch_allwheeldriveconfiguration },
    { { 11, 0 }, { 0, 0 }, "AmpStory", sch_ampstory },
    { { 3, 5 }, { 0, 0 }, "AMRadioChannel", sch_amradiochannel },
    { { 0, 10 }, { 0, 0 }, "AmusementPark", sch_amusementpark },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "AnaerobicActivity", sch_anaerobicactivity },
    { { 3, 5 }, { 0, 0 }, "AnalysisNewsArticle", sch_analysisnewsarticle },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "AnatomicalStructure", sch_anatomicalstructure },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "AnatomicalSystem", sch_anatomicalsystem },
    { { 14, 0 }, { 0, 0 }, "AndroidPlatform", sch_androidplatform },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Anesthesia", sch_anaesthesia },
    { { 0, 10 }, { 0, 0 }, "Animalshelter", sch_animalshelter },
    { { 1, 20 }, { 0, 0 }, "Answer", sch_answer },
    { { 3, 1 }, { 0, 0 }, "Apartment", sch_apartment },
    { { 0, 10 }, { 0, 0 }, "Apartmentcomplex", sch_apartmentcomplex },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Appearance", sch_appearance },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "AppendAction", sch_appendaction },
    { { 1, 2 }, { 0, 0 }, "ApplyAction", sch_applyaction },
    { { 2, 0 }, { 0, 0 }, "ApprovedIndication", sch_approvedindication },
    { { 0, 10 }, { 0, 0 }, "Aquarium", sch_aquarium },
    { { 3, 5 }, { 0, 0 }, "ArchiveComponent", sch_archivecomponent },
    { { 3, 5 }, { 0, 0 }, "ArchiveOrganization", sch_archiveorganisation },
    { { 1, 2 }, { 0, 0 }, "ArriveAction", sch_arriveaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Artery", sch_artery },
    { { 0, 10 }, { 0, 0 }, "ArtGallery", sch_artgallery },
    { { 0, 10 }, { 0, 0 }, "Article", sch_article },
    { { 1, 2 }, { 0, 0 }, "AskAction", sch_askaction },
    { { 3, 5 }, { 0, 0 }, "AskPublicNewsArticle", sch_askpublicnewsarticle },
    { { 1, 2 }, { 0, 0 }, "AssessAction", sch_assessaction },
    { { 1, 2 }, { 0, 0 }, "AssignAction", sch_assignaction },
    { { 3, 5 }, { 0, 0 }, "Atlas", sch_atlas },
    { { 0, 10 }, { 0, 0 }, "Attorney", sch_attorney },
    { { 0, 95 }, { 0, 0 }, "Audience", sch_audience },
    { { 3, 5 }, { 0, 0 }, "Audiobook", sch_audiobook },
    { { 3, 5 }, { 0, 0 }, "AudiobookFormat", sch_audiobookformat },
    { { 0, 10 }, { 0, 0 }, "AudioObject", sch_audioobject },
    { { 13, 0 }, { 0, 0 }, "AudioObjectSnapshot", sch_audioobjectsnapshot },
    { { 7, 0 }, { 0, 0 }, "AuthenticContent", sch_authenticcontent },
    { { 3, 5 }, { 0, 0 }, "AuthoritativeLegalValue", sch_authoritativelegalvalue },
    { { 1, 2 }, { 0, 0 }, "AuthorizeAction", sch_authoriseaction },
    { { 0, 10 }, { 0, 0 }, "AutoBodyShop", sch_autobodyshop },
    { { 0, 10 }, { 0, 0 }, "AutoDealer", sch_autodealer },
    { { 0, 10 }, { 0, 0 }, "AutomatedTeller", sch_automatedteller },
    { { 0, 10 }, { 0, 0 }, "AutomotiveBusiness", sch_automotivebusiness },
    { { 0, 10 }, { 0, 0 }, "AutoPartsStore", sch_carpartsshop },
    { { 0, 10 }, { 0, 0 }, "AutoRental", sch_autorental },
    { { 0, 10 }, { 0, 0 }, "AutoRepair", sch_autorepair },
    { { 0, 10 }, { 0, 0 }, "AutoWash", sch_autowash },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Ayurvedic", sch_ayurvedic },
    { { 3, 5 }, { 0, 0 }, "BackgroundNewsArticle", sch_backgroundnewsarticle },
    { { 12, 0 }, { 0, 0 }, "BackOrder", sch_backorder },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Bacteria", sch_bacteria },
    { { 0, 10 }, { 0, 0 }, "Bakery", sch_bakery },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Balance", sch_balance },
    { { 3, 0 }, { 0, 0 }, "BankAccount", sch_bankaccount },
    { { 0, 10 }, { 0, 0 }, "BankOrCreditUnion", sch_bankorcreditunion },
    { { 2, 2 }, { 0, 0 }, "BarCode", sch_barcode },
    { { 0, 10 }, { 0, 0 }, "BarOrPub", sch_barorpub },
    { { 7, 4 }, { 0, 0 }, "BasicIncome", sch_basicincome },
    { { 0, 10 }, { 0, 0 }, "Beach", sch_beach },
    { { 0, 10 }, { 0, 0 }, "BeautySalon", sch_beautysalon },
    { { 0, 10 }, { 0, 0 }, "BedAndBreakfast", sch_bedandbreakfast },
    { { 3, 1 }, { 0, 0 }, "BedDetails", sch_beddetails },
    { { 3, 5 }, { 0, 0 }, "BedType", sch_bedtype },
    { { 1, 2 }, { 0, 0 }, "BefriendAction", sch_befriendaction },
    { { 5, 0 }, { 0, 0 }, "BenefitsHealthAspect", sch_benefitshealthaspect },
    { { 0, 10 }, { 0, 0 }, "BikeStore", sch_bikeshop },
    { { 13, 0 }, { 0, 0 }, "BioChemEntity", sch_biochementity },
    { { 0, 10 }, { 0, 0 }, "Blog", sch_blog },
    { { 0, 10 }, { 0, 0 }, "BlogPosting", sch_blogposting },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "BloodTest", sch_bloodtest },
    { { 2, 0 }, { 0, 0 }, "BoardingPolicyType", sch_boardingpolicytype, s_schema, SF_ENUMERATION },
    { { 9, 0 }, { 0, 0 }, "BoatReservation", sch_boatreservation },
    { { 9, 0 }, { 0, 0 }, "BoatTerminal", sch_boatterminal },
    { { 9, 0 }, { 0, 0 }, "BoatTrip", sch_boattrip },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementArm", sch_bodymeasurementarm },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementBust", sch_bodymeasurementbust },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementChest", sch_bodymeasurementchest },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementFoot", sch_bodymeasurementfoot },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementHand", sch_bodymeasurementhand },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementHead", sch_bodymeasurementhead },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementHeight", sch_bodymeasurementheight },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementHips", sch_bodymeasurementhips },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementInsideLeg", sch_bodymeasurementinsideleg },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementTypeEnumeration", sch_bodymeasurementtypeenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementNeck", sch_bodymeasurementneck },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementUnderbust", sch_bodymeasurementunderbust },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementWaist", sch_bodymeasurementwaist },
    { { 12, 0 }, { 0, 0 }, "BodyMeasurementWeight", sch_bodymeasurementweight },
    { { 0, 10 }, { 0, 0 }, "BodyOfWater", sch_bodyofwater },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Bone", sch_bone },
    { { 0, 10 }, { 0, 0 }, "Book", sch_book },
    { { 0, 10 }, { 0, 0 }, "BookFormatType", sch_bookformattype, s_schema, SF_ENUMERATION },
    { { 1, 2 }, { 0, 0 }, "BookMarkAction", sch_bookmarkaction },
    { { 2, 0 }, { 0, 0 }, "BookSeries", sch_bookseries },
    { { 0, 10 }, { 0, 0 }, "BookStore", sch_bookshop },
    { { 0, 10 }, { 0, 0 }, "Boolean", sch_boolean, s_schema, SF_ENUMERATION | MAKE_SIMPLE_TYPE (t_bool) },
    { { 1, 2 }, { 0, 0 }, "BorrowAction", sch_borrowaction },
    { { 0, 10 }, { 0, 0 }, "BowlingAlley", sch_bowlingalley },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "BrainStructure", sch_brainstructure },
    { { 0, 99 }, { 0, 0 }, "Brand", sch_brand },
    { { 1, 93 }, { 0, 0 }, "BreadCrumblist", sch_breadcrumblist },
    { { 0, 10 }, { 0, 0 }, "Brewery", sch_brewery },
    { { 2, 1 }, { 0, 0 }, "Bridge", sch_bridge },
    { { 2, 0 }, { 0, 0 }, "BroadcastChannel", sch_broadcastchannel },
    { { 1, 3 }, { 0, 0 }, "BroadcastEvent", sch_broadcastevent },
    { { 3, 5 }, { 0, 0 }, "BroadcastFrequencySpecification", sch_broadcastfrequencyspecification },
    { { 1, 92 }, { 0, 0 }, "BroadcastRelease", sch_broadcastrelease },
    { { 1, 3 }, { 0, 0 }, "BroadcastService", sch_broadcastservice },
    { { 3, 5 }, { 0, 0 }, "BrokerageAccount", sch_brokerageaccount },
    { { 0, 10 }, { 0, 0 }, "BuddhistTemple", sch_buddhisttemple },
    { { 1, 3 }, { 0, 0 }, "BusinessAudience", sch_businessaudience },
    { { 0, 99 }, { 0, 0 }, "BusinessEntityType", sch_businessentitytype, s_schema, SF_EXTERNAL_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "BusinessEvent", sch_businessevent },
    { { 2, 0 }, { 0, 0 }, "BusinessFunction", sch_businessfunction, s_schema, SF_EXTERNAL_ENUMERATION },
    { { 7, 4 }, { 0, 0 }, "BusinessSupport", sch_businesssupport },
    { { 3, 5 }, { 0, 0 }, "BusOrCoach", sch_busorcoach },
    { { 1, 10 }, { 0, 0 }, "BusReservation", sch_busreservation },
    { { 0, 10 }, { 0, 0 }, "BusStation", sch_busstation },
    { { 0, 10 }, { 0, 0 }, "BusStop", sch_busstop },
    { { 1, 20 }, { 0, 0 }, "BusTrip", sch_bustrip },
    { { 1, 2 }, { 0, 0 }, "BuyAction", sch_buyaction },
    { { 2, 0 }, { 0, 0 }, "CableOrSatelliteService", sch_cableorsatelliteservice },
    { { 0, 10 }, { 0, 0 }, "CafeOrCoffeeShop", sch_cafeorcoffeeshop },
    { { 0, 10 }, { 0, 0 }, "Campground", sch_campground },
    { { 3, 1 }, { 0, 0 }, "CampingPitch", sch_campingpitch },
    { { 0, 10 }, { 0, 0 }, "Canal", sch_canal },
    { { 1, 2 }, { 0, 0 }, "CancelAction", sch_cancelaction },
    { { 1, 20 }, { 0, 0 }, "Car", sch_car },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Cardiovascular", sch_cardiovascular },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "CardiovascularExam", sch_cardiovascularexam },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "CarUsageType", sch_carusagetype, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "CaseSeries", sch_caseseries },
    { { 0, 10 }, { 0, 0 }, "Casino", sch_casino },
    { { 3, 5 }, { 0, 0 }, "CassetteFormat", sch_cassetteformat },
    { { 3, 5 }, { 0, 0 }, "CategoryCode", sch_categorycode },
    { { 3, 5 }, { 0, 0 }, "CategoryCodeSet", sch_categorycodeset },
    { { 0, 10 }, { 0, 0 }, "CatholicChurch", sch_catholicchurch },
    { { 5, 0 }, { 0, 0 }, "CausesHealthAspect", sch_causeshealthaspect },
    { { 7, 3 }, { 0, 0 }, "CDCPMDRecord", sch_cdcpmdrecord },
    { { 3, 5 }, { 0, 0 }, "CDFormat", sch_cdformat },
    { { 0, 10 }, { 0, 0 }, "Cemetery", sch_cemetery },
    { { 3, 5 }, { 0, 0 }, "Chapter", sch_chapter },
    { { 9, 0 }, { 0, 0 }, "CharitableIncorporatedOrganization", sch_charitableincorporatedorganization },
    { { 1, 2 }, { 0, 0 }, "CheckAction", sch_checkaction },
    { { 1, 2 }, { 0, 0 }, "CheckInAction", sch_checkinaction },
    { { 1, 2 }, { 0, 0 }, "CheckOutAction", sch_checkoutaction },
    { { 0, 10 }, { 0, 0 }, "CheckoutPage", sch_checkoutpage },
    { { 13, 0 }, { 0, 0 }, "ChemicalSubstance", sch_chemicalsubstance },
    { { 0, 10 }, { 0, 0 }, "ChildCare", sch_childcare },
    { { 0, 10 }, { 0, 0 }, "ChildrensEvent", sch_childrensevent },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Chiropractic", sch_chiropractic },
    { { 1, 2 }, { 0, 0 }, "ChooseAction", sch_chooseaction },
    { { 0, 10 }, { 0, 0 }, "Church", sch_church },
    { { 0, 10 }, { 0, 0 }, "City", sch_city },
    { { 0, 10 }, { 0, 0 }, "CityHall", sch_townhall },
    { { 0, 10 }, { 0, 0 }, "CivicStructure", sch_civicstructure },
    { { 3, 5 }, { 0, 0 }, "Claim", sch_claim },
    { { 3, 2 }, { 0, 0 }, "ClaimReview", sch_claimreview },
    { { 1, 0, SV_NOT_3034 }, { 0, 0 }, "Class", sch_class },
    { { 11, 0 }, { 0, 0 }, "CleaningFee", sch_cleaningfee },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Clinician", sch_clinician },
    { { 1, 3 }, { 0, 0 }, "Clip", sch_clip },
    { { 0, 10 }, { 0, 0 }, "ClothingStore", sch_clothesshop },
    { { 1, 0 }, { 2, 1 }, "Code", sch_code },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "CohortStudy", sch_cohortstudy },
    { { 3, 5 }, { 0, 0 }, "Collection", sch_collection },
    { { 0, 10 }, { 0, 0 }, "CollectionPage", sch_collectionpage },
    { { 0, 10 }, { 0, 0 }, "CollegeOrUniversity", sch_collegeoruniversity },
    { { 0, 10 }, { 0, 0 }, "ComedyClub", sch_comedyclub },
    { { 0, 10 }, { 0, 0 }, "ComedyEvent", sch_comedyevent },
    { { 3, 5 }, { 0, 0 }, "ComicCoverArt", sch_comiccoverart },
    { { 3, 5 }, { 0, 0 }, "ComicIssue", sch_comicissue },
    { { 3, 5 }, { 0, 0 }, "ComicSeries", sch_comicseries },
    { { 3, 5 }, { 0, 0 }, "ComicStory", sch_comicstory },
    { { 0, 95 }, { 0, 0 }, "Comment", sch_comment },
    { { 1, 2 }, { 0, 0 }, "CommentAction", sch_commentaction },
    { { 3, 5 }, { 0, 0 }, "CommentPermission", sch_commentpermission },
    { { 1, 2 }, { 0, 0 }, "CommunicateAction", sch_communicateaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "CommunityHealth", sch_communityhealth },
    { { 1, 92 }, { 0, 0 }, "CompilationAlbum", sch_compilationalbum },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Completed", sch_completed },
    { { 1, 20 }, { 0, 0 }, "CompletedActionStatus", sch_completedactionstatus },
    { { 3, 5 }, { 0, 0 }, "CompleteDatafeed", sch_completedatafeed },
    { { 3, 0 }, { 0, 0 }, "CompoundPriceSpecification", sch_compoundpricespecification },
    { { 3, 0 }, { 0, 0 }, "ComputerLanguage", sch_computerlanguage },
    { { 0, 10 }, { 0, 0 }, "ComputerStore", sch_computershop },
    { { 1, 2 }, { 0, 0 }, "ConfirmAction", sch_confirmaction },
    { { 3, 5 }, { 0, 0 }, "Consortium", sch_consortium },
    { { 1, 2 }, { 0, 0 }, "ConsumeAction", sch_consumeaction },
    { { 0, 10 }, { 0, 0 }, "ContactPage", sch_contactpage },
    { { 0, 10 }, { 0, 0 }, "ContactPoint", sch_contactpoint },
    { { 1, 3 }, { 0, 0 }, "ContactPointOption", sch_contactpointoption, s_schema, SF_ENUMERATION },
    { { 5, 0 }, { 0, 0 }, "ContagiousnessHealthAspect", sch_contagiousnesshealthaspect },
    { { 0, 10 }, { 0, 0 }, "Continent", sch_continent },
    { { 1, 92 }, { 0, 0 }, "ControlAction", sch_controlaction },
    { { 0, 10 }, { 0, 0 }, "ConvenienceStore", sch_convenienceshop },
    { { 3, 0 }, { 0, 0 }, "Conversation", sch_conversation },
    { { 1, 2 }, { 0, 0 }, "CookAction", sch_cookaction },
    { { 1, 92 }, { 0, 0 }, "Coop", sch_coop },
    { { 0, 10 }, { 0, 0 }, "Corporation", sch_corporation },
    { { 3, 5 }, { 0, 0 }, "CorrectionComment", sch_correctioncomment },
    { { 0, 10 }, { 0, 0 }, "Country", sch_country },
    { { 3, 2 }, { 0, 0 }, "Course", sch_course },
    { { 3, 2 }, { 0, 0 }, "CourseInstance", sch_courseinstance },
    { { 0, 10 }, { 0, 0 }, "CourtHouse", sch_courthouse },
    { { 3, 5 }, { 0, 0 }, "CoverArt", sch_coverart },
    { { 7, 0 }, { 0, 0 }, "CovidTestingFacility", sch_covidtestingfacility },
    { { 1, 2 }, { 0, 0 }, "CreateAction", sch_createaction },
    { { 0, 10 }, { 0, 0 }, "CreativeWork", sch_creativework },
    { { 2, 0 }, { 0, 0 }, "CreativeWorkSeason", sch_creativeworkseason },
    { { 2, 0 }, { 0, 0 }, "CreativeWorkSeries", sch_creativeworkseries },
    { { 0, 99 }, { 0, 0 }, "CreditCard", sch_creditcard },
    { { 0, 10 }, { 0, 0 }, "Crematorium", sch_crematorium },
    { { 3, 5 }, { 0, 0 }, "CriticReview", sch_criticreview },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "CrossSectional", sch_crosssectional },
    { { 3, 5 }, { 0, 0 }, "CssSelectorType", sch_cssselectortype },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "CT", sch_ct },
    { { 3, 1 }, { 0, 0 }, "CurrencyConversionService", sch_currencyconversionservice },
    { { 0, 10 }, { 0, 0 }, "DamagedCondition", sch_damagedcondition },
    { { 0, 10 }, { 0, 0 }, "DanceEvent", sch_danceevent },
    { { 0, 10 }, { 0, 0 }, "DanceGroup", sch_dancegroup },
    { { 14, 0 }, { 0, 0 }, "DangerousGoodConsideration", sch_dangerousgoodconsideration },
    { { 1, 0 }, { 0, 0 }, "DataCatalog", sch_datacatalogue },
    { { 1, 0 }, { 0, 0 }, "DataDownload", sch_datadownload },
    { { 3, 0 }, { 0, 0 }, "DataFeed", sch_datafeed },
    { { 3, 0 }, { 0, 0 }, "DataFeedItem", sch_datafeeditem },
    { { 1, 0 }, { 0, 0 }, "Dataset", sch_dataset },
    { { 0, 10 }, { 0, 0 }, "DataType", sch_datatype },
    { { 0, 10 }, { 0, 0 }, "Date", sch_date, s_schema, MAKE_SIMPLE_TYPE (t_just_date) },
    { { 1, 92 }, { 3, 4 }, "DatedMoneySpecification", sch_datedmoneyspecification },
    { { 0, 95 }, { 0, 0 }, "DateTime", sch_datetime, s_schema, MAKE_SIMPLE_TYPE (t_datetime) },
    { { 0, 99 }, { 0, 0 }, "DayOfWeek", sch_dayofweek, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "DaySpa", sch_dayspa },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DDxElement", sch_ddxelement },
    { { 1, 92 }, { 0, 0 }, "DeactivateAction", sch_deactivateaction },
    { { 12, 0 }, { 0, 0 }, "DecontextualizedContent", sch_decontextualisedcontent },
    { { 0, 10 }, { 0, 0 }, "DefenceEstablishment", sch_defenceestablishment },
    { { 7, 4 }, { 0, 0 }, "DefinedRegion", sch_definedregion },
    { { 3, 5 }, { 0, 0 }, "DefinedTerm", sch_definedterm },
    { { 3, 5 }, { 0, 0 }, "DefinedTermSet", sch_definedtermset },
    { { 3, 5 }, { 0, 0 }, "DefinitiveLegalValue", sch_definitivelegalvalue },
    { { 1, 2 }, { 0, 0 }, "DeleteAction", sch_deleteaction },
    { { 0, 99 }, { 0, 0 }, "DeliveryChargeSpecification", sch_deliverychargespecification },
    { { 1, 4 }, { 0, 0 }, "DeliveryEvent", sch_deliveryevent },
    { { 0, 99 }, { 0, 0 }, "DeliveryMethod", sch_deliverymethod, s_schema, SF_ENUMERATION },
    { { 8, 0 }, { 0, 0 }, "DeliveryTimeSettings", sch_deliverytimesettings },
    { { 0, 99 }, { 0, 0 }, "Demand", sch_demand },
    { { 1, 92 }, { 0, 0 }, "DemoAlbum", sch_demoalbum },
    { { 14, 0 }, { 0, 0 }, "DemoGameAvailability", sch_demogameavailability },
    { { 0, 10 }, { 0, 0 }, "Dentist", sch_dentist },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Dentistry", sch_dentistry },
    { { 1, 2 }, { 0, 0 }, "DepartAction", sch_departaction },
    { { 0, 10 }, { 0, 0 }, "DepartmentStore", sch_departmentstore },
    { { 3, 0 }, { 0, 0 }, "DepositAccount", sch_depositaccount },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Dermatology", sch_dermatology },
    { { 14, 0 }, { 0, 0 }, "DesktopWebPlatform", sch_desktopwebplatform },
    { { 3, 5 }, { 0, 0 }, "DiabeticDiet", sch_diabeticdiet },
    { { 0, 95, SV_NOT_3034 }, { 0, 0}, "Diagnostic", sch_diagnostic },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DiagnosticLab", sch_diagnosticlab },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DiagnosticProcedure", sch_diagnosticprocedure },
    { { 0, 95 }, { 0, 0 }, "Diet", sch_diet },
    { { 2, 0 }, { 0, 0 }, "DietarySupplement", sch_dietarysupplement },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DietNutrition", sch_dietnutrition },
    { { 3, 5 }, { 0, 0 }, "DigitalAudioTapeFormat", sch_digitalaudiotapeformat },
    { { 3, 0 }, { 0, 0 }, "DigitalDocument", sch_digitaldocument },
    { { 3, 0 }, { 0, 0 }, "DigitalDocumentPermission", sch_digitaldocumentpermission },
    { { 3, 0 }, { 0, 0 }, "DigitalDocumentPermissionType", sch_digitaldocumentpermissiontype, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "DigitalFormat", sch_digitalformat },
    { { 14, 0 }, { 0, 0 }, "DigitalPlatformEnumeration", sch_digitalplatformenumeration, s_schema, SF_ENUMERATION },
    { { 7, 4 }, { 0, 0 }, "DisabilitySupport", sch_disabilitysupport },
    { { 1, 2 }, { 0, 0 }, "DisagreeAction", sch_disagreeaction },
    { { 0, 10 }, { 0, 0 }, "Discontinued", sch_discontinued },
    { { 1, 2 }, { 0, 0 }, "DiscoverAction", sch_discoveraction },
    { { 2, 1 }, { 0, 0 }, "DiscussionForumPosting", sch_discussionforumposting },
    { { 1, 2 }, { 0, 0 }, "DislikeAction", sch_dislikeaction },
    { { 0, 10 }, { 0, 0 }, "Distance", sch_distance },
    { { 11, 0 }, { 0, 0 }, "DistanceFee", sch_distancefee },
    { { 3, 5 }, { 0, 0 }, "Distillery", sch_distillery },
    { { 1, 92 }, { 0, 0 }, "DJMixAlbum", sch_djmixalbum },
    { { 1, 2 }, { 0, 0 }, "DonateAction", sch_donateaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DoseSchedule", sch_doseschedule },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DoubleBlindedTrial", sch_doubleblindedtrial },
    { { 2, 0 }, { 0, 0 }, "DownloadaAtion", sch_downloadaction },
    { { 11, 0 }, { 0, 0 }, "Downpayment", sch_downpayment },
    { { 1, 2 }, { 0, 0 }, "DrawAction", sch_drawaction },
    { { 3, 5 }, { 0, 0 }, "Drawing", sch_drawing },
    { { 1, 2 }, { 0, 0 }, "DrinkAction", sch_drinkaction },
    { { 2, 0 }, { 0, 0 }, "DriveWheelConfigurationValue", sch_drivewheelconfigurationvalue, s_schema, SF_ENUMERATION },
    { { 3, 6 }, { 6, 0 }, "DrivingSchoolVehicleUsage", sch_drivingschoolvehicleusage },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Drug", sch_drug },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DrugClass", sch_drugclass },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DrugCost", sch_drugcost },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DrugCostCategory", sch_drugcostcategory, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DrugLegalStatus", sch_druglegalstatus },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DrugPregnancyCategory", sch_drugpregnancycategory, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DrugPrescriptionStatus", sch_drugprescriptionstatus },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "DrugStrength", sch_drugstrength },
    { { 0, 10 }, { 0, 0 }, "DryCleaningOrLaundry", sch_drycleaningorlaundry },
    { { 0, 10 }, { 0, 0 }, "Duration", sch_duration, s_schema, MAKE_SIMPLE_TYPE (t_duration) },
    { { 3, 5 }, { 0, 0 }, "DVDFormat", sch_dvdformat },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Ear", sch_ear },
    { { 1, 2 }, { 0, 0 }, "EatAction", sch_eataction },
    { { 0, 10 }, { 0, 0 }, "EBook", sch_ebook },
    { { 12, 0 }, { 0, 0 }, "EditedOrCroppedContent", sch_editedorcroppedcontent },
    { { 1, 0 }, { 0, 0 }, "EducationalAudience", sch_educationaudience },
    { { 3, 5 }, { 0, 0 }, "EducationalOccupationalCredential", sch_educationoccupationcredential },
    { { 3, 9 }, { 0, 0 }, "EducationalOccupationalProgram", sch_educationoccupationprogramme },
    { { 0, 10 }, { 0, 0 }, "EducationalOrganization", sch_educationorganisation },
    { { 0, 10 }, { 0, 0 }, "EducationEvent", sch_educationevent },
    { { 12, 0 }, { 0, 0 }, "EffectivenessHealthAspect", sch_effectivenesshealthaspect },
    { { 0, 10 }, { 0, 0 }, "Electrician", sch_electrician },
    { { 0, 10 }, { 0, 0 }, "ElectronicsStore", sch_electronicsshop },
    { { 0, 10 }, { 0, 0 }, "ElementarySchool", sch_elementaryschool },
    { { 1, 10 }, { 0, 0 }, "EmailMessage", sch_emailmessage },
    { { 0, 10 }, { 0, 0 }, "Embassy", sch_embassy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Emergency", sch_emergency },
    { { 0, 10 }, { 0, 0 }, "EmergencyService", sch_emergencyservice },
    { { 1, 92 }, { 0, 0 }, "EmployeeRole", sch_employeerole },
    { { 3, 5 }, { 0, 0 }, "EmployerAggregateRating", sch_employeraggregaterating },
    { { 3, 5 }, { 0, 0 }, "EmployerReview", sch_employerreview },
    { { 0, 10 }, { 0, 0 }, "EmploymentAgency", sch_employmentagency },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Endocrine", sch_endocrine },
    { { 1, 2 }, { 0, 0 }, "EndorseAction", sch_endorseaction },
    { { 3, 5 }, { 0, 0 }, "EndorsementRating", sch_endorsementrating },
    { { 0, 15 }, { 0, 0 }, "Energy", sch_energy },
    { { 10, 0 }, { 0, 0 }, "EnergyConsumptionDetails", sch_energyconsumptiondetails },
    { { 10, 0 }, { 0, 0 }, "EnergyEfficiencyEnumeration", sch_energyefficiencyenumeration, s_schema, SF_ENUMERATION },
    { { 10, 0 }, { 0, 0 }, "EnergyStarCertified", sch_energystarcertified },
    { { 2, 0 }, { 0, 0 }, "EngineSpecification", sch_enginespecification },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "EnrollingByInvitation", sch_enrollingbyinvitation },
    { { 0, 10 }, { 0, 0 }, "EntertainmentBusiness", sch_entertainmentbusiness },
    { { 1, 20 }, { 0, 0 }, "EntryPoint", sch_entrypoint },
    { { 0, 10 }, { 0, 0 }, "Enumeration", sch_enumeration },
    { { 1, 3 }, { 0, 0 }, "Episode", sch_episode },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryA", sch_euenergyefficiencycategorya },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryA1Plus", sch_euenergyefficiencycategorya1plus },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryA2Plus", sch_euenergyefficiencycategorya2plus },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryA3Plus", sch_euenergyefficiencycategorya3plus },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryB", sch_euenergyefficiencycategoryb },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryC", sch_euenergyefficiencycategoryc },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryD", sch_euenergyefficiencycategoryd },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryE", sch_euenergyefficiencycategorye },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryF", sch_euenergyefficiencycategoryf },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyCategoryG", sch_euenergyefficiencycategoryg },
    { { 10, 0 }, { 0, 0 }, "EUEnergyEfficiencyEnumeration", sch_euenergyefficiencyenumeration, s_schema, SF_ENUMERATION },
    { { 1, 92 }, { 0, 0 }, "EPRelease", sch_eprelease },
    { { 0, 10 }, { 0, 0 }, "Event", sch_event },
    { { 7, 0 }, { 0, 0 }, "EventAttendanceModeEnumeration", sch_eventattendancemodeenumeration, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "EventCancelled", sch_eventcancelled },
    { { 7, 0 }, { 0, 0 }, "EventMovedOnline", sch_eventmovedonline },
    { { 3, 5 }, { 0, 0 }, "EventPostponed", sch_eventpostponed },
    { { 3, 5 }, { 0, 0 }, "EventRescheduled", sch_eventrescheduled },
    { { 1, 10 }, { 0, 0 }, "EventReservation", sch_eventreservation },
    { { 3, 5 }, { 0, 0 }, "EventScheduled", sch_eventscheduled },
    { { 3, 5 }, { 0, 0 }, "EventSeries", sch_eventseries },
    { { 1, 3 }, { 0, 0 }, "EventStatusType", sch_eventstatustype, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "EventVenue", sch_eventvenue },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "EvidenceLevelA", sch_evidencelevela },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "EvidenceLevelB", sch_evidencelevelb },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "EvidenceLevelC", sch_evidencelevelc },
    { { 3, 5 }, { 0, 0 }, "ExchangeRateSpecification", sch_exchangeratespecification },
    { { 3, 8 }, { 0, 0 }, "ExchangeRefund", sch_exchangerefund },
    { { 1, 2 }, { 0, 0 }, "ExerciseAction", sch_exerciseaction },
    { { 0, 10 }, { 0, 0 }, "ExerciseGym", sch_exercisegym },
    { { 0, 95 }, { 0, 0 }, "ExercisePlan", sch_exerciseplan },
    { { 2, 2 }, { 0, 0 }, "ExhibitionEvent", sch_exhibitionevent },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Eye", sch_eye },
    { { 1, 20 }, { 0, 0 }, "FailedActionStatus", sch_failedactionstatus },
    { { 1, 92 }, { 0, 0 }, "False", sch_false },
    { { 3, 5 }, { 0, 0 }, "FAQPage", sch_faqpage },
    { { 0, 10 }, { 0, 0 }, "FastFoodRestaurant", sch_fastfoodrestaurant },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "FDACategoryA", sch_fdacategorya },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "FDACategoryB", sch_fdacategoryb },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "FDACategoryC", sch_fdacategoryc },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "FDACategoryD", sch_fdacategoryd },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "FDACategoryX", sch_fdacategoryx },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "FDANotEvaluated", sch_fdanotevaluated },
    { { 3, 5 }, { 0, 0 }, "Female", sch_female },
    { { 0, 10 }, { 0, 0 }, "Festival", sch_festival },
    { { 1, 2 }, { 0, 0 }, "FilmAction", sch_filmaction },
    { { 3, 0 }, { 0, 0 }, "FinancialProduct", sch_financialproduct },
    { { 0, 10 }, { 0, 0 }, "FinancialService", sch_financialservice },
    { { 1, 2 }, { 0, 0 }, "FindAction", sch_findaction },
    { { 0, 10 }, { 0, 0 }, "FireStation", sch_firestation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Flexibility", sch_flexibility },
    { { 1, 20 }, { 0, 0 }, "Flight", sch_flight },
    { { 1, 10 }, { 0, 0 }, "FlightReservation", sch_flightreservation },
    { { 0, 10 }, { 0, 0 }, "Float", sch_float, s_schema, MAKE_SIMPLE_TYPE (t_real) },
    { { 6, 0 }, { 0, 0 }, "FloorPlan", sch_floorplan },
    { { 0, 10 }, { 0, 0 }, "Florist", sch_florist },
    { { 3, 5 }, { 0, 0 }, "FMRadioChannel", sch_fmradiochannel },
    { { 1, 2 }, { 0, 0 }, "FollowAction", sch_followaction },
    { { 0, 10 }, { 0, 0 }, "FoodEstablishment", sch_foodestablishment },
    { { 1, 10 }, { 0, 0 }, "FoodEstablishmentReservation", sch_foodestablishmentreservation },
    { { 0, 10 }, { 0, 0 }, "FoodEvent", sch_foodevent },
    { { 3, 1 }, { 0, 0 }, "FoodService", sch_foodservice },
    { { 3, 6 }, { 6, 0 }, "FourWheelDriveConfiguration", sch_fourwheeldriveconfiguration },
    { { 13, 0 }, { 0, 0 }, "FreeReturn", sch_freereturn },
    { { 3, 0 }, { 0, 0 }, "Friday", sch_friday },
    { { 3, 6 }, { 6, 0 }, "FrontWheelDriveConfiguration", sch_frontwheeldriveconfiguration },
    { { 14, 0 }, { 0, 0 }, "FullGameAvailability", sch_fullgameavailability },
    { { 3, 8 }, { 0, 0 }, "FullRefund", sch_fullrefund },
    { { 3, 5 }, { 0, 0 }, "FundingAgency", sch_fundingagency },
    { { 3, 5 }, { 0, 0 }, "FundingScheme", sch_fundingscheme },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Fungus", sch_fungus },
    { { 0, 10 }, { 0, 0 }, "FurnitureStore", sch_furnitureshop },
    { { 2, 0 }, { 0, 0 }, "Game", sch_game },
    { { 14, 0 }, { 0, 0 }, "GameAvailabilityEnumeration", sch_gameavailabilityenumeration, s_schema, SF_ENUMERATION },
    { { 1, 92 }, { 0, 0 }, "GamePlayMode", sch_gameplaymode, s_schema, SF_ENUMERATION },
    { { 1, 92 }, { 0, 0 }, "GameServer", sch_gameserver },
    { { 1, 92 }, { 0, 0 }, "GameServerStatus", sch_gameserverstatus, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "GardenStore", sch_gardencentre },
    { { 0, 10 }, { 0, 0 }, "GasStation", sch_petrolstation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Gastroenterologic", sch_gastroenterologic },
    { { 0, 10 }, { 0, 0 }, "GatedResidenceCommunity", sch_gatedresidencecommunity },
    { { 3, 0 }, { 0, 0 }, "GenderType", sch_gendertype, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "GeneralContractor", sch_generalcontractor },
    { { 13, 0 }, { 0, 0 }, "Gene", sch_gene },
    { { 14, 0 }, { 0, 0 }, "GenericWebPlatform", sch_genericwebplatform },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Genetic", sch_genetic },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Genitourinary", sch_genitourinary },
    { { 2, 2 }, { 0, 0 }, "GeoCircle", sch_geocircle },
    { { 0, 10 }, { 0, 0 }, "GeoCoordinates", sch_geocoordinates },
    { { 0, 30 }, { 0, 0 }, "GeoShape", sch_geoshape },
    { { 3, 5 }, { 0, 0 }, "GeospatialGeometry", sch_geospatialgeometry },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Geriatric", sch_geriatric },
    { { 12, 0 }, { 0, 0 }, "GettingAccessHealthAspect", sch_gettingaccesshealthaspect },
    { { 1, 2 }, { 0, 0 }, "GiveAction", sch_giveaction },
    { { 3, 5 }, { 0, 0 }, "GlutenFreeDiet", sch_glutenfreediet },
    { { 0, 10 }, { 0, 0 }, "GolfCourse", sch_golfcourse },
    { { 7, 4 }, { 0, 0 }, "GovernmentBenefitsType", sch_governmentbenefitstype, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "GovernmentBuilding", sch_governmentbuilding },
    { { 0, 10 }, { 0, 0 }, "GovernmentOffice", sch_governmentoffice },
    { { 0, 10 }, { 0, 0 }, "GovernmentOrganization", sch_governmentorganisation },
    { { 1, 4 }, { 0, 0 }, "GovernmentPermit", sch_governmentpermit },
    { { 1, 3 }, { 0, 0 }, "GovernmentService", sch_governmentservice },
    { { 3, 5 }, { 0, 0 }, "Grant", sch_grant },
    { { 3, 5 }, { 0, 0 }, "GraphicNovel", sch_graphicnovel },
    { { 0, 10 }, { 0, 0 }, "GroceryStore", sch_grocers },
    { { 3, 5 }, { 0, 0 }, "GroupBoardingPolicy", sch_groupboardingpolicy },
    { { 6, 0 }, { 0, 0 }, "Guide", sch_guide },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Gynecologic", sch_gynaecologic },
    { { 7, 4 }, { 0, 0 }, "Hackathon", sch_hackathon },
    { { 0, 10 }, { 0, 0 }, "HairSalon", sch_hairsalon },
    { { 3, 5 }, { 0, 0 }, "HalalDiet", sch_halaldiet },
    { { 0, 10 }, { 0, 0 }, "Hardcover", sch_hardcover },
    { { 0, 10 }, { 0, 0 }, "HardwareStore", sch_hardwareshop },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Head", sch_head },
    { { 0, 10 }, { 0, 0 }, "HealthAndBeautyBusiness", sch_healthandbeautybusiness },
    { { 5, 0 }, { 0, 0 }, "HealthAspectEnumeration", sch_healthaspectenumeration, s_schema, SF_ENUMERATION },
    { { 7, 4 }, { 0, 0 }, "HealthCare", sch_healthcare },
    { { 14, 0 }, { 0, 0 }, "HealthcareConsideration", sch_healthcareconsideration },
    { { 0, 10 }, { 0, 0 }, "HealthClub", sch_healthclub },
    { { 3, 5 }, { 0, 0 }, "HealthInsurancePlan", sch_dystopianhealthinsuranceplan },
    { { 3, 5 }, { 0, 0 }, "HealthPlanCostSharingSpecification", sch_dystopianhealthplancostsharingspecification },
    { { 3, 5 }, { 0, 0 }, "HealthPlanFormulary", sch_dystopianhealthplanformulary },
    { { 3, 5 }, { 0, 0 }, "HealthPlanNetwork", sch_dystopianhealthplannetwork },
    { { 5, 0 }, { 0, 0 }, "HealthTopicContent", sch_healthtopiccontent },
    { { 3, 5 }, { 0, 0 }, "HearingImpairedSupported", sch_hearingimpairedsupported },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Hematologic", sch_haematologic },
    { { 0, 10 }, { 0, 0 }, "HighSchool", sch_highschool },
    { { 3, 5 }, { 0, 0 }, "HinduDiet", sch_hindudiet },
    { { 0, 10 }, { 0, 0 }, "HinduTemple", sch_hindutemple },
    { { 0, 10 }, { 0, 0 }, "HobbyShop", sch_hobbyshop },
    { { 0, 10 }, { 0, 0 }, "HomeAndConstructionBusiness", sch_homeandconstructionbusiness },
    { { 0, 10 }, { 0, 0 }, "HomeGoodsStore", sch_householdgoodsshop },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Homeopathic", sch_homeopathic },
    { { 0, 10 }, { 0, 0 }, "Hospital", sch_hospital },
    { { 0, 10 }, { 0, 0 }, "Hostel", sch_hostel },
    { { 0, 10 }, { 0, 0 }, "Hotel", sch_hotel },
    { { 3, 1 }, { 0, 0 }, "HotelRoom", sch_hotelroom },
    { { 3, 1 }, { 0, 0 }, "House", sch_house },
    { { 0, 10 }, { 0, 0 }, "HousePainter", sch_housepainter },
    { { 12, 0 }, { 0, 0 }, "HowItWorksHealthAspect", sch_howitworkshealthaspect },
    { { 5, 0 }, { 0, 0 }, "HowOrWhereHealthAspect", sch_howorwherehealthaspect },
    { { 3, 3 }, { 0, 0 }, "HowTo", sch_howto },
    { { 3, 3 }, { 0, 0 }, "HowToDirection", sch_howtodirection },
    { { 3, 3 }, { 0, 0 }, "HowToItem", sch_howtoitem },
    { { 3, 3 }, { 0, 0 }, "HowToSection", sch_howtosection },
    { { 3, 3 }, { 0, 0 }, "HowToStep", sch_howtostep },
    { { 3, 3 }, { 0, 0 }, "HowToSupply", sch_howtosupply },
    { { 3, 3 }, { 0, 0 }, "HowToTip", sch_howtotip },
    { { 3, 3 }, { 0, 0 }, "HowToTool", sch_howtotool },
    { { 0, 10 }, { 0, 0 }, "HVACBusiness", sch_hvacbusiness },
    { { 11, 0 }, { 0, 0 }, "HyperToc", sch_hypertoc },
    { { 11, 0 }, { 0, 0 }, "HyperTocEntry", sch_hypertocentry },
    { { 0, 10 }, { 0, 0 }, "IceCreamShop", sch_icecreamshop },
    { { 1, 2 }, { 0, 0 }, "IgnoreAction", sch_ignoreaction },
    { { 0, 10 }, { 0, 0 }, "ImageGallery", sch_imagegallery },
    { { 0, 10 }, { 0, 0 }, "ImageObject", sch_imageobject },
    { { 13, 0 }, { 0, 0 }, "ImageObjectSnapshot", sch_imageobjectsnapshot },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "ImagingTest", sch_imagingtest },
    { { 0, 99 }, { 0, 0 }, "IndividualProduct", sch_individualproduct },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Infectious", sch_infectious },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "InfectiousAgentClass", sch_infectiousagentclass, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "InfectiousDisease", sch_infectiousdisease },
    { { 3, 5 }, { 0, 0 }, "InForce", sch_inforce },
    { { 1, 2 }, { 0, 0 }, "InformAction", sch_informaction },
    { { 12, 0 }, { 0, 0 }, "IngredientsHealthAspect", sch_ingredientshealthaspect },
    { { 1, 2 }, { 0, 0 }, "InsertAction", sch_insertaction },
    { { 1, 2 }, { 0, 0 }, "InstallAction", sch_installaction },
    { { 11, 0 }, { 0, 0 }, "Installment", sch_installment },
    { { 0, 10 }, { 0, 0 }, "InStock", sch_instock },
    { { 0, 10 }, { 0, 0 }, "InStoreOnly", sch_instoreonly },
    { { 0, 10 }, { 0, 0 }, "InsuranceAgency", sch_insuranceagency },
    { { 0, 10 }, { 0, 0 }, "Intangible", sch_intangible },
    { { 0, 10 }, { 0, 0 }, "Integer", sch_integer, s_schema, MAKE_SIMPLE_TYPE (t_integer) },
    { { 1, 2 }, { 0, 0 }, "InteractAction", sch_interactaction },
    { { 2, 1 }, { 0, 0 }, "InteractionCounter", sch_interactioncounter },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "InternationalTrial", sch_internationaltrial },
    { { 0, 10 }, { 0, 0 }, "InternetCafe", sch_internetcafe },
    { { 3, 5 }, { 0, 0 }, "InvestmentFund", sch_investmentfund },
    { { 3, 1 }, { 0, 0 }, "InvestmentOrDeposit", sch_investmentordeposit },
    { { 1, 2 }, { 0, 0 }, "InviteAction", sch_inviteaction },
    { { 1, 93 }, { 0, 0 }, "Invoice", sch_invoice },
    { { 11, 0 }, { 0, 0 }, "InvoicePrice", sch_invoiceprice },
    { { 14, 0 }, { 0, 0 }, "IOSPlatform", sch_iosplatform },
    { { 0, 10 }, { 0, 0 }, "ItemAvailability", sch_itemavailability, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "ItemList", sch_itemlist },
    { { 1, 92 }, { 0, 0 }, "ItemListOrderAscending", sch_itemlistorderascending },
    { { 1, 92 }, { 0, 0 }, "ItemListOrderDescending", sch_itemlistorderdescending },
    { { 1, 92 }, { 0, 0 }, "ItemListOrderType", sch_itemlistordertype, s_schema, SF_ENUMERATION },
    { { 1, 92 }, { 0, 0 }, "ItemListUnordered", sch_itemlistunordered },
    { { 0, 10 }, { 0, 0 }, "ItemPage", sch_itempage },
    { { 0, 10 }, { 0, 0 }, "JewelryStore", sch_jewellers },
    { { 0, 60 }, { 0, 0 }, "JobPosting", sch_jobposting },
    { { 1, 2 }, { 0, 0 }, "JoinAction", sch_joinaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Joint", sch_joint },
    { { 3, 5 }, { 0, 0 }, "KosherDiet", sch_kosherdiet },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "LaboratoryScience", sch_laboratoryscience },
    { { 0, 10 }, { 0, 0 }, "LakeBodyOfWater", sch_lakebodyofwater },
    { { 0, 10 }, { 0, 0 }, "Landform", sch_landform },
    { { 0, 10 }, { 0, 0 }, "LandmarksOrHistoricalBuildings", sch_landmarksorhistoricalbuildings },
    { { 0, 10 }, { 0, 0 }, "Language", sch_language },
    { { 3, 5 }, { 0, 0 }, "LaserDiscFormat", sch_laserdiscformat },
    { { 9, 0 }, { 0, 0 }, "LearningResource", sch_learningresource },
    { { 1, 2 }, { 0, 0 }, "LeaveAction", sch_leaveaction },
    { { 3, 6 }, { 6, 0 }, "LeftHandDriving", sch_lefthanddriving },
    { { 3, 5 }, { 0, 0 }, "LegalForceStatus", sch_legalforcestatus, s_schema, SF_ENUMERATION },
    { { 2, 2 }, { 0, 0 }, "LegalService", sch_legalservice },
    { { 3, 5 }, { 0, 0 }, "LegalValueLevel", sch_legalvaluelevel, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "Legislation", sch_legislation },
    { { 3, 5 }, { 0, 0 }, "LegislationObject", sch_legislationobject },
    { { 0, 10 }, { 0, 0 }, "LegislativeBuilding", sch_legislativebuilding },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "LeisureTimeActivity", sch_leisuretimeactivity },
    { { 1, 2 }, { 0, 0 }, "LendAction", sch_lendaction },
    { { 0, 10 }, { 0, 0 }, "Library", sch_library },
    { { 3, 5 }, { 0, 0 }, "LibrarySystem", sch_librarysystem },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "LifestyleModification", sch_lifestylemodification },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Ligament", sch_ligament },
    { { 1, 2 }, { 0, 0 }, "LikeAction", sch_likeaction },
    { { 3, 5 }, { 0, 0 }, "LimitedAvailability", sch_limitedavailability },
    { { 9, 0 }, { 0, 0 }, "LimitedByGuaranteeCharity", sch_limitedbyguaranteecharity },
    { { 3, 5 }, { 0, 0 }, "LinkRole", sch_linkrole },
    { { 0, 10 }, { 0, 0 }, "LiquorStore", sch_offlicence },
    { { 1, 2 }, { 0, 0 }, "ListenAction", sch_listenaction },
    { { 1, 92 }, { 0, 0 }, "ListItem", sch_listitem },
    { { 11, 0 }, { 0, 0 }, "ListPrice", sch_listprice },
    { { 0, 10 }, { 0, 0 }, "LiteraryEvent", sch_literaryevent },
    { { 1, 92 }, { 0, 0 }, "LiveAlbum", sch_livealbum },
    { { 2, 1 }, { 0, 0 }, "LiveBlogPosting", sch_liveblogposting },
    { { 5, 0 }, { 0, 0 }, "LivingWithHealthAspect", sch_livingwithhealthaspect },
    { { 3, 0 }, { 0, 0 }, "LoanOrCredit", sch_loanorcredit },
    { { 0, 10 }, { 0, 0 }, "LocalBusiness", sch_localbusiness },
    { { 3, 1 }, { 0, 0 }, "LocationFeatureSpecification", sch_locationfeaturespecification },
    { { 1, 3 }, { 0, 0 }, "LockerDelivery", sch_lockerdelivery },
    { { 0, 10 }, { 0, 0 }, "Locksmith", sch_locksmith },
    { { 0, 10 }, { 0, 0 }, "LodgingBusiness", sch_lodgingbusiness },
    { { 1, 10 }, { 0, 0 }, "LodgingReservation", sch_lodgingreservation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Longitudinal", sch_longitudinal },
    { { 1, 2 }, { 0, 0 }, "LoseAction", sch_loseaction },
    { { 3, 5 }, { 0, 0 }, "LowCalorieDiet", sch_lowcaloriediet },
    { { 3, 5 }, { 0, 0 }, "LowFatDiet", sch_lowfatdiet },
    { { 3, 5 }, { 0, 0 }, "LowLactoseDiet", sch_lowlactosediet },
    { { 3, 5 }, { 0, 0 }, "LowSaltDiet", sch_lowsaltdiet },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Lung", sch_lung },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "LymphaticVessel", sch_lymphaticvessel },
    { { 3, 5 }, { 0, 0 }, "Male", sch_male },
    { { 3, 5 }, { 0, 0 }, "Manuscript", sch_manuscript },
    { { 0, 10 }, { 0, 0 }, "Map", sch_map },
    { { 1, 92 }, { 0, 0 }, "MapCategoryType", sch_mapcategorytype, s_schema, SF_ENUMERATION },
    { { 1, 2 }, { 0, 0 }, "MarryAction", sch_marryaction },
    { { 0, 10 }, { 0, 0 }, "Mass", sch_mass },
    { { 11, 0 }, { 0, 0 }, "MathSolver", sch_mathsolver },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MaximumDoseSchedule", sch_maximumdoseschedule },
    { { 5, 0 }, { 0, 0 }, "MayTreatHealthAspect", sch_maytreathealthaspect },
    { { 12, 0 }, { 0, 0 }, "MeasurementTypeEnumeration", sch_measurementtypeenumeration, s_schema, SF_ENUMERATION },
    { { 6, 0 }, { 0, 0 }, "MediaGallery", sch_mediagallery },
    { { 7, 0 }, { 0, 0 }, "MediaManipulationRatingEnumeration", sch_mediamanipulationratingenumeration, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "MediaObject", sch_mediaobject },
    { { 7, 0 }, { 0, 0 }, "MediaReview", sch_mediareview },
    { { 13, 0 }, { 0, 0 }, "MediaReviewItem", sch_mediareviewitem },
    { { 3, 5 }, { 0, 0 }, "MediaSubscription", sch_mediasubscription },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalAudience", sch_medicalaudience },
    { { 9, 0 }, { 0, 0 }, "MedicalAudienceType", sch_medicalaudiencetype, s_schema, SF_ENUMERATION },
    { { 3, 0, SV_NOT_3134 }, { 0, 0 }, "MedicalBusiness", sch_medicalbusiness },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalCause", sch_medicalcause },
    { { 0, 10, SV_NOT_3034 }, { 0, 0 }, "MedicalClinic", sch_medicalclinic },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalCode", sch_medicalcode },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalCondition", sch_medicalcondition },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalConditionStage", sch_medicalconditionstage },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalContraindication", sch_medicalcontraindication },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalDevice", sch_medicaldevice },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalDevicePurpose", sch_medicaldevicepurpose, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalEntity", sch_medicalentity },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalEnumeration", sch_medicalenumeration },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalEvidenceLevel", sch_medicalevidencelevel, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalGuideline", sch_medicalguideline },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalGuidelineContraindication", sch_medicalguidelinecontraindication },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalGuidelineRecommendation", sch_medicalguidelinerecommendation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalImagingTechnique", sch_medicalimagingtechnique, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalIndication", sch_medicalindication },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalIntangible", sch_medicalintangible },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalObservationalStudy", sch_medicalobservationalstudy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalObservationalStudyDesign", sch_medicalobservationalstudydesign, s_schema, SF_ENUMERATION },
    { { 0, 10, SV_NOT_3034 }, { 0, 0 }, "MedicalOrganization", sch_medicalorganisation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalProcedure", sch_medicalprocedure },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalProcedureType", sch_medicalproceduretype, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalResearcher", sch_medicalresearcher },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalRiskCalculator", sch_medicalriskcalculator },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalRiskEstimator", sch_medicalriskestimator },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalRiskFactor", sch_medicalriskfactor },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalRiskScore", sch_medicalriskscore },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalScholarlyArticle", sch_medicalscholarlyarticle },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalSign", sch_medicalsign },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalSignOrSymptom", sch_medicalsignorsymptom },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalSpecialty", sch_medicalspeciality, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalStudy", sch_medicalstudy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalStudyStatus", sch_medicalstudystatus, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalSymptom", sch_medicalsymptom },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalTest", sch_medicaltest },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalTestPanel", sch_medicaltestpanel },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalTherapy", sch_medicaltherapy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalTrial", sch_medicaltrial },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalTrialDesign", sch_medicaltrialdesign, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicalWebPage", sch_medicalwebpage },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MedicineSystem", sch_medicinesystem, s_schema, SF_ENUMERATION },
    { { 3, 1 }, { 0, 0 }, "MeetingRoom", sch_meetingroom },
    { { 0, 10 }, { 0, 0 }, "MensClothingStore", sch_tailors },
    { { 3, 2 }, { 0, 0 }, "Menu", sch_menu },
    { { 3, 2 }, { 0, 0 }, "MenuItem", sch_menuitem },
    { { 3, 2 }, { 0, 0 }, "MenuSection", sch_menusection },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnEnumeration", sch_merchantreturnenumeration, s_schema, SF_ENUMERATION },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnFiniteReturnWindow", sch_merchantreturnfinitereturnwindow },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnNotPermitted", sch_merchantreturnnotpermitted },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnPolicy", sch_merchantreturnpolicy },
    { { 13, 0 }, { 0, 0 }, "MerchantReturnPolicySeasonalOverride", sch_merchantreturnpolicyseasonaloverride },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnUnlimitedWindow", sch_merchantreturnunlimitedwindow },
    { { 6, 0 }, { 0, 0 }, "MerchantReturnUnspecified", sch_merchantreturnunspecified },
    { { 3, 0 }, { 0, 0 }, "Message", sch_message },
    { { 0, 10 }, { 0, 0 }, "MiddleSchool", sch_middleschool },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Midwifery", sch_midwifery },
    { { 11, 0 }, { 0, 0 }, "MinimumAdvertisedPrice", sch_minimumadvertisedprice },
    { { 5, 0 }, { 0, 0 }, "MisconceptionsHealthAspect", sch_misconceptionshealthaspect },
    { { 7, 0 }, { 0, 0 }, "MissingContext", sch_missingcontext },
    { { 7, 0 }, { 0, 0 }, "MixedEventAttendanceMode", sch_mixedeventattendancemode },
    { { 1, 92 }, { 0, 0 }, "MixtapeAlbum", sch_mixtapealbum },
    { { 0, 95 }, { 0, 0 }, "MobileApplication", sch_mobileapplication },
    { { 0, 10 }, { 0, 0 }, "MobilePhoneStore", sch_mobilephoneshop },
    { { 14, 0 }, { 0, 0 }, "MobileWebPlatform", sch_mobilewebplatform },
    { { 13, 0 }, { 0, 0 }, "MolecularEntity", sch_molecularentity },
    { { 3, 0 }, { 0, 0 }, "Monday", sch_monday },
    { { 3, 0 }, { 0, 0 }, "MonetaryAmount", sch_monetaryamount },
    { { 3, 5 }, { 0, 0 }, "MonetaryAmountDistribution", sch_monetaryamountdistribution },
    { { 3, 5 }, { 0, 0 }, "MonetaryGrant", sch_monetarygrant },
    { { 3, 5 }, { 0, 0 }, "MoneyTransfer", sch_moneytransfer },
    { { 3, 5 }, { 0, 0 }, "MortgageLoan", sch_mortgageloan },
    { { 0, 10 }, { 0, 0 }, "Mosque", sch_mosque },
    { { 0, 10 }, { 0, 0 }, "Motel", sch_motel },
    { { 3, 5 }, { 0, 0 }, "Motorcycle", sch_motorcycle },
    { { 0, 10 }, { 0, 0 }, "MotorcycleDealer", sch_motorcycledealer },
    { { 0, 10 }, { 0, 0 }, "MotorcycleRepair", sch_motorcyclerepair },
    { { 3, 5 }, { 0, 0 }, "MotorizedBicycle", sch_motorisedbicycle },
    { { 0, 10 }, { 0, 0 }, "Mountain", sch_mountain },
    { { 1, 2 }, { 0, 0 }, "MoveAction", sch_moveaction },
    { { 0, 10 }, { 0, 0 }, "Movie", sch_movie },
    { { 2, 0 }, { 0, 0 }, "MovieClip", sch_movieclip },
    { { 0, 10 }, { 0, 0 }, "MovieRentalStore", sch_filmhireshop },
    { { 2, 0 }, { 0, 0 }, "MovieSeries", sch_movieseries },
    { { 0, 10 }, { 0, 0 }, "MovieTheater", sch_movietheatre },
    { { 0, 10 }, { 0, 0 }, "MovingCompany", sch_removers },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Mri", sch_mri },
    { { 11, 0 }, { 0, 0 }, "MSRP", sch_msrp },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MulticellularParasite", sch_multicellularparasite },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MulticenterTrial", sch_multicentretrial },
    { { 1, 92 }, { 0, 0 }, "Multiplayer", sch_multiplayer },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Muscle", sch_muscle },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Musculoskeletal", sch_musculoskeletal },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "MusculoskeletalExam", sch_musculoskeletalexam },
    { { 0, 10  }, { 0, 0 }, "Museum", sch_museum },
    { { 0, 10 }, { 0, 0 }, "MusicalBum", sch_musicalbum }, // sorry, can't resist the childishness
    { { 1, 92 }, { 0, 0 }, "MusicAlbumProductionType", sch_musicalbumproductiontype, s_schema, SF_ENUMERATION },
    { { 1, 92 }, { 0, 0 }, "MusicAlbumReleaseType", sch_musicalbumreleasetype, s_schema, SF_ENUMERATION },
    { { 2, 0 }, { 0, 0 }, "MusicComposition", sch_musiccomposition },
    { { 0, 10 }, { 0, 0 }, "MusicEvent", sch_musicevent },
    { { 0, 10 }, { 0, 0 }, "MusicGroup", sch_musicgroup },
    { { 0, 10 }, { 0, 0 }, "MusicPlaylist", sch_musicplaylist },
    { { 0, 10 }, { 0, 0 }, "MusicRecording", sch_musicrecording },
    { { 2, 0 }, { 0, 0 }, "MusicRelease", sch_musicrelease },
    { { 1, 92 }, { 0, 0 }, "MusicReleaseFormatType", sch_musicreleaseformattype, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "MusicStore", sch_musicshop },
    { { 0, 10  }, { 0, 0 }, "MusicVenue", sch_musicvenue },
    { { 0, 10 }, { 0, 0 }, "MusicVideoObject", sch_musicvideoobject },
    { { 0, 10 }, { 0, 0 }, "NailSalon", sch_nailsalon },
    { { 14, 0 }, { 0, 0 }, "NarcoticConsideration", sch_narcoticconsideration },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Neck", sch_neck },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Nerve", sch_nerve },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Neuro", sch_neuro },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Neurologic", sch_neurologic },
    { { 0, 10 }, { 0, 0 }, "NewCondition", sch_newcondition },
    { { 0, 10 }, { 0, 0 }, "NewsArticle", sch_newsarticle },
    { { 3, 5 }, { 0, 0 }, "NewsMediaOrganization", sch_newsmediaorganisation },
    { { 3, 5 }, { 0, 0 }, "Newspaper", sch_newspaper },
    { { 0, 10, SV_NOT_3034 }, { 0, 0 }, "NGO", sch_ngo },
    { { 0, 10 }, { 0, 0 }, "NightClub", sch_nightclub },
    { { 8, 0 }, { 0, 0 }, "NlNonprofitType", sch_nlnonprofittype, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "NoninvasiveProcedure", sch_noninvasiveprocedure },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501a", sch_nonprofit501a },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c1", sch_nonprofit501c1 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c10", sch_nonprofit501c10 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c11", sch_nonprofit501c11 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c12", sch_nonprofit501c12 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c13", sch_nonprofit501c13 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c14", sch_nonprofit501c14 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c15", sch_nonprofit501c15 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c16", sch_nonprofit501c16 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c17", sch_nonprofit501c17 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c18", sch_nonprofit501c18 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c19", sch_nonprofit501c19 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c2", sch_nonprofit501c2 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c20", sch_nonprofit501c20 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c21", sch_nonprofit501c21 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c22", sch_nonprofit501c22 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c23", sch_nonprofit501c23 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c24", sch_nonprofit501c24 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c25", sch_nonprofit501c25 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c26", sch_nonprofit501c26 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c27", sch_nonprofit501c27 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c28", sch_nonprofit501c28 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c3", sch_nonprofit501c3 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c4", sch_nonprofit501c4 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c5", sch_nonprofit501c5 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c6", sch_nonprofit501c6 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c7", sch_nonprofit501c7 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c8", sch_nonprofit501c8 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501c9", sch_nonprofit501c9 },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501d", sch_nonprofit501d },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501e", sch_nonprofit501e },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501f", sch_nonprofit501f },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501k", sch_nonprofit501k },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501n", sch_nonprofit501n },
    { { 9, 0 }, { 0, 0 }, "Nonprofit501q", sch_nonprofit501q },
    { { 9, 0 }, { 0, 0 }, "Nonprofit527", sch_nonprofit527 },
    { { 9, 0 }, { 0, 0 }, "Nonprofitanbi", sch_nonprofitanbi },
    { { 9, 0 }, { 0, 0 }, "Nonprofitsbbi", sch_nonprofitsbbi },
    { { 8, 0 }, { 0, 0 }, "Nonprofittype", sch_nonprofittype },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Nose", sch_nose },
    { { 0, 10 }, { 0, 0 }, "Notary", sch_notary },
    { { 3, 0 }, { 0, 0 }, "NoteDigitalDocument", sch_notedigitaldocument },
    { { 3, 5 }, { 0, 0 }, "NotInForce", sch_notinforce },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "NotYetRecruiting", sch_notyetrecruiting },
    { { 0, 10 }, { 0, 0 }, "Number", sch_number, s_schema, MAKE_SIMPLE_TYPE (t_fixedpoint) },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Nursing", sch_nursing },
    { { 0, 10 }, { 0, 0 }, "NutritionInformation", sch_nutritioninformation },
    { { 3, 9 }, { 0, 0 }, "Observation", sch_observation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Observational", sch_observational },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Obstetric", sch_obstetric },
    { { 3, 5 }, { 0, 0 }, "Occupation", sch_occupation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "OccupationalActivity", sch_occupationalactivity },
    { { 12, 0 }, { 0, 0 }, "OccupationalExperienceRequirements", sch_occupationalexperiencerequirements },
    { { 3, 5 }, { 0, 0 }, "OccupationalTherapy", sch_occupationaltherapy },
    { { 0, 10 }, { 0, 0 }, "OceanBodyOfWater", sch_oceanbodyofwater },
    { { 0, 10 }, { 0, 0 }, "Offer", sch_offer },
    { { 2, 2 }, { 0, 0 }, "OfferCatalog", sch_offercatalogue },
    { { 4, 0 }, { 0, 0 }, "OfferForLease", sch_offerforlease },
    { { 4, 0 }, { 0, 0 }, "OfferForPurchase", sch_offerforpurchase },
    { { 0, 10 }, { 0, 0 }, "OfferItemCondition", sch_offeritemcondition, s_schema, SF_ENUMERATION },
    { { 7, 2 }, { 0, 0 }, "OfferShippingDetails", sch_offershippingdetails },
    { { 0, 10 }, { 0, 0 }, "OfficeEquipmentStore", sch_officeshop },
    { { 3, 5 }, { 0, 0 }, "OfficiallegalValue", sch_officiallegalvalue },
    { { 7, 0 }, { 0, 0 }, "OfflineEventAttendanceMode", sch_offlineeventattendancemode },
    { { 1, 92 }, { 0, 0 }, "OfflinePermanently", sch_offlinepermanently },
    { { 1, 92 }, { 0, 0 }, "OfflineTemporarily", sch_offlinetemporarily },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Oncologic", sch_oncologic },
    { { 1, 3 }, { 0, 0 }, "OnDemandEvent", sch_ondemandevent },
    { { 7, 4 }, { 0, 0 }, "OneTimePayments", sch_onetimepayments },
    { { 1, 92 }, { 0, 0 }, "Online", sch_online },
    { { 14, 0 }, { 0, 0 }, "OnlineBusiness", sch_onlinebusiness },
    { { 7, 0 }, { 0, 0 }, "OnlineEventAttendanceMode", sch_onlineeventattendancemode },
    { { 1, 92 }, { 0, 0 }, "OnlineFull", sch_onlinefull },
    { { 0, 10 }, { 0, 0 }, "OnlineOnly", sch_onlineonly },
    { { 14, 0 }, { 0, 0 }, "OnlineStore", sch_onlinestore },
    { { 1, 3 }, { 0, 0 }, "OnsitePickup", sch_onsitepickup },
    { { 0, 99 }, { 0, 0 }, "OpeningHoursSpecification", sch_openinghoursspecification },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "OpenTrial", sch_opentrial },
    { { 3, 5 }, { 0, 0 }, "OpinionNewsArticle", sch_opinionnewsarticle },
    { { 0, 10, SV_NOT_3034 }, { 0, 0 }, "Optician", sch_optician },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Optometric", sch_optometric },
    { { 1, 4 }, { 0, 0 }, "Order", sch_order },
    { { 2, 0 }, { 0, 0 }, "Orderaction", sch_orderaction },
    { { 1, 92 }, { 0, 0 }, "OrderCancelled", sch_ordercancelled },
    { { 1, 92 }, { 0, 0 }, "OrderDelivered", sch_orderdelivered },
    { { 1, 92 }, { 0, 0 }, "OrderIntransit", sch_orderintransit },
    { { 2, 0 }, { 0, 0 }, "OrderItem", sch_orderitem },
    { { 1, 92 }, { 0, 0 }, "OrderPaymentDue", sch_orderpaymentdue },
    { { 1, 92 }, { 0, 0 }, "OrderPickupAvailable", sch_orderpickupavailable },
    { { 1, 92 }, { 0, 0 }, "OrderProblem", sch_orderproblem },
    { { 1, 92 }, { 0, 0 }, "OrderProcessing", sch_orderprocessing },
    { { 1, 92 }, { 0, 0 }, "OrderReturned", sch_orderreturned },
    { { 1, 92 }, { 0, 0 }, "OrderStatus", sch_orderstatus, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "Organization", sch_organisation },
    { { 1, 92 }, { 0, 0 }, "OrganizationRole", sch_organisationrole },
    { { 1, 2 }, { 0, 0 }, "OrganizeAction", sch_organiseaction },
    { { 12, 0 }, { 0, 0 }, "OriginalMediaContent", sch_originalmediacontent },
    { { 3, 8 }, { 0, 0 }, "OriginalShippingSees", sch_originalshippingfees },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Osteopathic", sch_osteopathic },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Otc", sch_otc },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Otolaryngologic", sch_otolaryngologic },
    { { 0, 10 }, { 0, 0 }, "OutletStore", sch_outletstore },
    { { 0, 10 }, { 0, 0 }, "OutOfStock", sch_outofstock },
    { { 5, 0 }, { 0, 0 }, "OverviewHealthAspect", sch_overviewhealthaspect },
    { { 0, 99 }, { 0, 0 }, "OwnershipInfo", sch_ownershipinfo },
    { { 7, 4 }, { 0, 0 }, "PaidLeave", sch_paidleave },
    { { 1, 2 }, { 0, 0 }, "PaintAction", sch_paintaction },
    { { 0, 10 }, { 0, 0 }, "Painting", sch_painting },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PalliativeProcedure", sch_palliativeprocedure },
    { { 0, 10 }, { 0, 0 }, "Paperback", sch_paperback },
    { { 1, 4 }, { 0, 0 }, "ParcelDelivery", sch_parceldelivery },
    { { 0, 99 }, { 0, 0 }, "ParcelService", sch_parcelservice },
    { { 7, 4 }, { 0, 0 }, "ParentalSupport", sch_parentalsupport },
    { { 1, 0 }, { 0, 0 }, "ParentAudience", sch_parentaudience },
    { { 0, 10  }, { 0, 0 }, "Park", sch_park },
    { { 0, 10  }, { 0, 0 }, "ParkingFacility", sch_parkingfacility },
    { { 1, 92 }, { 0, 0 }, "ParkingMap", sch_parkingmap },
    { { 3, 5 }, { 0, 0 }, "PartiallyInforce", sch_partiallyinforce },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Pathology", sch_pathology },
    { { 2, 0 }, { 0, 0 }, "PathologyTest", sch_pathologytest },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Patient", sch_patient },
    { { 5, 0 }, { 0, 0 }, "PatientExperienceHealthAspect", sch_patientexperiencehealthaspect },
    { { 0, 10 }, { 0, 0 }, "PawnShop", sch_pawnshop },
    { { 1, 2 }, { 0, 0 }, "PayAction", sch_payaction },
    { { 3, 5 }, { 0, 0 }, "Paymentautomaticallyapplied", sch_paymentautomaticallyapplied },
    { { 3, 0 }, { 0, 0 }, "PaymentCard", sch_paymentcard },
    { { 0, 99 }, { 0, 0 }, "PaymentChargeSpecification", sch_paymentchargespecification },
    { { 3, 5 }, { 0, 0 }, "PaymentComplete", sch_paymentcomplete },
    { { 3, 5 }, { 0, 0 }, "PaymentDeclined", sch_paymentdeclined },
    { { 3, 5 }, { 0, 0 }, "PaymentDue", sch_paymentdue },
    { { 0, 99 }, { 0, 0 }, "PaymentMethod", sch_paymentmethod, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "PaymentPastDue", sch_paymentpastdue },
    { { 3, 1 }, { 0, 0 }, "PaymentService", sch_paymentservice },
    { { 2, 1 }, { 0, 0 }, "PaymentStatusType", sch_paymentstatustype, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Pediatric", sch_paediatric },
    { { 1, 0 }, { 0, 0 }, "PeopleAudience", sch_peopleaudience },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PercutaneousProcedure", sch_percutaneousprocedure },
    { { 1, 2 }, { 0, 0 }, "PerformAction", sch_performaction },
    { { 1, 92 }, { 0, 0 }, "PerformanceRole", sch_performancerole },
    { { 0, 10  }, { 0, 0 }, "PerformingArtsTheater", sch_tyreshop },
    { { 0, 10 }, { 0, 0 }, "PerformingGroup", sch_performinggroup },
    { { 1, 90 }, { 0, 0 }, "Periodical", sch_periodical },
    { { 1, 4 }, { 0, 0 }, "Permit", sch_permit },
    { { 0, 10 }, { 0, 0 }, "Person", sch_person },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Pet", sch_pet },
    { { 0, 10 }, { 0, 0 }, "PetStore", sch_petshop },
    { { 0, 10 }, { 0, 0 }, "Pharmacy", sch_pharmacy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PharmacySpeciality", sch_pharmacyspeciality },
    { { 0, 10 }, { 0, 0 }, "Photograph", sch_photograph },
    { { 1, 2 }, { 0, 0 }, "PhotographAction", sch_photographaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PhysicalActivity", sch_physicalactivity },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PhysicalActivityCategory", sch_physicalactivitycategory, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PhysicalExam", sch_physicalexam, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PhysicalTherapy", sch_physicaltherapy },
    { { 0, 10 }, { 0, 0 }, "Physician", sch_physician },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Physiotherapy", sch_physiotherapy },
    { { 0, 10 }, { 0, 0 }, "Place", sch_place },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PlaceboControlledTrial", sch_placebocontrolledtrial },
    { { 0, 10  }, { 0, 0 }, "PlaceOfWorship", sch_placeofworship },
    { { 1, 2 }, { 0, 0 }, "PlanAction", sch_planaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PlasticSurgery", sch_plasticsurgery },
    { { 3, 5 }, { 0, 0 }, "Play", sch_play },
    { { 1, 2 }, { 0, 0 }, "PlayAction", sch_playaction },
    { { 14, 0 }, { 0, 0 }, "PlayGameAction", sch_playgameaction },
    { { 0, 10  }, { 0, 0 }, "Playground", sch_playground },
    { { 0, 10 }, { 0, 0 }, "Plumber", sch_plumber },
    { { 4, 0 }, { 0, 0 }, "PodcastEpisode", sch_podcastepisode },
    { { 4, 0 }, { 0, 0 }, "PodcastSeason", sch_podcastseason },
    { { 4, 0 }, { 0, 0 }, "PodcastSeries", sch_podcastseries },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Podiatric", sch_podiatric },
    { { 0, 10 }, { 0, 0 }, "PoliceStation", sch_policestation },
    { { 0, 10 }, { 0, 0 }, "Pond", sch_pond },
    { { 0, 10 }, { 0, 0 }, "PostalAddress", sch_postaladdress },
    { { 8, 0 }, { 0, 0 }, "PostalcodeRangeSpecification", sch_postalcoderangespecification },
    { { 3, 5 }, { 0, 0 }, "Poster", sch_poster },
    { { 0, 10 }, { 0, 0 }, "PostOffice", sch_postoffice },
    { { 1, 20 }, { 0, 0 }, "PotentialActionStatus", sch_potentialactionstatus },
    { { 12, 0 }, { 0, 0 }, "PregnancyHealthAspect", sch_pregnancyhealthaspect },
    { { 0, 10 }, { 0, 0 }, "PreOrder", sch_preorder },
    { { 3, 5 }, { 0, 0 }, "PreOrderAction", sch_preorderaction },
    { { 1, 2 }, { 0, 0 }, "PrependAction", sch_prependaction },
    { { 3, 5 }, { 0, 0 }, "Presale", sch_presale },
    { { 0, 10 }, { 0, 0 }, "Preschool", sch_preschool },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PrescriptionOnly", sch_prescriptiononly },
    { { 3, 0 }, { 0, 0 }, "PresentationDigitalDocument", sch_presentationdigitaldocument },
    { { 5, 0 }, { 0, 0 }, "PreventionHealthAspect", sch_preventionhealthaspect },
    { { 2, 0 }, { 0, 0 }, "PreventionIndication", sch_preventionindication },
    { { 11, 0 }, { 0, 0 }, "PriceComponentTypeEnumeration", sch_pricecomponenttypeenumeration },
    { { 0, 99 }, { 0, 0 }, "PriceSpecification", sch_pricespecification },
    { { 11, 0 }, { 0, 0 }, "PriceTypeEnumeration", sch_pricetypeenumeration },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PrimaryCare", sch_primarycare },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Prion", sch_prion },
    { { 0, 10 }, { 0, 0 }, "Product", sch_product },
    { { 9, 0 }, { 0, 0 }, "ProductCollection", sch_productcollection },
    { { 9, 0 }, { 0, 0 }, "ProductGroup", sch_productgroup },
    { { 0, 99 }, { 0, 0 }, "ProductModel", sch_productmodel },
    { { 3, 8, SV_DEPRECATED }, { 0, 0 }, "ProductReturnEnumeration", sch_productreturnenumeration, s_schema, SF_DEPRECATED },
    { { 3, 8, SV_DEPRECATED }, { 0, 0 }, "ProductReturnFiniteReturnWindow", sch_productreturnfinitereturnwindow, s_schema, SF_DEPRECATED },
    { { 3, 8, SV_DEPRECATED }, { 0, 0 }, "ProductReturnNotPermitted", sch_productreturnnotpermitted, s_schema, SF_DEPRECATED },
    { { 3, 8, SV_DEPRECATED }, { 0, 0 }, "ProductReturnPolicy", sch_productreturnpolicy, s_schema, SF_DEPRECATED },
    { { 3, 8, SV_DEPRECATED }, { 0, 0 }, "ProductReturnUnlimitedWindow", sch_productreturnunlimitedwindow, s_schema, SF_DEPRECATED },
    { { 3, 8, SV_DEPRECATED }, { 0, 0 }, "ProductReturnUnspecified", sch_productreturnunspecified, s_schema, SF_DEPRECATED },
    { { 0, 10 }, { 0, 0 }, "ProfessionalService", sch_professionalservice },
    { { 0, 10 }, { 0, 0 }, "ProfilePage", sch_profilepage },
    { { 5, 0 }, { 0, 0 }, "PrognosisHealthAspect", sch_prognosishealthaspect },
    { { 1, 20 }, { 0, 0 }, "ProgramMembership", sch_programmemembership },
    { { 3, 5 }, { 0, 0 }, "Project", sch_project },
    { { 6, 0 }, { 0, 0 }, "PronounceableText", sch_pronounceabletext },
    { { 1, 0, SV_NOT_3034 }, { 0, 0 }, "Property", sch_property },
    { { 2, 0 }, { 0, 0 }, "PropertyValue", sch_propertyvalue },
    { { 1, 92 }, { 0, 0 }, "PropertyValueSpecification", sch_propertyvaluespecification },
    { { 13, 0 }, { 0, 0 }, "Protein", sch_protein },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Protozoa", sch_protozoa },
    { { 3, 5 }, { 0, 0 }, "Psychiatric", sch_psychiatric },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "PsychologicalTreatment", sch_psychologicaltreatment },
    { { 1, 3 }, { 0, 0 }, "PublicationEvent", sch_publicationevent },
    { { 1, 90 }, { 0, 0 }, "PublicationIssue", sch_publicationissue },
    { { 2, 0 }, { 0, 0 }, "PublicationVolume", sch_publicationvolume },
    { { 3, 5 }, { 0, 0 }, "PublicHealth", sch_publichealth },
    { { 3, 0 }, { 0, 0 }, "PublicHolidays", sch_publicholidays },
    { { 0, 10 }, { 0, 0 }, "PublicSwimmingPool", sch_publicswimmingpool },
    { { 3, 5 }, { 0, 0 }, "PublicToilet", sch_publictoilet },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Pulmonary", sch_pulmonary },
    { { 2, 0 }, { 0, 0 }, "Qapage", sch_qapage },
    { { 0, 99 }, { 0, 0 }, "QualitativeValue", sch_qualitativevalue },
    { { 0, 99 }, { 0, 0 }, "QuantitativeValue", sch_quantitativevalue },
    { { 3, 5 }, { 0, 0 }, "QuantitativeValueDistribution", sch_quantitativevaluedistribution },
    { { 0, 10 }, { 0, 0 }, "Quantity", sch_quantity },
    { { 1, 20 }, { 0, 0 }, "Question", sch_question },
    { { 9, 0 }, { 0, 0 }, "Quiz", sch_quiz },
    { { 3, 5 }, { 0, 0 }, "Quotation", sch_quotation },
    { { 2, 0 }, { 0, 0 }, "Quoteaction", sch_quoteaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "RadiationTherapy", sch_radiationtherapy },
    { { 3, 5 }, { 0, 0 }, "RadioBroadcastService", sch_radiobroadcastservice },
    { { 2, 0 }, { 0, 0 }, "RadioChannel", sch_radiochannel },
    { { 1, 3 }, { 0, 0 }, "RadioClip", sch_radioclip },
    { { 1, 3 }, { 0, 0 }, "RadioEpisode", sch_radioepisode },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Radiography", sch_radiography },
    { { 1, 3 }, { 0, 0 }, "RadioSeason", sch_radioseason },
    { { 1, 3 }, { 0, 0 }, "RadioSeries", sch_radioseries },
    { { 0, 10 }, { 0, 0 }, "RadioStation", sch_radiostation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "RandomizedTrial", sch_randomisedtrial },
    { { 0, 10 }, { 0, 0 }, "Rating", sch_rating },
    { { 1, 2 }, { 0, 0 }, "ReactAction", sch_reactaction },
    { { 1, 2 }, { 0, 0 }, "ReadAction", sch_readaction },
    { { 3, 5 }, { 0, 0 }, "ReadPermission", sch_readpermission },
    { { 0, 10 }, { 0, 0 }, "RealEstateAgent", sch_realestateagent },
    { { 4, 0 }, { 0, 0 }, "RealEstateListing", sch_realestatelisting },
    { { 3, 6 }, { 6, 0 }, "RearWheelDriveConfiguration", sch_rearwheeldriveconfiguration },
    { { 1, 2 }, { 0, 0 }, "ReceiveAction", sch_receiveaction },
    { { 0, 10 }, { 0, 0 }, "Recipe", sch_recipe },
    { { 6, 0 }, { 0, 0 }, "Recommendation", sch_recommendation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "RecommendedDoseSchedule", sch_recommendeddoseschedule },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Recruiting", sch_recruiting },
    { { 0, 10 }, { 0, 0 }, "RecyclingCenter", sch_recyclingcentre },
    { { 14, 0 }, { 0, 0 }, "ReducedRelevanceForChildrenConsideration", sch_reducedrelevanceforchildrenconsideration },
    { { 3, 8 }, { 0, 0 }, "RefundTypeEnumeration", sch_refundtypeenumeration, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "RefurbishedCondition", sch_refurbishedcondition },
    { { 1, 2 }, { 0, 0 }, "RegisterAction", sch_registeraction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Registry", sch_registry },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "ReimbursementCap", sch_reimbursementcap },
    { { 1, 2 }, { 0, 0 }, "RejectAction", sch_rejectaction },
    { { 5, 0 }, { 0, 0 }, "RelatedTopicsHealthAspect", sch_relatedtopicshealthaspect },
    { { 1, 92 }, { 0, 0 }, "RemixAlbum", sch_remixalbum },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Renal", sch_renal },
    { { 2, 0 }, { 0, 0 }, "RentAction", sch_rentaction },
    { { 1, 10 }, { 0, 0 }, "RentalCarReservation", sch_rentalcarreservation },
    { { 3, 6 }, { 6, 0 }, "RentalVehicleUsage", sch_rentalvehicleusage },
    { { 3, 5 }, { 0, 0 }, "RepaymentSpecification", sch_repaymentspecification },
    { { 1, 2 }, { 0, 0 }, "ReplaceAction", sch_replaceaction },
    { { 2, 0 }, { 0, 0 }, "Replyaction", sch_replyaction },
    { { 2, 1 }, { 0, 0 }, "Report", sch_report },
    { { 3, 5 }, { 0, 0 }, "ReportageNewsArticle", sch_reportagenewsarticle },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "ReportedDoseSchedule", sch_reporteddoseschedule },
    { { 3, 5 }, { 0, 0 }, "ResearchProject", sch_researchproject },
    { { 1, 10 }, { 0, 0 }, "Reservation", sch_reservation },
    { { 1, 92 }, { 0, 0 }, "ReservationCancelled", sch_reservationcancelled },
    { { 1, 92 }, { 0, 0 }, "ReservationConfirmed", sch_reservationconfirmed },
    { { 1, 92 }, { 0, 0 }, "ReservationHold", sch_reservationhold },
    { { 1, 10 }, { 0, 0 }, "ReservationPackage", sch_reservationpackage },
    { { 1, 92 }, { 0, 0 }, "ReservationPending", sch_reservationpending },
    { { 1, 92 }, { 0, 0 }, "ReservationStatusType", sch_reservationstatustype, s_schema, SF_ENUMERATION },
    { { 1, 2 }, { 0, 0 }, "ReserveAction", sch_reserveaction },
    { { 0, 10 }, { 0, 0 }, "Reservoir", sch_reservoir },
    { { 0, 10 }, { 0, 0 }, "Residence", sch_residence },
    { { 3, 1 }, { 0, 0 }, "Resort", sch_resort },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "RespiratoryTherapy", sch_respiratorytherapy },
    { { 0, 10 }, { 0, 0 }, "Restaurant", sch_restaurant },
    { { 3, 8 }, { 0, 0 }, "RestockingFees", sch_restockingfees },
    { { 3, 0 }, { 0, 0 }, "RestrictedDiet", sch_restricteddiet, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "ResultsAvailable", sch_resultsavailable },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "ResultsNotAvailable", sch_resultsnotavailable },
    { { 1, 92 }, { 0, 0 }, "ResumeAction", sch_resumeaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Retail", sch_retail },
    { { 1, 2 }, { 0, 0 }, "ReturnAction", sch_returnaction },
    { { 13, 0 }, { 0, 0 }, "ReturnAtKiosk", sch_returnatkiosk },
    { { 13, 0 }, { 0, 0 }, "ReturnByMail", sch_returnbymail },
    { { 3, 8 }, { 0, 0 }, "ReturnFeesEnumeration", sch_returnfeesenumeration, s_schema, SF_ENUMERATION },
    { { 13, 0 }, { 0, 0 }, "ReturnFeesCustomerResponsibility", sch_returnfeescustomerresponsibility },
    { { 13, 0 }, { 0, 0 }, "ReturnInStore", sch_returninstore },
    { { 13, 0 }, { 0, 0 }, "ReturnLabelCustomerResponsibility", sch_returnlabelcustomerresponsibility },
    { { 13, 0 }, { 0, 0 }, "ReturnLabelDownloadAndPrint", sch_returnlabeldownloadandprint },
    { { 13, 0 }, { 0, 0 }, "ReturnLabelInBox", sch_returnlabeldownloadandprint },
    { { 13, 0 }, { 0, 0 }, "ReturnLabelSourceEnumeration", sch_returnlabelsourceenumeration, s_schema, SF_ENUMERATION },
    { { 13, 0 }, { 0, 0 }, "ReturnMethodEnumeration", sch_returnmethodenumeration, s_schema, SF_ENUMERATION },
    { { 3, 8 }, { 0, 0 }, "ReturnShippingFees", sch_returnshippingfees },
    { { 0, 10 }, { 0, 0 }, "Review", sch_review },
    { { 1, 2 }, { 0, 0 }, "ReviewAction", sch_reviewaction },
    { { 3, 5 }, { 0, 0 }, "ReviewNewsArticle", sch_reviewnewsarticle },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Rheumatologic", sch_rheumatologic },
    { { 3, 6 }, { 6, 0 }, "Righthanddriving", sch_righthanddriving },
    { { 5, 0 }, { 0, 0 }, "Risksorcomplicationshealthaspect", sch_risksorcomplicationshealthaspect },
    { { 0, 10 }, { 0, 0 }, "RiverBodyOfWater", sch_riverbodyofwater },
    { { 1, 92 }, { 0, 0 }, "Role", sch_role },
    { { 0, 10 }, { 0, 0 }, "RoofingContractor", sch_roofingcontractor },
    { { 3, 1 }, { 0, 0 }, "Room", sch_room },
    { { 1, 2 }, { 0, 0 }, "RsvpAction", sch_rsvpaction },
    { { 1, 9 }, { 0, 0 }, "RsvpResponseMaybe", sch_rsvpresponsemaybe },
    { { 1, 9 }, { 0, 0 }, "RsvpResponseNo", sch_rsvpresponseno },
    { { 1, 92 }, { 0, 0 }, "RsvpResponseType", sch_rsvpresponsetype, s_schema, SF_ENUMERATION },
    { { 1, 9 }, { 0, 0 }, "RsvpResponseYes", sch_rsvpresponseyes },
    { { 0, 10 }, { 0, 0 }, "RVPark", sch_rvpark },
    { { 12, 0 }, { 0, 0 }, "SafetyHealthAspect", sch_safetyhealthaspect },
    { { 0, 10 }, { 0, 0 }, "SaleEvent", sch_saleevent },
    { { 11, 0 }, { 0, 0 }, "SalePrice", sch_saleprice },
    { { 12, 0 }, { 0, 0 }, "SatireOrParodyContent", sch_satireorparodycontent },
    { { 3, 5 }, { 0, 0 }, "SatiricalArticle", sch_satiricalarticle },
    { { 3, 0 }, { 0, 0 }, "Saturday", sch_saturday },
    { { 3, 5 }, { 0, 0 }, "Schedule", sch_schedule },
    { { 1, 2 }, { 0, 0 }, "ScheduleAction", sch_scheduleaction },
    { { 0, 10 }, { 0, 0 }, "ScholarlyArticle", sch_scholarlyarticle },
    { { 0, 10 }, { 0, 0 }, "School", sch_school },
    { { 7, 1 }, { 0, 0 }, "SchoolDistrict", sch_schooldistrict },
    { { 2, 0 }, { 0, 0 }, "ScreeningEvent", sch_screeningevent },
    { { 5, 0 }, { 0, 0 }, "ScreeningHealthAspect", sch_screeninghealthaspect },
    { { 0, 10 }, { 0, 0 }, "Sculpture", sch_sculpture },
    { { 0, 10 }, { 0, 0 }, "SeabodyOfWater", sch_seabodyofwater },
    { { 1, 2 }, { 0, 0 }, "SearchAction", sch_searchaction },
    { { 14, 0 }, { 0, 0 }, "SearchRescueOrganization", sch_searchrescueorganisation },
    { { 0, 10 }, { 0, 0 }, "SearchResultsPage", sch_searchresultspage },
    { { 1, 3 }, { 3, 4 }, "Season", sch_season },
    { { 1, 20 }, { 0, 0 }, "Seat", sch_seat },
    { { 1, 92 }, { 0, 0 }, "SeatingMap", sch_seatingmap },
    { { 5, 0 }, { 0, 0 }, "SeeDoctorHealthAspect", sch_seedoctorhealthaspect },
    { { 11, 0 }, { 0, 0 }, "SeekToAction", sch_seektoaction },
    { { 5, 0 }, { 0, 0 }, "SelfCareHealthAspect", sch_selfcarehealthaspect },
    { { 0, 10 }, { 0, 0 }, "SelfStorage", sch_selfstorage },
    { { 1, 2 }, { 0, 0 }, "SellAction", sch_sellaction },
    { { 2, 0 }, { 0, 0 }, "Sendaction", sch_sendaction },
    { { 1, 3 }, { 0, 0 }, "Series", sch_series },
    { { 1, 3 }, { 0, 0 }, "Service", sch_service },
    { { 1, 3 }, { 0, 0 }, "ServiceChannel", sch_servicechannel },
    { { 1, 2 }, { 0, 0 }, "ShareAction", sch_shareaction },
    { { 3, 5 }, { 0, 0 }, "SheetMusic", sch_sheetmusic },
    { { 8, 0 }, { 0, 0 }, "ShippingDeliverytime", sch_shippingdeliverytime },
    { { 8, 0 }, { 0, 0 }, "ShippingRatesettings", sch_shippingratesettings },
    { { 0, 10 }, { 0, 0 }, "ShoppingCenter", sch_shoppingcentre },
    { { 0, 10 }, { 0, 0 }, "ShoeStore", sch_shoeshop },
    { { 14, 0 }, { 0, 0 }, "SexualContentConsideration", sch_sexualcontentconsideration },
    { { 3, 5 }, { 0, 0 }, "ShortStory", sch_shortstory },
    { { 5, 0 }, { 0, 0 }, "SideEffectsHealthAspect", sch_sideeffectshealthaspect },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "SingleBlindedTrial", sch_singleblindedtrial },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "SingleCenterTrial", sch_singlecentretrial },
    { { 0, 10 }, { 0, 0 }, "SingleFamilyResidence", sch_singlefamilyresidence },
    { { 1, 92 }, { 0, 0 }, "SinglePlayer", sch_singleplayer },
    { { 1, 92 }, { 0, 0 }, "SingleRelease", sch_singlerelease },
    { { 0, 10 }, { 0, 0 }, "SiteNavigationElement", sch_sitenavigationelement },
    { { 12, 0 }, { 0, 0 }, "SizeGroupEnumeration", sch_sizegroupenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "SizeSpecification", sch_sizespecification },
    { { 12, 0 }, { 0, 0 }, "SizeSystemEnumeration", sch_sizesystemenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "SizeSystemImperial", sch_sizesystemimperial },
    { { 12, 0 }, { 0, 0 }, "SizeSystemMetric", sch_sizesystemmetric },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Skin", sch_skin },
    { { 0, 10 }, { 0, 0 }, "SkiResort", sch_skiresort },
    { { 0, 10 }, { 0, 0 }, "SocialEvent", sch_socialevent },
    { { 2, 1 }, { 0, 0 }, "SocialMediaPosting", sch_socialmediaposting },
    { { 0, 95 }, { 0, 0 }, "SoftwareApplication", sch_softwareapplication },
    { { 2, 0 }, { 0, 0 }, "SoftwareSourceCode", sch_softwaresourcecode },
    { { 3, 5 }, { 0, 0 }, "Soldout", sch_soldout },
    { { 11, 0 }, { 0, 0 }, "SolveMathAction", sch_solvemathaction },
    { { 0, 99 }, { 0, 0 }, "Someproducts", sch_someproducts },
    { { 1, 92 }, { 0, 0 }, "Soundtrackalbum", sch_soundtrackalbum },
    { { 3, 5 }, { 0, 0 }, "Speakablespecification", sch_speakablespecification },
    { { 7, 0 }, { 0, 0 }, "Specialannouncement", sch_specialannouncement },
    { { 0, 95 }, { 0, 0 }, "Specialty", sch_speciality, s_schema, SF_ENUMERATION },
    { { 0, 95, SV_NOT_3034 }, { 0, 00 }, "SpeechPathology", sch_speechpathology },
    { { 1, 92 }, { 0, 0 }, "SpokenWordAlbum", sch_spokenwordalbum },
    { { 0, 10 }, { 0, 0 }, "SportingGoodsStore", sch_sportsshop },
    { { 0, 10 }, { 0, 0 }, "SportsActivityLocation", sch_sportsactivitylocation },
    { { 0, 10 }, { 0, 0 }, "SportsClub", sch_sportsclub },
    { { 0, 10 }, { 0, 0 }, "SportsEvent", sch_sportsevent },
    { { 2, 0 }, { 0, 0 }, "SportsOrganization", sch_sportsorganisation },
    { { 0, 10 }, { 0, 0 }, "SportsTeam", sch_sportsteam },
    { { 3, 0 }, { 0, 0 }, "SpreadsheetDigitalDocument", sch_spreadsheetdigitaldocument },
    { { 11, 0 }, { 0, 0 }, "SRP", sch_srp },
    { { 0, 10 }, { 0, 0 }, "StadiumOrArena", sch_stadiumorarena },
    { { 12, 0 }, { 0, 0 }, "StagedContent", sch_stagedcontent },
    { { 5, 0 }, { 0, 0 }, "StagesHealthAspect", sch_stageshealthaspect },
    { { 0, 10 }, { 0, 0 }, "State", sch_state },
    { { 13, 0 }, { 0, 0 }, "Statement", sch_statement },
    { { 3, 9 }, { 0, 0 }, "StatisticalPopulation", sch_statisticalpopulation },
    { { 9, 0 }, { 0, 0 }, "StatusEnumeration", sch_statusenumeration },
    { { 2, 1 }, { 0, 0 }, "SteeringPositionValue", sch_steeringpositionvalue, s_schema, SF_ENUMERATION },
    { { 0, 10 }, { 0, 0 }, "Store", sch_store },
    { { 3, 8 }, { 0, 0 }, "StoreCreditRefund", sch_storecreditrefund },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "StrengthTraining", sch_strengthtraining },
    { { 0, 10 }, { 0, 0 }, "StructuredValue", sch_structuredvalue },
    { { 1, 92 }, { 0, 0 }, "StudioAlbum", sch_studioalbum },
    { { 3, 5, SV_DEPRECATED }, { 0, 0 }, "StupidType", sch_stupidtype, s_schema, SF_DEPRECATED },
    { { 1, 2 }, { 0, 0 }, "SubscribeAction", sch_subscribeaction },
    { { 11, 0 }, { 0, 0 }, "Subscription", sch_subscription },
    { { 3, 5 }, { 0, 0 }, "Substance", sch_substance },
    { { 0, 10 }, { 0, 0 }, "SubwayStation", sch_metrostation },
    { { 3, 1 }, { 0, 0 }, "Suite", sch_suite },
    { { 3, 0 }, { 0, 0 }, "Sunday", sch_sunday },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "SuperficialAnatomy", sch_superficialanatomy },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Surgical", sch_surgical },
    { { 3, 5 }, { 0, 0 }, "SurgicalProcedure", sch_surgicalprocedure },
    { { 1, 92 }, { 0, 0 }, "SuspendAction", sch_suspendaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Suspended", sch_suspended },
    { { 5, 0 }, { 0, 0 }, "SymptomsHealthAspect", sch_symptomshealthaspect },
    { { 0, 10 }, { 0, 0 }, "Synagogue", sch_synagogue },
    { { 14, 0 }, { 0, 0 }, "TobaccoNicotineConsideration", sch_tobacconicotineconsideration },
    { { 0, 10 }, { 0, 0 }, "Table", sch_table },
    { { 1, 2 }, { 0, 0 }, "TakeAction", sch_takeaction },
    { { 0, 10 }, { 0, 0 }, "TattooParlor", sch_tattooparlour },
    { { 1, 20 }, { 3, 4 }, "Taxi", sch_taxi },
    { { 2, 0 }, { 0, 0 }, "TaxiReservation", sch_taxireservation },
    { { 2, 0 }, { 0, 0 }, "TaxiService", sch_taxiservice },
    { { 0, 10 }, { 0, 0 }, "TaxiStand", sch_taxistand },
    { { 3, 6 }, { 6, 0 }, "TaxiVehicleUsage", sch_taxivehicleusage },
    { { 13, 0 }, { 0, 0 }, "Taxon", sch_taxon },
    { { 1, 0 }, { 0, 0 }, "TechArticle", sch_techarticle },
    { { 2, 0 }, { 0, 0 }, "TelevisionChannel", sch_televisionchannel },
    { { 0, 10 }, { 0, 0 }, "TelevisionStation", sch_televisionstation },
    { { 0, 10 }, { 0, 0 }, "TennisComplex", sch_tenniscomplex },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Terminated", sch_terminated },
    { { 0, 10 }, { 0, 0 }, "Text", sch_text, s_schema, MAKE_SIMPLE_TYPE (t_text) },
    { { 3, 0 }, { 0, 0 }, "TextDigitalDocument", sch_textdigitaldocument },
    { { 0, 10 }, { 0, 0 }, "TheaterEvent", sch_theatreevent },
    { { 0, 10 }, { 0, 0 }, "TheaterGroup", sch_theatregroup },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Therapeutic", sch_therapeutic },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "TherapeuticProcedure", sch_therapeuticprocedure },
    { { 3, 5 }, { 0, 0 }, "Thesis", sch_thesis },
    { { 0, 10 }, { 0, 0 }, "Thing", sch_thing },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Throat", sch_throat },
    { { 3, 0 }, { 0, 0 }, "Thursday", sch_thursday },
    { { 1, 20 }, { 0, 0 }, "Ticket", sch_ticket },
    { { 2, 0 }, { 0, 0 }, "Tieaction", sch_tieaction },
    { { 0, 95 }, { 0, 0 }, "Time", sch_time, s_schema, MAKE_SIMPLE_TYPE (t_just_time) },
    { { 1, 2 }, { 0, 0 }, "TipAction", sch_tipaction },
    { { 0, 10 }, { 0, 0 }, "TireShop", sch_tyreshop },
    { { 3, 5 }, { 0, 0 }, "Tollfree", sch_tollfree },
    { { 0, 10 }, { 0, 0 }, "TouristAttraction", sch_touristattraction },
    { { 3, 5 }, { 0, 0 }, "TouristDestination", sch_touristdestination },
    { { 0, 10 }, { 0, 0 }, "TouristInformationCenter", sch_touristinformationcentre },
    { { 3, 5 }, { 0, 0 }, "TouristTrip", sch_touristtrip },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Toxicologic", sch_toxicologic },
    { { 0, 10 }, { 0, 0 }, "ToyStore", sch_toyshop },
    { { 1, 2 }, { 0, 0 }, "TrackAction", sch_trackaction },
    { { 1, 2 }, { 0, 0 }, "TradeAction", sch_tradeaction },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "TraditionalChinese", sch_traditionalchinese },
    { { 1, 10 }, { 0, 0 }, "TrainReservation", sch_trainreservation },
    { { 0, 10 }, { 0, 0 }, "TrainStation", sch_trainstation },
    { { 1, 20 }, { 0, 0 }, "TrainTrip", sch_traintrip },
    { { 1, 2 }, { 0, 0 }, "TransferAction", sch_transferaction },
    { { 12, 0 }, { 0, 0 }, "TransformedContent", sch_transformedcontent },
    { { 1, 92 }, { 0, 0 }, "TransitMap", sch_transitmap },
    { { 1, 2 }, { 0, 0 }, "TravelAction", sch_travelaction },
    { { 0, 10 }, { 0, 0 }, "TravelAgency", sch_travelagency },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "TreatmentIndication", sch_treatmentindication },
    { { 5, 0 }, { 0, 0 }, "TreatmentsHealthAspect", sch_treatmentshealthaspect },
    { { 3, 4 }, { 0, 0 }, "Trip", sch_trip },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "TripleBlindedTrial", sch_tripleblindedtrial },
    { { 1, 92 }, { 0, 0 }, "True", sch_true },
    { { 3, 0 }, { 0, 0 }, "Tuesday", sch_tuesday },
    { { 1, 3 }, { 0, 0 }, "TVClip", sch_tvclip },
    { { 0, 10 }, { 0, 0 }, "TVEpisode", sch_tvepisode },
    { { 0, 10 }, { 0, 0 }, "TVSeason", sch_tvseason },
    { { 0, 10 }, { 0, 0 }, "TVSeries", sch_tvseries },
    { { 0, 99 }, { 0, 0 }, "TypeAndQuantityNode", sch_typeandquantitynode },
    { { 5, 0 }, { 0, 0 }, "TypesHealthAspect", sch_typeshealthaspect },
    { { 8, 0 }, { 0, 0 }, "UkNonProfitType", sch_uknonprofittype, s_schema, SF_ENUMERATION },
    { { 9, 0 }, { 0, 0 }, "UkTrust", sch_uktrust },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Ultrasound", sch_ultrasound },
    { { 14, 0 }, { 0, 0 }, "UnclassifiedAdultConsideration", sch_unclassifiedadultconsideration },
    { { 7, 4 }, { 0, 0 }, "UnemploymentSupport", sch_unemploymentsupport },
    { { 9, 0 }, { 0, 0 }, "UnincorporatedAssociationCharity", sch_unincorporatedassociationcharity },
    { { 0, 99 }, { 0, 0 }, "UnitPriceSpecification", sch_unitpricespecification },
    { { 3, 5 }, { 0, 0 }, "UnofficialLegalValue", sch_unofficiallegalvalue },
    { { 1, 2 }, { 0, 0 }, "UnregisterAction", sch_unregisteraction },
    { { 1, 2 }, { 0, 0 }, "UpdateAction", sch_updateaction },
    { { 0, 10 }, { 0, 0 }, "URL", sch_url, s_schema, MAKE_SIMPLE_TYPE (t_url) },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Urologic", sch_urologic },
    { { 5, 0 }, { 0, 0 }, "UsageOrScheduleHealthAspect", sch_usageorschedulehealthaspect },
    { { 1, 2, SV_DEP_3034 }, { 3, 4 }, "UseAction", sch_useaction },
    { { 0, 10 }, { 0, 0 }, "UsedCondition", sch_usedcondition },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, "UserBlocks", sch_userblocks },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, "UserCheckins", sch_usercheckins },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, "UserComments", sch_usercomments },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, "UserDownloads", sch_userdownloads },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, "UserInteraction", sch_userinteraction },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, "UserLikes", sch_userlikes },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, "UserPageVisits", sch_userpagevisits },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, "UserPlays", sch_userplays },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, "UserPlusOnes", sch_userplusones },
    { { 3, 5 }, { 0, 0 }, "UserReview", sch_userreview },
    { { 0, 10, SV_DEP_3034 }, { 3, 4 }, "UserTweets", sch_usertweets },
    { { 8, 0 }, { 0, 0 }, "USNonprofitType", sch_usnonprofittype, s_schema, SF_ENUMERATION },
    { { 3, 5 }, { 0, 0 }, "VeganDiet", sch_vegandiet },
    { { 3, 5 }, { 0, 0 }, "VegetarianDiet", sch_vegetariandiet },
    { { 1, 20 }, { 0, 0 }, "Vehicle", sch_vehicle },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Vein", sch_vein },
    { { 1, 92 }, { 0, 0 }, "VenueMap", sch_venuemap },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Vessel", sch_vessel },
    { { 0, 10, SV_NOT_3034 }, { 0, 0 }, "VeterinaryCare", sch_veterinarycare },
    { { 0, 10 }, { 0, 0 }, "VideoGallery", sch_videogallery },
    { { 2, 0 }, { 0, 0 }, "VideoGame", sch_videogame },
    { { 2, 0 }, { 0, 0 }, "VideoGameClip", sch_videogameclip },
    { { 2, 0 }, { 0, 0 }, "VideoGameSeries", sch_videogameseries },
    { { 0, 10 }, { 0, 0 }, "VideoObject", sch_videoobject },
    { { 13, 0 }, { 0, 0 }, "VideoObjectSnapshot", sch_videoobjectsnapshot },
    { { 1, 2 }, { 0, 0 }, "ViewAction", sch_viewaction },
    { { 3, 5 }, { 0, 0 }, "VinylFormat", sch_vinylformat },
    { { 14, 0 }, { 0, 0 }, "ViolenceConsideration", sch_violenceconsideration },
    { { 7, 0 }, { 0, 0 }, "VirtualLocation", sch_virtuallocation },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Virus", sch_virus },
    { { 0, 10 }, { 0, 0 }, "VisualArtsEvent", sch_visualartsevent },
    { { 1, 93 }, { 0, 0 }, "VisualArtwork", sch_visualartwork },
    { { 3, 5 }, { 0, 0 }, "VitalSign", sch_vitalsign },
    { { 0, 10 }, { 0, 0 }, "Volcano", sch_volcano },
    { { 1, 2 }, { 0, 0 }, "VoteAction", sch_voteaction },
    { { 1, 2 }, { 0, 0 }, "WantAction", sch_wantaction },
    { { 0, 99 }, { 0, 0 }, "WarrantyPromise", sch_warrantypromise },
    { { 0, 99 }, { 0, 0 }, "WarrantyScope", sch_warrantyscope, s_schema, SF_EXTERNAL_ENUMERATION },
    { { 1, 2 }, { 0, 0 }, "WatchAction", sch_watchaction },
    { { 0, 10 }, { 0, 0 }, "Waterfall", sch_waterfall },
    { { 14, 0 }, { 0, 0 }, "WeaponConsideration", sch_weaponconsideration },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementTypeEnumeration", sch_wearablemeasurementtypeenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementBack", sch_wearablemeasurementback },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementChestOrBust", sch_wearablemeasurementchestorbust },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementCollar", sch_wearablemeasurementcollar },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementCup", sch_wearablemeasurementcup },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementHeight", sch_wearablemeasurementheight },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementHips", sch_wearablemeasurementhips },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementInseam", sch_wearablemeasurementinseam },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementLength", sch_wearablemeasurementlength },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementOutsideLeg", sch_wearablemeasurementoutsideleg },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementSleeve", sch_wearablemeasurementsleeve },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementWaist", sch_wearablemeasurementwaist },
    { { 12, 0 }, { 0, 0 }, "WearableMeasurementWidth", sch_wearablemeasurementwidth },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupBig", sch_wearablesizegroupbig },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupBoys", sch_wearablesizegroupboys },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupEnumeration", sch_wearablesizegroupenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupExtraShort", sch_wearablesizegroupextrashort },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupExtraTall", sch_wearablesizegroupextratall },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupGirls", sch_wearablesizegroupgirls },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupHusky", sch_wearablesizegrouphusky },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupInfants", sch_wearablesizegroupinfants },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupJuniors", sch_wearablesizegroupjuniors },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupMaternity", sch_wearablesizegroupmaternity },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupMens", sch_wearablesizegroupmens },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupMisses", sch_wearablesizegroupmisses },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupPetite", sch_wearablesizegrouppetite },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupPlus", sch_wearablesizegroupplus },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupRegular", sch_wearablesizegroupregular },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupShort", sch_wearablesizegroupshort },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupTall", sch_wearablesizegrouptall },
    { { 12, 0 }, { 0, 0 }, "WearableSizeGroupWomens", sch_wearablesizegroupwomens },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemAU", sch_wearablesizesystemau },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemBR", sch_wearablesizesystembr },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemCN", sch_wearablesizesystemcn },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemContinental", sch_wearablesizesystemcontinental },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemDE", sch_wearablesizesystemde },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemEN13402", sch_wearablesizesystemen13402 },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemEnumeration", sch_wearablesizesystemenumeration, s_schema, SF_ENUMERATION },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemEurope", sch_wearablesizesystemeurope },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemFR", sch_wearablesizesystemfr },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemGS1", sch_wearablesizesystemgs1 },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemIT", sch_wearablesizesystemit },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemJP", sch_wearablesizesystemjp },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemMX", sch_wearablesizesystemmx },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemUK", sch_wearablesizesystemuk },
    { { 12, 0 }, { 0, 0 }, "WearableSizeSystemUS", sch_wearablesizesystemus },
    { { 2, 0 }, { 0, 0 }, "Wearaction", sch_wearaction },
    { { 3, 5 }, { 0, 0 }, "WebAPI", sch_webapi },
    { { 0, 95 }, { 0, 0 }, "WebApplication", sch_webapplication },
    { { 5, 0 }, { 0, 0 }, "WebContent", sch_webcontent },
    { { 0, 10 }, { 0, 0 }, "WebPage", sch_webpage },
    { { 0, 10 }, { 0, 0 }, "WebPageElement", sch_webpageelement },
    { { 2, 0 }, { 0, 0 }, "WebSite", sch_website },
    { { 3, 0 }, { 0, 0 }, "Wednesday", sch_wednesday },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "WesternConventional", sch_westernconventional },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Wholesale", sch_wholesale },
    { { 0, 10 }, { 0, 0 }, "WholesaleStore", sch_wholesalers },
    { { 2, 0 }, { 0, 0 }, "Winaction", sch_winaction },
    { { 0, 10 }, { 0, 0 }, "Winery", sch_winery },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "Withdrawn", sch_withdrawn },
    { { 3, 9 }, { 0, 0 }, "WorkBasedProgram", sch_workbasedprogramme },
    { { 3, 5 }, { 0, 0 }, "WorkersUnion", sch_workersunion },
    { { 0, 10 }, { 0, 0 }, "WPAdBlock", sch_wpadblock },
    { { 0, 10 }, { 0, 0 }, "WPFooter", sch_wpfooter },
    { { 0, 10 }, { 0, 0 }, "WPHeader", sch_wpheader },
    { { 0, 10 }, { 0, 0 }, "WPSideBar", sch_wpsidebar },
    { { 1, 2 }, { 0, 0 }, "WriteAction", sch_writeaction },
    { { 3, 5 }, { 0, 0 }, "WritePermission", sch_writepermission },
    { { 3, 5 }, { 0, 0 }, "XPathType", sch_xpathtype },
    { { 0, 95, SV_NOT_3034 }, { 0, 0 }, "XRay", sch_xray },
    { { 0, 10 }, { 0, 0 }, "Zoo", sch_zoo },
    { { 3, 5 }, { 0, 0 }, "ZoneBoardingPolicy", sch_zoneboardingpolicy },

    // sd
    { { s_sd, 1, 0 }, { 0, 0 }, "Aggregate", sd_aggregate, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "BasicFederatedQuery", sd_basicfederatedquery, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "Dataset", sd_dataset, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "DereferencesURIs", sd_dereferencesuris, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "EmptyGraphs", sd_emptygraphs, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "EntailmentProfile", sd_entailmentprofile, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "EntailmentRegime", sd_entailmentregime, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "Feature", sd_feature, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "Function", sd_function, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "Graph", sd_graph, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "GraphCollection", sd_graphcollection, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "Language", sd_language, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "NamedGraph", sd_namedgraph, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "RequiresDataset", sd_requiresdataset, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "Service", sd_service, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "SPARQL10Query", sd_sparql10query, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "SPARQL11Query", sd_sparql11query, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "SPARQL11Update", sd_sparql11update, s_sd },
    { { s_sd, 1, 0 }, { 0, 0 }, "UnionDefaultGraph", sd_uniondefaultgraph, s_sd },

    // sioc
    { { s_sioc, 1, 0 }, { 0, 0 }, "Community", sioc_community, s_sioc },
    { { s_sioc, 1, 0 }, { 0, 0 }, "Container", sioc_container, s_sioc },
    { { s_sioc, 1, 0 }, { 0, 0 }, "Forum", sioc_forum, s_sioc },
    { { s_sioc, 1, 0 }, { 0, 0 }, "Item", sioc_item, s_sioc },
    { { s_sioc, 1, 0 }, { 0, 0 }, "Post", sioc_post, s_sioc },
    { { s_sioc, 1, 0 }, { 0, 0 }, "Role", sioc_role, s_sioc },
    { { s_sioc, 1, 0 }, { 0, 0 }, "Site", sioc_site, s_sioc },
    { { s_sioc, 1, 0 }, { 0, 0 }, "Space", sioc_space, s_sioc },
    { { s_sioc, 1, 0 }, { 0, 0 }, "Thread", sioc_thread, s_sioc },
    { { s_sioc, 1, 0 }, { 0, 0 }, "User", sioc_user, s_sioc },
    { { s_sioc, 1, 0 }, { 0, 0 }, "Usergroup", sioc_usergroup, s_sioc },

    // skos
    { { s_skos, 1, 0 }, { 0, 0 }, "Collection", skos_collection, s_skos },
    { { s_skos, 1, 0 }, { 0, 0 }, "Concept", skos_concept, s_skos },
    { { s_skos, 1, 0 }, { 0, 0 }, "ConceptScheme", skos_conceptscheme, s_skos },
    { { s_skos, 1, 0 }, { 0, 0 }, "OrderedCollection", skos_orderedcollection, s_skos },

    // skosxl
    { { s_skosxl, 1, 0 }, { 0, 0 }, "Label", skosxl_label, s_skosxl },

    // sosa
    { { s_sosa, 1, 0 }, { 0, 0 }, "Actuation", sosa_actuation, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "ActuatableProperty", sosa_actuatableproperty, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "Actuator", sosa_actuator, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "FeatureOfInterest", sosa_featureofinterest, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "ObservableProperty", sosa_observableproperty, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "Observation", sosa_observation, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "Platform", sosa_platform, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "Procedure", sosa_procedure, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "Result", sosa_result, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "Sample", sosa_sample, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "Sampler", sosa_sampler, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "Sampling", sosa_sampling, s_sosa },
    { { s_sosa, 1, 0 }, { 0, 0 }, "Sensor", sosa_sensor, s_sosa },

    // ssn
    { { s_ssn, 1, 0 }, { 0, 0 }, "Deployment", ssn_deployment, s_ssn },
    { { s_ssn, 1, 0 }, { 0, 0 }, "Input", ssn_input, s_ssn },
    { { s_ssn, 1, 0 }, { 0, 0 }, "Output", ssn_output, s_ssn },
    { { s_ssn, 1, 0 }, { 0, 0 }, "Property", ssn_property, s_ssn },
    { { s_ssn, 1, 0 }, { 0, 0 }, "Stimulus", ssn_stimulus, s_ssn },
    { { s_ssn, 1, 0 }, { 0, 0 }, "System", ssn_system, s_ssn },

    // time
    { { s_time, 1, 0 }, { 0, 0 }, "April", time_april, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "August", time_august, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "DateTimeDescription", time_datetimedescription, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "DateTimeInterval", time_datetimeinterval, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "DayOfWeek", time_dayofweek, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "December", time_december, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Duration", time_duration, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "DurationDescription", time_durationdescription, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "February", time_february, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Friday", time_friday, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "GeneralDateTimeDescription", time_datetimeinterval, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "GeneralDurationDescription", time_generaldurationdescription, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Instant", time_instant, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Interval", time_interval, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "January", time_january, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "July", time_july, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "June", time_june, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "March", time_march, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "May", time_may, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Monday", time_monday, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "MonthOfYear", time_monthofyear, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "November", time_november, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "October", time_october, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Position", time_position, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "ProperInterval", time_properinterval, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "September", time_september, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "TemporalDuration", time_temporalduration, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "TemporalEntity", time_temporalentity, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "TemporalPosition", time_temporalposition, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "TemporalUnit", time_temporalunit, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Thursday", time_thursday, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "TimePosition", time_timeposition, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Timezone", time_timezone, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "TRS", time_trs, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Tuesday", time_tuesday, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Saturday", time_saturday, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Sunday", time_sunday, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "unitday", time_unitday, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "unithour", time_unithour, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "unitminute", time_unitminute, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "unitmonth", time_unitmonth, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "unitsecond", time_unitsecond, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "unitweek", time_unitweek, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "unityear", time_unityear, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Wednesday", time_wednesday, s_time },
    { { s_time, 1, 0 }, { 0, 0 }, "Year", time_year, s_time },

    // data vocabulary
    { { s_v, 1, 0 }, { 0, 0 }, "Breadcrumb", v_breadcrumb, s_v },
    { { s_v, 1, 0 }, { 0, 0 }, "Event", v_event, s_v },
    { { s_v, 1, 0 }, { 0, 0 }, "Offer", v_offer, s_v },
    { { s_v, 1, 0 }, { 0, 0 }, "Offer-aggregate", v_offeraggregate, s_v },
    { { s_v, 1, 0 }, { 0, 0 }, "Organisation", v_organisation, s_v },
    { { s_v, 1, 0 }, { 0, 0 }, "Person", v_person, s_v },
    { { s_v, 1, 0 }, { 0, 0 }, "Product", v_product, s_v },
    { { s_v, 1, 0 }, { 0, 0 }, "Review", v_review, s_v },
    { { s_v, 1, 0 }, { 0, 0 }, "Review-aggregate", v_reviewaggregate, s_v },

    // vcard
    { { s_vcard, 4, 0 }, { 0, 0 }, "Acquaintance", vcard_cacquaintance, s_vcard },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, "ADRTYPES", vcard_adrtypes, s_vcard },
    { { s_vcard, 2, 0 }, { 0, 0 }, "Address", vcard_address, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Agent", vcard_cagent, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 0, 0 }, "BBS", vcard_bbs, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 0, 0 }, "Car", vcard_car, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 }, { 0, 0 }, "Cell", vcard_mobile, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Child", vcard_cchild, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Colleague", vcard_ccolleague, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Contact", vcard_ccontact, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Coresident", vcard_ccoresident, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Coworker", vcard_ccoworker, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Crush", vcard_ccrush, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Date", vcard_cdate, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 0, 0 }, "Dom", vcard_dom, s_vcard },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, "EMAILTYPES", vcard_emailtypes, s_vcard },
    { { s_vcard, 3, 0, SV_VC_DEP4 }, { 0, 0 }, "Email", vcard_email, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Emergency", vcard_cemergency, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 }, { 0, 0 }, "Fax", vcard_fax, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Female", vcard_female, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Friend", vcard_cfriend, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Gender", vcard_cgender, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Group", vcard_cgroup, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 }, { 0, 0 }, "Home", vcard_home, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Individual", vcard_individual, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 }, { 0, 0 }, "Internet", vcard_internet, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 0, 0 }, "Intl", vcard_intl, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 0, 0 }, "ISDN", vcard_isdn, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Kin", vcard_ckin, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Kind", vcard_vkind, s_vcard },
    { { s_vcard, 3, 0 }, { 0, 0 }, "Label", vcard_label, s_vcard },
    { { s_vcard, 2, 0 }, { 0, 0 }, "Location", vcard_location, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Male", vcard_male, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Me", vcard_cme, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Met", vcard_cmet, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 0, 0 }, "Modem", vcard_modem, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 0, 0 }, "Msg", vcard_msg, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Muse", vcard_cmuse, s_vcard },
    { { s_vcard, 2, 0 }, { 0, 0 }, "Name", vcard_name, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Neighbor", vcard_cneighbour, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "None", vcard_none, s_vcard },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, "NPROPERTIES", vcard_nproperties, s_vcard },
    { { s_vcard, 2, 0 }, { 0, 0 }, "Organization", vcard_organisation, s_vcard },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, "ORGPROPERTIES", vcard_orgproperties, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Other", vcard_cother, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 }, { 0, 0 }, "Pager", vcard_pager, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 0, 0 }, "Parcel", vcard_parcel, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Parent", vcard_cparent, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 01, 0 }, "PCS", vcard_pcs, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 }, { 0, 0 }, "Postal", vcard_postal, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 0, 0 }, "Pref", vcard_pref, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "RelatedType", vcard_relatedtype, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Sibling", vcard_csibling, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Spouse", vcard_cspouse, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Sweetheart", vcard_csweetheart, s_vcard },
    { { s_vcard, 2, 0, SV_VC_DEP4 }, { 0, 0 }, "Tel", vcard_tel, s_vcard },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, "TELTYPES", vcard_teltypes, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "TelephoneType", vcard_telephonetype, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Text", vcard_ctext, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Textphone", vcard_ctextphone, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Type", vcard_ctype, s_vcard },
    { { s_vcard, 1, 0 }, { s_vcard, 1, 0 }, "TZTYPES", vcard_tztypes, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "Unknown", vcard_unknown, s_vcard },
    { { s_vcard, 2, 0 }, { 0, 0 }, "VCard", vcard_vcard, s_vcard },
    { { s_vcard, 4, 0 }, { 0, 0 }, "vCardType", vcard_vcardtype, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 }, { 0, 0 }, "Video", vcard_video, s_vcard },
    { { s_vcard, 3, 0 }, { 0, 0 }, "Voice", vcard_voice, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 }, { 0, 0 }, "Work", vcard_work, s_vcard },
    { { s_vcard, 1, 0, SV_VC_NOT2 | SV_VC_DEP4 }, { 0, 0 }, "X400", vcard_x400, s_vcard },

    // void
    { { s_void, 1, 0 }, { 0, 0 }, "Dataset", void_dataset, s_void },
    { { s_void, 1, 0 }, { 0, 0 }, "DatasetDescription", void_datasetdescription, s_void },
    { { s_void, 1, 0 }, { 0, 0 }, "Linkset", void_linkset, s_void },
    { { s_void, 1, 0 }, { 0, 0 }, "TechnicalFeature", void_technicalfeature, s_void },

    // WDR
    { { s_wdr, 1, 0 }, { 0, 0 }, "AttributionType", wdr_attributiontype, s_wdr },
    { { s_wdr, 1, 0 }, { 0, 0 }, "IRISetType", wdr_irisettype, s_wdr },
    { { s_wdr, 1, 0 }, { 0, 0 }, "QueryContainsType", wdr_querycontainstype, s_wdr },

    // wdrs
    { { s_wdrs, 1, 0 }, { 0, 0 }, "Document", wdrs_document, s_wdrs },
    { { s_wdrs, 1, 0 }, { 0, 0 }, "Processor", wdrs_processor, s_wdrs },

    // living standard
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/hcalendar#vevent", mf_vevent, s_microformats, SF_NO_ITEMID },
    { { s_microformats, 1, 0 }, { 0, 0 }, "profile/hcard", mft_hcard, s_microformats, SF_NO_ITEMID },
    { { s_whatwg, 1, 0 }, { 0, 0 }, "work", wwg_work, s_whatwg },

    // XML schema types
    { { s_xsd, 1, 1 }, { 0, 0 }, "anyAtomicType", xsd_anyatomictype, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "anySimpleType", xsd_anysimpletype, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "anyType", xsd_anytype, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "anyURI", xsd_anyuri, s_xsd, MAKE_SIMPLE_TYPE (t_url) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "base64Binary", xsd_base64binary, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "boolean", xsd_boolean, s_xsd, MAKE_SIMPLE_TYPE (t_xsdbool) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "byte", xsd_byte, s_xsd, MAKE_SIMPLE_TYPE (t_integer) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "date", xsd_date, s_xsd, MAKE_SIMPLE_TYPE (t_just_date) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "dateTime", xsd_datetime, s_xsd, MAKE_SIMPLE_TYPE (t_datetime) },
    { { s_xsd, 1, 1 }, { 0, 0 }, "dateTimeDuration", xsd_datetimeduration, s_xsd, MAKE_SIMPLE_TYPE (t_duration) },
    { { s_xsd, 1, 1 }, { 0, 0 }, "dateTimeStamp", xsd_datetimestamp, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "decimal", xsd_decimal, s_xsd, MAKE_SIMPLE_TYPE (t_real) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "double", xsd_double, s_xsd, MAKE_SIMPLE_TYPE (t_real) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "duration", xsd_duration, s_xsd, MAKE_SIMPLE_TYPE (t_duration) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "ENTITIES", xsd_entities, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "ENTITY", xsd_entity, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "float", xsd_float, s_xsd, MAKE_SIMPLE_TYPE (t_real) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "gDay", xsd_gday, s_xsd, MAKE_SIMPLE_TYPE (t_day) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "gMonth", xsd_gmonth, s_xsd, MAKE_SIMPLE_TYPE (t_month) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "gMonthDay", xsd_gmonthday, s_xsd, MAKE_SIMPLE_TYPE (t_monthday) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "gYear", xsd_gyear, s_xsd, MAKE_SIMPLE_TYPE (t_year) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "gYearMonth", xsd_gyearmonth, s_xsd, MAKE_SIMPLE_TYPE (t_yearmonth) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "hexBinary", xsd_hexbinary, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "ID", xsd_id, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "IDREF", xsd_idref, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "IDREFS", xsd_idrefs, s_xsd, MAKE_SIMPLE_TYPE (t_idrefs) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "int", xsd_int, s_xsd, MAKE_SIMPLE_TYPE (t_integer) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "integer", xsd_integer, s_xsd, MAKE_SIMPLE_TYPE (t_integer) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "language", xsd_language, s_xsd, MAKE_SIMPLE_TYPE (t_lang) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "long", xsd_long, s_xsd, MAKE_SIMPLE_TYPE (t_integer) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "Name", xsd_name, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "NCName", xsd_ncname, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "negativeInteger", xsd_negativeinteger, s_xsd, MAKE_SIMPLE_TYPE (t_negative) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "NMTOKEN", xsd_nmtoken, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "NMTOKENS", xsd_nmtokens, s_xsd },
    { { s_xsd, 1, 0 }, { 0, 0 }, "nonNegativeInteger", xsd_nonnegativeinteger, s_xsd, MAKE_SIMPLE_TYPE (t_not_neg) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "nonPositiveInteger", xsd_nonpositiveinteger, s_xsd, MAKE_SIMPLE_TYPE (t_not_pos) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "normalizedString", xsd_normalisedstring, s_xsd, MAKE_SIMPLE_TYPE (t_text) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "NOTATION", xsd_notation, s_xsd, MAKE_SIMPLE_TYPE (t_namespace) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "positiveInteger", xsd_positiveinteger, s_xsd, MAKE_SIMPLE_TYPE (t_positive) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "QName", xsd_qname, s_xsd, MAKE_SIMPLE_TYPE (t_namespace) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "short", xsd_short, s_xsd, MAKE_SIMPLE_TYPE (t_short) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "string", xsd_string, s_xsd, MAKE_SIMPLE_TYPE (t_text) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "time", xsd_time, s_xsd, MAKE_SIMPLE_TYPE (t_just_time) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "token", xsd_token, s_xsd, MAKE_SIMPLE_TYPE (t_text) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "unsignedByte", xsd_unsignedbyte, s_xsd, MAKE_SIMPLE_TYPE (t_unsigned_byte) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "unsignedInt", xsd_unsignedint, s_xsd, MAKE_SIMPLE_TYPE (t_unsigned) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "unsignedLong", xsd_unsignedlong, s_xsd, MAKE_SIMPLE_TYPE (t_unsigned) },
    { { s_xsd, 1, 0 }, { 0, 0 }, "unsignedShort", xsd_unsignedshort, s_xsd, MAKE_SIMPLE_TYPE (t_unsigned_short) },
    { { s_xsd, 1, 1 }, { 0, 0 }, "yearMonthDuration", xsd_yearmonthduration, s_xsd },

    { { 0, 0 }, { 0, 0 }, "", sty_illegal } };

sch::sch (nitpick& nits, const html_version& v, const ::std::string& x, const e_schema root)
{   const e_schema_type es = parse (nits, v, x, root);
    if (es != sty_illegal) set (v, es); }

void sch::init (nitpick& nits)
{   symbol < schema_version, e_schema_type, e_schema, s_schema > ::
        init (nits, schema_type_symbol_table, sizeof (schema_type_symbol_table) / sizeof (symbol_entry < schema_version, e_schema_type, e_schema, s_schema >), true); }

e_schema_type sch::parse (nitpick& nits, const html_version& v, const ::std::string& x, const e_schema root)
{   ::std::string lc (trim_the_lot_off (x));
    if (lc.empty ())
    {   nits.pick (nit_empty, ed_jul20, "5.2.2 Items", es_error, ec_microdata, "A schema type cannot be empty");
        return sty_illegal; }
    if (lc.at (0) == '/') lc = lc.substr (1);
    if (! v.xhtml ()) lc = ::boost::algorithm::to_lower_copy (lc);
    symbol < schema_version, e_schema_type, e_schema, s_schema > s (default_schema, lc, root);
    for (int i = s_none; (s.unknown () || (s.get () == sty_context)) && (i < s_error); ++i)
        if (i != root)
            s.reset (symbol < schema_version, e_schema_type, e_schema, s_schema > (default_schema, lc, static_cast < e_schema > (i)));
    if (s.unknown () || (s.get () == sty_context))
    {   if (! check_identifier_spelling (nits, v, lc))
            nits.pick (nit_unrecognised_schema, es_error, ec_microdata, "Unrecognised type ", quote (x)); }
    else
    {   if (s.first ().deprecated ())
            nits.pick (nit_deprecated_schema, es_warning, ec_schema, quote (x), " is deprecated");
        const schema_version sv = corresponding_schema_version (s.ns (), v);
        if (is_faux_schema (s.ns ()))
            nits.pick (nit_unrecognised_schema, es_error, ec_schema, quote (x), " requires a schema");
        else if (may_apply (sv, s.first (), s.last ()))
        {   if ((s.ns () == s_v) && (v >= html_jan12))
                nits.pick (nit_data_vocabulary, es_warning, ec_schema, "data-vocabulary is deprecated in ", v.report (), " in favour of schema.org");
            return s.get (); }
        else nits.pick (nit_unrecognised_schema, es_error, ec_schema, quote (x), " is invalid in ", sv.report ()); }
    return sty_illegal; }

e_schema sch::root () const noexcept
{ return first ().root (); }

e_schema sch::root (const e_schema_type st)
{ return first_version (st).root (); }

bool sch::enumerated () const noexcept
{ return enumerated_schema_type (flags ()); }

bool sch::has_simple_type () const noexcept
{ return has_simple_schema_type (flags ()); }

bool sch::external_enumerated () const noexcept
{ return external_enumerated_schema_type (flags ()); }

e_type sch::get_simple_type () const noexcept
{ return get_simple_schema_type (flags ()); }
